Package: seriation
Type: Package
Title: Infrastructure for Ordering Objects Using Seriation
Version: 1.5.8
Date: 2025-08-20
Authors@R: c(
	person("Michael", "Hahsler", role = c("aut", "cre", "cph"),
	  email = "mhahsler@lyle.smu.edu",
	  comment = c(ORCID = "0000-0003-2716-1405")),
	person("Christian", "Buchta", role = c("aut", "cph")),
	person("Kurt", "Hornik", role = c("aut", "cph"),
	  comment = c(ORCID = "0000-0003-4198-9911")),
	person("David", "Barnett", role = c("ctb")),
	person("Michael", "Brusco", role = c("ctb", "cph")),
	person("Michael", "Friendly", role = c("ctb")),
	person("Hans-Friedrich", "Koehn", role = c("ctb", "cph")),
	person("Fionn", "Murtagh", role = c("ctb", "cph")),
	person("Stephanie", "Stahl", role = c("ctb", "cph")))
Description: Infrastructure for ordering objects with an implementation of several
    seriation/sequencing/ordination techniques to reorder matrices, dissimilarity
    matrices, and dendrograms. Also provides (optimally) reordered heatmaps,
    color images and clustering visualizations like dissimilarity plots, and
    visual assessment of cluster tendency plots (VAT and iVAT). Hahsler et al (2008) <doi:10.18637/jss.v025.i03>.
Classification/ACM: G.1.6, G.2.1, G.4
URL: https://github.com/mhahsler/seriation
BugReports: https://github.com/mhahsler/seriation/issues
Depends: R (>= 2.14.0)
Imports: ca, cluster, colorspace, foreach, gclus, grDevices, grid,
        MASS, qap, registry, stats, TSP, vegan
Suggests: dbscan, DendSer, dendextend, doParallel, GA, ggplot2, keras,
        Rtsne, scales, smacof, tensorflow, testthat, umap
Encoding: UTF-8
RoxygenNote: 7.3.2
License: GPL-3
Copyright: The code in src/bbwrcg.f, src/arsa.f and src/bburcg.f are
        Copyright (C) 2005 M. Brusco, H.F. Koehn, and S. Stahl. All
        other code is Copyright (C) Michael Hahsler, Christian Buchta,
        and Kurt Hornik.
Collate: 'AAA_check_installed.R' 'AAA_color_palette.R' 'AAA_defaults.R'
        'AAA_map.R' 'AAA_parameters.R' 'AAA_registry_criterion.R'
        'AAA_registry_seriate.R' 'AAA_seriation-package.R'
        'Chameleon.R' 'Irish.R' 'Munsingen.R' 'Psych24.R'
        'SupremeCourt.R' 'Townships.R' 'VAT.R' 'Wood.R' 'Zoo.R'
        'bertinplot.R' 'criterion.R' 'criterion.array.R'
        'criterion.dist.R' 'criterion.matrix.R' 'dissplot.R'
        'get_order.R' 'ggVAT.R' 'ggbertinplot.R' 'ggdissplot.R'
        'hmap.R' 'gghmap.R' 'pimage.R' 'ggpimage.R' 'grid_helpers.R'
        'lines_and_ordered_data.R' 'lle.R' 'permute.R'
        'register_DendSer.R' 'register_GA.R' 'register_optics.R'
        'register_smacof.R' 'register_tsne.R' 'register_umap.R'
        'reorder.hclust.R' 'robinson.R' 'ser_dist.R'
        'ser_permutation.R' 'ser_permutation_vector.R'
        'ser_permutation_vector2matrix.R' 'seriate.R' 'seriate.dist.R'
        'seriate.matrix.R' 'seriate.array.R' 'seriate.data.frame.R'
        'seriate.table.R' 'seriate_AOE.R' 'seriate_ARSA_Branch-Bound.R'
        'seriate_TSP.R' 'seriate_BEA.R' 'seriate_BK.R' 'seriate_CA.R'
        'seriate_GSA.R' 'seriate_HC.R' 'seriate_LLE.R' 'seriate_MDS.R'
        'seriate_Mean.R' 'seriate_PCA.R' 'seriate_QAP.R'
        'seriate_R2E.R' 'seriate_SGD.R' 'seriate_SPIN.R'
        'seriate_VAT.R' 'seriate_best.R' 'seriate_enumerate.R'
        'seriate_heatmap.R' 'seriate_identity.R' 'seriate_random.R'
        'seriate_reverse.R' 'seriate_spectral.R' 'seriate_vegan.R'
        'uniscale.R'
NeedsCompilation: yes
Packaged: 2025-08-20 17:01:36 UTC; mhahsler
Author: Michael Hahsler [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-2716-1405>),
  Christian Buchta [aut, cph],
  Kurt Hornik [aut, cph] (ORCID: <https://orcid.org/0000-0003-4198-9911>),
  David Barnett [ctb],
  Michael Brusco [ctb, cph],
  Michael Friendly [ctb],
  Hans-Friedrich Koehn [ctb, cph],
  Fionn Murtagh [ctb, cph],
  Stephanie Stahl [ctb, cph]
Maintainer: Michael Hahsler <mhahsler@lyle.smu.edu>
Repository: CRAN
Date/Publication: 2025-08-20 18:10:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 21:43:39 UTC; unix
Archs: seriation.so.dSYM

Package: seriation
Type: Package
Title: Infrastructure for Ordering Objects Using Seriation
Version: 1.5.8
Date: 2025-08-20
Authors@R: c(
	person("Michael", "Hahsler", role = c("aut", "cre", "cph"),
	  email = "mhahsler@lyle.smu.edu",
	  comment = c(ORCID = "0000-0003-2716-1405")),
	person("Christian", "Buchta", role = c("aut", "cph")),
	person("Kurt", "Hornik", role = c("aut", "cph"),
	  comment = c(ORCID = "0000-0003-4198-9911")),
	person("David", "Barnett", role = c("ctb")),
	person("Michael", "Brusco", role = c("ctb", "cph")),
	person("Michael", "Friendly", role = c("ctb")),
	person("Hans-Friedrich", "Koehn", role = c("ctb", "cph")),
	person("Fionn", "Murtagh", role = c("ctb", "cph")),
	person("Stephanie", "Stahl", role = c("ctb", "cph")))
Description: Infrastructure for ordering objects with an implementation of several
    seriation/sequencing/ordination techniques to reorder matrices, dissimilarity
    matrices, and dendrograms. Also provides (optimally) reordered heatmaps,
    color images and clustering visualizations like dissimilarity plots, and
    visual assessment of cluster tendency plots (VAT and iVAT). Hahsler et al (2008) <doi:10.18637/jss.v025.i03>.
Classification/ACM: G.1.6, G.2.1, G.4
URL: https://github.com/mhahsler/seriation
BugReports: https://github.com/mhahsler/seriation/issues
Depends: R (>= 2.14.0)
Imports: ca, cluster, colorspace, foreach, gclus, grDevices, grid,
        MASS, qap, registry, stats, TSP, vegan
Suggests: dbscan, DendSer, dendextend, doParallel, GA, ggplot2, keras,
        Rtsne, scales, smacof, tensorflow, testthat, umap
Encoding: UTF-8
RoxygenNote: 7.3.2
License: GPL-3
Copyright: The code in src/bbwrcg.f, src/arsa.f and src/bburcg.f are
        Copyright (C) 2005 M. Brusco, H.F. Koehn, and S. Stahl. All
        other code is Copyright (C) Michael Hahsler, Christian Buchta,
        and Kurt Hornik.
Collate: 'AAA_check_installed.R' 'AAA_color_palette.R' 'AAA_defaults.R'
        'AAA_map.R' 'AAA_parameters.R' 'AAA_registry_criterion.R'
        'AAA_registry_seriate.R' 'AAA_seriation-package.R'
        'Chameleon.R' 'Irish.R' 'Munsingen.R' 'Psych24.R'
        'SupremeCourt.R' 'Townships.R' 'VAT.R' 'Wood.R' 'Zoo.R'
        'bertinplot.R' 'criterion.R' 'criterion.array.R'
        'criterion.dist.R' 'criterion.matrix.R' 'dissplot.R'
        'get_order.R' 'ggVAT.R' 'ggbertinplot.R' 'ggdissplot.R'
        'hmap.R' 'gghmap.R' 'pimage.R' 'ggpimage.R' 'grid_helpers.R'
        'lines_and_ordered_data.R' 'lle.R' 'permute.R'
        'register_DendSer.R' 'register_GA.R' 'register_optics.R'
        'register_smacof.R' 'register_tsne.R' 'register_umap.R'
        'reorder.hclust.R' 'robinson.R' 'ser_dist.R'
        'ser_permutation.R' 'ser_permutation_vector.R'
        'ser_permutation_vector2matrix.R' 'seriate.R' 'seriate.dist.R'
        'seriate.matrix.R' 'seriate.array.R' 'seriate.data.frame.R'
        'seriate.table.R' 'seriate_AOE.R' 'seriate_ARSA_Branch-Bound.R'
        'seriate_TSP.R' 'seriate_BEA.R' 'seriate_BK.R' 'seriate_CA.R'
        'seriate_GSA.R' 'seriate_HC.R' 'seriate_LLE.R' 'seriate_MDS.R'
        'seriate_Mean.R' 'seriate_PCA.R' 'seriate_QAP.R'
        'seriate_R2E.R' 'seriate_SGD.R' 'seriate_SPIN.R'
        'seriate_VAT.R' 'seriate_best.R' 'seriate_enumerate.R'
        'seriate_heatmap.R' 'seriate_identity.R' 'seriate_random.R'
        'seriate_reverse.R' 'seriate_spectral.R' 'seriate_vegan.R'
        'uniscale.R'
NeedsCompilation: yes
Packaged: 2025-08-20 17:01:36 UTC; mhahsler
Author: Michael Hahsler [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-2716-1405>),
  Christian Buchta [aut, cph],
  Kurt Hornik [aut, cph] (ORCID: <https://orcid.org/0000-0003-4198-9911>),
  David Barnett [ctb],
  Michael Brusco [ctb, cph],
  Michael Friendly [ctb],
  Hans-Friedrich Koehn [ctb, cph],
  Fionn Murtagh [ctb, cph],
  Stephanie Stahl [ctb, cph]
Maintainer: Michael Hahsler <mhahsler@lyle.smu.edu>
Repository: CRAN
Date/Publication: 2025-08-20 18:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 05:23:40 UTC; unix
Archs: seriation.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method("[",ser_permutation)
S3method(c,ser_permutation)
S3method(c,ser_permutation_vector)
S3method(criterion,array)
S3method(criterion,data.frame)
S3method(criterion,default)
S3method(criterion,dist)
S3method(criterion,matrix)
S3method(criterion,table)
S3method(get_order,default)
S3method(get_order,dendrogram)
S3method(get_order,hclust)
S3method(get_order,integer)
S3method(get_order,numeric)
S3method(get_order,ser_permutation)
S3method(get_order,ser_permutation_vector)
S3method(ggpimage,default)
S3method(ggpimage,dist)
S3method(ggpimage,matrix)
S3method(length,ser_permutation_vector)
S3method(permute,array)
S3method(permute,character)
S3method(permute,data.frame)
S3method(permute,default)
S3method(permute,dendrogram)
S3method(permute,dist)
S3method(permute,hclust)
S3method(permute,list)
S3method(permute,matrix)
S3method(permute,numeric)
S3method(permute,table)
S3method(pimage,data.frame)
S3method(pimage,default)
S3method(pimage,dist)
S3method(pimage,matrix)
S3method(pimage,table)
S3method(plot,reordered_cluster_dissimilarity_matrix)
S3method(print,criterion_method)
S3method(print,reordered_cluster_dissimilarity_matrix)
S3method(print,ser_permutation)
S3method(print,ser_permutation_vector)
S3method(print,seriation_method)
S3method(reorder,hclust)
S3method(rev,ser_permutation_vector)
S3method(seriate,array)
S3method(seriate,data.frame)
S3method(seriate,default)
S3method(seriate,dist)
S3method(seriate,matrix)
S3method(seriate,table)
S3method(summary,ser_permutation)
S3method(summary,ser_permutation_vector)
export(LS_insert)
export(LS_mixed)
export(LS_reverse)
export(LS_swap)
export(MDS_stress)
export(VAT)
export(bertin_cut_line)
export(bertinplot)
export(bluered)
export(blues)
export(create_lines_data)
export(create_ordered_data)
export(criterion)
export(dissplot)
export(gaperm_mixedMutation)
export(get_config)
export(get_criterion_method)
export(get_method)
export(get_order)
export(get_permutation_matrix)
export(get_rank)
export(get_seriation_method)
export(ggVAT)
export(ggbertinplot)
export(ggdissplot)
export(gghmap)
export(ggiVAT)
export(ggpimage)
export(grays)
export(greenred)
export(greens)
export(greys)
export(hmap)
export(iVAT)
export(is.robinson)
export(list_criterion_methods)
export(list_seriation_methods)
export(lle)
export(panel.bars)
export(panel.blocks)
export(panel.circles)
export(panel.lines)
export(panel.rectangles)
export(panel.squares)
export(panel.tiles)
export(path_dist)
export(permutation_matrix2vector)
export(permutation_vector2matrix)
export(permute)
export(pimage)
export(plot_config)
export(random.robinson)
export(reds)
export(register_DendSer)
export(register_GA)
export(register_optics)
export(register_smacof)
export(register_tsne)
export(register_umap)
export(registry_criterion)
export(registry_seriate)
export(ser_align)
export(ser_cor)
export(ser_dist)
export(ser_permutation)
export(ser_permutation_vector)
export(seriate)
export(seriate_best)
export(seriate_improve)
export(seriate_rep)
export(set_criterion_method)
export(set_seriation_method)
export(uniscale)
import("TSP")
import("grid")
import(registry)
importFrom(ca,ca)
importFrom(foreach,`%do%`)
importFrom(foreach,`%dopar%`)
importFrom(foreach,times)
importFrom(graphics,plot)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(stats,as.dist)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,order.dendrogram)
importFrom(stats,prcomp)
importFrom(stats,reorder)
importFrom(stats,rnorm)
importFrom(stats,runif)
useDynLib(seriation, .registration=TRUE)

# seriation
This software is an efficient implementation of the [Seriation](http://www.jstatsoft.org/v25/i03) problem which 'finds a suitable linear order for a set of objects'. It has been used to order a network of proteins such that 'related' nodes are closer in the other.

[![Build Status](https://travis-ci.org/amamory/seriation.svg?branch=master)](https://travis-ci.org/amamory/seriation)
[![DOI](https://zenodo.org/badge/92440644.svg)](https://zenodo.org/badge/latestdoi/92440644)


## Authors

The Seriation Package was developed by [Felipe Kuentzer](http://lattes.cnpq.br/1979213773480902), in collaboration with 
Douglas G. Ávila, Alexandre Pereira, Gabriel Perrone, Samoel da Silva, [Alexandre Amory](http://lattes.cnpq.br/2609000874577720), and [Rita de Almeida](http://lattes.cnpq.br/4672766298301524).

**Contact information**: Alexandre Amory (*amamory @ gmail com*)

## Inputs

The input file is a textual file describing an undirected network of nodes (in our examples the nodes are protein names). Example:
<pre>
L7007 L7008
L7008 L7007
L7010 L7011
L7011 L7010
L7014 L7015
L7015 L7014
L7017 Z1275
</pre>

In the tab Files you can find networks for different species such as [Escherichia coli](data/Escherichia_coli.dat), [Mus musculus](data/Mus_musculus.dat), [Saccharomyces cerevisiae](data/Saccharomyces_cerevisiae.dat), [Homo sapiens](data/Homo_sapiens.dat), among others.

## Outputs

The output is a text file with the order of the network nodes. Example:

<pre>
Protein	dim1
Z5822	0
Z5823	1
Z2911	2
Z2910	3
Z2909	4
Z4123	5
Z4124	6
Z3105	7
Z3106	8
...
</pre>

The following image represents the Homo sapiens network with a *random ordering*.

![initial](initial.png)

The next image represents the Homo sapiens network **'seriated'**.

![final](final.png)


## Download and Instalation

The Seriation Package is developed in C and tested on Ubuntu 14.04.
* Download the [package](https://github.com/amamory/seriation/releases/latest) .
* Is recommended to update your packages before the instalation:
> sudo apt-get install update
* To install, you can double-click it or execute:
> sudo dpkg -i cfm-seriation_1.0-1_amd64.deb
* In case of missing dependencies, try: 
> sudo apt-get install -f
* To unistall:
> sudo dpkg -r cfm-seriation

* this distribution has the following files

<pre>
/usr/share/cfm-seriation/bin/             Executable file
/usr/share/cfm-seriation/etc/             Auxiliar used to plot charts with GNUPLOT
/usr/share/cfm-seriation/data/            Biological input networks
/usr/share/cfm-seriation/src/             Source code in C
</pre>


## Download and Compilation

> sudo apt-get install git

> git clone https://github.com/amamory/seriation.git

> cd seriation

> gcc cfm-seriation.c -lm -lpthread -lrt -o cfm-seriation


## How to Use

type 'cfm-seriation' to show the options:

<pre>
cfm-seriation

Usage: cfm-seriation [OPTION...]

 Seriation Parameters:
   f=[NETWORK FILE].dat       Network file path name
   o=[ORDER FILE].dat         Apply initial order
   i=[INTERVAL]               Number of isothermal steps
   m=[STEPS]                  Number of steps
   c=[FACTOR]                 Cooling factor
   a=[ALPHA]                  Alpha value
   p=[PERCENTUAL]             Percentual energy for initial temperature
   s=[SEDD]                   Random seed
   P                          Plot graphs
   v                          Generate video
</pre>

type to execute the seriation. This process can take about 12 minutes, depending on the CPU.

> ./cfm-seriation f=data/Homo_sapiens.dat m=3000 P

In case you want a video of the process, type to execute the seriation. 

> ./cfm-seriation f=/usr/share/cfm-seriation/data/Homo_sapiens.dat m=3000 P v

This will consume some extra time.

<pre>
Usage: cfm-seriation [OPTION...]

 Seriation Parameters:
   f=[NETWORK FILE].dat       Network file path name
   o=[ORDER FILE].dat         Apply initial order
   i=[INTERVAL]               Number of isothermal steps
   m=[STEPS]                  Number of steps
   c=[FACTOR]                 Cooling factor
   a=[ALPHA]                  Alpha value
   p=[PERCENTUAL]             Percentual energy for initial temperature
   s=[SEDD]                   Random seed
   P                          Plot graphs
   v                          Generate video

Reading file...
	Proteins: 9684
	Interactions: 163509
Applying random order...
Saving and plotting initial order...
INITIAL Energy: 4123514310
Ordering...
100% [====================================================================================================]
</pre>

For a quicker test you can execute smaller dataset, like the Escherichia Coli.

> ./cfm-seriation f=Escherichia_coli.dat

<pre>
Reading file...
	Proteins: 3598
	Interactions: 13687
Applying random order...
Saving initial order...
INITIAL Energy: 129449102
Ordering...
100% [====================================================================================================]
FINAL Energy: 129025784
Saving final order...
Done!
</pre>

The results are save in a different directory for each execution. 

## Further Information


* [Optimization and Analysis of Seriation Algorithm for Ordering Protein Networks](http://ieeexplore.ieee.org/document/7033586/). This paper describes the optimizations implemented in this package.


* Felipe's Master Thesis [Otimização e análise de algoritmos de ordenamento de redes proteicas](http://hdl.handle.net/10923/6663). Full description of the optimizations implemented in this package (in Portuguese).

## License

The source code is distributed under the terms of the GNU General Public License v3 [GPL](http://www.gnu.org/copyleft/gpl.html).

## How to Cite this Package

If you are using this package on your research, please cite our paper:
* [Optimization and Analysis of Seriation Algorithm for Ordering Protein Networks](http://ieeexplore.ieee.org/document/7033586/)

<pre>
KUENTZER, Felipe A. et al. Optimization and analysis of seriation algorithm for ordering protein networks. 
In: IEEE International Conference on Bioinformatics and Bioengineering (BIBE), 2014. p. 231-237.
</pre>

## Where Seriation is Used

If you are using the Seriation Package, please send an email to *alexandre.amory at pucrs.br* so we can update this list of users:
* [Transcriptogrammer](http://lief.if.ufrgs.br/pub/biosoftwares/transcriptogramer/)

## Similar Packages

* [R Package seriation](http://www.jstatsoft.org/v25/i03), available at http://cran.r-project.org/web/packages/seriation/index.html.

# <img src="man/figures/logo.svg" align="right" height="139" /> R package seriation - Infrastructure for Ordering Objects Using Seriation

[![Package on
CRAN](https://www.r-pkg.org/badges/version/seriation)](https://CRAN.R-project.org/package=seriation)
[![CRAN RStudio mirror
downloads](https://cranlogs.r-pkg.org/badges/seriation)](https://CRAN.R-project.org/package=seriation)
![License](https://img.shields.io/cran/l/seriation)
[![Anaconda.org](https://anaconda.org/conda-forge/r-seriation/badges/version.svg)](https://anaconda.org/conda-forge/r-seriation)
[![r-universe
status](https://mhahsler.r-universe.dev/badges/seriation)](https://mhahsler.r-universe.dev/seriation)
[![StackOverflow](https://img.shields.io/badge/stackoverflow-seriation+r-orange.svg)](https://stackoverflow.com/questions/tagged/seriation+r)

## Introduction

Seriation arranges a set of objects into a linear order given available
data with the goal of revealing structural information. This package
provides the infrastructure for ordering objects with an implementation
of many
[seriation](https://en.wikipedia.org/wiki/Seriation_(archaeology))/[ordination](https://en.wikipedia.org/wiki/Ordination_(statistics))
techniques to reorder data matrices, dissimilarity matrices, correlation
matrices, and dendrograms (see below for a complete list). The package
provides several visualizations (grid and ggplot2) to reveal structural
information, including permuted image plots, reordered heatmaps, Bertin
plots, clustering visualizations like dissimilarity plots, and visual
assessment of cluster tendency plots (VAT and iVAT).

Here are some quick guides on applications of seriation:

- [Introduction the R package
  seriation](https://cran.r-project.org/package=seriation/vignettes/seriation.pdf)
- [How to reorder
  heatmaps](https://mhahsler.github.io/seriation/heatmaps.html)
- [How to reorder correlation
  matrices](https://mhahsler.github.io/seriation/correlation_matrix.html)
- [How to evaluate clusters using dissimilarity
  plots](https://mhahsler.github.io/seriation/clustering.html)

Implemented seriation methods and criteria:

- [Documentation of the implemented seriation
  methods](https://mhahsler.github.io/seriation/seriation_methods.html)
- [Documentation of the implemented seriation
  criteria](https://mhahsler.github.io/seriation/seriation_criteria.html)
- [A visual comparison between seriation
  methods](https://mhahsler.github.io/seriation/comparison.html)

The following R packages use `seriation`:
[adepro](https://CRAN.R-project.org/package=adepro),
[arulesViz](https://CRAN.R-project.org/package=arulesViz),
[baizer](https://CRAN.R-project.org/package=baizer),
[ChemoSpec](https://CRAN.R-project.org/package=ChemoSpec),
[corrgram](https://CRAN.R-project.org/package=corrgram),
[corrplot](https://CRAN.R-project.org/package=corrplot),
[corrr](https://CRAN.R-project.org/package=corrr),
[dendextend](https://CRAN.R-project.org/package=dendextend),
[DendSer](https://CRAN.R-project.org/package=DendSer),
[dendsort](https://CRAN.R-project.org/package=dendsort),
[disclapmix](https://CRAN.R-project.org/package=disclapmix),
[elaborator](https://CRAN.R-project.org/package=elaborator),
[flexclust](https://CRAN.R-project.org/package=flexclust),
[GAPR](https://CRAN.R-project.org/package=GAPR),
[ggraph](https://CRAN.R-project.org/package=ggraph),
[heatmaply](https://CRAN.R-project.org/package=heatmaply),
[MEDseq](https://CRAN.R-project.org/package=MEDseq),
[ockc](https://CRAN.R-project.org/package=ockc),
[protti](https://CRAN.R-project.org/package=protti),
[RMaCzek](https://CRAN.R-project.org/package=RMaCzek),
[SFS](https://CRAN.R-project.org/package=SFS),
[tidygraph](https://CRAN.R-project.org/package=tidygraph),
[treeheatr](https://CRAN.R-project.org/package=treeheatr),
[vcdExtra](https://CRAN.R-project.org/package=vcdExtra)

To cite package ‘seriation’ in publications use:

> Hahsler M, Hornik K, Buchta C (2008). “Getting things in order: An
> introduction to the R package seriation.” *Journal of Statistical
> Software*, *25*(3), 1-34. ISSN 1548-7660, <doi:10.18637/jss.v025.i03>
> <https://doi.org/10.18637/jss.v025.i03>.

    @Article{,
      title = {Getting things in order:  An introduction to the R package seriation},
      author = {Michael Hahsler and Kurt Hornik and Christian Buchta},
      year = {2008},
      journal = {Journal of Statistical Software},
      volume = {25},
      number = {3},
      pages = {1--34},
      doi = {10.18637/jss.v025.i03},
      month = {March},
      issn = {1548-7660},
    }

## Available seriation methods to reorder dissimilarity data

Seriation methods for dissimilarity data reorder the set of objects in
the data. The methods fall into several groups based on the criteria
they try to optimize, constraints (like dendrograms), and the
algorithmic approach.

### Dendrogram leaf order

These methods create a dendrogram using hierarchical clustering and then
derive the seriation order from the leaf order in the dendrogram. Leaf
reordering may be applied.

- **DendSer** - Dendrogram seriation heuristic to optimize various
  criteria
- **GW** - Hierarchical clustering reordered by the Gruvaeus and Wainer
  heuristic
- **HC** - Hierarchical clustering (single link, avg. link, complete
  link)
- **OLO** - Hierarchical clustering with optimal leaf ordering

### Dimensionality reduction

Find a seriation order by reducing the dimensionality to 1 dimension.
This is typically done by minimizing a stress measure or the
reconstruction error.

- **MDS** - classical metric multidimensional scaling
- **MDS_angle** - order by the angular order in the 2D MDS projection
  space split by the larges gap
- **isoMDS** - 1D Krusakl’s non-metric multidimensional scaling
- **isomap** - 1D isometric feature mapping ordination
- **monoMDS** - order along 1D global and local non-metric
  multidimensional scaling using monotone regression (NMDS)
- **metaMDS** - 1D non-metric multidimensional scaling (NMDS) with
  stable solution from random starts
- **Sammon** - Order along the 1D Sammon’s non-linear mapping
- **smacof** - 1D MDS using majorization (ratio MDS, interval MDS,
  ordinal MDS)
- **TSNE** - Order along the 1D t-distributed stochastic neighbor
  embedding (t-SNE)
- **UMAP** - Order along the 1D embedding produced by uniform manifold
  approximation and projection

### Optimization

These methods try to optimize a seriation criterion directly, typically
using a heuristic approach.

- **ARSA** - optimize the linear seriation criterion using simulated
  annealing  
- **Branch-and-bound** to minimize the unweighted/weighted column
  gradient
- **GA** - Genetic algorithm with warm start to optimize any seriation
  criteria
- **GSA** - General simulated annealing to optimize any seriation
  criteria
- **SGD** - stochastic gradient descent to find a local optimum given an
  initial order and a seriation criterion.
- **QAP** - Quadratic assignment problem heuristic (optimizes 2-SUM,
  linear seriation, inertia, banded anti-Robinson form)
- **Spectral** seriation to optimize the 2-SUM criterion (unnormalized,
  normalized)
- **TSP** - Traveling salesperson solver to minimize the Hamiltonian
  path length

### Other Methods

- **Identity** permutation
- **OPTICS** - Order of ordering points to identify the clustering
  structure
- **R2E** - Rank-two ellipse seriation
- **Random** permutation
- **Reverse** order
- **SPIN** - Sorting points into neighborhoods (neighborhood algorithm,
  side-to-site algorithm)
- **VAT** - Order of the visual assessment of clustering tendency

A detailed comparison of the most popular methods is available in the
paper [An experimental comparison of seriation methods for one-mode
two-way data.](http://dx.doi.org/10.1016/j.ejor.2016.08.066) (read the
[preprint](https://michael.hahsler.net/research/paper/EJOR_seriation_2016.pdf)).

## Available seriation methods to reorder data matrices, count tables, and data.frames

For matrices, rows and columns are reordered.

### Seriating rows and columns simultaneously

Row and column order influence each other.

- **BEA** - Bond Energy Algorithm to maximize the measure of
  effectiveness (ME)
- **BEA_TSP** - TSP to optimize the measure of effectiveness
- **BK_unconstrained** - Algorithm by Brower and Kyle (1988) to arrange
  binary matrices.
- **CA** - calculates a correspondence analysis of a matrix of
  frequencies (count table) and reorders according to the scores on a
  correspondence analysis dimension

### Seriating rows and columns separately using dissimilarities

- **Heatmap** - reorders rows and columns independently by calculating
  row/column distances and then applying a seriation method for
  dissimilarities (see above)

### Seriate rows in a data matrix

These methods need access to the data matrix instead of dissimilarities
to reorder objects (rows). The same approach can be applied to columns.

- **PCA_angle** - order by the angular order in the 2D PCA projection
  space split by the larges gap
- **LLE** reorder along a 1D locally linear embedding
- **Means** - reorders using row means
- **PCA** - orders along the first principal component
- **TSNE** - Order along the 1D t-distributed stochastic neighbor
  embedding (t-SNE)
- **UMAP** - Order along the 1D embedding produced by uniform manifold
  approximation and projection

### Other methods

- **AOE** - order by the angular order of the first two eigenvectors for
  correlation matrices.
- **Identity** permutation
- **Random** permutation
- **Reverse** order

## Installation

**Stable CRAN version:** Install from within R with

``` r
install.packages("seriation")
```

**Current development version:** Install from
[r-universe.](https://mhahsler.r-universe.dev/seriation)

``` r
install.packages("seriation",
    repos = c("https://mhahsler.r-universe.dev",
              "https://cloud.r-project.org/"))
```

## Usage

The used example dataset contains the joint probability of disagreement
between Supreme Court Judges from 1995 to 2002. The goal is to reveal
structural information in this data. We load the library, read the data,
convert the data to a distance matrix, and then use the default
seriation method to reorder the objects.

``` r
library(seriation)
data("SupremeCourt")

d <- as.dist(SupremeCourt)
d
```

    ##           Breyer Ginsburg Kennedy OConnor Rehnquist Scalia Souter Stevens
    ## Ginsburg   0.120                                                         
    ## Kennedy    0.250    0.267                                                
    ## OConnor    0.209    0.252   0.156                                        
    ## Rehnquist  0.299    0.308   0.122   0.162                                
    ## Scalia     0.353    0.370   0.188   0.207     0.143                      
    ## Souter     0.118    0.096   0.248   0.220     0.293  0.338               
    ## Stevens    0.162    0.145   0.327   0.329     0.402  0.438  0.169        
    ## Thomas     0.359    0.368   0.177   0.205     0.137  0.066  0.331   0.436

``` r
order <- seriate(d)
order
```

    ## object of class 'ser_permutation', 'list'
    ## contains permutation vectors for 1-mode data
    ## 
    ##   vector length seriation method
    ## 1             9         Spectral

Here is the resulting permutation vector.

``` r
get_order(order)
```

    ##    Scalia    Thomas Rehnquist   Kennedy   OConnor    Souter    Breyer  Ginsburg 
    ##         6         9         5         3         4         7         1         2 
    ##   Stevens 
    ##         8

Next, we visualize the original and permuted distance matrix.

``` r
pimage(d, main = "Judges (original alphabetical order)")
pimage(d, order, main = "Judges (reordered by seriation)")
```

<img src="inst/README_files/seriation-1.png" width="50%" /><img src="inst/README_files/seriation-2.png" width="50%" />

Darker squares around the main diagonal indicate groups of similar
objects. After seriation, two groups are visible.

We can compare the available seriation criteria. Seriation improves all
measures. Note that some measures are merit measures while others
represent cost. See the manual page for details.

``` r
rbind(alphabetical = criterion(d), seriated = criterion(d, order))
```

    ##              2SUM AR_deviations AR_events BAR Gradient_raw Gradient_weighted
    ## alphabetical  872        10.304        80 1.8            8              0.54
    ## seriated      811         0.064         5 1.1          158             19.76
    ##              Inertia Lazy_path_length Least_squares LS MDS_stress  ME
    ## alphabetical     267              6.9           967 99       0.62  99
    ## seriated         364              4.6           942 86       0.17 101
    ##              Moore_stress Neumann_stress Path_length RGAR   Rho
    ## alphabetical          7.0            3.9         1.8 0.48 0.028
    ## seriated              2.5            1.3         1.1 0.03 0.913

Some seriation methods also return a linear configuration where more
similar objects are located closer to each other.

``` r
get_config(order)
```

    ##    Breyer  Ginsburg   Kennedy   OConnor Rehnquist    Scalia    Souter   Stevens 
    ##      0.24      0.28     -0.15     -0.11     -0.27     -0.42      0.21      0.61 
    ##    Thomas 
    ##     -0.41

``` r
plot_config(order)
```

<img src="inst/README_files/configuration-1.png" style="display: block; margin: auto;" />

We can see a clear divide between the two groups in the configuration.

## References

- Michael Hahsler, Kurt Hornik and Christian Buchta, [Getting Things in
  Order: An Introduction to the R Package
  seriation,](http://dx.doi.org/10.18637/jss.v025.i03) *Journal of
  Statistical Software,* 25(3), 2008. DOI: 10.18637/jss.v025.i03
- Michael Hahsler. [An experimental comparison of seriation methods for
  one-mode two-way data.](http://dx.doi.org/10.1016/j.ejor.2016.08.066)
  *European Journal of Operational Research,* 257:133-143, 2017. DOI:
  10.1016/j.ejor.2016.08.066 (read the
  [preprint](https://michael.hahsler.net/research/paper/EJOR_seriation_2016.pdf))
- Hahsler, M. and Hornik, K. (2011): [Dissimilarity plots: A visual
  exploration tool for partitional
  clustering.](http://dx.doi.org/10.1198/jcgs.2010.09139) *Journal of
  Computational and Graphical Statistics,* **10**(2):335–354. DOI:
  10.1198/jcgs.2010.09139 (read the
  [preprint](https://michael.hahsler.net/research/paper/dissplot_JCGS2011_preprint.pdf);
  [code
  examples](https://mhahsler.github.io/seriation/seriation_cluster_evaluation.html))
- [Reference manual for package
  seriation.](https://mhahsler.r-universe.dev/seriation/doc/manual.html#seriation-package)

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

## uses a sequence of correlation matrices and finds  the first matrix
## with rank 2. The elements are projected into the plane spanned by the
## first two eigenvectors. All points are lying on a ellipse. The order
## of the elements on the ellipse is returned (see Chen 2002).
seriate_dist_chen <- function(x, control = NULL){
  .get_parameters(control, NULL)

  x <- as.matrix(x)

  rank <- qr(x)$rank

  ## find the first correlation matrix of rank 2
  n <- 0
  while(rank > 2){
    x <- cor(x)
    n <- n + 1
    rank <- qr(x)$rank
  }

  ## project the matrix on the first 2 eigenvectors
  e <- eigen(x)$vectors[,1:2]

  ## extract the order
  ## chen says that he uses the one of the two possible cuts
  ## that separate the points at rank 1. Since the points just
  ## separate further towards right and left, cutting on the vertical
  ## axis of the ellipse yields the same result.

  right <- which(e[,1] >= 0)
  right <- right[order(e[right,2], decreasing = TRUE)]
  left <- which(e[,1] < 0)
  left <- left[order(e[left,2])]

  o <- c(right,left)
  names(o) <- labels(x)[o]
  o
}

#set_seriation_method("dist", "Chen", seriate_dist_chen,
#  "Rank-two ellipse seriation")

set_seriation_method("dist", "R2E", seriate_dist_chen,
  "Rank-two ellipse seriation")

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


.dist_methods <- c("spearman", "kendall", "manhattan", "euclidean", "hamming",
  "ppc", "aprd")

ser_cor <- function(x, y = NULL, method = "spearman",
  reverse = TRUE, test=FALSE) {
  ## Note: not all .dist_methods are implemented!
  method <- match.arg(tolower(method), .dist_methods)

  ## make sure everything is a permutation vector
  if(!is.null(y))
    x <- list(ser_permutation_vector(x), ser_permutation_vector(y))
  else x <- lapply(x, ser_permutation_vector)

  m <- .lget_rank(x)

  if(method == "ppc") {
    if(test) stop("No test for association available for PPC!")
    return(.ppc(x))
  }

  ## cor based methods
  co <- cor(m, method = method)
  if(reverse) co <- abs(co)

  ## add a correlation test?
  if(test) {
    p <- outer(1:ncol(m), 1:ncol(m), FUN =
      Vectorize(
        function(i, j) cor.test(m[,i], m[,j], method = method)$p.value))
    dimnames(p) <- dimnames(co)
    attr(co, "p-value") <- p
  }

  co
}

ser_dist <- function(x, y = NULL, method = "spearman", reverse = TRUE, ...) {

  method <- match.arg(tolower(method), .dist_methods)

  ## make sure everything is a permutation vector
  if(!is.null(y))
    x <- list(ser_permutation_vector(x), ser_permutation_vector(y))
  else x <- lapply(x, ser_permutation_vector)

  if(!reverse) switch(method,
    spearman = as.dist(1-ser_cor(x, method="spearman", reverse = FALSE)),
    kendall = as.dist(1-ser_cor(x, method="kendal", reverse = FALSE)),

    ### Manhattan == Spearman's footrule
    manhattan = dist(t(.lget_rank(x)), method="manhattan"),
    euclidean = dist(t(.lget_rank(x)), method="euclidean"),
    hamming   = .dist_hamming(t(.lget_rank(x))),
    ppc       = as.dist(1-ser_cor(x, method="ppc", reverse = FALSE)),
    aprd      = as.dist(.aprd(x, ...))
  )

  else switch(method,
    spearman  = as.dist(1-ser_cor(x, method="spearman", reverse = TRUE)),
    kendall   =  as.dist(1-ser_cor(x, method="kendal", reverse = TRUE)),

    ### Manhattan == Spearman's footrule
    manhattan = .find_best(dist(t(.lget_rank(.add_rev(x))),
      method="manhattan")),
    euclidean = .find_best(dist(t(.lget_rank(.add_rev(x))),
      method="euclidean")),
    hamming   = .find_best(.dist_hamming(t(.lget_rank(.add_rev(x))))),

    ### positional proximity coefficient is direction invariant
    ppc       = as.dist(1-ser_cor(x, method="ppc", reverse = FALSE)),
    aprd      = as.dist(.aprd(x, ...))
  )
}

ser_align <- function(x, method = "spearman") {
    if(!is.list(x) || any(!sapply(x, inherits, "ser_permutation_vector")))
      stop("x needs to be a list with elements of type 'ser_permutation_vector'")

    .do_rev(x, .alignment(x, method=method))
}

.dist_hamming <- function(x) {
  n <- nrow(x)
  m <- matrix(nrow=n, ncol=n)
  for(i in seq_len(n))
    for(j in seq(i, n))
      m[j, i] <- m[i, j] <- sum(x[i,] != x[j,])
  mode(m) <- "numeric"
  dimnames(m) <- list(rownames(x), rownames(x))
  as.dist(m)
}

### make a permutation list into a rank matrix (cols are permutations)
.lget_rank <- function(x) sapply(x, get_rank)

### add reversed permutations to a list of permutations
.add_rev <- function(x) {
  os <- append(x, lapply(x, rev))
  names(os) <- c(labels(x), paste(labels(x), "_rev", sep=""))
  os
}

### reverses permutations in the list given a logical indicator vector
.do_rev <- function(x, rev) {
  for(i in which(rev)) x[[i]] <- rev(x[[i]])
  x
}

### finds the smallest distance in lists with reversed orders present
.find_best <- function(d) {
  ### find smallest values
  m <- as.matrix(d)
  n <- nrow(m)/2
  m1 <- m[1:n, 1:n]
  m2 <- m[(n+1):(2*n), (n+1):(2*n)]
  m3 <- m[1:n, (n+1):(2*n)]
  m4 <- m[(n+1):(2*n), 1:n]
  as.dist(pmin(m1, m2, m3, m4))
}

### find largest values in matrix
.find_best_max <- function(d) {
  m <- as.matrix(d)
  n <- nrow(m)/2
  m1 <- m[1:n, 1:n]
  m2 <- m[(n+1):(2*n), (n+1):(2*n)]
  m3 <- m[1:n, (n+1):(2*n)]
  m4 <- m[(n+1):(2*n), 1:n]
  pmax(m1, m2, m3, m4)
}


### returns TRUE for sequences which should be reversed
.alignment <- function(x, method = "spearman") {
    if(!is.list(x) || any(!sapply(x, inherits, "ser_permutation_vector"))) stop("x needs to be a list with elements of type 'ser_permutation_vector'")

    method <- match.arg(tolower(method), .dist_methods)

    n <- length(x)

    ## calculate dist (orders + reversed orders)
    d <- as.matrix(ser_dist(.add_rev(x), method=method, reverse=FALSE))
    diag(d) <- NA
    for(i in 1:n) {
      d[i, n+i] <- NA
      d[n+i, i] <- NA
    }

    ## start with closest pair
    take <- which(d == min(d, na.rm = TRUE), arr.ind = TRUE)[1,]
    #d[, c(take, (take+n) %% (2*n))] <- NA

    ## mark order and complement as taken
    d[, c(take, (take+n) %% (2*n))] <- Inf

    ## keep adding the closest
    while(length(take) < n) {
      t2 <- which(d[take,] == min(d[take,], na.rm = TRUE), arr.ind = TRUE)[1, 2]
      #d[, c(t2,  (t2+n) %% (2*n))] <- NA

      ### closest to all
      #t2 <- which.min(colSums(d[take,], na.rm = T))
      d[, c(t2,  (t2+n) %% (2*n))] <- Inf

      take <- append(take, t2)
    }

    ## create indicator vector for the orders which need to be reversed
    take_ind <- logical(n)
    take_ind[take[take>n]-n] <- TRUE
    names(take_ind) <- names(x)
    take_ind
}

## Propositional Proximity Coefficient (1 - generalized corr. coef.)
## Goulermas, Kostopoulos and Mu (2016). A new measure for analyzing and fusing
## sequences of objects, IEEE Transactions on Pattern Analysis and Machine
## Intelligence 38(5):833-48.
##
## x,y ... permutation vectors (ranks)
.vppc <- Vectorize(function(x, y) {
  x <- get_rank(x)
  y <- get_rank(y)
  n <- length(x)

  #sum <- 0
  #for(j in 2:n) for(i in 1:(j-1)) sum <- sum + (x[i]-x[j])^2 * (y[i]-y[j])^2

  ## use fast matrix algebra instead
  Ax <- (x %*% rbind(rep_len(1, n)) - tcrossprod(cbind(rep_len(1, n)), x))^2
  Ay <- (y %*% rbind(rep_len(1, n)) - tcrossprod(cbind(rep_len(1, n)), y))^2
  ## note: Ay is symetric
  sum <- sum(diag(Ax %*% Ay))

  ## scale by theoretical maximum
  zapsmall(sum / (n^6/15 - n^4/6 + n^2/10))
})

.ppc <- function(x) outer(x, x, .vppc)

# Sum of differences of rank differences
#
# distance(R, S) =
#  \sum_{i,j} | |\pi_R(i)-\pi_R(j)| - |\pi_S(i)-\pi_S(j)| |^p
#

.vaprd <- Vectorize(function(x, y, p = 2) {
  x <- get_rank(x)
  y <- get_rank(y)
  n <- length(x)

  sum <- 0
  for(j in 2:n) for(i in 1:(j-1))
    sum <- sum + abs(abs(x[i]-x[j]) - abs(y[i]-y[j]))^p

  ## FIXME: scale by theoretical maximum?
  sum
})

.aprd <- function(x, p = 2) outer(x, x, .vaprd, p=p)

library(seriation)


## FIXME add tests for ser_align

set.seed(0)

x <- list(
  a = 1:100,
  b = 100:1,
  c = sample(100),
  d = sample(100)
)


context("ser_dist")

## Default is Spearman
## first two are equal with reverse
d <- ser_dist(x)
expect_true(all(d >= 0))
expect_equal(d[1], 0)

## first two are largest distance (2) w/o reverse
d_norev <- ser_dist(x, reverse = FALSE)
expect_true(all(d_norev >= 0))
expect_equal(d_norev[1], 2)

## x,y interface
d <- ser_dist(x[[1]], x[[2]])
expect_equal(d[1], 0)

## Manhattan is 100 times 50 difference
d <- ser_dist(x, method = "Manhattan", reverse = FALSE)
expect_true(all(d >=0))
expect_equal(d[1], 100*50)

d <- ser_dist(x, method = "Manhattan")
expect_true(all(d >=0))
expect_equal(d[1], 0)

## Hamming is 100
d <- ser_dist(x, method = "Hamming", reverse = FALSE)
expect_true(all(d >=0))
expect_equal(d[1], 100)

d <- ser_dist(x, method = "Hamming")
expect_true(all(d >=0))
expect_equal(d[1], 0)

## PPC (reverse has no effect on PPC)
d <- ser_dist(x, method = "PPC")
expect_true(all(d >=0))
expect_equal(d[1], 0)

## test correlations
context("ser_cor")

## Default is Spearman
## sequence with its reverse
co <- ser_cor(x[[1]], x[[2]], reverse = FALSE)
expect_equal(co, rbind(c(1,-1), c(-1,1)))

co <- ser_cor(x, reverse = FALSE)
expect_identical(dim(co), rep(length(x), 2))
expect_true(all(co >=-1 & co <=1))
expect_equivalent(co[1:2,1:2], rbind(c(1,-1), c(-1,1)))

co <- ser_cor(x)
expect_true(all(co >=-1 & co <=1))
expect_equivalent(co[1:2,1:2], rbind(c(1,1), c(1,1)))

### PPC
co <- ser_cor(x, method ="PPC")
expect_true(all(co >=-1 & co <=1))
expect_equivalent(co[1:2,1:2], rbind(c(1,1), c(1,1)))

## test p-value
co <- ser_cor(x, test = TRUE)
expect_equivalent(attr(co, "p-value")[1:2,1:2], matrix(0, nrow=2, ncol=2))

co <- ser_cor(x, reverse = TRUE, test = TRUE)
expect_equivalent(attr(co, "p-value")[1:2,1:2], matrix(0, nrow=2, ncol=2))

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



## wrapper to greedy ordering inspired by F. Murtagh
## actually a hierarchical cluster algorithm.

## ceeboo 2005

hclust_greedy <- function(dist) {
    if (!inherits(dist, "dist"))
        stop("Argument 'dist' is not of class 'dist'.")
    if (!is.double(dist))
    mode(dist) <- "double"
    obj <- .Call("order_greedy", dist)
    names(obj) <- c("merge", "order", "height");
    class(obj) <- "hclust"
    obj
}

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' Create Simulated Data for Seriation Evaluation
#'
#' Several functions to create simulated data to evaluate different aspects of
#' seriation algorithms and criterion functions.
#'
#' `create_lines_data()` creates the lines data set used in for [iVAT()] in
#' Havens and Bezdeck (2012).
#'
#' `create_ordered_data()` is a versatile function which creates "orderable"
#' 2D data using Gaussian components along a linear or circular path. The
#' components are equally spaced (`spacing`) along the path. The default
#' spacing of 6 ensures that 2 adjacent components with a standard deviation of
#' one along the direction of the path will barely touch. The standard
#' deviation along the path is set by `sd1`. The standard deviation
#' perpendicular to the path is set by `sd2`. A value larger than zero
#' will result in the data not being perfectly orderable (i.e., the resulting
#' distance matrix will not be a perfect pre-anti-Robinson matrix and contain
#' anti-Robinson violation events after seriation). Note that a circular path
#' always creates anti-Robinson violation since the circle has to be broken at
#' some point to create a linear order. This function was created for this package
#' (Hahsler et al, 2021).
#'
#' @param n number of data points to create.
#' @param k number of Gaussian components.
#' @param size relative size (number of points) of components (length of k).
#' If `NULL` then all components have the same size.
#' @param spacing space between the centers of components. The default of 6
#' means that the components will barely touch at `ds1 = 1` (3 standard
#' deviations for each Gaussian component).
#' @param path Are the components arranged along a `"linear"` or
#' `"circular"` path?
#' @param sd1 variation in the direction along the components.  A value greater
#' than one means the components are mixing.
#' @param sd2 variation perpendicular to the direction along the components.  A
#' value greater than 0 will introduce anti-Robinson violation events.
#' @returns a data.frame with the created data.
#'
#' @author Michael Hahsler
#' @seealso [seriate()], [criterion()], [iVAT()].
#' @references
#' Havens, T.C. and Bezdek, J.C. (2012): An Efficient Formulation
#' of the Improved Visual Assessment of Cluster Tendency (iVAT) Algorithm,
#' \emph{IEEE Transactions on Knowledge and Data Engineering,} \bold{24}(5),
#' 813--822.
#'
#' Michael Hahsler, Christian Buchta and Kurt Hornik (2021). seriation: Infrastructure for
#' Ordering Objects Using Seriation. R package version 1.3.2.
#' \url{https://github.com/mhahsler/seriation}
#' @keywords datasets
#' @examples
#'
#' ## lines data set from Havens and Bezdek (2011)
#' x <- create_lines_data(250)
#' plot(x, xlim = c(-5, 5), ylim = c(-3, 3), cex = .2, col = attr(x, "id"))
#' d <- dist(x)
#' pimage(d, seriate(d, "OLO_single"), col = bluered(100, bias = .5), key = TRUE)
#'
#' ## create_ordered_data can produce many types of "orderable" data
#'
#' ## perfect pre-Anti-Robinson matrix (with a single components)
#' x <- create_ordered_data(250, k = 1)
#' plot(x, cex = .2, col = attr(x, "id"))
#' d <- dist(x)
#' pimage(d, seriate(d, "MDS"), col = bluered(100, bias=.5), key = TRUE)
#'
#' ## separated components
#' x <- create_ordered_data(250, k = 5)
#' plot(x, cex =.2, col = attr(x, "id"))
#' d <- dist(x)
#' pimage(d, seriate(d, "MDS"), col = bluered(100, bias = .5), key = TRUE)
#'
#' ## overlapping components
#' x <- create_ordered_data(250, k = 5, sd1 = 2)
#' plot(x, cex = .2, col = attr(x, "id"))
#' d <- dist(x)
#' pimage(d, seriate(d, "MDS"), col = bluered(100, bias = .5), key = TRUE)
#'
#' ## introduce anti-Robinson violations (a non-zero y value)
#' x <- create_ordered_data(250, k = 5, sd1 = 2, sd2 = 5)
#' plot(x, cex = .2, col = attr(x, "id"))
#' d <- dist(x)
#' pimage(d, seriate(d, "MDS"), col = bluered(100, bias = .5), key = TRUE)
#'
#' ## circular path (has always violations)
#' x <- create_ordered_data(250, k = 5, path = "circular", sd1 = 2)
#' plot(x, cex = .2, col = attr(x, "id"))
#' d <- dist(x)
#' pimage(d, seriate(d, "OLO"), col = bluered(100, bias = .5), key = TRUE)
#'
#' ## circular path (with more violations violations)
#' x <- create_ordered_data(250, k = 5, path = "circular", sd1 = 2, sd2 = 1)
#' plot(x, cex=.2, col = attr(x, "id"))
#' d <- dist(x)
#' pimage(d, seriate(d, "OLO"), col = bluered(100, bias = .5), key = TRUE)
#' @export
create_lines_data <- function(n = 250) {
  n1 <- n / 5 * 2
  n2 <- n / 5
  n3 <- n / 5 * 2

  x1 <-
    data.frame(x = runif(n1, -5, 5), y = rnorm(n1, mean = 2, sd = .1))
  x2 <-
    data.frame(x = runif(n2, -3, 3), y = rnorm(n2, mean = 0, sd = .1))
  x3 <-
    data.frame(x = runif(n3, -5, 5), y = rnorm(n3, mean = -2, sd = .1))
  id <-
    c(rep(1, times = n1), rep(2, times = n2), rep(3, times = n3))


  x <- rbind(x1, x2, x3)
  o <- sample(nrow(x))
  x <- x[o,]
  id <- id[o]

  rownames(x) <- 1:nrow(x)
  attr(x, "id") <- id

  x
}

#' @rdname create_lines_data
#' @export
create_ordered_data <- function(n = 250,
  k = 2,
  size = NULL,
  spacing = 6,
  path = "linear",
  sd1 = 1,
  sd2 = 0) {
  if (k > n)
    stop("k needs to be less than n!")
  path <- match.arg(path, c("linear", "circular"))

  ## size
  if (is.null(size))
    size <- rep(1, k)
  else if (length(size) != k)
    stop("length of size vector and k do not agree!")
  size <- round(size / sum(size) * n)
  size[1] <- n - sum(size[-1])

  ## create data
  ids <- rep(1:k, times = size)

  x <- data.frame(x = rnorm(n, mean = ids * spacing, sd = sd1),
    y = rnorm(n, mean = 0, sd = sd2))

  ## transform
  if (path == "circular") {
    p <- k * spacing
    theta <- x[, 1] / p * 2 * pi
    r <-  p / (2 * pi) + x[, 2]
    x <- cbind(x = r * sin(theta), y = r * cos(theta))
  }

  ## randomize order
  o <- sample(nrow(x))
  x <- x[o , , drop = FALSE]
  ids <- ids[o]
  attr(x, "id") <- ids

  x
}

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



## seriate matrices

seriate.matrix <- function(x, method = "PCA", control = NULL,
  margin = c(1,2), ...)
  .seriate_array_helper(x, method, control, margin,
    datatype = "matrix", defmethod = "BEA_TSP", ...)


seriate_matrix_identity <- function(x, control) {
  control <- .get_parameters(control, NULL)

  l <- lapply(dim(x), seq)
  for(i in 1:length(dim(x))) names(l[[i]]) <- labels(x)[[i]]
  l
}

seriate_matrix_random <- function(x, control) {
  control <- .get_parameters(control, NULL)

  l <- lapply(dim(x), FUN = function(l) sample(seq(l)))
  for(i in 1:length(dim(x))) names(l[[i]]) <- labels(x)[[i]][l[[i]]]
  l
}


set_seriation_method("matrix", "Identity", seriate_matrix_identity,
  "Identity permutation")
set_seriation_method("matrix", "Random", seriate_matrix_random,
  "Random permutation")


## these also work for general arrays!
set_seriation_method("array", "Identity", seriate_matrix_identity,
  "Identity permutation")
set_seriation_method("array", "Random", seriate_matrix_random,
  "Random permutation")

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



## Criterion for the quality of a permutation of a matrix

criterion.matrix <-
function(x, order = NULL, method = NULL, force_loss = FALSE, ...)
  .criterion_array_helper(x, order, method, "matrix", force_loss)

## Bond energy (BEA)
criterion_ME <- function(x, order = NULL, ...) {
    ### ... unused

    if(any(x < 0))
        stop("Bond energy (ME) is only defined for nonnegative matrices.")

    n <- nrow(x)
    m <- ncol(x)

    if(!is.null(order)) x <- permute(x, order)

    mode(x) <- "single"
    ener <- 0.0

    energy <- .Fortran("energy",
                       n = as.integer(n),
                       m = as.integer(m),
                       b = as.matrix(x),
                       ener = as.single(ener),
		       PACKAGE="seriation")

    0.5 * as.numeric(energy$ener)
}


## the interface to the stress functions allows for
## arbitrary subsetting (see the wrapper in C).
## (C) ceeboo 2005, 2006

.stress <- function(x, order, type="moore") {
    TYPE <- c(1,2)
    names(TYPE) <- c("moore", "neumann")
    if (inherits(x, "dist")) x <- as.matrix(x)
    if (!is.matrix(x))
        stop("Argument 'x' must be a matrix.")
    if (!is.double(x)) mode(x) <- "double"

    if(is.null(order)) {
        rows <- as.integer(1:dim(x)[1])
        cols <- as.integer(1:dim(x)[2])
    }else{
        rows <- get_order(order, 1)
        cols <- get_order(order, 2)
    }

    type <- as.integer(TYPE[type])

    x <- .Call("stress", x, rows, cols, type)

    ## does only half of the matrix!
    2 * x
}

criterion_stress_moore <- function(x, order, ...) .stress(x, order, "moore")
criterion_stress_neumann <- function(x, order, ...) .stress(x, order, "neumann")

### A MEASURE OF EFFECTIVENESS FOR THE MOMENT ORDERING ALGORITHM
### by Deutsch & Martin (1971)
### Correlation coefficient R for matrices.
criterion_R_matrix <- function(x, order, ...) {
  if(!is.null(order)) x <- permute(x, order)

  M <- nrow(x)
  N <- ncol(x)

  ## total sum
  T <- sum(x)

  ## X_i = i/M; Y_j = j/N
  X_i <- (1:M)/M
  Y_j <- (1:N)/N

  ## X_bar = 1/T sum_i,j a_ij X_i
  X_bar <- 1/T * sum(crossprod(x, X_i))
  ## Y_bar = 1/T sum_i,j a_ij Y_j
  Y_bar <- 1/T * sum(crossprod(t(x), Y_j))

  ## S_X2 = 1/(T-1) sum_i,j a_ij (X_i - X_bar)^2
  S_X2 <- 1/(T-1) * sum(crossprod(x, (X_i-X_bar)^2))
  ## S_Y2 = 1/(T-1) sum_i,j a_ij (Y_j - Y_bar)^2
  S_Y2 <- 1/(T-1) * sum(crossprod(t(x), (Y_j-Y_bar)^2))

  ## S_XY = 1/(T-1) sum_i,j a_ij  (X_i - X_bar) (Y_j - Y_bar)
  S_XY <- 1/(T-1) * sum(x * outer(X_i-X_bar, Y_j-Y_bar))

  ## R = S_XY/(S_X S_Y)
  S_XY/(sqrt(S_X2)*sqrt(S_Y2))
}

## register built-ins
set_criterion_method("matrix", "ME", criterion_ME,
    "Measure of effectiveness", TRUE)
set_criterion_method("matrix", "Cor_R", criterion_R_matrix,
    "Correlation Coefficient R", TRUE)

set_criterion_method("matrix", "Moore_stress", criterion_stress_moore,
    "Stress (Moore neighborhood)", FALSE)
set_criterion_method("matrix", "Neumann_stress", criterion_stress_neumann,
    "Stress (Neumann neighborhood)", FALSE)

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


## Spectral Seriation
## Ding, C. and Xiaofeng He (2004): Linearized cluster assignment via
## spectral orderingProceedings of the Twenty-first.
## International Conference on Machine learning (ICML '04)

## Minimizes: sum_{i,j} (i-j)^2 * d_{pi_i,pi_j}

seriate_dist_spectral <- function(x, control = NULL) {
  #param <- .get_parameters(control, NULL)
  .get_parameters(control, NULL)

  ### calculate Laplacian
  W <- 1/(1+as.matrix(x))
  D <- diag(rowSums(W))
  L <- D - W

  ## The Fiedler vector is the eigenvector with the smallest eigenvalue
  ## eigen reports eigenvectors/values in decreasing order
  q <- eigen(L)
  fiedler <- q$vectors[,ncol(W)-1L]
  o <- order(fiedler)
  names(o) <- names(x)[o]
  o
}

seriate_dist_spectral_norm <- function(x, control = NULL) {
  #param <- .get_parameters(control, NULL)
  .get_parameters(control, NULL)

  ### calculate normalized Laplacian
  W <- 1/(1+as.matrix(x))
  D_sqrt<- diag(rowSums(1/W^.5))
  L <- D_sqrt %*% W %*% D_sqrt

  z <- eigen(L)$vectors
  q <- D_sqrt %*% z

  ## look for the vector with the largest eigenvalue
  largest_ev <- q[,2L]
  o <- order(largest_ev)
  names(o) <- names(x)[o]
  o
}


set_seriation_method("dist", "Spectral", seriate_dist_spectral,
  "Spectral seriation (Ding and He 2004)  uses a relaxation to minimize the 2-Sum Problem (Barnard, Pothen, and Simon 1993). It uses the order of the Fiedler vector of the similarity matrix's Laplacian.")
set_seriation_method("dist", "Spectral_norm", seriate_dist_spectral_norm,
  "Spectral seriation (Ding and He 2004)  uses a relaxation to minimize the 2-Sum Problem (Barnard, Pothen, and Simon 1993). It uses the order of the Fiedler vector of the similarity matrix's normalized Laplacian.")

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' Create and Recognize Robinson and Pre-Robinson Matrices
#'
#' Provides functions to create and recognize (anti) Robinson and pre-Robinson
#' matrices. A (anti) Robinson matrix has strictly decreasing (increasing)
#' values when moving away from the main diagonal. A pre-Robinson matrix is a
#' matrix which can be transformed into a perfect Robinson matrix using
#' simultaneous permutations of rows and columns.
#'
#' Note that the default matrices are anti Robinson matrices. This is done
#' because distance matrices (the default in R) are typically anti Robinson
#' matrices with values increasing when moving away from the diagonal.
#'
#' Robinson matrices are recognized using the fact that they have zero anti
#' Robinson events. For pre-Robinson matrices we use spectral seriation first
#' since spectral seriation is guaranteed to perfectly reorder pre-Robinson
#' matrices (see Laurent and Seminaroti, 2015).
#'
#' Random pre-Robinson matrices are generated by reversing the process of
#' unidimensional scaling. We randomly (uniform distribution with range
#' \eqn{[0,1]}) choose \eqn{x} coordinates for \code{n} points on a straight
#' line and calculate the pairwise distances. For Robinson matrices, the points
#' are sorted first according to \eqn{x}. For noise, \eqn{y} coordinates is
#' added. The coordinates are chosen uniformly between 0 and \code{noise}, with
#' \code{noise} between 0 and 1.
#'
#' @aliases Robinson
#' @param x a symmetric, positive matrix or a dissimilarity matrix (a
#' \code{dist} object).
#' @param anti logical; check for anti Robinson structure? Note that for
#' distances, anti Robinson structure is appropriate.
#' @param pre logical; recognize/create pre-Robinson matrices.
#' @param n number of objects.
#' @param noise noise intensity between 0 and 1. Zero means no noise.  Noise
#' more than zero results in non-Robinson matrices.
#' @return A single logical value.
#' @references M. Laurent, M. Seminaroti (2015): The quadratic assignment
#' problem is easy for Robinsonian matrices with Toeplitz structure,
#' \emph{Operations Research Letters} 43(1), 103--109.
#' @examples
#'
#' ## create a perfect anti Robinson structure
#' m <- random.robinson(10)
#' pimage(m)
#'
#' is.robinson(m)
#'
#' ## permute the structure to make it not Robinsonian. However,
#' ## it is still pre-Robinson.
#' o <- sample(10)
#' m2 <- permute(m, ser_permutation(o,o))
#' pimage(m2)
#'
#' is.robinson(m2)
#' is.robinson(m2, pre = TRUE)
#'
#' ## create a binary random Robinson matrix (not anti Robinson)
#' m3 <- random.robinson(10, anti = FALSE) > .7
#' pimage(m3)
#' is.robinson(m3, anti = FALSE)
#'
#' ## create matrices with noise (as distance matrices)
#' m4 <- as.dist(random.robinson(50, pre = FALSE, noise = .1))
#' pimage(m4)
#' criterion(m4, method = "AR")
#'
#' m5 <- as.dist(random.robinson(50, pre = FALSE, noise = .5))
#' pimage(m5)
#' criterion(m5, method = "AR")
#' @export
is.robinson <- function(x, anti = TRUE, pre = FALSE) {
  if (is.matrix(x) && !isSymmetric(unname(x)))
    stop("x needs to be a symmetric matrix!")

  d <- as.dist(x)
  if (!anti)
    d <- max(d) - d

  ## pre Robinson matrix can be perfectly seriated using
  ## spectral seriation!
  if (pre)
    d <- permute(d, seriate(d, method = "spectral"))

  unname(criterion(d, method = "AR_events") == 0)
}

#' @rdname is.robinson
#' @export
random.robinson <-
  function(n,
    anti = TRUE,
    pre = FALSE,
    noise = 0) {
    if (noise < 0 | noise > 1)
      stop("noise has to be beween 0 and 1.")

    x <- runif(n)
    if (!pre)
      x <- sort(x)

    if (noise)
      x <- cbind(x, runif(n, min = 0, max = noise))

    m <- as.matrix(stats::dist(x))

    if (!anti)
      m <- max(m) - m

    m
  }

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



#' Reorder Dendrograms using Optimal Leaf Ordering
#'
#' Reorder method for dendrograms for optimal leaf ordering.
#'
#' Minimizes the distance between neighboring objects (leaf nodes) in the
#' dendrogram by flipping the order of subtrees. The algorithm by Gruvaeus and
#' Wainer is implemented in package \pkg{gclus} (Hurley 2004).
#'
#' @aliases reorder reorder.hclust
#' @param x an object of class \code{hclust}.
#' @param dist an object of class \code{dist} with dissimilarities between the
#' objects in \code{x}.
#' @param method a character string with the name of the used measure.
#' Available are:
#'   - \code{"OLO"} (optimal leaf ordering; Bar-Joseph et al., 2001) implemented in this package and
#'   - \code{"GW"} (Gruvaeus and Wainer, 1972) from package \pkg{gclus}.
#' @param ...  further arguments are currently ignored.
#' @return A reordered \code{hclust} object.
#' @author Michael Hahsler
#' @seealso [gclus::reorder.hclust()]
#' @references Bar-Joseph, Z., E. D. Demaine, D. K. Gifford, and T. Jaakkola.
#' (2001): Fast Optimal Leaf Ordering for Hierarchical Clustering.
#' \emph{Bioinformatics,} \bold{17}(1), 22--29.
#'
#' Gruvaeus, G. and Wainer, H. (1972): Two Additions to Hierarchical Cluster
#' Analysis, \emph{British Journal of Mathematical and Statistical Psychology,}
#' \bold{25}, 200--206.
#'
#' Hurley, Catherine B. (2004): Clustering Visualizations of Multidimensional
#' Data. \emph{Journal of Computational and Graphical Statistics,}
#' \bold{13}(4), 788--806.
#' @keywords optimize cluster
#' @examples
#' ## cluster European cities by distance
#' data("eurodist")
#' d <- as.dist(eurodist)
#' hc <- hclust(eurodist)
#'
#' ## plot original dendrogram and the reordered dendrograms
#' plot(hc)
#' plot(reorder(hc, d, method = "GW"))
#' plot(reorder(hc, d, method = "OLO"))
#' @export
reorder.hclust <- function(x, dist, method = "OLO", ...) {
  method <- match.arg(tolower(method), choices = c("olo", "gw"))

  ## no reordering for less than 3 objects!
  if (length(x$order) < 3)
    return(x)

  switch(method,
    olo = .seriate_optimal(x, dist),
    gw  = .seriate_gruvaeus(x, dist))
}

## wrapper for reorder.hclust in gclus
.seriate_gruvaeus <- function(hclust, dist)
  gclus::reorder.hclust(hclust, dist)

## wrapper to the optimal leaf ordering algorithm
##
## ceeboo 2005
.seriate_optimal <- function(hclust, dist) {
  ## check hclust
  merge <- hclust$merge
  if (!is.matrix(merge))
    stop("Component 'merge' of argument 'hclust' must be a matrix.")
  if (length(dim(merge)) != 2)
    stop("Component 'merge' of argument 'hclust' is invalid.")
  if (dim(merge)[1] != attr(dist, "Size") - 1)
    stop("Argument 'dist' and component 'merge' of argument 'hclust' do not conform.")
  mode(merge) <- "integer"

  obj <- .Call("order_optimal", dist, merge)

  names(obj) <- c("merge", "order", "length")
  ##names(obj$order) <- attr(dist,"Labels")
  hclust$merge <- obj$merge
  hclust$order <- obj$order

  hclust
}

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


#' Registry for Seriation Methods
#'
#' A registry to manage methods used by [seriate()].
#'
#' The functions below are convenience function for the registry
#' \code{registry_seriate}.
#'
#' \code{list_seriation_method()} lists all available methods for a given data
#' type (\code{kind}) (e.g., "dist", "matrix").
#' The result is a vector of character strings with the
#' method names that can be used in function `seriate()`.
#' If \code{kind} is missing, then a list of
#' methods is returned.
#'
#' \code{get_seriation_method()} returns detailed information for a given method in
#' form of an object of class \code{"seriation_method"}.
#' The information includes a description, parameters and the
#' implementing function.
#'
#' With \code{set_seriation_method()} new seriation methods can be added by the
#' user. The implementing function (\code{definition}) needs to have the formal
#' arguments \code{x, control}, where \code{x} is the data object and
#' \code{control} contains a list with additional information for the method
#' passed on from \code{seriate()}.  The implementation has to return a list of
#' objects which can be coerced into \code{ser_permutation_vector} objects
#' (e.g., integer vectors). The elements in the list have to be in
#' corresponding order to the dimensions of \code{x}.
#'
#' @import registry
#' @family seriation
#'
#' @param kind the data type the method works on. For example, \code{"dist"},
#' \code{"matrix"} or \code{"array"}. If missing, then methods for any type are
#' shown.
#' @param name the name for the method used to refer to the method in
#' [seriate()].
#' @param definition a function containing the method's code.
#' @param description a description of the method. For example, a long name.
#' @param control a list with control arguments and default values.
#' @param x an object of class  "seriation_method" to be printed.
#' @param ... further information that is stored for the method in the
#' registry.
#' @returns
#' - \code{list_seriation_method()} result is a vector of character strings with the
#'   names of the methods. These names are used for methods in `seriate()`.
#' - \code{get_seriation_method()} returns a given method in form of an object of class
#'   \code{"seriation_method"}.
#'
#' @author Michael Hahsler
#' @seealso This registry uses [registry()] in package \pkg{registry}.
#' @keywords misc
#' @examples
#' # Registry
#' registry_seriate
#'
#' # List all seriation methods by type
#' list_seriation_methods()
#'
#' # List methods for matrix seriation
#' list_seriation_methods("matrix")
#'
#' get_seriation_method(name = "BEA")
#'
#' # Example for defining a new seriation method (reverse identity function for matrix)
#'
#' # 1. Create the seriation method
#' seriation_method_reverse <- function(x, control) {
#'    # return a list of order vectors, one for each dimension
#'    list(seq(nrow(x), 1), seq(ncol(x), 1))
#' }
#'
#' # 2. Register new method
#' set_seriation_method("matrix", "Reverse", seriation_method_reverse,
#'     description = "Reverse identity order", control = list())
#'
#' list_seriation_methods("matrix")
#' get_seriation_method("matrix", "reverse")
#'
#' # 3. Use the new seriation methods
#' seriate(matrix(1:12, ncol=3), "reverse")
#' @export
registry_seriate <- registry(registry_class = "seriation_registry",
  entry_class = "seriation_method")

registry_seriate$set_field("kind",
  type = "character",
  is_key = TRUE,
  index_FUN = match_partial_ignorecase)
registry_seriate$set_field("name",
  type = "character",
  is_key = TRUE,
  index_FUN = match_partial_ignorecase)
registry_seriate$set_field("fun", type = "function",
  is_key = FALSE)
registry_seriate$set_field("description", type = "character",
  is_key = FALSE)
registry_seriate$set_field("control", type = "list",
  is_key = FALSE)

#' @rdname registry_seriate
#' @export
list_seriation_methods <- function(kind) {
  if (missing(kind)) {
    kinds <- unique(sort(as.vector(
      sapply(registry_seriate$get_entries(), "[[", "kind")
    )))

    sapply(
      kinds,
      FUN = function(k)
        list_seriation_methods(k)
    )

  } else{
    sort(as.vector(sapply(
      registry_seriate$get_entries(kind = kind), "[[", "name"
    )))
  }
}

#' @rdname registry_seriate
#' @export
get_seriation_method <- function(kind, name) {
  if (missing(kind))
    method <- registry_seriate$get_entry(name = name)
  else
    method <- registry_seriate$get_entry(kind = kind, name = name)

  if (is.null(method))
    stop(
      "Unknown seriation method ",
      name,
      " for data type ",
      kind,
      ". Check list_seriation_methods(\"",
      kind,
      "\")"
    )

  method
}

#' @rdname registry_seriate
#' @export
set_seriation_method <- function(kind,
  name,
  definition,
  description = NULL,
  control = list(),
  ...) {
  ## check formals
  if (!identical(names(formals(definition)),
    c("x", "control")))
    stop("Seriation methods must have formals 'x' and 'control'.")

  ## check if entry already exists
  r <- registry_seriate$get_entry(kind = kind, name = name)
  if (!is.null(r) && r$name == name) {
    # warning(
    #   "Entry with name \"",
    #   name,
    #   "\" for kind \"",
    #   kind,
    #   "\" already exists! Modifying entry."
    # )
    registry_seriate$modify_entry(
      kind = kind,
      name = name,
      fun = definition,
      description = description,
      control = control
    )
  } else {
    registry_seriate$set_entry(
      name = name,
      kind = kind,
      fun = definition,
      description = description,
      control = control
    )
  }
}


#' @rdname registry_seriate
#' @export
print.seriation_method <- function(x, ...) {
  writeLines(c(
    gettextf("name:        %s", x$name),
    gettextf("kind:        %s", x$kind),
    gettextf("description: %s", x$description)
  ))

  if (length(x$control) > 0) {
    writeLines("control (default values):")

    contr <- lapply(
      x$control,
      FUN =
        function(p)
          utils::capture.output(dput(p, control = list()))[1]
    )

    print(as.data.frame(contr))
  } else
    writeLines("control: no parameters registered.")

  invisible(x)
}

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

## Brusco: simulated annealing for the Linear Seriation Criterion
.arsa_control <- list(
  cool = 0.5,                  ## Brusco: 0.95
  tmin = 0.0001,               ## Brusco: 0.0001
  swap_to_inversion = .5,      ## Brusco: .5
  try_multiplier = 100,        ## Brusco: 100
  reps = 1L,                   ## Brusco: 20
  verbose = FALSE
)

seriate_dist_arsa <- function(x, control = NULL) {
  param <- .get_parameters(control, .arsa_control)

  A <- as.matrix(x)
  # SUBROUTINE arsa(N, A, COOL, TMIN, NREPS, IPERM, R1, R2, D, U,
  #      S, T, SB, ZBEST, verbose)
  N <- ncol(A)
  NREPS <- as.integer(param$reps)
  IPERM <- integer(N)
#  R1 <- double(N*N/2)
#  R2 <- double(N*N/2)
  D <- double(N*N)
  U <- integer(N)
  S <- integer(N)
  T <- integer(NREPS*N)
  SB <- integer(N)
  ZBEST <- double(1)

  ret <- .Fortran("arsa", N, A,
    as.numeric(param$cool), as.numeric(param$tmin), NREPS,
    IPERM, D, U, S, T, SB, ZBEST, as.numeric(param$swap_to_insertion),
    as.numeric(param$try_multiplier), as.integer(param$verbose),
    PACKAGE="seriation")

  o <- ret[[6]]
  names(o) <- labels(x)[o]

  ### ARSA returns all 0's in some cases
  if(all(o == 0)) {
    o <- 1:N
    warning("ARSA has returned an invalid permutation vector! Check the supplied dissimilarity matrix.")
  }

  o
}


## Brusco: branch-and-bound - unweighted row gradient
.bb_control <- list(
  eps = 1e-7,
  verbose = FALSE
)

seriate_dist_bburcg <- function(x, control = NULL) {
  param <- .get_parameters(control, .bb_control)

  A <- as.matrix(x)
  N <- ncol(A)

  # SUBROUTINE bburcg(N, A, EPS, X, Q, D, DD, S, UNSEL, IVERB)
  X <- integer(N)
  Q <- integer(N)
  D <- integer(N*N*N)
  DD <- integer(N*N*N)
  S <- integer(N)
  UNSEL <- integer(N)

  ret <- .Fortran("bburcg", N, A, param$eps, X, Q, D, DD, S, UNSEL,
    param$verbose)

  o <- ret[[4]]
  names(o) <- labels(x)[o]
  o
}


## Brusco: branch-and-bound - weighted row gradient
seriate_dist_bbwrcg <- function(x, control = NULL) {
  param <- .get_parameters(control, .bb_control)

  A <- as.matrix(x)
  N <- ncol(A)

  # SUBROUTINE bbwrcg(N, A, EPS, X, Q, D, DD, S, UNSEL, IVERB)
  X <- integer(N)
  Q <- integer(N)
  D <- double(N*N*N)
  DD <- double(N*N*N)
  S <- integer(N)
  UNSEL <- integer(N)

  ret <- .Fortran("bbwrcg", N, A, param$eps, X, Q, D, DD, S, UNSEL,
    param$verbose)

  o <- ret[[4]]
  names(o) <- labels(x)[o]
  o
}

set_seriation_method("dist", "ARSA", seriate_dist_arsa,
  "Minimize the linear seriation criterion using simulated annealing (Brusco et al, 2008).\ncontrol parameters:\n - cool (cooling rate)\n - tmin (minimum temperature)\n - swap_to_inversion (proportion of swaps to inversions for local neighborhood search)\n - try_multiplier (local search tries per temperature; multiplied with the number of objects)\n - reps (repeat the algorithm with random initialization)\n", control = .arsa_control)

set_seriation_method("dist", "BBURCG", seriate_dist_bburcg,
  "Minimize the unweighted row/column gradient by branch-and-bound (Brusco and Stahl 2005). This is only feasible for a relatively small number of objects.", control  = .bb_control)

set_seriation_method("dist", "BBWRCG", seriate_dist_bbwrcg,
  "Minimize the weighted row/column gradient by branch-and-bound (Brusco and Stahl 2005). This is only feasible for a relatively small number of objects.", control  = .bb_control)

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


## Hierarchical clustering related seriations
.hc_control <- list(
  hclust = NULL,
  method = "average"
)

.hclust_helper <- function(d, control = NULL){
  control <- .get_parameters(control, .hc_control)

  if(!is.null(control$hclust)) return(control$hclust)
  return(hclust(d, method = control$method))
}

seriate_dist_hc <- function(x, control = NULL) .hclust_helper(x, control)
seriate_dist_hc_single <- function(x, control = NULL)
  .hclust_helper(x, control=list(method="single"))
seriate_dist_hc_average <- function(x, control = NULL)
  .hclust_helper(x, control=list(method="average"))
seriate_dist_hc_complete <- function(x, control = NULL)
  .hclust_helper(x, control=list(method="complete"))
seriate_dist_hc_ward <- function(x, control = NULL)
  .hclust_helper(x, control=list(method="ward.D2"))

## workhorses are in seriation.hclust
seriate_dist_gw <- function(x, control = NULL)
  reorder(seriate_dist_hc(x, control), x, method="GW")
seriate_dist_gw_single <- function(x, control = NULL)
  reorder(seriate_dist_hc_single(x, control), x, method="GW")
seriate_dist_gw_average <- function(x, control = NULL)
  reorder(seriate_dist_hc_average(x, control), x, method="GW")
seriate_dist_gw_complete <- function(x, control = NULL)
  reorder(seriate_dist_hc_complete(x, control), x, method="GW")
seriate_dist_gw_ward <- function(x, control = NULL)
  reorder(seriate_dist_hc_ward(x, control), x, method="GW")


seriate_dist_olo <- function(x, control = NULL)
  reorder(seriate_dist_hc(x, control), x, method="OLO")
seriate_dist_olo_single <- function(x, control = NULL)
  reorder(seriate_dist_hc_single(x, control), x, method="OLO")
seriate_dist_olo_average <- function(x, control = NULL)
  reorder(seriate_dist_hc_average(x, control), x, method="OLO")
seriate_dist_olo_complete <- function(x, control = NULL)
  reorder(seriate_dist_hc_complete(x, control), x, method="OLO")
seriate_dist_olo_ward <- function(x, control = NULL)
  reorder(seriate_dist_hc_ward(x, control), x, method="OLO")


.hc_desc <- "Using the order of the leaf nodes in a dendrogram obtained by hierarchical clustering"
set_seriation_method("dist", "HC", seriate_dist_hc,
  .hc_desc, .hc_control)
set_seriation_method("dist", "HC_single", seriate_dist_hc_single,
  paste(.hc_desc, "(single link)"))
set_seriation_method("dist", "HC_complete", seriate_dist_hc_complete,
  paste(.hc_desc, "(complete link)."))
set_seriation_method("dist", "HC_average", seriate_dist_hc_average,
  paste(.hc_desc, "(avg. link)."))
set_seriation_method("dist", "HC_ward", seriate_dist_hc_ward,
  paste(.hc_desc, "(Ward's method)."))

.gw_desc <- "Using the order of the leaf nodes in a dendrogram obtained by hierarchical clustering and reordered by the Gruvaeus and Wainer (1972) heuristic"
set_seriation_method("dist", "GW", seriate_dist_gw,
  .gw_desc, .hc_control)
set_seriation_method("dist", "GW_single", seriate_dist_gw_single,
  paste(.gw_desc, "(single link)"))
set_seriation_method("dist", "GW_average", seriate_dist_gw_average,
  paste(.gw_desc, "(avg.link)"))
set_seriation_method("dist", "GW_complete", seriate_dist_gw_complete,
  paste(.gw_desc, "(complete link)"))
set_seriation_method("dist", "GW_ward", seriate_dist_gw_ward,
  paste(.gw_desc, "(Ward's method)"))

.olo_desc <- "Using the order of the leaf nodes in a dendrogram obtained by hierarchical clustering and reordered by with optimal leaf ordering (Bar-Joseph et al., 2001)"
set_seriation_method("dist", "OLO", seriate_dist_olo,
  .olo_desc, .hc_control)
set_seriation_method("dist", "OLO_single", seriate_dist_olo_single,
  paste(.olo_desc, "(single link)"))
set_seriation_method("dist", "OLO_average", seriate_dist_olo_average,
  paste(.olo_desc, "(avg. link)"))
set_seriation_method("dist", "OLO_complete", seriate_dist_olo_complete,
  paste(.olo_desc, "(complete link)"))
set_seriation_method("dist", "OLO_ward", seriate_dist_olo_ward,
  paste(.olo_desc, "(Ward's method)"))

## This is a modified version from package rlang that only uses base R functionality.

## action can be "install" (from CRAN), "stop" (with message), "check" (returns TRUE/FALSE)
## manual can be either TRUE or a string with installation instructions.
check_installed <-
  function (pkg,
    action = "install",
    message = NULL)
  {
    action <- match.arg(action, choices = c("install", "stop", "check"))

    if (!is.character(pkg))
      stop("`pkg` must be a package name or a vector of package names.")

    needs_install <-
      sapply(pkg, function(x)
        ! requireNamespace(x,
          quietly = TRUE))

    if (action == "check")
      return(!any(needs_install))

    if (any(needs_install)) {
      if (!interactive())
        stop(info)

      missing_pkgs <- pkg[needs_install]
      missing_pkgs_enum <- paste(missing_pkgs, collapse = ", ")

      info <-
        paste("The", missing_pkgs_enum,
          "package(s) is/are required.")

      if (action == "install") {
        question <-
          "Would you like to install the package(s)?"
        cat(info, "\n", question, sep = '')
        if (utils::menu(c("Yes", "No")) != 1) {
          invokeRestart("abort")
        }

        utils::install.packages(missing_pkgs)
      } else {
        ### this is stop
        cat(info,
          "\n",
          message,
          sep = '')

        invokeRestart("abort")
      }
    }

    invisible(TRUE)
  }

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2015 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


## registers seriation methods and criteria from package DendSer

register_DendSer <- function() {
  if(!.installed("DendSer")) stop("Package 'DendSer' needs to be  installed!")

  ## seriation methods

  ## control:
  # cost (default: costBAR)
  #   ## costLS, costPL, costLPL, costED, costARc, costBAR
  # h (default is NULL -> complete)


  .DendSer_control <- list(
    h = NULL,
    method = "complete",
    criterion = NULL,
    cost = DendSer::costBAR,
    DendSer_args = NULL,
    verbose = FALSE
  )

  DendSer_helper <- function(x, control) {
    n <- attr(x, "Size")

    control <- .get_parameters(control, .DendSer_control)

    ## fix cost if it is a criterion from seriation
    if(!is.null(control$criterion))
      control$cost <- DendSer::crit2cost(crit = control$criterion)

    ## produce hclust
    if(is.null(control$h))
      control$h <- hclust(x, control$method)

    control$method <- NULL
    control$criterion <- NULL
    control$ser_weight <- x

    if(!is.null(control$DendSer_args)) {
      control <- c(control, control$DendSer_args)
      control$DendSer_args <- NULL
    }

    do.call(DendSer::DendSer, control)
  }


  DendSer_BAR <- DendSer_helper

  DendSer_PL <- function(x, control) {
    #control$cost <- DendSer::costPL
    control$criterion <- "Path_length"
    DendSer_helper(x, control)
  }

  DendSer_LPL <- function(x, control) {
    #control$cost <- DendSer::costLPL
    control$criterion <- "Lazy_path_length"
    DendSer_helper(x, control)
  }

  DendSer_ARc <- function(x, control) {
    control$cost <- DendSer::costARc
    DendSer_helper(x, control)
  }

  ## This is not Least Squares!
  #  DendSer_LS <- function(x, control) {
  #    control$cost <- DendSer::costLS
  #    control$criterion <- "LS"
  #    control$h <- hclust(x)
  #    DendSer_helper(as.matrix(x)[,1], control)
  #  }

  seriation::set_seriation_method("dist", "DendSer",
    DendSer_helper, "Dendrogram seriation (Earle and Hurley, 2015).", .DendSer_control)

  seriation::set_seriation_method("dist", "DendSer_BAR",
    DendSer_BAR, "Dendrogram seriation  with BAR (Earle and Hurley, 2015).", .DendSer_control)
  seriation::set_seriation_method("dist", "DendSer_PL",
    DendSer_PL, "Dendrogram seriation (Path length)", .DendSer_control)
  seriation::set_seriation_method("dist", "DendSer_LPL",
    DendSer_PL, "Dendrogram seriation (Lazy path length)", .DendSer_control)
  seriation::set_seriation_method("dist", "DendSer_ARc",
    DendSer_ARc, "Dendrogram seriation (ARc)", .DendSer_control)
  #  seriation::set_seriation_method("dist", "DendSer_LS",
  #    DendSer_LS, "Dendrogram seriation (Leaf sort)")


  ## criteria

  DendSer_crit_ARc <- function(x, order, ...) {
    x <- as.matrix(x)
    if (is.null(order)) order <- 1:nrow(x)
    else order <- get_order(order)
    DendSer::costARc(x,order,...)
  }

  seriation::set_criterion_method("dist", "ARc", DendSer_crit_ARc,
    "AR cost", FALSE)

  ## Already in seriation
  #  DendSer_crit_BAR <- function(x, order, ...) {
  #    x <- as.matrix(x)
  #    if (is.null(order)) order <- 1:nrow(x)
  #    else order <- get_order(order)
  #    DendSer::costBAR(x,order,...)
  #  }
  #
  #  seriation::set_criterion_method("dist", "BAR", DendSer_crit_BAR,
  #    "Banded AR cost", FALSE)


  #  criterion_method_dist_LPL <- function(x, order, ...) {
  #    x <- as.matrix(x)
  #    if (is.null(order)) order <- 1:nrow(x)
  #    else order <- get_order(order)
  #    DendSer::costLPL(x,order,...)
  #  }
  #
  #  seriation::set_criterion_method("dist", "LPL", criterion_method_dist_LPL,
  #    "Lazy path cost", FALSE)
  #}
}

#' 2D Data Sets used for the CHAMELEON Clustering Algorithm
#'
#' Several 2D data sets created to evaluate the CHAMELEON clustering algorithm in
#' the paper by Karypis et al (1999).
#'
#' @name Chameleon
#' @aliases Chameleon chameleon chameleon_ds4 chameleon_ds5 chameleon_ds7
#' chameleon_ds8
#' @docType data
#' @format
#' `chameleon_ds4`: The format is a 8,000 x 2 data.frame.
#'
#' `chameleon_ds5`: The format is a 8,000 x 2 data.frame.
#'
#' `chameleon_ds7`: The format is a 10,000 x 2 data.frame.
#'
#' `chameleon_ds8`: The format is a 8,000 x 2 data.frame.
#' @references Karypis, G., EH. Han, V. Kumar (1999): CHAMELEON: A Hierarchical
#' Clustering Algorithm Using Dynamic Modeling, _IEEE Computer,_
#' **32**(8): 68--75.
#' \doi{10.1109/2.781637}
#' @keywords datasets
#' @examples
#' data(Chameleon)
#'
#' plot(chameleon_ds4, cex = .1)
#' plot(chameleon_ds5, cex = .1)
#' plot(chameleon_ds7, cex = .1)
#' plot(chameleon_ds8, cex = .1)
NULL

# link does not work
# @source The data was obtained from
# \url{http://glaros.dtc.umn.edu/gkhome/cluto/cluto/download}

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


## Algorithm B
##  F. Murtagh (1985). Multidimensional Cluster Algorithms. Lectures
##  in Computational Statistics, Physica Verlag, pp. 15.
#
# this is actually just the same as BEA
#
#.seriate_matrix_murtagh <- function(x, control) {
#
#    if(any(x < 0)) stop("Requires a nonnegative matrix.")
#
#    criterion <- as.dist(tcrossprod(x))
#    row <- hclust_greedy(-criterion)$order
#    criterion <- as.dist(crossprod(x))
#    col <- hclust_greedy(-criterion)$order
#
#    list(row = row, col = col)
#}

seriate_matrix_bea_tsp <- function(x, control) {

  if(any(x < 0)) stop("Requires a nonnegative matrix.")

  criterion <- as.dist(tcrossprod(x))
  row <- seriate(max(criterion)-criterion,
    method = "TSP", control = control)[[1]]

  criterion <- as.dist(crossprod(x))
  col <- seriate(max(criterion)-criterion,
    method = "TSP", control = control)[[1]]

  attr(row, "method") <- "BEA_TSP"
  attr(col, "method") <- "BEA_TSP"

  list(row = row, col = col)
}


## Bond Energy Algorithm (McCormick 1972)
.bea_contr <- list(
  istart = 0,
  jstart = 0,
  rep = 1
)

seriate_matrix_bea <- function(x, control = NULL){
  control <- .get_parameters(control, .bea_contr)

  if(any(x < 0)) stop("Requires a nonnegative matrix.")
  istart <- control$istart
  jstart <- control$jstart
  rep  <- control$rep

  res <- replicate(rep, bea(x, istart = istart, jstart = jstart),
    simplify = FALSE)

  best <- which.max(sapply(res, "[[", "e"))
  res <- res[[best]]

  row <- res$ib
  col <- res$jb

  names(row) <- rownames(x)[row]
  names(col) <- colnames(x)[col]

  list(row = row, col = col)
}

## register methods
set_seriation_method("matrix", "BEA", seriate_matrix_bea,
  "Bond Energy Algorithm (BEA; McCormick 1972) to maximize the Measure of Effectiveness of a non-negative matrix.", .bea_contr)
set_seriation_method("matrix", "BEA_TSP", seriate_matrix_bea_tsp,
  "Use a TSP to optimize the Measure of Effectiveness (Lenstra 1974). Control is passed on to the seriation method TSP.")

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2015 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' Register Seriation Based on 1D t-SNE
#'
#' Use t-distributed stochastic neighbor embedding (t-SNE) for [seriate()].
#'
#' Registers the method \code{"tsne"} for [seriate()]. This method applies
#' 1D t-SNE to data represented by a distance matrix and extracts the order
#' from the 1D embedding. To speed up the process, an initial embedding is
#' created using multi-dimensional scaling (MDS) which is improved by t-SNE.
#'
#' The \code{control} parameter \code{mds} controls if MDS is used to create an
#' initial embedding. See [Rtsne::Rtsne()] to learn about the other
#' available \code{control} parameters.
#'
#' \bold{Note:} Package \pkg{Rtsne} needs to be installed.
#'
#' @aliases register_tsne tsne tSNE
#' @seealso [Rtsne::Rtsne()] in \pkg{Rtsne}.
#' @family seriation
#' @returns Nothing.
#'
#' @references van der Maaten, L.J.P. & Hinton, G.E., 2008. Visualizing
#' High-Dimensional Data Using t-SNE. Journal of Machine Learning Research, 9,
#' pp.2579-2605.
#' @keywords optimize cluster
#' @examples
#'
#' \dontrun{
#' register_tsne()
#' get_seriation_method("dist", "tsne")
#'
#' d <- dist(random.robinson(50, pre=TRUE, noise=.1))
#'
#' o <- seriate(d, method = "tsne")
#' pimage(d, o)
#' }
#'
#' @export
register_tsne <- function() {
  check_installed("Rtsne")

  .contr <- list(
    max_iter = 1000,
    theta = 0,
    perplexity = 30,
    eta = 200,
    mds = TRUE
  )

  tsne_order <- function(x, control) {
    control <- .get_parameters(control, .contr)

    # start with MDS
    if(control$mds) Y_init <- stats::cmdscale(x, k = 1)
    else Y_init <- NULL

    # calculate the maximal value for perplexity
    perplexity <- min(control$perplexity, floor(attr(x, "Size") / 3) - 1)

    embedding <- Rtsne::Rtsne(x, dims = 1, is_distance = TRUE,
      max_iter = control$max_iter, theta = control$theta, eta = control$eta,
      perplexity = perplexity, Y_init = Y_init)
    order(embedding$Y)
  }

  set_seriation_method(
    "dist",
    "tsne",
    tsne_order,
    "Use 1D  t-distributed stochastic neighbor embedding (t-SNE) to one dimension to create an order",
    .contr
  )
}

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2017 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

# unidimensional scaling: Defrays Decomposition (1978)




#' Unidimensional Scaling from Seriation Results
#'
#' Performs (approximate) unidimensional scaling by first performing seriation
#' to obtain a permutation and the using the permutation to calculate the
#' configuration.
#'
#' Uses the method describes in Maier and De Leeuw (2015) to calculate the
#' minimum stress configuration for either a given
#' configuration/permutation/order or for a permutation computed via a
#' seriation method.
#'
#' The code is similar to `uniscale()` in \pkg{smacof}, but scales to larger
#' datasets since it does not check all permutations.
#'
#' @param d a dissimilarity matrix.
#' @param order a precomputed permutation (configuration) order.  If
#' \code{NULL}, then seriation is performed using the method specified in
#' \code{method}.
#' @param method seriation method used if \code{o} is \code{NULL}.
#' @param rep Number of repetitions of the seriation heuristic.
#' @param \dots additional arguments are passed on to the seriation method.
#' @return A vector with the fitted configuration.
#' @author Michael Hahsler with code from Patrick Mair (from \pkg{smacof}).
#' @references Mair P., De Leeuw J. (2015). Unidimensional scaling. In
#' \emph{Wiley StatsRef: Statistics Reference Online,} Wiley, New York.
#' \doi{10.1002/9781118445112.stat06462.pub2}
#' @keywords optimize
#' @examples
#' data(SupremeCourt)
#'
#' d <- as.dist(SupremeCourt)
#'
#' sc <- uniscale(d)
#' sc
#'
#' orderplot(sc)
#' @export
uniscale <-
  function(d,
    order = NULL,
    method = "QAP_LS",
    rep = 10,
    ...) {
    if (is.null(order))
      order <- seriate(d, method = method, rep = rep, ...)
    x <- get_rank(order)
    n <- length(x)

    w <- 1 - diag(n)

    normDissN <- function (diss, wghts, m) {
      N <- length(diss) * m
      dissnorm <- diss / sqrt(sum(wghts * diss ^ 2, na.rm = TRUE)) *
        sqrt(N)
      return(dissnorm)
    }

    delta <- as.matrix(normDissN(as.dist(d), as.dist(w),
      1))
    v <- as.matrix(solve((diag(rowSums(
      w
    )) - w) + (1 / n)) - (1 / n))
    s <- sign(outer(x, x, "-"))

    t <- as.vector(v %*% rowSums(delta * w * s))

    names(t) <- attr(d, "Labels")
    t
  }

#' @rdname uniscale
#' @param x a scaling returned by `uniscale()`.
#' @param main main plot label
#' @param pch print character
#' @export
orderplot <- function (x, main, pch = 19, ...) {
  if (missing(main))
    main <- "Configuration"

  n <- length(x)
  plot(
    x,
    rep(0, n),
    axes = FALSE,
    ann = FALSE,
    pch = pch,
    type = "o",
    ylim = c(-0.2, 0.8),
    ...
  )
  title(main)

  labs <- names(x)
  if (is.null(labs))
    labs <- 1:n
  text(x,
    rep(0, n) + 0.05,
    labs,
    srt = 90,
    adj = c(0, 0.5))
}

C   ANTI-ROBINSON SERIATION
C   branch-and-bound
C   by Brusco, M. and Stahl, S.
C   R Interface by Michael Hahsler


C      PROGRAM DYNAMIC
C      SUBROUTINE dynamic(N, A, EPS, X)
      SUBROUTINE bburcg(N, A, EPS, X, Q, D, DD, S, UNSEL, IVERB)

      IMPLICIT INTEGER(A-Z)
C      DOUBLE PRECISION TIMEA,TIMEB,TIMTOT,A(50,50),EPS
      DOUBLE PRECISION A(N,N), EPS
      REAL S1
      INTEGER X(N),Q(N),D(N,N,N),S(N),DD(N,N,N),UNSEL(N)

C       Initialize R RNG
      CALL getrngstate()

      OLDM=0
      CHECKS=0
C
C #################################################################
C 10/13/01 This program fits an "unweighted" row gradient criterion
C         to a symmetric proximity matrix.  Count +1 if the anti-
C         Robinson triple is satisfied, -1 if its not, and 0 for
C         ties.  Only look at upper half of matrix
C 07/20/02: Improved symmetry test implemented.
C 07/26/03: Fixed the incorrect symmetry test, added an interchange test
C          avoid use of so many "IF" statements using F & D matrices
C 12/24/03: Add insertion test to interchange test.
C 07/09/15: Fixed memory issue (MFH)
C #################################################################
C
C      OPEN(1,FILE='AMAT.DAT')          ! Dissimilarity matrix
C      OPEN(2,FILE='SEQ.OUT')           ! Output file
C      EPS = 1.0d-07
C      READ(1,*) N                      ! Read number of objects
C      WRITE(*,*) 'TYPE 1 FOR HALF MATRIX OR TYPE 2 FOR FULL MATRIX'
C      READ(*,*) ITYPE
C      ITYPE = 2
C      IF(ITYPE.EQ.2) THEN
C        READ(1,*) ((A(I,J),J=1,N),I=1,N)
C      ELSE
C        DO J = 2,N
C          READ(1,*) (A(I,J),I=1,J-1)
C        END DO
C        DO J = 2,N
C          DO I = 1,J-1
C            A(J,I) = A(I,J)
C          END DO
C        END DO
C      END IF
C      CALL GETTIM (IHR, IMIN, ISEC, I100)
C      CALL GETDAT (IYR, IMON, IDAY)
C      TIMEA=DFLOAT(86400*IDAY+3600*IHR+60*IMIN+ISEC)+DFLOAT(I100)/100.
      DO I = 1,N
        A(I,I) = 0.0D0
      END DO
C
      DO 848 I = 1,N
        DO 849 J = 1,N
          IF(I.EQ.J) GO TO 849
          DO 850 K = 1,N
            IF(I.EQ.K.OR.J.EQ.K) GO TO 850
            IF(A(I,K).GT.A(I,J)+EPS) D(I,J,K)=1
            IF(A(I,K).LT.A(I,J)-EPS) D(I,J,K)=-1
C for bburcg
            IF(A(I,K).GT.A(J,K)+EPS) D(I,J,K)=D(I,J,K)+1
            IF(A(I,K).LT.A(J,K)-EPS) D(I,J,K)=D(I,J,K)-1
C

 850      CONTINUE
 849    CONTINUE
 848  CONTINUE
C

      DO 851 I = 1,N
        DO 852 J = 1,N
          IF(I.EQ.J) GO TO 852
          DO 853 K = 1,N
            IF(I.EQ.K.OR.J.EQ.K) GO TO 853
            ACT=D(I,J,K)
            IF(D(I,K,J).GT.ACT) ACT = D(I,K,J)
            IF(D(J,I,K).GT.ACT) ACT = D(J,I,K)
            DD(I,J,K) = ACT
 853      CONTINUE
 852    CONTINUE
 851  CONTINUE
C
      ZBEST = 0
      DO 3500 JJJ = 1,20
        DO I = 1,N
          UNSEL(I) = I
          Q(I) = 0
        END DO
        NNSEL = N
C 3501   CALL RANDOM(S1)
C 3501   S1 = rand()
 3501   CALL unifrand(S1)
        ISEL = INT(1. + S1*FLOAT(NNSEL))
        IF(ISEL.GT.NNSEL) ISEL = NNSEL
        Q(NNSEL) = UNSEL(ISEL)
        DO J = ISEL,NNSEL-1
          UNSEL(J) = UNSEL(J+1)
        END DO
        NNSEL = NNSEL - 1
        IF(NNSEL.GT.0) GO TO 3501
C        WRITE(*,72) (Q(J),J=1,N)
C 72     FORMAT(20I3)
        Z = 0
        DO I = 1,N-2
          R1 = Q(I)
          DO J = I+1,N-1
            R2 = Q(J)
            DO K = J+1,N
              R3 = Q(K)
              Z = Z + D(R1,R2,R3)
            END DO
          END DO
        END DO
 3502   ITRIG = 0
        DO II = 1,N-1
          DO JJ = II+1,N

C   R interrupt
            CALL rchkusr()
C

            R3 = Q(JJ)
            R2 = Q(II)
            DELTA=0
            DO I = 1,II-1
              R1 = Q(I)
              DELTA = DELTA + D(R1,R3,R2) - D(R1,R2,R3)
              DO J = II+1,JJ-1
                R4 = Q(J)
                DELTA = DELTA + D(R1,R3,R4) - D(R1,R2,R4)
                DELTA = DELTA + D(R1,R4,R2) - D(R1,R4,R3)
              END DO
            END DO
            DO J = II+1,JJ-1
              R4 = Q(J)
              DELTA = DELTA + D(R3,R4,R2) - D(R2,R4,R3)
              DO K = JJ+1,N
                R5 = Q(K)
                DELTA = DELTA + D(R4,R2,R5) - D(R4,R3,R5)
                DELTA = DELTA + D(R3,R4,R5) - D(R2,R4,R5)
              END DO
            END DO
            DO K = JJ + 1,N
              R5 = Q(K)
              DELTA = DELTA + D(R3,R2,R5) - D(R2,R3,R5)
            END DO
            DO I = II+1,JJ-2
              DO J = I+1,JJ-1
                R4A = Q(I)
                R4B = Q(J)
                DELTA = DELTA + D(R4A,R4B,R2) - D(R4A,R4B,R3)
                DELTA = DELTA + D(R3,R4A,R4B) - D(R2,R4A,R4B)
              END DO
            END DO
            IF(DELTA.GT.0) THEN
              Z = Z + DELTA
              Q(II) = R3
              Q(JJ) = R2
              ITRIG = 1
            END IF
          END DO
        END DO
        IF(ITRIG.EQ.1) GO TO 3502
        IF(Z.GT.ZBEST) ZBEST = Z
 3500 CONTINUE
C      WRITE(2,3505) ZBEST
      IF (IVERB == 1) THEN
C          WRITE(*,3505) ZBEST
          CALL dblepr('HEURISTIC OBJ VALUE', -1, DBLE(ZBEST), 1)
      ENDIF
C 3505 FORMAT(' HEURISTIC OBJ VALUE ',I12)
      Z = ZBEST-1
      DO I = 1,N
        Q(I) = 0
      END DO
C
      M=1
      Q(M)=1
      S(1)=1
      trig=1
      DO K = 2,N
        Q(K)=0
      END DO
C
  1   M = M + 1
C
C
      CHECKS=CHECKS+1
      IF (IVERB == 1 .AND. M .GT. OLDM) THEN
C          WRITE (*,6000) M+1, CHECKS
          CALL intpr('reached position', -1, M+1, 1)
          CALL intpr('with following number checks', -1, CHECKS, 1)
C 6000 FORMAT(' reached position ', I5, ' with ', I9, ' checks')

          OLDM=M
      ENDIF
C
C   R interrupt
      CALL rchkusr()
C
C   main loop
C
  2   Q(M)=Q(M)+1
C
C MFH: Make sure to not get out of bounds with S(Q(M)) - 9/24/12
      IF(Q(M).GT.N) GO TO 222
      IF(S(Q(M)).EQ.1) GO TO 2               ! REDUNDANCY
 222  IF(M.EQ.1.AND.Q(M).GT.N) GO TO 9       ! TERMINATE
      IF(M.GT.1.AND.Q(M).GT.N) GO TO 7       ! GO TO RETRACTION
C only for bburcg
      IF(TRIG.EQ.0.AND.Q(M).EQ.2) GO TO 2    ! SYMMETRY FATHOM
C
      S(Q(M))=1
      IF(M.EQ.1) GO TO 1
      IF(M.EQ.N-1) THEN
        CALL EVALBBURCG(ZBD,Q,N,D)
        IF(ZBD.GT.Z) THEN
          Z=ZBD
          IF (IVERB == 1) THEN
C              WRITE(*,*) 'Eval =',z
              CALL dblepr('Eval', -1, DBLE(Z), 1)
          ENDIF
          DO I = 1,N
            X(I)=Q(I)
          END DO
        END IF
        Q(N)=0
        S(Q(M))=0
        GO TO 2
      ELSE
        DO 251 MM = M-1,1,-1    ! Insertion Test
          R3=Q(M)
          IDX1=0
          IDX2=0
          DO I = 1,MM-1
            R1=Q(I)
            DO J = MM,M-1
              R4=Q(J)
              IDX1=IDX1+D(R1,R4,R3)
              IDX2=IDX2+D(R1,R3,R4)
C
            END DO
C
          END DO
C
          DO 250 I = 1,N
            IF(S(I).EQ.1) GO TO 250
            R5=I
C
            DO J = MM,M-1
              R4=Q(J)
              IDX1=IDX1+D(R4,R3,R5)
              IDX2=IDX2+D(R3,R4,R5)
            END DO
C
  250     CONTINUE
C
          DO J = MM, M-2
            DO JJ = J+1, M-1
              R4A=Q(J)
              R4B=Q(JJ)
              IDX1=IDX1+D(R4A,R4B,R3)
              IDX2=IDX2+D(R3,R4A,R4B)
            END DO
          END DO
          IF(IDX1.LT.IDX2) THEN
            S(Q(M))=0
C            ism2 = ism2 + 1
            GO TO 2
          END IF
  251   CONTINUE
C        go to 253
C
        DO 151 MM = M-2,1,-1    ! Interchange Test
          R3=Q(M)
          R2=Q(MM)
          IDX1=0
          IDX2=0
          DO J = MM+1,M-1
            R4 = Q(J)
            IDX1=IDX1+D(R2,R4,R3)
            IDX2=IDX2+D(R3,R4,R2)
          END DO
          DO I = 1,MM-1
            R1=Q(I)
            IDX1=IDX1+D(R1,R2,R3)
            IDX2=IDX2+D(R1,R3,R2)
            DO J = MM+1,M-1
              R4=Q(J)
              IDX1=IDX1+D(R1,R2,R4)
              IDX2=IDX2+D(R1,R3,R4)
C
              IDX1=IDX1+D(R1,R4,R3)
              IDX2=IDX2+D(R1,R4,R2)
C
            END DO
C
          END DO
C
          DO 150 I = 1,N
            IF(S(I).EQ.1) GO TO 150
            R5=I
            IDX1=IDX1+D(R2,R3,R5)
            IDX2=IDX2+D(R3,R2,R5)
C
            DO J = MM+1,M-1
              R4=Q(J)
              IDX1=IDX1+D(R2,R4,R5)
              IDX2=IDX2+D(R3,R4,R5)
C
              IDX1=IDX1+D(R4,R3,R5)
              IDX2=IDX2+D(R4,R2,R5)
            END DO
C
  150     CONTINUE
C
          DO J = MM+1, M-2
            DO JJ = J+1, M-1
              R4A=Q(J)
              R4B=Q(JJ)
              IDX1=IDX1+D(R4A,R4B,R3)
              IDX2=IDX2+D(R4A,R4B,R2)
              IDX1=IDX1+D(R2,R4A,R4B)
              IDX2=IDX2+D(R3,R4A,R4B)
            END DO
          END DO
          IF(IDX1.LT.IDX2) THEN
C            ism = ism + 1
            S(Q(M))=0
            GO TO 2
          END IF
  151   CONTINUE
C
        CALL BOUND2BBURCG(ZBD,N,Q,M,D,S,DD)
        IF(ZBD.LE.Z) THEN
          S(Q(M))=0
C          ism3 = ism3 + 1
          GO TO 2
        END IF
        IF(Q(M).EQ.1) TRIG=1
        GO TO 1
      END IF
C
   7  IF(Q(M).EQ.1) TRIG=0
C MFH: Make sure to not get out of bounds with S(Q(M)) - 6/9/15
      IF(Q(M).GT.N) GO TO 777
      S(Q(M))=0
  777 Q(M)=0
      M=M-1
      IF(Q(M).EQ.1) TRIG=0
      S(Q(M))=0
C
C      WRITE(*,*) 'X',(X(J),J=1,N)
C      WRITE(*,*) 'Q',(Q(J),J=1,N)
C

      GO TO 2
C   9  CALL GETTIM (IHR, IMIN, ISEC, I100)
C      CALL GETDAT (IYR, IMON, IDAY)
C      TIMEB=DFLOAT(86400*IDAY+3600*IHR+60*IMIN+ISEC)+DFLOAT(I100)/100.
C      TIMTOT=TIMEB-TIMEA
C      write(*,*) ism,ism2,ism3
C      WRITE(*,69) Z,TIMTOT
C  9   WRITE(*,69) Z
C      WRITE(2,69) Z,TIMTOT
C      WRITE(2,70) (X(I),I=1,N)
C 69   FORMAT(' MAXIMUM UNWEIGHTED ROW GRADIENT INDEX ',I20)
C 69   FORMAT(' MAXIMUM UNWEIGHTED ROW GRADIENT INDEX ',I7,' CPU TIME ',
C     1        F8.2)
C 70   FORMAT(30I3)
C
  9    IF (IVERB == 1) THEN
C          PRINT *, 'total number of checks: ', CHECKS
          CALL intpr('total number of checks', -1, CHECKS, 1)
       ENDIF

C    Return R RNG
      CALL Putrngstate()

      RETURN
      END
C
      SUBROUTINE BOUND2BBURCG(ZBD,N,Q,M,D,S,DD)
      IMPLICIT INTEGER(A-Z)
      INTEGER Q(N),D(N,N,N),S(N),DD(N,N,N)
      Z1=0
      DO I = 1,M-2
        R1=Q(I)
        DO J = I+1,M-1
          R2=Q(J)
          DO K = J+1,M
            R3=Q(K)
            Z1=Z1+D(R1,R2,R3)
          END DO
        END DO
      END DO
C
      Z2=0
      DO I = 1,M-1
        R1=Q(I)
        DO J = I+1,M
          R2=Q(J)
          DO 60 K = 1,N
            IF(S(K).EQ.1) GO TO 60
            R3=K
            Z2=Z2+D(R1,R2,R3)
 60       CONTINUE
        END DO
      END DO
C
      Z3=0
      DO 90 I = 1,N-1
        IF(S(I).EQ.1) GO TO 90
        R2=I
        DO 91 J = I+1,N
          IF(S(J).EQ.1) GO TO 91
          R3=J
          ZA=0
          ZB=0
          DO 92 K = 1,M
            R1=Q(K)
            ZA=ZA+D(R1,R2,R3)
            ZB=ZB+D(R1,R3,R2)
 92       CONTINUE
          ZCT=ZA
          IF(ZB.GT.ZCT) ZCT=ZB
          Z3=Z3+ZCT
 91     CONTINUE
 90   CONTINUE
C
      N4=0
      DO 93 I = 1,N-2
        IF(S(I).EQ.1) GO TO 93
        R1=I
        DO 94 J = I+1,N-1
          IF(S(J).EQ.1) GO TO 94
          DO 95 K = J+1,N
            IF(S(K).EQ.1) GO TO 95
C            ACT=D(I,J,K)
C            IF(D(I,K,J).GT.ACT) ACT=D(I,K,J)
C            IF(D(J,I,K).GT.ACT) ACT=D(J,I,K)
C            N4=N4+ACT
             N4 = N4 + DD(I,J,K)
  95      CONTINUE
  94    CONTINUE
  93  CONTINUE
C      N1=N*(N-1)*(N-2)/3    ! This bound is OK!  The N1 is total
C      N2=M*(M-1)*(M-2)/3    ! and N2 and N3 are truly computed terms.
C      N3=(N-M)*(M*(M-1))  ! So N1-N2-N3 assumes +1 for rest, which
      ZBD=Z1+Z2+Z3+n4   ! (N-M)*(N-M-1)*(N-M-2)/3      +n4
C      WRITE(*,98) N,M,N1,N2,N3,Z1,Z2,N1-N2-N3,ZBD
C 98   FORMAT(9I7)
      RETURN
      END
C
      SUBROUTINE EVALBBURCG(ZBD,Q,N,D)
      IMPLICIT INTEGER(A-Z)
      INTEGER Q(N),D(N,N,N)
      ZBD=0
      DO 85 I = 1,N
        DO J = 1,N-1
          IF(Q(J).EQ.I) GO TO 85
        END DO
        Q(N)=I
 85   CONTINUE
      DO I = 1,N-2
        R1=Q(I)
        DO J = I+1,N-1
          R2=Q(J)
          DO K = J+1,N
            R3=Q(K)
            ZBD=ZBD+D(R1,R2,R3)
          END DO
        END DO
      END DO
      RETURN
      END

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


## calculate distances for rows and columns, perform hclust and reorder.
.heatmap_contr <- list(
  dist_fun = list(row = dist, col = dist),
  seriation_method = list(row = "OLO", col = "OLO"),
  seriation_control = list(row = NULL, col = NULL),
  scale = "none",
  verbose = FALSE
)

seriate_matrix_heatmap <- function(x, control = NULL) {
  control <- .get_parameters(control, .heatmap_contr)

  if (length(control$dist_fun) == 1L)
    control$dist_fun <-
      list(row = control$dist_fun,
        col = control$dist_fun)
  if (length(control$seriation_method) == 1L)
    control$seriation_method <-
      list(row = control$seriation_method,
        col = control$seriation_method)
  if (length(control$seriation_control) == 1L)
    control$seriation_control <-
      list(row = control$seriation_control,
        col = control$seriation_control)

  if (!is.null(control$scale)) {
    if (control$scale == "row")
      x <- t(scale(t(x)))
    if (control$scale == "col")
      x <- scale(x)
  }

  d <- control$dist_fun$row(x)
  o_row <- seriate(d,
    method = control$seriation_method$row,
    control = control$seriation_control$row)

  d <- control$dist_fun$col(t(x))
  o_col <- seriate(d,
    method = control$seriation_method$col,
    control = control$seriation_control$col)

  #names(row) <- rownames(x)[get_order(o_row)]
  #names(col) <- colnames(x)[get_order(o_col)]

  list(row = o_row[[1]], col = o_col[[1]])
}

set_seriation_method(
  "matrix",
  "Heatmap",
  seriate_matrix_heatmap,
  "Calculate distances for row and column vectors, and seriate. If only a single distance function or seriation method is specified, then it is used for rows and columns. The default seriation method is optimal leaf ordering (OLO) which  perform hierarchical clustering and reorder the dentrograms.",
  .heatmap_contr
)

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


######################################################
## permutations

## constructor
ser_permutation <- function(x, ...) {
  x <- c(list(x), list(...))

  x <- lapply(x, FUN = function(obj) {
    if(inherits(obj, "ser_permutation")) return(obj)
    if(inherits(obj, "ser_permutation_vector")) return(list(obj))
    return(list(ser_permutation_vector(obj)))
    })

  x <- unlist(x, recursive = FALSE)
  class(x) <- c("ser_permutation", "list")
  x
}

## so we can say get_order to permutations
get_order.ser_permutation <- function(x, dim = 1, ...) get_order(x[[dim]])

## print et al
print.ser_permutation <- function(x, ...) {
  writeLines(c(
    gettextf("object of class %s",
      paste(sQuote(class(x)), collapse = ", ")),
    gettextf("contains permutation vectors for %d-mode data\n",
      length(x))
  ))

  print(data.frame("vector length" = sapply(x,
    FUN = function(o) if(.is_identity_permutation(o)) NA_integer_ else length(o)),
    "seriation method" =
      sapply(x, get_method, printable = TRUE),
    check.names = FALSE))

  invisible(x)
}

## fake summary (we dont really provide a summary,
## but summary produces now a reasonable result --- same as print)
summary.ser_permutation <- function(object, ...)
  object

c.ser_permutation <- function(..., recursive = FALSE)
  do.call("ser_permutation", list(...))

## fixme [[<- needs to check for ser_permutation_vector

"[.ser_permutation" <- function(object, i, ...)
  do.call("ser_permutation", unclass(object)[i])

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

## image method that makes a proper image plot of a matrix.
## the rows and columns are swapped and the order of the
## columns (original rows) is reversed.



#' Permutation Image Plot
#'
#' Provides methods for matrix shading, i.e., displaying a color image for
#' matrix (including correlation matrices) and \code{dist} objects given an
#' optional permutation.  The plot arranges colored rectangles to represent the
#' matrix value. Columns and rows appear in the order in the matrix.  This
#' visualization is also know asi a heatmap.  Implementations based on the
#' \pkg{grid} graphics engine and based n \pkg{ggplot2} are provided.
#'
#' Plots a matrix in its original row and column orientation. This means, in a
#' plot the columns become the x-coordinates and the rows the y-coordinates (in
#' reverse order).
#'
#' If \code{x} is of class \code{dist} it is converted to full-storage
#' representation before plotting.
#'
#' \bold{Grid-based plot:} The viewports used for plotting are called:
#' \code{"plot"}, \code{"image"} and \code{"colorkey"}.  \emph{Note:} Since
#' \code{pimage} uses \pkg{grid}, it should not be mixed with base R primitive
#' plotting functions, but the appropriate functions in
#' \code{\link[grid]{grid-package}}.
#'
#' \bold{ggplot2-based plot:} A ggplot2 object is returned. Colors, axis limits
#' and other visual aspects can be added using standard ggplot2 functions
#' (\code{labs}, \code{scale_fill_continuous}, \code{labs}, etc.).
#'
#' @family plots
#'
#' @param x a matrix or an object of class \code{dist}.
#' @param order an object of class \code{ser_permutation} or the name of a seriation method.
#'   If \code{NULL} the order in \code{x} is plotted.
#' @param col a list of colors used. If \code{NULL}, a gray scale is used (for
#' matrix larger values are displayed darker and for \code{dist} smaller
#' distances are darker). For matrices containing logical data, black and white
#' is used. For matrices containing negative values a symmetric diverging color
#' palette is used.
#' @param main plot title.
#' @param xlab,ylab labels for the x and y axes.
#' @param zlim vector with two elements giving the range (min, max) for
#' representing the values in the matrix.
#' @param key logical; add a color key? No key is available for logical
#' matrices.
#' @param keylab string plotted next to the color key.
#' @param symkey logical; if \code{x} contains negative values, should the
#' color palate be symmetric (zero is in the middle)>
#' @param upper_tri,lower_tri,diag a logical indicating whether to show the
#' upper triangel, the lower triangle or the diagonal of the (distance) matrix.
#' @param row_labels,col_labels a logical indicating if row and column labels
#' in \code{x} should be displayed.  If \code{NULL} then labels are displayed
#' if the \code{x} contains the appropriate dimname and the number of labels is
#' 25 or less. A character vector of the appropriate length with labels can
#' also be supplied.
#' @param prop logical; change the aspect ratio so cells in the image have a
#' equal width and height.
#' @param flip_axes logical; exchange rows and columns for plotting.
#' @param reverse_columns logical; revers the order of how the columns are
#' displayed.
#' @param \dots further arguments are ignored.
#' @param newpage,pop,gp Start plot on a new page, pop the viewports after
#' plotting, and use the supplied \code{gpar} object (see \pkg{grid}).
#' @returns Nothing.
#'
#' @author Christian Buchta and Michael Hahsler
#' @keywords hplot
#' @examples
#' set.seed(1234)
#'
#' ## Example: Logical Matrix
#' x <- matrix(sample(c(FALSE, TRUE), 300, rep = TRUE), ncol = 10,
#'   dimnames = list(1:30, LETTERS[1:10]))
#'
#' # Matrix for logical values. TRUE values are dark and no color key is shown. There are too many
#' # Row labels (>25) so they are suppressed.
#' pimage(x)
#'
#' # Show all labels and flip axes or reverse columns
#' pimage(x, row_labels = TRUE, col_labels = TRUE, flip_axes = TRUE)
#' pimage(x, row_labels = TRUE, col_labels = TRUE, reverse_columns = TRUE)
#'
#' # Reorder matrix, use custom colors, and add a title.
#' pimage(x, order = seriate(x), row_labels = TRUE, col_labels = TRUE,
#'   col = c("white", "red"), main = "Random Data (Reordered)")
#'
#' ## Example: Positive Matrix
#' x <- matrix(runif(100), ncol = 10,
#'   dimnames = list(LETTERS[1:10], paste0("X", 1:10)))
#'
#' pimage(x)
#'
#' ## Example: Pos/Neg. Matrix
#' x <- matrix(rnorm(100), ncol = 10,
#'   dimnames = list(LETTERS[1:10], paste0("X", 1:10)))
#'
#' pimage(x)
#'
#' ## Example: Distance Matrix
#' # Show a reordered distance matrix (distances between rows).
#' # Dark means low distance. The aspect ratio is automatically fixed to 1:1.
#' # The upper triangle is suppressed triangle
#' d <- dist(x)
#' pimage(d,  order = seriate(d),
#'   main = "Random Data (Distances)")
#'
#' # Show only distances that are smaller than 4 using limits on z.
#' pimage(d, order = seriate(d),
#'   main = "Random Data (Distances + Theshold)", zlim = c(0, 4))
#'
#' ## Example: Correlation Matrix
#' # we calculate correlation between rows and seriate the matrix
#' r <- cor(t(x))
#' r <- permute(r, seriate(r))
#' pimage(r, upper = FALSE, diag = FALSE, zlim = c(-1, 1), reverse_columns = TRUE,
#'   main = "Random Data (Correlation)")
#'
#' # Add to the plot using functions in package grid
#' # Note: pop = FALSE allows us to manipulate viewports
#' library("grid")
#' pimage(x, pop = FALSE)
#'
#' # available viewports are: "main", "colorkey", "plot", "image"
#' current.vpTree()
#'
#' # Highlight cell column 7 (G) / row 5 (from top)/col with a red arrow starting at 5/2
#' # Note: columns are x and rows are y.
#' downViewport(name = "image")
#' grid.lines(x = c(5, 7), y = c(2, 5), arrow = arrow(),
#'   default.units = "native", gp = gpar(col = "red", lwd = 3))
#'
#' # add a red box around rows 15 and 16
#' grid.rect(x = 0.5, y = 5.5, width = ncol(x), height = 2,
#'   just = "left",
#'   default.units = "native", gp = gpar(col = "red", lwd = 3, fill = NA))
#'
#' ## remove the viewports
#' popViewport(0)
#'
#' ## put several pimages on a page (use grid viewports and newpage = FALSE)
#' # set up grid layout
#' library(grid)
#' grid.newpage()
#' top_vp <- viewport(layout = grid.layout(nrow = 1, ncol = 2,
#'   widths = unit(c(.4, .6), unit = "npc")))
#' col1_vp <- viewport(layout.pos.row = 1, layout.pos.col = 1, name = "col1_vp")
#' col2_vp <- viewport(layout.pos.row = 1, layout.pos.col = 2, name = "col2_vp")
#' splot <- vpTree(top_vp, vpList(col1_vp, col2_vp))
#' pushViewport(splot)
#'
#' seekViewport("col1_vp")
#' o <- seriate(x)
#' pimage(x, o, labCol = FALSE, main = "Random Data",
#'   newpage = FALSE)
#'
#' seekViewport("col2_vp")
#' ## add the reordered dissimilarity matrix for rows
#' d <- dist(x)
#' pimage(d, o[[1]], labCol = FALSE, main = "Random Data",
#'   newpage = FALSE)
#'
#' popViewport(0)
#'
#' ##-------------------------------------------------------------
#' ## ggplot2 Examples
#' if (require("ggplot2")) {
#'
#' library("ggplot2")
#'
#' ## Example: Logical Matrix
#' x <- matrix(sample(c(FALSE, TRUE), 300, rep = TRUE), ncol = 10,
#'   dimnames = list(1:30, LETTERS[1:10]))
#'
#' # Matrix for logical values. TRUE values are dark. There are too many
#' # Row labels (>25) so they are suppressed.
#' ggpimage(x)
#'
#' # Show all labels and flip axes or reverse columns
#' ggpimage(x, flip_axes = TRUE, row_labels = TRUE, col_labels = TRUE)
#' ggpimage(x, reverse_columns = TRUE, row_labels = TRUE, col_labels = TRUE)
#'
#' # Add lines
#' ggpimage(x) +
#'   geom_hline(yintercept = seq(0, nrow(x)) + .5) +
#'   geom_vline(xintercept = seq(0, ncol(x)) + .5)
#'
#' # Reorder matrix, use custom colors, add a title,
#' # and hide colorkey.
#' ggpimage(x, order = seriate(x), row_labels = TRUE, col_labels = TRUE) +
#'   scale_fill_manual(values = c("grey90", "red")) +
#'   theme(legend.position = "none") +
#'   labs(title = "Random Data")
#'
#' ## Example: Positive Matrix
#' x <- matrix(runif(100), ncol = 10,
#'   dimnames = list(LETTERS[1:10], paste0("X", 1:10)))
#'
#' ggpimage(x, order = seriate(x)) +
#'   labs(title = "Random Data")
#'
#' #' ## Example: Pos/Neg. Matrix
#' x <- matrix(rnorm(100), ncol = 10,
#'   dimnames = list(LETTERS[1:10], paste0("X", 1:10)))
#'
#' ggpimage(x, order = seriate(x)) +
#'   labs(title = "Random Data")
#'
#' ## Example: Distance Matrix
#' # Show a reordered distance matrix (distances between rows).
#' # Dark means low distance. The aspect ratio is automatically fixed to 1:1.
#' # The upper triangle is suppressed triangle
#' d <- dist(x)
#' ggpimage(d, order = seriate(d)) +
#'   labs(title = "Random Data", subtitle = "Distances")
#'
#' # Show also upper triangle and diagonal
#' ggpimage(d, order = seriate(d), upper_tri = TRUE, diag = TRUE) +
#'   labs(title = "Random Data", subtitle = "Distances")
#'
#' # Show only distances that are smaller than 4 using limits on fill.
#' ggpimage(d,  order = seriate(d), zlim = c(0, 4)) +
#'   labs(title = "Random Data (Distances + Theshold)")
#'
#' ## Example: Correlation Matrix
#' # we calculate correlation between rows and seriate the matrix
#' r <- cor(t(x))
#' r <- permute(r, seriate(r))
#' ggpimage(r,  zlim = c(-1, 1), upper = FALSE, diag = FALSE, reverse_columns = TRUE) +
#'   geom_text(aes(x = col, y = row, label = round(x, 2)), color = "black", size = 4) +
#'   labs(title = "Random Data", subtitle = "Correlation")
#'
#' ## Example: Custom themes and colors
#' # Use ggplot2 themes with theme_set
#' old_theme <- theme_set(theme_linedraw())
#' ggpimage(d, order = seriate(d)) +
#'   labs(title = "Random Data (Distances)")
#' theme_set(old_theme)
#'
#' # Use custom color palettes: Gray scale, Colorbrewer (provided in ggplot2) and colorspace
#' ggpimage(d, order = seriate(d), upper_tri = FALSE) +
#'   scale_fill_gradient(low = "black", high = "white", na.value = "white")
#'
#' ggpimage(d, order = seriate(d), upper_tri = FALSE) +
#'   scale_fill_distiller(palette = "Spectral", direction = +1, na.value = "white")
#'
#' ggpimage(d, order = seriate(d), upper_tri = FALSE) +
#'   colorspace::scale_fill_continuous_sequential("Reds", rev = FALSE, na.value = "white")
#' }
#' @export
pimage <-
  function(x,
    order = NULL,
    col = NULL,
    main = "",
    xlab = "",
    ylab = "",
    zlim = NULL,
    key = TRUE,
    keylab = "",
    symkey = TRUE,
    upper_tri = TRUE,
    lower_tri = TRUE,
    diag = TRUE,
    row_labels = NULL,
    col_labels = NULL,
    prop = TRUE,
    flip_axes = FALSE,
    reverse_columns = FALSE,
    ...,
    newpage = TRUE,
    pop = TRUE,
    gp = NULL)
UseMethod("pimage")

### Note for matrix large values are dark, for dist large values are light!
#' @rdname pimage
#' @export
pimage.matrix <-
  function(x,
    order = NULL,
    col = NULL,
    main = "",
    xlab = "",
    ylab = "",
    zlim = NULL,
    key = TRUE,
    keylab = "",
    symkey = TRUE,
    upper_tri = TRUE,
    lower_tri = TRUE,
    diag = TRUE,
    row_labels = NULL,
    col_labels = NULL,
    prop = TRUE,
    flip_axes = FALSE,
    reverse_columns = FALSE,
    ...,
    newpage = TRUE,
    pop = TRUE,
    gp = NULL) {
    x <- as.matrix(x)

    # check data
    if (all(is.na(x)))
      stop("all data missing in x.")
    if (any(is.infinite(x)))
      stop("x contains infinite entries.")

    # set default values
    # no key for logical data!
    if (is.logical(x))
      key <- FALSE

    if (is.null(col)) {
      if (is.logical(x))
        col <- c("white", "black")
      else if (any(x < 0, na.rm = TRUE))  {
        col <- .diverge_pal(100)
        if (is.null(zlim) && symkey)
          zlim <- max(abs(range(x, na.rm = TRUE))) * c(-1, 1)
      }
      else
        col <- .sequential_pal(100)
    }

    if (is.null(prop))
      prop <- FALSE

    if (is.null(gp))
      gp <- gpar()

    if (is.null(zlim))
      zlim <- range(x, na.rm = TRUE)


    # reorder
    if (!is.null(order))
      x <- permute(x, order)

    # mask triangles
    if (any(!upper_tri ||
        !lower_tri ||
        !diag) &&
        nrow(x) != ncol(x))
      stop("Upper triange, lower triangle or diagonal can only be suppressed for square matrices!")
    if (!upper_tri)
      x[upper.tri(x)] <- NA
    if (!lower_tri)
      x[lower.tri(x)] <- NA
    if (!diag)
      diag(x) <- NA

    # change x and y
    if (flip_axes) {
      x <- t(x)
      tmp <- row_labels
      row_labels <- col_labels
      col_labels <- tmp
    }

    # reverse order of columns
    if (reverse_columns)
      x <- x[, seq(ncol(x), 1)]

    # deal with row/col labels
    if (!is.null(row_labels) && !is.logical(row_labels)) {
      if (length(row_labels) != nrow(x))
        stop("Length of row_labels does not match the number of rows of x.")
      rownames(x) <- row_labels
      row_labels <- TRUE
    }

    if (!is.null(col_labels) && !is.logical(col_labels)) {
      if (length(col_labels) != ncol(x))
        stop("Length of col_labels does not match the number of columns of x.")
      colnames(x) <- col_labels
      col_labels <- TRUE
    }

    if (is.null(row_labels))
      if (!is.null(rownames(x)) &&
          nrow(x) < 25) {
        row_labels <- TRUE
      } else{
        row_labels <- FALSE
      }
    if (is.null(col_labels))
      if (!is.null(colnames(x)) &&
          ncol(x) < 25) {
        col_labels <- TRUE
      } else{
        col_labels <- FALSE
      }

    if (is.null(rownames(x)))
      rownames(x) <- seq(nrow(x))
    if (is.null(colnames(x)))
      colnames(x) <- seq(ncol(x))

    # create layout for plot
    bottom_mar <- if (col_labels)
      max(stringWidth(colnames(x))) + unit(3, "lines")
    else
      unit(1, "lines")

    left_mar <- if (row_labels)
      max(stringWidth(rownames(x))) + unit(3, "lines")
    else
      unit(1, "lines")

    if (newpage)
      grid.newpage()

    if (key) {
      .grid_basic_layout_with_colorkey(
        main = main,
        left = left_mar,
        right = unit(0, "lines"),
        bottom = bottom_mar,
        gp = gp
      )
       down <- downViewport("colorkey")
      .grid_colorkey(zlim,
        col = col,
        horizontal = FALSE,
        lab = keylab)
      upViewport(down)

    } else
      .grid_basic_layout(
        main = main,
        left = left_mar,
        right = unit(0, "lines"),
        bottom = bottom_mar,
        gp = gp
      )

    down <- downViewport("plot")
    .grid_image(
      x,
      col = col,
      zlim = zlim,
      prop = prop
    ) #, gp=gp)
    upViewport(down)

    ## axes and labs
    down <- downViewport("image")
    if (col_labels)
      grid.text(
        colnames(x),
        y = unit(-1, "lines"),
        x = unit(1:ncol(x), "native"),
        rot = 90,
        just = "right"
      ) #, gp=gp)
    #grid.xaxis(at=1:ncol(x),
    #	    label=colnames(x))
    if (row_labels)
      grid.text(
        rownames(x),
        x = unit(-1, "lines"),
        y = unit(1:nrow(x), "native"),
        just = "right"
      ) #, gp=gp)
    #grid.yaxis(at=1:nrow(x),
    #    label=rownames(x))


    if (xlab != "")
      grid.text(xlab, y = -1 * bottom_mar + unit(1, "lines"))
    #, gp=gp)
    if (ylab != "")
      grid.text(ylab,
        x = ,-1 * left_mar + unit(1, "lines"),
        rot = 90) #, gp=gp)

    # it is always 2 up from main
    seekViewport("main")
    down <- 2

    if (pop)
      popViewport(down)
    else
      upViewport(down)
  }

pimage.default <- pimage.matrix


# as.matrix does not work for table!
table2matrix <- function(M)
  matrix(M, ncol = ncol(M), dimnames = dimnames(M))

#' @rdname pimage
#' @export
pimage.table <- function(x, order = NULL, ...)
  pimage.matrix(table2matrix(x), order = order, ...)

## small values are dark
#' @rdname pimage
#' @export
pimage.dist <-
  function(x,
    order = NULL,
    col = NULL,
    main = "",
    xlab = "",
    ylab = "",
    zlim = NULL,
    key = TRUE,
    keylab = "",
    symkey = TRUE,
    upper_tri = FALSE,
    lower_tri = TRUE,
    diag = FALSE,
    row_labels = NULL,
    col_labels = NULL,
    prop = TRUE,
    flip_axes = FALSE,
    reverse_columns = FALSE,
    ...,
    newpage = TRUE,
    pop = TRUE,
    gp = NULL) {
    if (is.null(col))
      col <- rev(.sequential_pal(100))
    else
      col <- rev(col)

    if (is.null(prop))
      prop <- TRUE

    if (!is.null(order))
      x <- permute(x, order)

    if (flip_axes) warning("flip_axes has no effect for distance matrices.")

    pimage.matrix(
      x,
      order = NULL,  # already reordered
      main = main,
      xlab = xlab,
      ylab = ylab,
      col = col,
      zlim = zlim,
      key = key,
      keylab = keylab,
      symkey = symkey,
      upper_tri = upper_tri,
      lower_tri = lower_tri,
      diag = diag,
      row_labels = row_labels,
      col_labels = col_labels,
      prop = prop,
      flip_axes = FALSE,
      reverse_columns = reverse_columns,
      ...,
      newpage = newpage,
      pop = pop,
      gp = gp
    )
  }

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

.dist_methods <-
  c("spearman",
    "kendall",
    "manhattan",
    "euclidean",
    "hamming",
    "ppc",
    "aprd")

#' Dissimilarities and Correlations Between Seriation Orders
#'
#' Calculates dissimilarities/correlations between seriation orders in a list of type
#' [ser_permutation_vector].
#'
#' `ser_cor()` calculates the correlation between two sequences (orders).
#' Note that a seriation order and its reverse are identical and purely an
#' artifact due to the method that creates the order. This is a major
#' difference to rankings. For ranking-based correlation measures (Spearman and
#' Kendall) the absolute value of the correlation is returned for
#' `reverse = TRUE` (in effect returning the correlation for the reversed order). If
#' `test = TRUE` then the appropriate test for association is performed
#' and a matrix with p-values is returned as the attribute `"p-value"`.
#' Note that no correction for multiple testing is performed.
#'
#' For `ser_dist()`, the correlation coefficients (Kendall's tau and
#' Spearman's rho) are converted into a dissimilarity by taking one minus the
#' correlation value. Note that Manhattan distance between the ranks in a
#' linear order is equivalent to Spearman's footrule metric (Diaconis 1988).
#' `reverse = TRUE` returns the pairwise minima using also reversed
#' orders.
#'
#' The positional proximity coefficient (ppc) is a precedence invariant measure
#' based on product of the squared positional distances in two permutations
#' defined as (see Goulermas et al 2016):
#'
#' \deqn{d_{ppc}(R, S) = 1/h \sum_{j=2}^n \sum_{i=1}^{j-1}
#' (\pi_R(i)-\pi_R(j))^2 * (\pi_S(i)-\pi_S(j))^2,}
#'
#' where \eqn{R} and \eqn{S} are two seriation orders, \eqn{pi_R} and
#' \eqn{pi_S} are the associated permutation vectors and \eqn{h} is a
#' normalization factor. The associated generalized correlation coefficient is
#' defined as \eqn{1-d_{ppc}}. For this precedence invariant measure
#' `reverse` is ignored.
#'
#' The absolute pairwise rank difference (aprd) is also precedence invariant
#' and defined as a distance measure:
#'
#' \deqn{d_{aprd}(R, S) = \sum_{j=2}^n \sum_{i=1}^{j-1} | |\pi_R(i)-\pi_R(j)| -
#' |\pi_S(i)-\pi_S(j)| |^p,}
#'
#' where \eqn{p} is the power which can be passed on as parameter `p` and
#' is by default set to 2. For this precedence invariant measure `reverse`
#' is ignored.
#'
#' `ser_align()` tries to normalize the direction in a list of seriations
#' such that ranking-based methods can be used. We add for each permutation
#' also the reversed order to the set and then use a modified version of Prim's
#' algorithm for finding a minimum spanning tree (MST) to choose if the
#' original seriation order or its reverse should be used. We use the orders
#' first added to the MST. Every time an order is added, its reverse is removed
#' from the possible remaining orders.
#'
#' @family permutation
#'
#' @param x set of seriation orders as a list with elements which can be
#' coerced into [ser_permutation_vector] objects.
#' @param y if not `NULL` then a single seriation order can be specified.
#' In this case `x` has to be a single seriation order and not a list.
#' @param method a character string with the name of the used measure.
#' Available measures are: `"kendall"`, `"spearman"`,
#' `"manhattan"`, `"euclidean"`, `"hamming"`, `"ppc"`
#' (positional proximity coefficient), and `"aprd"` (absolute pairwise
#' rank differences).
#' @param reverse a logical indicating if the orders should also be checked in
#' reverse order and the best value (highest correlation, lowest distance) is
#' reported. This only affect ranking-based measures and not precedence
#' invariant measures (e.g., ppc, aprd).
#' @param test a logical indicating if a correlation test should be performed.
#' @param ... Further arguments passed on to the method.
#' @return
#' - `ser_dist()` returns an object of class [dist].
#' - `ser_align()` returns a new list with elements of class
#'   [ser_permutation].
#' @author Michael Hahsler
#' @references P. Diaconis (1988): Group Representations in Probability and
#' Statistics. Institute of Mathematical Statistics, Hayward, CA.
#'
#' J.Y. Goulermas, A. Kostopoulos, and T. Mu (2016): A New Measure for
#' Analyzing and Fusing Sequences of Objects. _IEEE Transactions on
#' Pattern Analysis and Machine Intelligence_ **38**(5):833-48.
#' \doi{10.1109/TPAMI.2015.2470671}
#' @keywords cluster
#' @examples
#' set.seed(1234)
#' ## seriate dist of 50 flowers from the iris data set
#' data("iris")
#' x <- as.matrix(iris[-5])
#' x <- x[sample(1:nrow(x), 50), ]
#' rownames(x) <- 1:50
#' d <- dist(x)
#'
#' ## Create a list of different seriations
#' methods <- c("HC_single", "HC_complete", "OLO", "GW", "R2E", "VAT",
#'   "TSP", "Spectral", "SPIN", "MDS", "Identity", "Random")
#'
#' os <- sapply(methods, function(m) {
#'   cat("Doing", m, "... ")
#'   tm <- system.time(o <- seriate(d, method = m))
#'   cat("took", tm[3],"s.\n")
#'   o
#' })
#'
#' ## Compare the methods using distances. Default is based on
#' ## Spearman's rank correlation coefficient where reverse orders are
#' ## also considered.
#' ds <- ser_dist(os)
#' hmap(ds, margin = c(7,7))
#'
#' ## Compare using correlation between orders. Reversed orders have
#' ## negative correlation!
#' cs <- ser_cor(os, reverse = FALSE)
#' hmap(cs, margin = c(7,7))
#'
#' ## Compare orders by allowing orders to be reversed.
#' ## Now all but random and identity are highly positive correlated
#' cs2 <- ser_cor(os, reverse = TRUE)
#' hmap(cs2, margin=c(7,7))
#'
#' ## A better approach is to align the direction of the orders first
#' ## and then calculate correlation.
#' os_aligned <- ser_align(os)
#' cs3 <- ser_cor(os_aligned, reverse = FALSE)
#' hmap(cs3, margin = c(7,7))
#'
#' ## Compare the orders using clustering. We use Spearman's foot rule
#' ## (Manhattan distance of ranks). In order to use rank-based method,
#' ## we align the direction of the orders.
#' os_aligned <- ser_align(os)
#' ds <- ser_dist(os_aligned, method = "manhattan")
#' plot(hclust(ds))
#' @export
ser_dist <-
function(x,
  y = NULL,
  method = "spearman",
  reverse = TRUE,
  ...) {
  method <- match.arg(tolower(method), .dist_methods)

  ## make sure everything is a permutation vector
  if (!is.null(y))
    x <- list(ser_permutation_vector(x), ser_permutation_vector(y))
  else
    x <- lapply(x, ser_permutation_vector)

  if (!reverse)
    switch(
      method,
      spearman = stats::as.dist(1 - ser_cor(
        x, method = "spearman", reverse = FALSE
      )),
      kendall = stats::as.dist(1 - ser_cor(
        x, method = "kendal", reverse = FALSE
      )),

      ### Manhattan == Spearman's footrule
      manhattan = stats::dist(t(.lget_rank(x)), method = "manhattan"),
      euclidean = stats::dist(t(.lget_rank(x)), method = "euclidean"),
      hamming   = .dist_hamming(t(.lget_rank(x))),
      ppc       = as.dist(1 - ser_cor(
        x, method = "ppc", reverse = FALSE
      )),
      aprd      = stats::as.dist(.aprd(x, ...))
    )

  else
    switch(
      method,
      spearman  = stats::as.dist(1 - ser_cor(
        x, method = "spearman", reverse = TRUE
      )),
      kendall   =  stats::as.dist(1 - ser_cor(
        x, method = "kendal", reverse = TRUE
      )),

      ### Manhattan == Spearman's footrule
      manhattan = .find_best(dist(t(
        .lget_rank(.add_rev(x))
      ),
        method = "manhattan")),
      euclidean = .find_best(dist(t(
        .lget_rank(.add_rev(x))
      ),
        method = "euclidean")),
      hamming   = .find_best(.dist_hamming(t(
        .lget_rank(.add_rev(x))
      ))),

      ### positional proximity coefficient is direction invariant
      ppc       = stats::as.dist(1 - ser_cor(
        x, method = "ppc", reverse = FALSE
      )),
      aprd      = stats::as.dist(.aprd(x, ...))
    )
}

#' @rdname ser_dist
#' @export
ser_cor <- function(x,
  y = NULL,
  method = "spearman",
  reverse = TRUE,
  test = FALSE) {
  ## Note: not all .dist_methods are implemented!
  method <- match.arg(tolower(method), .dist_methods)

  ## make sure everything is a permutation vector
  if (!is.null(y))
    x <- list(ser_permutation_vector(x), ser_permutation_vector(y))
  else
    x <- lapply(x, ser_permutation_vector)

  m <- .lget_rank(x)

  if (method == "ppc") {
    if (test)
      stop("No test for association available for PPC!")
    return(.ppc(x))
  }

  ## cor based methods
  co <- stats::cor(m, method = method)
  if (reverse)
    co <- abs(co)

  ## add a correlation test?
  if (test) {
    p <- outer(1:ncol(m), 1:ncol(m), FUN =
        Vectorize(function(i, j)
          stats::cor.test(m[, i], m[, j], method = method)$p.value))
    dimnames(p) <- dimnames(co)
    attr(co, "p-value") <- p
  }

  co
}

#' @rdname ser_dist
#' @export
ser_align <- function(x, method = "spearman") {
  if (!is.list(x) ||
      any(!sapply(x, inherits, "ser_permutation_vector")))
    stop("x needs to be a list with elements of type 'ser_permutation_vector'")

  .do_rev(x, .alignment(x, method = method))
}

.dist_hamming <- function(x) {
  n <- nrow(x)
  m <- matrix(nrow = n, ncol = n)
  for (i in seq_len(n))
    for (j in seq(i, n))
      m[j, i] <- m[i, j] <- sum(x[i, ] != x[j, ])
  mode(m) <- "numeric"
  dimnames(m) <- list(rownames(x), rownames(x))
  stats::as.dist(m)
}

### make a permutation list into a rank matrix (cols are permutations)
.lget_rank <- function(x)
  sapply(x, get_rank)

### add reversed permutations to a list of permutations
.add_rev <- function(x) {
  os <- append(x, lapply(x, rev))
  names(os) <- c(labels(x), paste(labels(x), "_rev", sep = ""))
  os
}

### reverses permutations in the list given a logical indicator vector
.do_rev <- function(x, rev) {
  for (i in which(rev))
    x[[i]] <- rev(x[[i]])
  x
}

### finds the smallest distance in lists with reversed orders present
.find_best <- function(d) {
  ### find smallest values
  m <- as.matrix(d)
  n <- nrow(m) / 2
  m1 <- m[1:n, 1:n]
  m2 <- m[(n + 1):(2 * n), (n + 1):(2 * n)]
  m3 <- m[1:n, (n + 1):(2 * n)]
  m4 <- m[(n + 1):(2 * n), 1:n]
  stats::as.dist(pmin(m1, m2, m3, m4))
}

### find largest values in matrix
.find_best_max <- function(d) {
  m <- as.matrix(d)
  n <- nrow(m) / 2
  m1 <- m[1:n, 1:n]
  m2 <- m[(n + 1):(2 * n), (n + 1):(2 * n)]
  m3 <- m[1:n, (n + 1):(2 * n)]
  m4 <- m[(n + 1):(2 * n), 1:n]
  pmax(m1, m2, m3, m4)
}


### returns TRUE for sequences which should be reversed
.alignment <- function(x, method = "spearman") {
  if (!is.list(x) ||
      any(!sapply(x, inherits, "ser_permutation_vector")))
    stop("x needs to be a list with elements of type 'ser_permutation_vector'")

  method <- match.arg(tolower(method), .dist_methods)

  n <- length(x)

  ## calculate dist (orders + reversed orders)
  d <-
    as.matrix(ser_dist(.add_rev(x), method = method, reverse = FALSE))
  diag(d) <- NA
  for (i in 1:n) {
    d[i, n + i] <- NA
    d[n + i, i] <- NA
  }

  ## start with closest pair
  take <- which(d == min(d, na.rm = TRUE), arr.ind = TRUE)[1, ]
  #d[, c(take, (take+n) %% (2*n))] <- NA

  ## mark order and complement as taken
  d[, c(take, (take + n) %% (2 * n))] <- Inf

  ## keep adding the closest
  while (length(take) < n) {
    t2 <-
      which(d[take, ] == min(d[take, ], na.rm = TRUE), arr.ind = TRUE)[1, 2]
    #d[, c(t2,  (t2+n) %% (2*n))] <- NA

    ### closest to all
    #t2 <- which.min(colSums(d[take,], na.rm = T))
    d[, c(t2,  (t2 + n) %% (2 * n))] <- Inf

    take <- append(take, t2)
  }

  ## create indicator vector for the orders which need to be reversed
  take_ind <- logical(n)
  take_ind[take[take > n] - n] <- TRUE
  names(take_ind) <- names(x)
  take_ind
}

## Propositional Proximity Coefficient (1 - generalized corr. coef.)
## Goulermas, Kostopoulos and Mu (2016). A new measure for analyzing and fusing
## sequences of objects, IEEE Transactions on Pattern Analysis and Machine
## Intelligence 38(5):833-48.
##
## x,y ... permutation vectors (ranks)
.vppc <- Vectorize(function(x, y) {
  x <- get_rank(x)
  y <- get_rank(y)
  n <- length(x)

  #sum <- 0
  #for(j in 2:n) for(i in 1:(j-1)) sum <- sum + (x[i]-x[j])^2 * (y[i]-y[j])^2

  ## use fast matrix algebra instead
  Ax <-
    (x %*% rbind(rep_len(1, n)) - tcrossprod(cbind(rep_len(1, n)), x)) ^ 2
  Ay <-
    (y %*% rbind(rep_len(1, n)) - tcrossprod(cbind(rep_len(1, n)), y)) ^ 2
  ## note: Ay is symetric
  sum <- sum(diag(Ax %*% Ay))

  ## scale by theoretical maximum
  zapsmall(sum / (n ^ 6 / 15 - n ^ 4 / 6 + n ^ 2 / 10))
})

.ppc <- function(x)
  outer(x, x, .vppc)

# Sum of differences of rank differences
#
# distance(R, S) =
#  \sum_{i,j} | |\pi_R(i)-\pi_R(j)| - |\pi_S(i)-\pi_S(j)| |^p
#

.vaprd <- Vectorize(function(x, y, p = 2) {
  x <- get_rank(x)
  y <- get_rank(y)
  n <- length(x)

  sum <- 0
  for (j in 2:n)
    for (i in 1:(j - 1))
      sum <- sum + abs(abs(x[i] - x[j]) - abs(y[i] - y[j])) ^ p

  ## FIXME: scale by theoretical maximum?
  sum
})

.aprd <- function(x, p = 2)
  outer(x, x, .vaprd, p = p)

C   ANTI-ROBINSON SERIATION
C   simulated annealing algorithm - provides an initial permutation
C   by Brusco, M., Koehn, H.F., and Stahl, S.
C   R Interface by Michael Hahsler

C      PROGRAM SANNEAL
      SUBROUTINE arsa(N, A, COOL, TMIN, NREPS, IPERM, D, U,
     1 S, T, SB, ZMAX, RULE, TRYMULT, IVERB)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(N,N)
      DIMENSION IPERM(N)
      DOUBLE PRECISION D(N,N)
      REAL S1, RCRIT
      INTEGER U(N), S(N), UNSEL, T(NREPS,N), SB(N), Q, NREPS

      EPS = 1.0D-08

C   Defaults
C      RULE = .5
C        COOL = .95
C        TMIN = .0001d0

C     Initialize R RNG
      CALL getrngstate()

      DO I = 1,N-1
        DO J = I+1,N
          D(I,J) = DFLOAT(J-I)
          D(J,I) = D(I,J)
        END DO
      END DO

      DO 999 III = 1,NREPS
        DO I = 1,N
          U(I) = I
          T(III,I) = 0
        END DO
        UNSEL = N
        DO 1 I = 1,N
C          S1 = rand()
          CALL unifrand(S1)
          ISET = INT(S1 * FLOAT(UNSEL) + 1.)
          IF(ISET.GT.UNSEL) ISET = UNSEL
          T(III,I) = U(ISET)
C          DO J = ISET,UNSEL
C          out of bounds error reported by Rohan Shah (9/13/12)
          DO J = ISET,UNSEL-1
            U(J) = U(J+1)
          END DO
          UNSEL = UNSEL - 1
  1     CONTINUE
 999  CONTINUE
C
      ZMIN = 9.9D+20
      ZAVG = 0.
      ZMAX = 0.
      DO 1000 III = 1,NREPS
        DO I = 1,N
          S(I) = T(III,I)
        END DO
        Z = 0.0D0
        DO I = 1,N-1
          K = S(I)
          DO J = I+1,N
            L = S(J)
            Z = Z + D(I,J) * A(K,L)
          END DO
        END DO
        ZBEST = Z
        TMAX = 0.0D0
C        DO LLL = 1,5000
C   Find initial TMAX using N*10 tries
        DO LLL = 1,N*10
C          S1 = rand()
          CALL unifrand(S1)
          I1 = INT(S1 * FLOAT(N) + 1.)
          IF(I1.GT.N) I1 = N
C 199      S1 = rand()
 199      CALL unifrand(S1)
          J1 = INT(S1 * FLOAT(N) + 1.)
          IF(J1.GT.N) J1 = N
          IF(I1.EQ.J1) GO TO 199
          IF(I1.GT.J1) THEN
            JDUM = J1
            J1 = I1
            I1 = JDUM
          END IF
          K = S(I1)
          M = S(J1)
          DELTA = 0.0D0
          DO 1250 L1 = 1,N
            IF(I1.EQ.L1.OR.J1.EQ.L1) GO TO 1250
            L=S(L1)
            DELTA=DELTA+(D(L1,I1)-D(L1,J1))*(A(L,M)-A(L,K))
 1250     CONTINUE
          IF(DELTA.LT.0) THEN
            IF(ABS(DELTA).GT.TMAX) TMAX = ABS(DELTA)
          END IF
        END DO
C        TMAX = Z
        ILOOP = INT(TRYMULT*N)
        NLOOP = INT((LOG(TMIN)-LOG(TMAX))/LOG(COOL))
        IF (IVERB == 1) THEN
            CALL intpr('Steps needed', -1, NLOOP, 1)
            CALL intpr('Temp', -1, NLOOP, 0)
        ENDIF
        TEMP = TMAX
        DO I = 1,N
          SB(I) = S(I)
        END DO
C
        DO 2000 IJK = 1,NLOOP
        IF (IVERB == 1) THEN
            CALL dblepr('', -1, DBLE(TEMP), 1)
        ENDIF

C   R interrupt
        CALL rchkusr()
C

          DO 2001 KKK = 1,ILOOP
C            S1 = rand()
            CALL unifrand(S1)
            IF(S1.LE.RULE) THEN     ! INTERCHANGE / INSERTION / OR BOTH
C            S1 = rand()
            CALL unifrand(S1)
            I1 = INT(S1 * FLOAT(N) + 1.)
            IF(I1.GT.N) I1 = N
C 99         S1 = rand()
 99         CALL unifrand(S1)
            J1 = INT(S1 * FLOAT(N) + 1.)
            IF(J1.GT.N) J1 = N
            IF(I1.EQ.J1) GO TO 99
            IF(I1.GT.J1) THEN
              JDUM = J1
              J1 = I1
              I1 = JDUM
            END IF
            K = S(I1)
            M = S(J1)
            DELTA = 0.0D0
            DO 250 L1 = 1,N
              IF(I1.EQ.L1.OR.J1.EQ.L1) GO TO 250
              L=S(L1)
              DELTA=DELTA+(D(L1,I1)-D(L1,J1))*(A(L,M)-A(L,K))
  250       CONTINUE
            IF(DELTA.GT.-EPS) THEN
              Z = Z + DELTA
              S(I1) = M
              S(J1) = K
              IF(Z.GT.ZBEST) THEN
                ZBEST = Z
                DO I = 1,N
                  SB(I) = S(I)
                END DO
              END IF
            ELSE
C              S1 = rand()
              CALL unifrand(S1)
              RCRIT = REAL(EXP(DELTA/TEMP))
              IF(S1.LE.RCRIT) THEN
                Z = Z + DELTA
                S(I1) = M
                S(J1) = K
              END IF
            END IF

            ELSE                ! INSERTION

C            S1 = rand()
            CALL unifrand(S1)
            I1 = INT(S1 * FLOAT(N) + 1.)      ! OBJECT POSITION IS I1
            IF(I1.GT.N) I1 = N
C 599        S1 = rand()
 599        CALL unifrand(S1)
            J1 = INT(S1 * FLOAT(N) + 1.)
            IF(J1.GT.N) J1 = N
            IF(I1.EQ.J1) GO TO 599
            K = S(I1)
            DELTA = 0.0D0
            IF(J1.GT.I1) THEN
              SPAN = DFLOAT(J1-I1)
              DO L = I1+1,J1
                Q = S(L)
                DO I = J1+1,N
                  M = S(I)
                  DELTA = DELTA + A(M,Q)
                END DO
                DO I = 1,I1-1
                  M = S(I)
                  DELTA = DELTA - A(M,Q)
                END DO
              END DO
              DO I = 1,I1-1
                M = S(I)
                DELTA = DELTA + SPAN*A(M,K)
              END DO
              DO I = J1+1,N
                M = S(I)
                DELTA = DELTA - SPAN*A(K,M)
              END DO
              SPAN2 = SPAN+1
              DO I = I1+1,J1
                SPAN2 = SPAN2-2
                M = S(I)
                DELTA = DELTA + SPAN2*A(K,M)
              END DO
            ELSE
              SPAN = DFLOAT(I1-J1)
              DO L = J1,I1-1
                Q = S(L)
                DO I = I1+1,N
                  M = S(I)
                  DELTA = DELTA - A(M,Q)
                END DO
                DO I = 1,J1-1
                  M = S(I)
                  DELTA = DELTA + A(M,Q)
                END DO
              END DO
              DO I = 1,J1-1
                M = S(I)
                DELTA = DELTA - SPAN*A(M,K)
              END DO
              DO I = I1+1,N
                M = S(I)
                DELTA = DELTA + SPAN*A(K,M)
              END DO
              SPAN2 = SPAN+1
              DO I = J1,I1-1
                SPAN2 = SPAN2-2
                M = S(I)
                DELTA = DELTA - SPAN2*A(K,M)
              END DO
            END IF
            IF(DELTA.GT.-EPS) THEN
              Z = Z + DELTA
              IF(J1.GT.I1) THEN
                DO L = I1,J1-1
                  S(L)=S(L+1)
                END DO
                S(J1) = K
              ELSE
                DO L = I1,J1+1,-1
                  S(L)=S(L-1)
                END DO
                S(J1) = K
              END IF

              IF(Z.GT.ZBEST) THEN
                ZBEST = Z
                DO I = 1,N
                  SB(I) = S(I)
                END DO
              END IF
            ELSE
C              S1 = rand()
              CALL unifrand(S1)
              RCRIT = REAL(EXP(DELTA/TEMP))
              IF(S1.LE.RCRIT) THEN
                Z = Z + DELTA
                IF(J1.GT.I1) THEN
                  DO L = I1,J1-1
                    S(L)=S(L+1)
                  END DO
                  S(J1) = K
                ELSE
                  DO L = I1,J1+1,-1
                    S(L)=S(L-1)
                  END DO
                  S(J1) = K
                END IF
              END IF
            END IF
C
            END IF
 2001     CONTINUE
          TEMP = TEMP*COOL
 2000   CONTINUE

        IF(ZBEST.LT.ZMIN) ZMIN = ZBEST
        IF(ZBEST.GT.ZMAX) THEN
          ZMAX = ZBEST
          DO I = 1,N
            IPERM(I) = SB(I)
          END DO
        END IF
        IF (IVERB == 1) THEN
          CALL intpr('Rep', -1, III, 1)
          CALL dblepr('ZMAX', -1, DBLE(ZMAX), 1)
        END IF
 1000 CONTINUE

C    Return R RNG
      CALL Putrngstate()

      RETURN
      END

#######################################################################
# Basic Grid helpers
# Copyrigth (C) 2011 Michael Hahsler
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



## grid helpers

.grid_basic_layout <- function(main = "",
  left=unit(4, "lines"), right = unit(4, "lines"),
  bottom = unit(4, "lines"),
  gp = gpar()){

  pushViewport(viewport(layout = grid.layout(4, 3,
    widths = unit.c(
      left,                             # space
      unit(1, "npc") - left - right,    # plot
      right                             # space
    ),
    heights = unit.c(
      unit(3, "lines"),                  # title
      unit(1, "lines"),                  # space
      unit(1, "npc") - unit(4, "lines") - bottom, # plot
      bottom                            # space
    )
  ), gp = gp))

  pushViewport(viewport(layout.pos.col = 2, layout.pos.row = 1,
    name="main"))

  gp$cex <- 1.3
  gp$fontface <- "bold"

  grid.text(main, gp = gp)
  upViewport(1)

  pushViewport(viewport(layout.pos.col = 2, layout.pos.row = 3,
    name="plot"))
  upViewport(2)
}

.grid_basic_layout_with_colorkey <- function(main = "",
  left=unit(4, "lines"), right = unit(4, "lines"),
  bottom = unit(4, "lines"),
  gp = gpar()){

  pushViewport(viewport(layout = grid.layout(4, 3,
    widths = unit.c(
      left,                             # space
      unit(1, "npc") - left - right,    # plot
      right                             # space
    ),
    heights = unit.c(
      unit(3, "lines"),                  # title
      unit(1, "lines"),                  # space
      unit(1, "npc") - unit(4, "lines") - bottom, # plot
      bottom                            # space
    )
  ), gp = gp))

  pushViewport(viewport(layout.pos.col = 2, layout.pos.row = 1,
    name="main"))

  gp$cex <- 1.3
  gp$fontface <- "bold"
  grid.text(main, gp = gp)
  upViewport(1)

  pushViewport(viewport(layout.pos.col = 2, layout.pos.row = 3))

  pushViewport(viewport(layout = grid.layout(1, 3,
    widths = unit.c(
      unit(1, "npc") - unit(8, "lines"),# plot
      unit(1, "lines"),                  # space
      unit(1, "lines")                  # colorkey
    ),
    heights = unit.c(
      unit(1, "npc") # plot
    )
  )))

  pushViewport(viewport(layout.pos.col = 1, layout.pos.row = 1,
    name="plot"))

  upViewport(1)

  pushViewport(viewport(layout.pos.col = 3, layout.pos.row = 1,
    name="colorkey"))

  upViewport(1)
  upViewport(2)
}



### new version below uses grid.raster
.grid_image_old <- function(x, y, z, zlim, col = grey.colors(12, 1, 0),
  name = "image", gp = gpar()) {

  if(is.matrix(x)){
    z <- x
    x <- 1:ncol(z)
    y <- 1:nrow(z)
  }

  if(missing(zlim)) zlim <- range(z, na.rm = TRUE)
  else {# fix data for limits
    z[z < zlim[1]] <- NA
    z[z > zlim[2]] <- NA
  }

  offset <- if(zlim[1] < 0) -zlim[1] else 0
  range <- diff(zlim)

  div <- 1/length(col)

  ## create a viewport
  vp <- viewport(
    xscale = c(0,(length(x)+1)), yscale = c((length(y)+1),0),
    default.units = "native", name = name)
  pushViewport(vp)

  ## make sure we have a color for the maximal value (see floor +1)
  col[length(col)+1] <- col[length(col)]

  ## the highest value is lightest color!
  xs <- sapply(x, "rep.int", times = length(y))
  grid.rect(x = xs, y = y, 1, 1,
    gp = gpar(fill = col[floor((z + offset)/range/div)+1], col=0),
    default.units = "native")

  ## make border
  gp_border       <- gp
  gp_border$fill  <- "transparent"
  grid.rect(x = (length(x)+1)/2, y = (length(y)+1)/2,
    width = length(x), height = length(y),
    default.units = "native", gp = gp_border)

  upViewport(1)
}

.grid_image <- function(x, zlim, col = grey.colors(12, 1, 0),
  prop = FALSE, name = "image", gp = gpar()) {

  if(missing(zlim)) zlim <- range(x, na.rm = TRUE)
  else {# fix data for limits
    x[x < zlim[1]] <- NA
    x[x > zlim[2]] <- NA
  }

  offset <- if(zlim[1] < 0) -zlim[1] else 0
  #range <- diff(zlim) ### not al plots have 0 as the minimum!
  range <- zlim[2]+offset

  div <- 1/length(col)

  ## create a viewport
  if(!prop) {
    vp <- viewport(
      #xscale = c(0,ncol(x)), yscale = c(nrow(x),0),
      xscale = c(0.5,ncol(x)+.5), yscale = c(nrow(x)+.5,0.5),
      default.units = "native", name = name)
    pushViewport(vp)
  }else{
    ## ratio
    if (nrow(x) > ncol(x)) { w <- ncol(x)/nrow(x); h <- 1 }
    else if (nrow(x) < ncol(x)) { h <- nrow(x)/ncol(x); w <- 1 }
    else { w <- 1; h <- 1 }
    vp <- viewport(
      #xscale = c(0,ncol(x)), yscale = c(nrow(x),0),
      xscale = c(0.5,ncol(x)+.5), yscale = c(nrow(x)+.5,0.5),
      width = unit(w, "snpc"), height = unit(h, "snpc"),
      default.units = "native", name = name)
    pushViewport(vp)
  }

  ## make sure we have a color for the maximal value (see floor +1)
  col[length(col)+1] <- col[length(col)]

  ## the highest value is lightest color!
  x[] <- col[floor((x + offset)/range/div)+1]
  grid.raster(x, interpolate=FALSE, default.units = "npc", width=1, height=1)

  ## make border
  gp_border       <- gp
  gp_border$fill  <- "transparent"
  grid.rect(gp = gp_border)

  upViewport(1)
}




.grid_barplot_horiz <- function(height, name = "barplot", xlab="",
  gp = gpar(), gp_bars = gpar(fill="lightgrey")) {

  n <-  length(height)

  ## these plots always start at x = 0 or below!
  lim <- c(min(c(height, 0)), max(height))

  ## create a viewport
  vp <- viewport(
    xscale = lim , yscale = c(n,0), default.units = "native", name = name,
    gp =gp)
  pushViewport(vp)

  grid.rect(x = 0, y = (1:n)-.5, width = height, height = 1,
    just = c("left", "center"), default.units = "native", gp =gp_bars)

  ## hopefully there is space outside for axes
  grid.xaxis()
  grid.text(xlab, y = unit(-3, "lines"))

  upViewport(1)
}

# new colorkey uses grid.raster
.grid_colorkey_old <- function(range, col, threshold = NULL,
  name = "colorkey", gp = gpar()) {

  vp <- viewport(
    xscale = range, yscale = c(0,1),
    default.units = "native", name = name)
  pushViewport(vp)

  n <- length(col)
  width <- diff(range)/n
  xs <- seq(range[1] + width/2, range[2] - width/2, length.out = n)

  ## do not display the part above the threshold
  col[xs > threshold] <- NA

  ## col
  gp_col      <- gp
  gp_col$col  <- 0
  gp_col$fill <- col
  grid.rect(x = xs, y = 0, width = width, height = 1,
    just = c("centre", "bottom"), default.units = "native",
    gp = gp_col)


  ## box
  gp_border       <- gp
  gp_border$fill  <- "transparent"
  grid.rect(x = 0, y = 0, width = 1, height = 1,
    just = c("left", "bottom"), default.units = "npc",
    gp = gp_border)

  grid.xaxis(gp = gp)

  upViewport(1)
}

.grid_colorkey <- function(range, col, threshold = NULL, lab = "",
  name = "colorkey", horizontal=TRUE, gp = gpar()) {

  ### no color key for only a single value
  if(diff(range) == 0) {
    vp <- viewport(
      xscale = c(0,1), yscale = c(0,1),
      default.units = "native", name = name)

    pushViewport(vp)
    grid.text(label = range[1], x = 0.5, y = 0.5, default.units = "native")
    return(upViewport(1))
  }

  if(horizontal)
    vp <- viewport(
      xscale = range, yscale = c(0,1),
      default.units = "native", name = name)
  else
    vp <- viewport(
      xscale = c(0,1), yscale = range,
      default.units = "native", name = name)

  pushViewport(vp)

  n <- length(col)
  #width <- diff(range)/n
  #xs <- seq(range[1] + width/2, range[2] - width/2, length.out = n)
  xs <- seq(range[1], range[2], length.out = n)

  ## do not display the part above the threshold
  col[xs > threshold] <- NA

  ## col
  if(horizontal) grid.raster(t(col), width=1, height=1, interpolate=FALSE)
  else grid.raster(rev(col), width=1, height=1, interpolate=FALSE)

  #gp_col      <- gp
  #gp_col$col  <- 0
  #gp_col$fill <- col
  #grid.rect(x = xs, y = 0, width = width, height = 1,
  #    just = c("centre", "bottom"), default.units = "native",
  #    gp = gp_col)


  ## box
  gp_border       <- gp
  gp_border$fill  <- "transparent"
  grid.rect(gp = gp_border)

  if(horizontal) grid.xaxis(gp = gp)
  else grid.yaxis(main = FALSE, gp = gp)

  if(horizontal) grid.text(lab, y= unit(-2.5, "lines"))
  else grid.text(lab, x= unit(4, "lines"), rot = 90)

  upViewport(1)
}

#######################################################################
# Basic Grid helpers
# Copyright (C) 2011 Michael Hahsler
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' @import "grid"

## grid helpers
## requires map.R

.grid_basic_layout <- function(main = "",
  left = unit(4, "lines"),
  right = unit(4, "lines"),
  top = unit(3, "lines"),
  bottom = unit(4, "lines"),
  gp = gpar()) {
  pushViewport(viewport(
    layout = grid.layout(
      nrow = 4,
      ncol = 3,
      widths = unit.c(left,               # space
        unit(1, "npc") - left - right,    # plot
        right),
      # space
      heights = unit.c(
        top,
        # title
        unit(1, "lines"),
        # space
        unit(1, "npc") - unit(1, "lines") - top - bottom,
        # plot
        bottom                            # space
      )
    ),
    gp = gp
  ))

  pushViewport(viewport(
    layout.pos.col = 2,
    layout.pos.row = 1,
    name = "main"
  ))

  gp$cex <- 1.3
  gp$fontface <- "bold"

  grid.text(main, gp = gp)
  upViewport(1)

  pushViewport(viewport(
    layout.pos.col = 2,
    layout.pos.row = 3,
    name = "plot"
  ))
  upViewport(2)
}

.grid_basic_layout_with_colorkey <- function(main = "",
  left = unit(4, "lines"),
  right = unit(0, "lines"),
  top = unit(3, "lines"),
  bottom = unit(4, "lines"),
  gp = gpar()) {
  pushViewport(viewport(
    layout = grid.layout(
      nrow = 4,
      ncol = 3,
      widths = unit.c(# space
        left,
        # plot
        unit(1, "npc") - left - right,
        # space
        right),

      heights = unit.c(
        # title
        top,
        # space
        unit(1, "lines"),
        # plot
        unit(1, "npc") - unit(1, "lines") - top - bottom,
        # space
        bottom
      )
    ),
    gp = gp
  ))

  pushViewport(viewport(
    layout.pos.col = 2,
    layout.pos.row = 1,
    name = "main"
  ))

  gp$cex <- 1.3
  gp$fontface <- "bold"
  grid.text(main, gp = gp)
  upViewport(1)

  pushViewport(viewport(layout.pos.col = 2, layout.pos.row = 3))

  pushViewport(viewport(layout = grid.layout(
    1,
    3,
    widths = unit.c(
      # plot
      unit(1, "npc") - unit(8, "lines"),
      # space
      unit(1, "lines"),
      # colorkey
      unit(1, "lines")
    ),
    # plot
    heights = unit.c(unit(1, "npc"))
  )))

  pushViewport(viewport(
    layout.pos.col = 1,
    layout.pos.row = 1,
    name = "plot"
  ))

  upViewport(1)

  pushViewport(viewport(
    layout.pos.col = 3,
    layout.pos.row = 1,
    name = "colorkey"
  ))

  upViewport(1)
  upViewport(2)
}


.grid_image <- function(x,
  zlim,
  col = grDevices::gray.colors(12),
  prop = FALSE,
  interpolate = FALSE,
  name = "image",
  gp = gpar()) {
  if (missing(zlim))
    zlim <- range(x, na.rm = TRUE)
  else {
    # fix data for limits
    x[x < zlim[1]] <- NA
    x[x > zlim[2]] <- NA
  }

  ## create a viewport
  if (!prop) {
    vp <- viewport(
      #xscale = c(0,ncol(x)), yscale = c(nrow(x),0),
      xscale = c(0.5, ncol(x) + .5),
      yscale = c(nrow(x) + .5, 0.5),
      default.units = "native",
      name = name
    )
    pushViewport(vp)
  } else{
    ## ratio
    if (nrow(x) > ncol(x)) {
      w <- ncol(x) / nrow(x)
      h <- 1
    }
    else if (nrow(x) < ncol(x)) {
      h <- nrow(x) / ncol(x)
      w <- 1
    }
    else {
      w <- 1
      h <- 1
    }
    vp <- viewport(
      xscale = c(0.5, ncol(x) + .5),
      yscale = c(nrow(x) + .5, 0.5),
      width = unit(w, "snpc"),
      height = unit(h, "snpc"),
      default.units = "native",
      name = name
    )
    pushViewport(vp)
  }

  grid.raster(
    .map_color(x, col),
    interpolate = interpolate,
    default.units = "npc",
    width = 1,
    height = 1
  )

  ## make border
  gp_border       <- gp
  gp_border$fill  <- "transparent"
  grid.rect(gp = gp_border)

  upViewport(1)
}

.grid_barplot_horiz <- function(height,
  name = "barplot",
  xlab = "",
  gp = gpar(),
  gp_bars = gpar(fill = "lightgrey")) {
  n <-  length(height)

  ## these plots always start at x = 0 or below!
  lim <- c(min(c(height, 0)), max(height))

  ## create a viewport
  vp <- viewport(
    xscale = lim ,
    yscale = c(n, 0),
    default.units = "native",
    name = name,
    gp = gp
  )
  pushViewport(vp)

  grid.rect(
    x = 0,
    y = (1:n) - .5,
    width = height,
    height = 1,
    just = c("left", "center"),
    default.units = "native",
    gp = gp_bars
  )

  ## hopefully there is space outside for axes
  grid.xaxis()
  grid.text(xlab, y = unit(-3, "lines"))

  upViewport(1)
}

.grid_colorkey <- function(range,
  col,
  threshold = NULL,
  lab = "",
  name = "colorkey",
  horizontal = TRUE,
  gp = gpar()) {
  ### no color key for only a single value
  if (diff(range) == 0) {
    vp <- viewport(
      xscale = c(0, 1),
      yscale = c(0, 1),
      default.units = "native",
      name = name
    )

    pushViewport(vp)
    grid.text(
      label = range[1],
      x = 0.5,
      y = 0.5,
      default.units = "native"
    )
    upViewport(1)
    return()
  }

  if (horizontal)
    vp <- viewport(
      xscale = range,
      yscale = c(0, 1),
      default.units = "native",
      name = name
    )
  else
    vp <- viewport(
      xscale = c(0, 1),
      yscale = range,
      default.units = "native",
      name = name
    )

  pushViewport(vp)

  n <- length(col)
  #width <- diff(range)/n
  #xs <- seq(range[1] + width/2, range[2] - width/2, length.out = n)
  xs <- seq(range[1], range[2], length.out = n)

  ## do not display the part above the threshold
  col[xs > threshold] <- NA

  ## col
  if (horizontal)
    grid.raster(t(col),
      width = 1,
      height = 1,
      interpolate = FALSE)
  else
    grid.raster(rev(col),
      width = 1,
      height = 1,
      interpolate = FALSE)

  #gp_col      <- gp
  #gp_col$col  <- 0
  #gp_col$fill <- col
  #grid.rect(x = xs, y = 0, width = width, height = 1,
  #    just = c("centre", "bottom"), default.units = "native",
  #    gp = gp_col)


  ## box
  gp_border       <- gp
  gp_border$fill  <- "transparent"
  grid.rect(gp = gp_border)

  if (horizontal)
    grid.xaxis(gp = gp)
  else
    grid.yaxis(main = FALSE, gp = gp)

  if (horizontal)
    grid.text(lab, y = unit(-2.5, "lines"))
  else
    grid.text(lab, x = unit(4, "lines"), rot = 90)

  upViewport(1)
}

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


## Seriation generic and default method.
seriate <- function(x, ...)
  UseMethod("seriate")
seriate.default <- function(x, ...)
  stop(gettextf("seriate not implemented for class '%s'.",
    class(x)))

## Seriation methods db.
get_seriation_method <- function(kind, name) {
  if(missing(kind)) method <- registry_seriate$get_entry(name=name)
  else method <- registry_seriate$get_entry(kind=kind, name=name)

  if(is.null(method))
    stop("Unknown seriation method. Check list_seriation_methods(\"",
      kind, "\")")

  method
}

set_seriation_method <- function(kind, name, definition,
  description = NULL, control = list(), ...){

  ## check formals
  if(!identical(names(formals(definition)),
    c("x", "control")))
    stop("Seriation methods must have formals 'x' and 'control'.")

  ## check if entry already exists
  r <- registry_seriate$get_entry(kind=kind, name=name)
  if(!is.null(r) && r$name==name) {
    warning("Entry with name \"", name, "\" for kind \"", kind,
      "\" already exists! Modifying entry.")
    registry_seriate$modify_entry(kind=kind, name=name, fun=definition,
      description=description, control = control)
  } else {
    registry_seriate$set_entry(name=name, kind=kind, fun=definition,
      description=description, control = control)
  }
}

list_seriation_methods <- function(kind) {
  if(missing(kind)) sort(as.vector(sapply(registry_seriate$get_entries(), "[[", "name")))
  else sort(as.vector(sapply(registry_seriate$get_entries(kind=kind), "[[", "name")))
}

show_seriation_methods <- function(kind) {
  if(missing(kind)) m <- registry_seriate$get_entries()
  else m <- registry_seriate$get_entries(kind=kind)
  m[sort(names(m))]
}

### NOTE: disabled snapshot testing since the direction of the order is not defined and randomized
###       for some methods.

library(seriation)
library(testthat)

x <- matrix(
  c(1, 1, 0, 0, 0,
    1, 1, 1, 0, 0,
    0, 0, 1, 1, 1,
    1, 0, 1, 1, 1),
  byrow = TRUE,
  ncol = 5,
  dimnames = list(letters[1:4], LETTERS[1:5])
)

d <- dist(x)

test_that("seriate.dist returns expected results", {
  #local_edition(3) # for snapshot testing

  cat("\n") # for cleaner testthat output
  methods <- list_seriation_methods(kind = "dist")
  os <- sapply(methods, function(m) {
    cat("   -> testing", format(m, width = 13), "... ")
    tm <- system.time(o <- seriate(d, method = m))
    cat("took", formatC(tm[3], digits = 4), "s.\n")
    o
  })
  # make sure they are all the right length
  expect_true(all(sapply(os, length) == nrow(x)))

  # check which methods produce hclusts and which integers
  hclusts <- os[sapply(os, function(x)
    inherits(x, "hclust"))]
  expect_setequal(
    object = names(hclusts),
    expected = c(
      "GW",
      "GW_average",
      "GW_complete",
      "GW_single",
      "GW_ward",
      "HC",
      "HC_average",
      "HC_complete",
      "HC_single",
      "HC_ward",
      "OLO",
      "OLO_average",
      "OLO_complete",
      "OLO_single",
      "OLO_ward"
    )
  )
  integers <- os[sapply(os, is.integer)]
  expect_setequal(
    object = names(integers),
    expected = c(
      "ARSA",
      "BBURCG",
      "BBWRCG",
      "Identity",
      "MDS",
      "MDS_angle",
      "MDS_metric",
      "MDS_nonmetric",
      "QAP_2SUM",
      "QAP_BAR",
      "QAP_Inertia",
      "QAP_LS",
      "R2E",
      "Random",
      "SA",
      "Spectral",
      "Spectral_norm",
      "SPIN_NH",
      "SPIN_STS",
      "TSP",
      "VAT"
    )
  )
  expect_setequal(c(names(hclusts), names(integers)), expected = names(os))

  # check all orders are integers
  ORDERS <-
    sapply(
      X = os,
      FUN = get_order,
      dim = 1,
      simplify = FALSE
    )
  for (n in names(ORDERS))
    expect_type(ORDERS[[!!n]], "integer") # see ?testthat::quasi_label RE "!!"

  # check names - first sanity check before more comprehensive checks
  expect_equal(names(get_order(os$Identity)), letters[1:4])

  # check names of integer seriation vectors correspond to correct integers
  for (n in names(integers)) {
    expect_type(names(ORDERS[[!!n]]), "character") # i.e. not NULL
    expect_mapequal(ORDERS[[!!n]], expected = c(
      a = 1,
      b = 2,
      c = 3,
      d = 4
    ))
  }

  # check $labels of hclust seriation vectors remain in original input order
  for (n in names(hclusts)) {
    expect_equal(hclusts[[!!n]][["labels"]], expected = letters[1:4])
  }

  # check names of get_order(<hclust seriation vector>) equal to ordered labels
  for (n in names(hclusts)) {
    expect_equal(object = names(ORDERS[[!!n]]),
      expected = hclusts[[n]][["labels"]][hclusts[[n]][["order"]]])
  }

  # check str snapshot of some deterministic methods
  deterMethods <- c(
    "BBURCG",
    "BBWRCG",
    "GW",
    "GW_average",
    "GW_complete",
    "GW_single",
    "GW_ward",
    "HC",
    "HC_average",
    "HC_complete",
    "HC_single",
    "HC_ward",
    "Identity",
    "MDS",
    "MDS_angle",
    "MDS_metric",
    "R2E",
    "Spectral",
    "Spectral_norm",
    "VAT"
  )

  #expect_snapshot(str(os[deterMethods]))

})

# check seriate errors for bad dist objects
test_that("negative distances and NAs prompt correct seriate.dist errors", {
  dNeg <- d
  dNeg[1] <- -1
  expect_error(seriate(dNeg), "Negative distances not supported")

  dNA <- d
  dNA[1] <- NA
  expect_error(seriate(dNA), "NAs not allowed in distance matrix x")
})

test_that("dist objects without Diag or Upper attributes can be permuted", {
  # eurodist is an object of class dist from built in R package "datasets"
  expect_s3_class(eurodist, "dist")
  expect_identical(attr(eurodist, "Diag"), NULL)
  expect_identical(attr(eurodist, "Upper"), NULL)
  s <- seriate(eurodist, method = "MDS")
  expect_s3_class(p <- permute(eurodist, order = s), "dist")
  expect_false(attr(p, "Diag")) # permutation adds Diag, is this desirable?
  expect_false(attr(p, "Upper"))
  expect_equal(labels(p), names(get_order(s)))
})

### Stress test to find memory access problems with randomized algorithms
#context("memory stress test")
#replicate(1000, seriate(d, method="bburcg"))
#replicate(1000, seriate(d, method="bbwrcg"))
#replicate(1000, seriate(d, method="arsa"))

test_that("seriate.matrix returns expected results", {
  #local_edition(3) # for snapshot testing

  cat("\n") # for cleaner testthat output
  methods <- list_seriation_methods(kind = "matrix")
  os <- sapply(methods, function(m) {
    cat("   -> testing", format(m, width = 13), "... ")
    tm <- system.time(o <- seriate(x, method = m))
    cat("took", formatC(tm[3], digits = 4), "s.\n")
    o
  }, simplify = FALSE)

  # check number and length of orders
  expect_true(all(sapply(os, length) == 2L))
  expect_true(all(sapply(
    os,
    FUN = function(o2)
      sapply(o2, length)
  ) == c(4L, 5L)))

  x_p <- permute(x, os[[1]]) # BEA method
  expect_equal(x_p, x[get_order(os[[1]], 1), get_order(os[[1]], 2)])

  # check labels
  expect_equal(get_order(os$Identity, 1), c(
    a = 1,
    b = 2,
    c = 3,
    d = 4
  ))
  expect_equal(get_order(os$Identity, 2), c(
    A = 1,
    B = 2,
    C = 3,
    D = 4,
    E = 5
  ))
  expect_equal(get_order(os$Reverse, 2), c(
    A = 5,
    B = 4,
    C = 3,
    D = 2,
    E = 1
  ))

  # check snapshot of some deterministic methods
  #deterMethods <- c("CA", "Identity", "PCA", "PCA_angle", "Reverse")
  #expect_snapshot(str(os[deterMethods]))

})

test_that("seriate.matrix with margin returns expected results", {
  #local_edition(3) # for snapshot testing

  cat("\n") # for cleaner testthat output
  methods <- list_seriation_methods(kind = "matrix")
  os <- sapply(methods, function(m) {
    cat("   -> testing", format(m, width = 13), "... ")
    tm <- system.time(o <- seriate(x, method = m, margin = 2))
    cat("took", formatC(tm[3], digits = 4), "s.\n")
    o
  }, simplify = FALSE)

  expect_true(all(sapply(os, length) == 2L))
  expect_true(all(sapply(
    os,
    FUN = function(o2)
      o2[[1]]
  ) == 1:4))
  expect_true(all(sapply(
    os,
    FUN = function(o2)
      length(o2[[2]]) == 5L
  )))

  x_p <- permute(x, os[[1]], margin = 2)
  expect_equal(x_p, x[, get_order(os[[1]], 2)])
})

test_that("data.frame seriation works as expected", {
  #local_edition(3) # for snapshot testing

  df <- as.data.frame(x)
  o <- seriate(df)
  expect_silent(permute(df, o)) # defaults work with no messages/warnings

  expect_message(
    permute(df, o[1]),
    # DEPRECATED: results in a message
    "permute for data.frames with a single seriation order is now deprecated"
  )

  o <- seriate(df, margin = 1)
  expect_equal(as.integer(o[[2]]), 1:5) # columns left in original order

  oPCA <- seriate(df, method = "PCA")
  #expect_snapshot(permute(df, oPCA))
})

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

## QAP 2SUM seriation
seriate_dist_2SUM <- function(x, control = NULL) {
  ## param are passed on to QAP

  do.call(qap::qap, c(list(A = .A_2SUM(attr(x, "Size")),
    B = 1/(1+as.matrix(x))), control))
}

## QAP Linear seriation
seriate_dist_LS <- function(x, control = NULL) {
  ## param are passed on to QAP

  do.call(qap::qap, c(list(A = .A_LS(attr(x, "Size")),
    B = as.matrix(x)), control))
}

## QAP Inertia
seriate_dist_Inertia <- function(x, control = NULL) {
  ## param are passed on to QAP
  n <- attr(x, "Size")

  ## inertia uses the same A matrix as 2SUM
  ## we use n^2 since A needs to be positive
  do.call(qap::qap, c(list(A = n^2-.A_2SUM(n),
    B = as.matrix(x)), control))
}

## QAP BAR
.qap_bar_contr <- list(b = function(n) max(1, floor(n/5)))
seriate_dist_BAR <- function(x, control = NULL) {
  ## param are passed on to QAP

  if(is.null(control)) control <- .qap_bar_contr
  if(is.null(control$b)) control$b <- .qap_bar_contr$b

  .A_BAR <- function(n, b) {
    b <- floor(b)
    if(b<1 || b>=n) stop("b: needs to be 1<=b<n!")
    A <- b+1- outer(1:n,1:n, FUN = function(i,j) abs(i-j))
    A[A<0] <- 0
    A
  }

  n <- attr(x, "Size")
  if(is.function(control$b)) b <- control$b(n)
  else b <- floor(control$b)

  if(b<1 || b>n) stop("BAR bandwidth is not between 1 and n!")

  control$b <- NULL

  ## inertia uses the same A matrix as 2SUM
  do.call(qap::qap, c(list(A = .A_BAR(n, b = b),
    B = as.matrix(x)), control))
}


set_seriation_method("dist", "QAP_2SUM", seriate_dist_2SUM,
  "Quadratic assignment problem formulation for seriation solved using a simulated annealing solver to minimize the 2-Sum Problem criterion (Barnard, Pothen, and Simon 1993). Control arguments are passed to qap in package qap.")
set_seriation_method("dist", "QAP_LS", seriate_dist_LS,
  "Quadratic assignment problem formulation for seriation solved using a simulated annealing solver to minimize the Linear Seriation Problem (LS) criterion (Hubert and Schultz 1976).Control arguments are passed to qap in package qap.")
set_seriation_method("dist", "QAP_BAR", seriate_dist_BAR,
  "Quadratic assignment problem formulation for seriation solved using a simulated annealing solver to minimize the banded anti-Robinson form (BAR). Control argument b is the BAR bandwidth (either a function of n or a number). The default for b is n/5. Additional control arguments are passed to qap in package qap.", .qap_bar_contr)
set_seriation_method("dist", "QAP_Inertia", seriate_dist_Inertia,
  "Quadratic assignment problem formulation for seriation solved using a simulated annealing solver to minimize the Inertia criterion. Control arguments are passed to qap in package qap.")

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2015 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

## register GA for seriation



#' Register a Genetic Algorithm Seriation Method
#'
#' Register a GA-based seriation metaheuristic for use with [seriate()].
#'
#' Registers the method \code{"GA"} for [seriate()]. This method can be used
#' to optimize any criterion in package \pkg{seriation}.
#'
#' \code{control} for
#' \code{seriate} with method \code{"GA"} accepts the following parameters:
#' - "criterion" criterion to optimize. Default: BAR
#' - "suggestions" suggestions to warm start the GA.
#'   \code{NA} means no warm start. Default: TSP, QAP_LS and Spectral.
#' - "selection" Selection operator.
#' - "crossover" Crossover operator.
#' - "mutation" Mutation operator. Default: a
#'   mixture of the simple insertion (80% chance) and simple inversion (20%
#'   chance) operators.
#' - "pmutation" probability for permutations. Default: .5
#' - "pcrossover" probability for crossover. Default: .2
#' - "popsize" the population size. Default: 100
#' - "maxiter" maximum number of generations. Default: 1000
#' - "run" stop after \code{run} generations without improvement. Default: 50
#' - "parallel" use multiple cores? Default: TRUE
#' - "verbose" a logical; report progress? Default: TRUE
#'
#' The GA uses by default the ordered cross-over (OX) operator. For mutation,
#' the GA uses a mixture of simple insertion and simple inversion operators.
#' This mixed operator is created using
#' \code{seriation::gaperm_mixedMutation(ismProb = .8)}, where \code{ismProb}
#' is the probability that the simple insertion mutation operator is used. See
#' package \pkg{GA} for a description of other available cross-over and
#' mutation operators for permutations. The appropriate operator functions in
#' \pkg{GA} start with \code{gaperm_}.
#'
#' We warm start the GA using \code{"suggestions"} given by several heuristics.
#' Set \code{"suggestions"} to \code{NA} to start with a purely random initial
#' population.
#'
#' \bold{Note:} Package \pkg{GA} needs to be installed.
#'
#' @aliases register_GA GA ga gaperm_mixedMutation
#' @family seriation
#' @returns Nothing.
#'
#' @author Michael Hahsler
#' @references Luca Scrucca (2013): GA: A Package for Genetic Algorithms in R.
#' \emph{Journal of Statistical Software,} \bold{53}(4), 1--37. URL
#' \doi{10.18637/jss.v053.i04}.
#' @keywords optimize cluster
#' @examples
#'
#' \dontrun{
#' register_GA()
#' get_seriation_method("dist", "GA")
#'
#' d <- dist(random.robinson(50, pre=TRUE, noise=.1))
#'
#' ## use default settings: Banded AR form
#' o <- seriate(d, "GA")
#' pimage(d, o)
#'
#' ## optimize for linear sertiation criterion (LS)
#' o <- seriate(d, "GA", control = list(criterion = "LS"))
#' pimage(d, o)
#'
#' ## no warm start
#' o <- seriate(d, "GA", control = list(criterion = "LS", suggestions = NA))
#' pimage(d, o)
#' }
#' @export
register_GA <- function() {
  check_installed("GA")

  .ga_contr <- list(
    criterion = "BAR",
    suggestions = c("TSP", "QAP_LS", "Spectral"),
    selection = GA::gaperm_nlrSelection,
    crossover = GA::gaperm_oxCrossover,
    mutation = gaperm_mixedMutation(.8),
    pcrossover = .2,
    pmutation = .5,
    popSize = 100,
    maxiter = 1000,
    run = 50,
    parallel = TRUE,
    verbose = TRUE
  )

  GA_helper <- function(x, control) {
    n <- attr(x, "Size")

    control <- .get_parameters(control, .ga_contr)

    if (control$verbose)
      cat("\nPreparing suggestions\n")

    if (is.na(control$suggestions[1]))
      suggestions <- NULL
    else
      suggestions <- t(sapply(control$suggestions,
        function(method)
          get_order(seriate(x, method = method))))

    if (control$verbose)
      cat("\nStarting GA\n")

    ### FIXME: need to be able to set bandwidth for BAR
    # fitness function
    f <-
      function(o)
        - criterion(x, o, method = control$criterion, force_loss = TRUE)

    result <- GA::ga(
      type = "permutation",
      fitness = f,
      lower = rep(1L, times = n),
      upper = rep(n, times = n),
      selection = control$selection,
      mutation = control$mutation,
      crossover = control$crossover,
      pmutation = control$pmutation,
      pcrossover = control$pcrossover,
      suggestions = suggestions,
      names = as.character(1:n),
      monitor = if (control$verbose)
        GA::gaMonitor
      else
        NULL,
      parallel = control$parallel,
      maxiter = control$maxiter,
      run = control$run,
      maxFitness = Inf,
      popSize = control$popSize
    )

    as.integer(result@solution[1,])
  }

  set_seriation_method(
    "dist",
    "GA",
    GA_helper,
    "Use a genetic algorithm to optimize for various criteria.",
    .ga_contr
  )
}


# Generates a mutation function which mixes simMutation (simple insertion)
# with ismMutation (inversion) given the probability.

#' @rdname register_GA
#' @param ismProb probability to use [GA::gaperm_ismMutation()] (inversion) versus [GA::gaperm_simMutation()] (simple insertion).
#' @export
gaperm_mixedMutation <- function(ismProb = .8) {
  function(object, parent, ...) {
    if (runif(1) > ismProb)
      GA::gaperm_simMutation(object, parent, ...)
    else
      GA::gaperm_ismMutation(object, parent, ...)
  }
}

      subroutine rbea(n,m,a,istart,b,ib,ifin)
c     Reorder rows using BEA, bond energy algorithm.
      dimension a(n,m), b(n,m), ib(n), ifin(n)
c------------------------------------------------------------------------------
c     a(n,m)         input matrix, rows of which are to be permuted
c     istart         1st row to be placed
c     b(n,m)         permuted rows to be stored in this array
c     ib(n)          integer list giving permutation carried out
c     ifin(n)        book-keeping vector: is row still active, or has it been 
c                    placed (resp. = 1, = 0).
c------------------------------------------------------------------------------
c     Bond energy algorithm -- see:
c
c     (1) W.T. McCormick, P.J. Schweitzer and T.W. White, 
c         "Problem decomposition and data reorganization by a clustering 
c         technique", Oper. Res., vol. 20, pp. 993-1009, Sept./Oct. 1972.
c     (2) P. Arabie and L.J. Hubert, 
c         "The bond energy algorithm revisited", IEEE Trans. Syst. Man 
c         Cybern., vol. 20, pp. 268-274, 1990.
c     (3) P. Arabie, S. Schleutermann, J. Daws and L. Hubert, 
c         "Marketing applications of sequencing and partitioning of 
c         nonsymmetric and/or two-mode matrices", in W. Gaul and M. Schader, 
c         Eds., Data Analysis, Decision Support, and Expert Knowledge 
c         Representation in Marketing, Springer Verlag, 1988, pp. 215-224.
c  
c     Implemented by F. Murtagh, Sept. 1991.
c------------------------------------------------------------------------------
c
c     Flags to indicate if row already chosen; 1 = not yet chosen/placed.
      do 200 i = 1, n
        ifin(i) = 1
 200  continue
c
c     Anticipate 1st placement. 'nplace' = # rows placed. 'nrem' = # remaining.
      nplace = 1
      nrem   = n-1
c
c     Place 1st row
      ifin(istart) = 0
      ib(nplace) = istart
      do 300 j = 1, m
         b(1,j)     = a(istart,j)
 300  continue
c
 400  continue
      sim   = -100000.0
      insrt = 0
c     'nplace' rows have been placed.
c     Now want next placement.  Have 'nrem' rows still to place.
      do 900 irow = 1, n
         if (ifin(irow).eq.1) then
c           For all still-to-be-placed rows...
c           1. Place right at beg.:
               sim1 = 0.0
               do 500 j = 1, m
                  sim1 = sim1 + a(irow,j)*b(1,j)
  500          continue
c           2. Place right at end:
               if (nplace.gt.1) then
                  sim2 = 0.0
                  do 600 j = 1, m
                     sim2 = sim2 + a(irow,j)*b(nplace,j)
  600             continue
               endif
c           3. Place between k and k+1, where k = 1, ..., nplace-1:
               if (nplace.ge.2) then
                  do 800 k = 1, nplace-1
c                    Path length involves sim with k'th and with k+1'th rows
c                    in 'b'; i.e. b(k,j) and b(k+1,j), for all j.
c                    Sim is with a(irow,j), for all j.
                     sim3 = 0.0
                     do 700 j = 1, m
                        sim3 = sim3 + a(irow,j)*(b(k,j)+b(k+1,j))
  700                continue
                     if (sim3.gt.sim) then
                        sim = sim3
                        insrt = k
                        iplrow = irow
                     endif
  800             continue
               endif
c
c              Scale up 'sim1' and 'sim2' relative to 'sim', since former
c              are based on one link only
               sim1 = sim1*2.0
               sim2 = sim2*2.0
c              Use 'sim' and 'insrt' to store final info on row to place.
               if (sim1.gt.sim) then
                  sim = sim1
                  insrt = 0
                  iplrow = irow
               endif
c              .ge. in following, to force tied case to end
               if (sim2.ge.sim) then
                  sim = sim2
                  insrt = nplace+1
                  iplrow = irow
               endif
            endif
 900     continue
c
c      So now, we want to make placement in location 'insrt+1'
c      1. This happens to be right at beginning:
       if (insrt.eq.0) then
c         Shift right
          do 1100 l = nplace+1, 2, -1
             ib(l)  = ib(l-1)
             do 1000 j = 1, m
                b(l,j) = b(l-1,j)
 1000        continue
 1100     continue
          do 1200 j = 1, m
             b(1,j) = a(iplrow,j)
 1200     continue
          ifin(iplrow) = 0
          nplace       = nplace + 1
          nrem         = nrem - 1
          ib(1)        = iplrow
          goto 1900
       endif
c
c      2. Placement happens to be right at end of all current placements:
       if (insrt.eq.nplace+1) then
c         Insert after all current placements.
          do 1300 j = 1, m
             b(nplace+1,j) = a(iplrow,j)
 1300     continue
          ifin(iplrow) = 0
          nplace       = nplace + 1
          nrem         = nrem -1
          ib(nplace)   = iplrow
          goto 1900
       endif
c
c      3. If we get to here, new placement is somewhere in the middle.
c      Shift rows 'insrt+1' to 'nplace', in 'b', right.
       do 1500 l = nplace+1, insrt+2, -1
          ib(l) = ib(l-1)
          do 1400 j = 1, m
             b(l,j) = b(l-1,j)
 1400     continue
 1500  continue
       do 1600 j = 1, m
          b(insrt+1,j) = a(iplrow,j)
 1600  continue
       nplace       = nplace + 1
       nrem         = nrem -1
       ifin(iplrow) = 0
       ib(insrt+1)   = iplrow
       goto 1900
c
 1900   continue
       if (nrem.ge.1) goto 400
c
       return
       end
                 
c------------------------------------------------------------------------------
      subroutine cbea(n,m,a,jstart,b,jb,jfin)
c     Reoder cols. using BEA, bond energy algorithm.
c     See references at beg. of routine 'rbea'.
      dimension a(n,m), b(n,m), jb(m), jfin(m)
c------------------------------------------------------------------------------
c     a(n,m)         input matrix, cols. of which are to be permuted
c     jstart         1st col. to be placed
c     b(n,m)         permuted cols. to be stored in this array
c     jb(m)          integer list giving permutation carried out
c     jfin(m)        book-keeping vector: is col. still active, or has it been 
c                    placed (resp. = 1, = 0).
c------------------------------------------------------------------------------
c
c     Flags to indicate if col. already chosen
      do 200 j = 1, m
        jfin(j) = 1
 200  continue   
c
c     'nplace' cols. placed (anticipating!); 'nrem' cols. still to be placed.
      nplace = 1
      nrem   = m-1
c
c     Place 1st col.
      jfin(jstart) = 0
      jb(nplace) = jstart
      do 300 i = 1, n
         b(i,1)     = a(i,jstart)
 300  continue
c
 400  continue
      sim   = -100000.0
      insrt = 0
c     'nplace' cols. have been placed.
c     Now want next placement.  Have 'nrem' cols. still to place.
      do 900 jcol = 1, m
         if (jfin(jcol).eq.1) then
c           For all still-to-be-placed cols...
c           1. Place right at beg.:
               sim1 = 0.0
               do 500 i = 1, n
                  sim1 = sim1 + a(i,jcol)*b(i,1)
  500          continue
c           2. Place right at end:
               if (nplace.gt.1) then
                  sim2 = 0.0
                  do 600 i = 1, n
                     sim2 = sim2 + a(i,jcol)*b(i,nplace)
  600             continue
               endif
c           3. Place between k and k+1, where k = 1, ..., nplace-1:
               if (nplace.ge.2) then
                  do 800 k = 1, nplace-1
c                    Path length involves sim with k'th and with k+1'th cols.
c                    in 'b'; i.e. b(i,k) and b(i,k+1), for all i.
c                    Sim is with a(i,jcol), for all i.
                     sim3 = 0.0
                     do 700 i = 1, n
                        sim3 = sim3 + a(i,jcol)*(b(i,k)+b(i,k+1))
  700                continue
                     if (sim3.gt.sim) then
                        sim = sim3
                        insrt = k
                        jplcol = jcol
                     endif
  800             continue
               endif
c
c              Scale up 'sim1' and 'sim2' rel. to 'sim' since former are
c              based on only one link
               sim1 = 2.0*sim1
               sim2 = 2.0*sim2
c              Use 'sim' and 'insrt' to store final info. on col. to be placed.
               if (sim1.gt.sim) then
                  sim = sim1
                  insrt = 0
                  jplcol = jcol
               endif
c              .ge. in following, to force tied case to end
               if (sim2.ge.sim) then
                  sim = sim2
                  insrt = nplace+1
                  jplcol = jcol
               endif
            endif
 900     continue
c
c      So now, we want to make placement in location 'insrt+1'.
c      1. This happens to be right at beginning.
       if (insrt.eq.0) then
c         Shift right
          do 1100 l = nplace+1, 2, -1
             jb(l) = jb(l-1)
             do 1000 i = 1, n
                b(i,l) = b(i,l-1)
 1000        continue
 1100     continue
          do 1200 i = 1, n
             b(i,1) = a(i,jplcol)
 1200     continue
          jfin(jplcol) = 0
          nplace       = nplace + 1
          nrem         = nrem - 1
          jb(1)   = jplcol
          goto 1900
       endif
c
c      2. Placement happens to be right at end of already placed cols.
       if (insrt.eq.nplace+1) then
          do 1300 i = 1, n
             b(i,nplace+1) = a(i,jplcol)
 1300     continue
          jfin(jplcol) = 0
          nplace       = nplace + 1
          nrem         = nrem -1
          jb(nplace)   = jplcol
          goto 1900
       endif
c
c      3. New col. placement happens to be somewhere in the middle.
c      Shift cols. 'insrt+1' to 'nplace', in 'b', right.
       do 1500 l = nplace+1, insrt+2, -1
          jb(l)  = jb(l-1)
          do 1400 i = 1, n
             b(i,l) = b(i,l-1)
 1400     continue
 1500  continue
       do 1600 i = 1, n
          b(i,insrt+1) = a(i,jplcol)
 1600  continue
       nplace       = nplace + 1
       nrem         = nrem -1
       jfin(jplcol) = 0
       jb(insrt+1)   = jplcol
       goto 1900
c
 1900   continue
       if (nrem.ge.1) goto 400
c
       return
       end
c--------------------------------------------------------------------------
       subroutine energy(n,m,b,ener)
       dimension b(n,m)
c      Det. "bond energy" of array b
c      I.e. product of each elt. with its 4 nearest neighbors,
c      summed over all elts.
       ener = 0.0
c      Corner elts.
       ener = ener + b(1,1)*(b(1,2)+b(2,1))
       ener = ener + b(1,m)*(b(1,m-1)+b(2,m))                 
       ener = ener + b(n,1)*(b(n-1,1)+b(n,2))
       ener = ener + b(n,m)*(b(n-1,m)+b(n,m-1))
c      Next non-corner border elts.
       do 100 j = 2, m-1
          ener = ener + b(1,j)*(b(1,j-1)+b(1,j+1)+b(2,j))
          ener = ener + b(n,j)*(b(n,j-1)+b(n,j+1)+b(n-1,j))
 100   continue
       do 200 i = 2, n-1
          ener = ener + b(i,1)*(b(i-1,1)+b(i+1,1)+b(i,2))
          ener = ener + b(i,m)*(b(i-1,m)+b(i+1,m)+b(i,m-1))
 200   continue
c      Finally, all non-border elts.
       do 400 i = 2, n-1
          do 300 j = 2, m-1
             ener = ener + b(i,j)*(b(i-1,j)+b(i+1,j)+b(i,j-1)+b(i,j+1))
 300      continue
 400   continue
c
       return
       end
       

#######################################################################
# Code to check parameter/control objects
# Copyrigth (C) 2011 Michael Hahsler
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


## helper to parse parameter lists with defaults
.nodots <- function(...) {
  l <- list(...)
  if(length(l) > 0L) warning("Unknown arguments: ",
    paste(names(l), "=",l, collapse=", "))
}

.get_parameters <- function(parameter, defaults) {
  defaults <- as.list(defaults)
  parameter <- as.list(parameter)

  ## add verbose
  if(is.null(defaults$verbose)) defaults$verbose <- FALSE

  if(length(parameter) != 0) {
    o <- pmatch(names(parameter), names(defaults))

    ## unknown parameter
    if(any(is.na(o))){
      warning(sprintf(ngettext(length(is.na(o)),
        "Unknown parameter: %s",
        "Unknown parameters: %s"),
        paste(names(parameter)[is.na(o)],
          collapse = ", ")), call. = FALSE, immediate. = TRUE)

      cat("Available parameter (with default values):\n")
      cat(rbind(names(defaults)," = ", gsub("\n"," ",as.character(defaults))),
        sep=c("\t"," ","\n"))
    }

    defaults[o[!is.na(o)]] <- parameter[!is.na(o)]
  }

  if(defaults$verbose) {
    cat("Used parameters:\n")
    cat(rbind(names(defaults)," = ",
      strtrim(gsub("\n"," ",as.character(defaults)), 50)),
      sep=c("\t"," ","\n"))
    cat("\n")
  }

  defaults
}

#######################################################################
# Code to map between ranges for contunuous variables
# Copyrigth (C) 2011 Michael Hahsler
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
    
## mapping helper

map <- function(x, range = c(0,1), from.range=NA) {
    if(any(is.na(from.range))) from.range <- range(x, na.rm=TRUE)
    
    ## check if all values are the same
    if(!diff(from.range)) return(
	    matrix(mean(range), ncol=ncol(x), nrow=nrow(x), 
		    dimnames = dimnames(x)))
    
    ## map to [0,1]
    x <- (x-from.range[1])
    x <- x/diff(from.range)
    ## handle single values
    if(diff(from.range) == 0) x <- 0 
    
    ## map from [0,1] to [range]
    if (range[1]>range[2]) x <- 1-x
    x <- x*(abs(diff(range))) + min(range)
    
    x[x<min(range) | x>max(range)] <- NA
    
    x
}

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



## Criterion for the quality of a permutation of a array

.criterion_array_helper <-
function(x, order = NULL, method = NULL, datatype = "array",
  force_loss = FALSE)
{
    ## check order
    if(!is.null(order)){
        if(!inherits(order, "ser_permutation"))
            stop("Argument 'order' has to be of class 'ser_permutation'.")
        .check_matrix_perm(x, order)
    }

    ## get methods
    if(is.null(method)) method <- list_criterion_methods(datatype)
    method <- lapply(method, function(m) get_criterion_method(datatype, m))

    crit <- sapply(method,
        function(m) structure(m$fun(x, order), names=m$name))

    if(force_loss) crit <- crit * sapply(method, FUN =
        function(m) ((as.integer(m$merit)*-2)+1))

    crit
}

criterion.array <-
function(x, order = NULL, method = NULL, force_loss = FALSE, ...)
    .criterion_array_helper(x, order, method, "array", force_loss)

## methods

## register built-ins

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



## Criterion generic.
criterion <- function(x, order = NULL, method = NULL, force_loss = FALSE, ...)
  UseMethod("criterion")

## Criterion method registry.

## <NOTE>
## For criterion() methods, argument 'method' really allows selecting
## *several* methods ... should perhaps be called 'methods'?
## We thus have a getter which returns a named list of methods from the
## registry, and a setter for single methods.
## </NOTE>


set_criterion_method <- function(kind, name, fun,
  description = NULL, merit = NA, ...) {
  ## check formals
  ##if(!identical(names(formals(definition)),
  ##              c("x", "order", "...")))
  ##    stop("Criterion methods must have formals 'x', 'order', and '...'.")

  ## check if criterion is already in registry
  r <- registry_criterion$get_entry(kind=kind, name=name)
  if(!is.null(r) && r$name==name) {
    warning("Entry with name ", name, " already exists! Modifying entry.")
    registry_criterion$modify_entry(kind=kind, name=name, fun=fun,
      description=description, merit = merit)
  } else {
    registry_criterion$set_entry(
      kind = kind, name=name, fun = fun,
      description = description, merit = merit)
  }

}

get_criterion_method <- function(kind, name) {
  method <- registry_criterion$get_entry(kind=kind, name=name)
  if(is.null(method))
    stop("Unknown criterion. Check list_criterion_methods(\"", kind, "\")")

  method
}

list_criterion_methods <- function(kind){
  if(missing(kind)) m <- registry_criterion$get_entries()
  else m <- registry_criterion$get_entries(kind=kind)

  sort(as.vector(sapply(m, "[[", "name")))
}

show_criterion_methods <- function(kind) {
  if(missing(kind)) m <- registry_criterion$get_entries()
  else m <- registry_criterion$get_entries(kind=kind)
  m[sort(names(m))]
}

library(seriation)

m <- matrix(c(
		1,1,0,0,0,
		1,1,1,0,0,
		0,0,1,1,1,
		1,0,1,1,1
		), byrow=TRUE, ncol=5)

d <- dist(m)
as.matrix(d)

context("criterion")
expect_equal(criterion(d,method="AR_events"), structure(2, names="AR_events"))
## 2

expect_equal(criterion(d,method="Path_length"), structure(4, names="Path_length"))
## 1+2+1=4

expect_equal(criterion(d,method="Lazy_path_length"),
  structure(8, names="Lazy_path_length"))
## (4-1)*1 + (4-2)*2+ (4-3)*1 = 8

expect_true(round(criterion(d, method="AR_deviations"), 6) - 0.504017 < 1e-10)
## 2.000000 - 1.732051 +  2.236068 - 2.000000 = 0.504017

expect_equal(criterion(d, method="Gradient_raw"),
	structure(4,names="Gradient_raw"))
## 6 - 2 = 4

expect_true(round(criterion(d, method="Gradient_weighted"), 6) - 3.968119 < 1e-10)
## -1 *(1.000000 - 2.236068 + 1.000000 - 2.000000 + 2.236068 - 2.000000 + 2.000000 - 1.732051 + 1.000000 - 1.732051 + 1.000000 - 2.000000 + 1.732051 - 2.000000 + 2.000000 - 2.236068)
## = 3.968119

## test stress
expect_equal(round(criterion(d, method="Neumann"), 3),
	structure(7.787, names="Neumann_stress"))
expect_equal(round(criterion(d, method="Moore"), 3),
	structure(11.539, names="Moore_stress"))

expect_equal(criterion(m, method="Neumann"),
	structure(22, names="Neumann_stress"))
expect_equal(criterion(m, method="Moore"),
	structure(44, names="Moore_stress"))


## RGAR
## for w = 2 -> 1/4
## for w = 3 -> 2/8
expect_error(criterion(d, method="RGAR", w=1))
expect_error(criterion(d, method="RGAR", w=4))

expect_equivalent(criterion(d, method="RGAR", pct=0), .25)
expect_equivalent(criterion(d, method="RGAR", w=2), .25)

expect_equivalent(round(criterion(d, method="RGAR", pct=100), 3), .25)
expect_equivalent(round(criterion(d, method="RGAR", w=3), 3), .25)

expect_equivalent(criterion(d, method="RGAR", w=3, relative = FALSE), 2)

### BAR
expect_error(criterion(d, method="BAR", b=0), "Band")
expect_error(criterion(d, method="BAR", b=4), "Band")

# b=1 -> Ham. path length
expect_equivalent(criterion(d, method="BAR", b=1),
  criterion(d, method="Path_length"))
# b = n-1 -> ARc
expect_equivalent(round(criterion(d, method="BAR", b=3), 3), 21.936)

### Cor R
m <- diag(100)

expect_equivalent(criterion(m, method="Cor_R"), 1.0)
expect_equivalent(criterion(m[nrow(m):1,], method="Cor_R"), -1.0)

# this should be close to 0
set.seed(1234)
r <- replicate(100, criterion(m[sample(nrow(m)),], method="Cor_R"))
# hist(r)
expect_true(abs(mean(r)) < 0.1)

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' Different Useful Color Palettes
#'
#' Defines several color palettes for [pimage()], [dissplot()] and
#' [hmap()].
#'
#' The color palettes are created with [sequential_hcl()] and
#' [diverging_hcl()] from package \pkg{colorspace}.
#'
#' The two sequential palettes are: `reds()` and `grays()` (or
#' `greys()`).
#'
#' The two diverging palettes are: `bluered()` and `greenred()`.
#'
#' @name palette
#' @aliases palette, colors
#' @family plots
#'
#' @param n number of different colors produces.
#' @param power used to control how chroma and luminance is increased (1 =
#' linear, 2 = quadratic, etc.)
#' @param bias a positive number. Higher values give more widely spaced colors
#' at the high end.
#' @param ...  further parameters are passed on to [sequential_hcl()]
#' or [diverging_hcl()].
#' @return A vector with `n` colors.
#' @author Michael Hahsler
#' @keywords hplot
#' @examples
#' m <- outer(1:10,1:10)
#' m
#'
#' pimage(m)
#' pimage(m, col = greys(100, power = 2))
#' pimage(m, col = greys(100, bias = 2))
#' pimage(m, col = bluered(100))
#' pimage(m, col = bluered(100, power = .5))
#' pimage(m, col = bluered(100, bias = 2))
#' pimage(m - 25, col = greenred(20, bias = 2))
#'
#' ## choose your own color palettes
#' library(colorspace)
#' hcl_palettes(plot = TRUE)
#'
#' ## blues (with 20 shades)
#' pimage(m,
#'   col = colorspace::sequential_hcl(20, "Blues", rev = TRUE))
#' ## blue to green (aka "Cork")
#' pimage(m,
#'   col = colorspace::diverging_hcl(100, "Cork"))
#' @export
bluered <- function(n = 100,
  bias = 1,
  power = 1,
  ...)
  grDevices::colorRampPalette(colorspace::diverging_hcl(n, palette = "Blue-Red", power = power, ...),
    bias = bias)(n)

#hclplot(bluered(10))
#plot(1:20, col = bluered(20), pch = 19, cex = 4)

#' @rdname palette
#' @export
greenred <- function(n = 100,
  bias = 1,
  power = 1,
  ...)
  grDevices::colorRampPalette(rev(
    colorspace::diverging_hcl(n, palette = "Red-Green", power = power, ...)
  ), bias = bias)(n)

#hclplot(greenred(10))
#plot(1:20, col = greenred(20), pch = 19, cex = 4)

#' @rdname palette
#' @export
reds <- function(n = 100,
  bias = 1,
  power = 1,
  ...)
  grDevices::colorRampPalette(rev(
    colorspace::sequential_hcl(n, palette = "Reds", power = power, ...)
  ), bias = bias)(n)

#hclplot(reds(10))
#plot(1:20, col = reds(20), pch = 19, cex = 4)

#' @rdname palette
#' @export
blues <- function(n = 100,
  bias = 1,
  power = 1,
  ...)
  grDevices::colorRampPalette(rev(
    colorspace::sequential_hcl(n, palette = "Blues 2", power = power, ...)
  ), bias = bias)(n)

#hclplot(blues(10))
#plot(1:20, col = blues(20), pch = 19, cex = 4)

#' @rdname palette
#' @export
greens <- function(n = 100,
  bias = 1,
  power = 1,
  ...)
  grDevices::colorRampPalette(rev(
    colorspace::sequential_hcl(n, palette = "Greens", power = power, ...)
  ), bias = bias)(n)

#hclplot(greens(10))
#plot(1:20, col = greens(20), pch = 19, cex = 4)

#' @rdname palette
#' @export
greys <- function(n = 100,
  bias = 1,
  power = 1,
  ...)
  grDevices::colorRampPalette(rev(
    colorspace::sequential_hcl(n, palette = "Grays", power = power, ...)
  ), bias = bias)(n)

#hclplot(greys(10))
#plot(1:20, col = greys(20), pch = 19, cex = 4)

#' @rdname palette
#' @export
grays <- greys


.map_color_01 <- function(x, col) {
  x[] <- col[map_int(x, length(col), from.range = c(0, 1))]
  x
}

# translate all data to a color
.map_color <- function(x, col, from.range = NA) {
  x[] <- col[map_int(x, length(col), from.range)]
  x
}

## define default colors
#.sequential_pal <- grays
.sequential_pal <- blues
.diverge_pal <- bluered

## define default ggplot2 colors
.gg_logical_pal <- function()
  ggplot2::scale_fill_manual(values = c("white", "black"), na.value = "white")

.gg_sequential_pal <- function(dist = FALSE, limits = NULL) {
  if (dist)
    ggplot2::scale_fill_gradient(low = scales::muted("blue"),
      high = "white",
      na.value = "white",
      limits = limits)
  else
    ggplot2::scale_fill_gradient(low = "white",
      high = scales::muted("blue"),
      na.value = "white",
      limits = limits)
}

.gg_diverge_pal <- function(limits = NULL)
  ggplot2::scale_fill_gradient2(
    low = scales::muted("blue"),
    mid = "white",
    high = scales::muted("red"),
    na.value = "white",
    midpoint = 0,
    limits = limits
  )

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

## seriate general arrays

.seriate_array_helper <- function(x,
  method = "PCA",
  control = NULL,
  margin = seq(ndim(x)),
  datatype = "array",
  ...) {
  ## add ... to control
  if (any(!margin %in% seq(ndim(x))))
    stop("illegal margin specified.")

  control <- c(control, list(...))

  if (!is.character(method) || (length(method) != 1L))
    stop("Argument 'method' must be a character string.")

  method <- get_seriation_method(datatype, method)

  if (!is.null(control$verbose) &&
      control$verbose)
    cat(method$name, ": ",
      method$description, "\n\n", sep = "")

  order <- method$fun(x, control)

  for (i in seq(ndim(x)))
    if (!is.null(dimnames(x)[[i]]) &&
        is.integer(order[[i]]))
      names(order[[i]]) <- dimnames(x)[[i]]
  perm <- do.call("ser_permutation",
    unname(lapply(
      order, "ser_permutation_vector", method$name
    )))

  ### make non seriated margins identity permutations
  rem <- which(!seq(ndim(x)) %in% margin)
  if (length(rem) > 0) {
    perm_ident <- seriate(x, method = "Identity")
    perm[[rem]] <- perm_ident[[rem]]
  }

  perm
}

#' @rdname seriate
#' @export
seriate.array <- function(x,
  method = "PCA",
  control = NULL,
  margin = seq(length(dim(x))),
  ...)
  .seriate_array_helper(x,
    method,
    control,
    margin,
    datatype = "array",
    ...)
## we currently have no method and therefore also no default method!


## methods
## Identity is defined in seriate.matrix.R
## no other methods available right now

## register methods
## no methods available right now

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



## VAT: a tool for visual assessment of (cluster) tendency
## Bezdek, J.C., Hathaway, R.J.
## Proceedings of the 2002 International Joint Conference on
## Neural Networks, 2002. IJCNN '02. (Volume:3)
seriate_dist_VAT <- function(x, control = NULL) {
  #param <- .get_parameters(control, NULL)
  .get_parameters(control, NULL)

  D <- as.matrix(x)
  N <- nrow(D)
  P <- rep(NA_integer_, N)
  I <- rep(FALSE, N)
  ### J is !I

  i <- which(D == max(D, na.rm = TRUE), arr.ind = TRUE)[1,1]
  P[1] <- i
  I[i] <- TRUE

  for(r in 2:N) {
    D2 <- D[I,!I, drop=FALSE]
    j <- which(D2 == min(D2, na.rm = TRUE), arr.ind = TRUE)[1,2]
    j <- which(!I)[j]
    P[r] <- j
    I[j] <- TRUE
  }

  names(P) <- labels(x)[P]
  P
}


set_seriation_method("dist", "VAT", seriate_dist_VAT,
  "Visual assesment of clustering tendency (Bezdek and Hathaway (2002). Creates an order based on Prim's algorithm for finding a minimum spanning tree (MST) in a weighted connected graph representing the distance matrix. The order is given by the order in which the nodes (objects) are added to the MST.")

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' Dissimilarity Plot
#'
#' Visualizes a dissimilarity matrix using seriation and matrix shading using
#' the method developed by Hahsler and Hornik (2011). Entries with lower
#' dissimilarities (higher similarity) are plotted darker. Dissimilarity plots
#' can be used to uncover hidden structure in the data and judge cluster
#' quality.
#'
#' The plot can also be used to visualize cluster quality (see Ling 1973).
#' Objects belonging to the same cluster are displayed in consecutive order.
#' The placement of clusters and the within cluster order is obtained by a
#' seriation algorithm which tries to place large similarities/small
#' dissimilarities close to the diagonal. Compact clusters are visible as dark
#' squares (low dissimilarity) on the diagonal of the plot. Additionally, a
#' Silhouette plot (Rousseeuw 1987) is added. This visualization is similar to
#' CLUSION (see Strehl and Ghosh 2002), however, allows for using arbitrary
#' seriating algorithms.
#'
#' **Note:** Since [pimage()] uses \pkg{grid}, it should not be mixed
#' with base R primitive plotting functions.
#'
#' @family plots
#'
#' @param x an object of class [dist].
#' @param labels `NULL` or an integer vector of the same length as
#' rows/columns in `x` indicating the cluster membership for each object
#' in `x` as consecutive integers starting with one. The labels are used
#' to reorder the matrix.
#' @param method A single character string indicating the seriation method used
#' to reorder the clusters (inter cluster seriation) as well as the objects
#' within each cluster (intra cluster seriation).  If different algorithms for
#' inter and intra cluster seriation are required, `method` can be a
#' `list` of two named elements (`inter_cluster` and
#' `intra_cluster` each containing the name of the respective seriation
#' method. Use [list_seriation_methods()] with `kind = "dist"` to find available algorithms.
#'
#' Set method to `NA` to plot the matrix as is (no or, if cluster labels
#' are supplied, only coarse seriation). For intra cluster reordering with the
#' special method `"silhouette width"` is available (for `dissplot()`
#' only). Objects in clusters are then ordered by silhouette width (from
#' silhouette plots).  If no `method` is given, the default method of
#' [seriate.dist()] is used.
#'
#' A third list element (named `aggregation`) can be added to control how
#' inter cluster dissimilarities are computed from from the given dissimilarity
#' matrix. The choices are `"avg"` (average pairwise dissimilarities;
#' average-link), `"min"` (minimal pairwise dissimilarities; single-link),
#' `"max"` (maximal pairwise dissimilarities; complete-link), and
#' `"Hausdorff"` (pairs up each point from one cluster with the most
#' similar point from the other cluster and then uses the largest dissimilarity
#' of paired up points).
#' @param control a list of control options passed on to the seriation
#' algorithm.  In case of two different seriation algorithms, `control`
#' can contain a list of two named elements (`inter_cluster` and
#' `intra_cluster`) containing each a list with the control options for
#' the respective algorithm.
#' @param upper_tri,lower_tri,diag a logical indicating whether to show the upper triangle, the
#' lower triangle or the diagonal of the distance matrix. The string "average" can also be used
#' to display within and between cluster averages in the two triangles.
#' @param cluster_labels a logical indicating whether to display cluster labels
#' in the plot.
#' @param cluster_lines a logical indicating whether to draw lines to separate
#' clusters.
#' @param reverse_columns a logical indicating if the clusters are displayed on
#' the diagonal from north-west to south-east (`FALSE`; default) or from
#' north-east to south-west (`TRUE`).
#' @param options a list with options for plotting the matrix (`dissplot`
#' only).
#' - `plot` a logical indicating if a plot should
#'   be produced.  if `FALSE`, the returned object can be plotted later
#'   using the function `plot` which takes as the second argument a list of
#'   plotting options (see `options` below).
#' - `silhouettes` a logical indicating whether to include a silhouette plot
#'   (see Rousseeuw, 1987).
#' - `threshold` a numeric. If used, only plot distances
#'   below the threshold are displayed. Consider also using `zlim` for this
#'   purpose.
#' - `col` colors used for the image plot.
#' - `key` a logical indicating whether to place a color key below the plot.
#' - `zlim` range of values to display (defaults to range `x`).
#' - `axes` `"auto"` (default; enabled for less than 25 objects), `"y"` or `"none"`.
#' - `main` title for the plot.
#' - `newpage` a logical indicating whether to start plot on a new page
#'   (see [grid.newpage()].
#' - `pop` a logical indicating whether to pop the created viewports?
#'   (see package \pkg{grid})
#' - `gp`, `gp_lines`, `gp_labels` objects of class `gpar` containing graphical parameters for the plot
#'   lines and labels (see [gpar()].
#' @param ... `dissplot()`: further arguments are added to `options`.
#'   `ggdissplot()` further arguments are passed on to [ggpimage()].
#' @return `dissplot()` returns an invisible object of class
#' `cluster_proximity_matrix` with the following elements:
#' \item{order}{`NULL` or integer vector giving the order used to plot `x`.}
#' \item{cluster_order}{ `NULL` or integer vector giving the order of the
#'   clusters as plotted.}
#' \item{method}{ vector of character strings indicating
#'   the seriation methods used for plotting `x`.}
#' \item{k}{ `NULL` or integer scalar giving the number of clusters generated.}
#' \item{description}{ a `data.frame` containing information (label, size, average
#'   intra-cluster dissimilarity and the average silhouette) for the clusters as
#'   displayed in the plot (from top/left to bottom/right).}
#'
#' This object can be used for plotting via `plot(x, options = NULL, ...)`,
#' where `x` is the object and `options` contains a list with
#' plotting options (see above).
#'
#' `ggdissplot()` returns a ggplot2 object representing the plot.
#'
#' @returns The plot description as an object of class `reordered_cluster_dissimilarity_matrix`.
#'
#' @author Michael Hahsler
#' @references
#' Hahsler, M. and Hornik, K. (2011): Dissimilarity plots: A visual
#' exploration tool for partitional clustering. \emph{Journal of Computational
#' and Graphical Statistics,} \bold{10}(2):335--354.
#' \doi{10.1198/jcgs.2010.09139}
#'
#' Ling, R.F. (1973): A computer generated aid for cluster analysis.
#' \emph{Communications of the ACM,} \bold{16}(6), 355--361.
#' \doi{10.1145/362248.362263}
#'
#' Rousseeuw, P.J. (1987): Silhouettes: A graphical aid to the interpretation
#' and validation of cluster analysis. \emph{Journal of Computational and
#' Applied Mathematics,} \bold{20}(1), 53--65.
#' \doi{10.1016/0377-0427(87)90125-7}
#'
#' Strehl, A. and Ghosh, J. (2003): Relationship-based clustering and
#' visualization for high-dimensional data mining. \emph{INFORMS Journal on
#' Computing,} \bold{15}(2), 208--230.
#' \doi{10.1287/ijoc.15.2.208.14448}
#' @keywords hplot cluster
#' @examples
#' data("iris")
#'
#' # shuffle rows
#' x_iris <- iris[sample(seq(nrow(iris))), -5]
#' d <- dist(x_iris)
#'
#' # Plot original matrix
#' dissplot(d, method = NA)
#'
#' # Plot reordered matrix using the nearest insertion algorithm (from tsp)
#' dissplot(d, method = "TSP", main = "Seriation (TSP)")
#'
#' # Cluster iris with k-means and 3 clusters and reorder the dissimality matrix
#' l <- kmeans(x_iris, centers = 3)$cluster
#' dissplot(d, labels = l, main = "k-means")
#'
#' # show only distances as lower triangle
#' dissplot(d, labels = l, main = "k-means", lower_tri = TRUE, upper_tri = FALSE)
#'
#' # Use a grid layout to place several plots on a page
#' library("grid")
#' grid.newpage()
#' pushViewport(viewport(layout=grid.layout(nrow = 2, ncol = 2),
#'     gp = gpar(fontsize = 8)))
#' pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 1))
#'
#' # Visualize the clustering (using Spectral between clusters and MDS within)
#' res <- dissplot(d, l, method = list(inter = "Spectral", intra = "MDS"),
#'   main = "K-Means + Seriation", newpage = FALSE)
#'
#' popViewport()
#' pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 2))
#'
#' # More visualization options. Note that we reuse the reordered object res!
#' # color: use 10 shades red-blue, biased towards small distances
#' plot(res, main = "K-Means + Seriation (red-blue + biased)",
#'     col= bluered(10, bias = .5), newpage = FALSE)
#'
#' popViewport()
#' pushViewport(viewport(layout.pos.row = 2, layout.pos.col = 1))
#'
#' # Threshold (using zlim) and cubic scale to highlight differences
#' plot(res, main = "K-Means + Seriation (cubic + threshold)",
#'     zlim = c(0, 2), col = grays(100, power = 3), newpage = FALSE)
#'
#' popViewport()
#' pushViewport(viewport(layout.pos.row = 2, layout.pos.col = 2))
#'
#' # Use gray scale with logistic transformation
#' plot(res, main = "K-Means + Seriation (logistic scale)",
#'   col = gray(
#'     plogis(seq(max(res$x_reordered), min(res$x_reordered), length.out = 100),
#'       location = 2, scale = 1/2, log = FALSE)
#'     ),
#'   newpage = FALSE)
#'
#' popViewport(2)
#'
#' # The reordered_cluster_dissimilarity_matrix object
#' res
#' names(res)
#'
#' ## --------------------------------------------------------------------
#' ## ggplot-based dissplot
#' if (require("ggplot2")) {
#'
#' library("ggplot2")
#'
#' # Plot original matrix
#' ggdissplot(d, method = NA)
#'
#' # Plot seriated matrix
#' ggdissplot(d, method = "TSP") +
#'   labs(title = "Seriation (TSP)")
#'
#' # Cluster iris with k-means and 3 clusters
#' l <- kmeans(x_iris, centers = 3)$cluster
#'
#' ggdissplot(d, labels = l) +
#'   labs(title = "K-means + Seriation")
#'
#' # show only lower triangle
#' ggdissplot(d, labels = l, lower_tri = TRUE, upper_tri = FALSE) +
#'   labs(title = "K-means + Seriation")
#'
#' # No lines or cluster labels and add a label for the color key (fill)
#' ggdissplot(d, labels = l, cluster_lines = FALSE, cluster_labels = FALSE) +
#'   labs(title = "K-means + Seriation", fill = "Distances\n(Euclidean)")
#'
#' # Diverging color palette with manual set midpoint and different seriation methods
#' ggdissplot(d, l, method = list(inter = "Spectral", intra = "MDS")) +
#'   labs(title = "K-Means + Seriation", subtitle = "biased color scale") +
#'   scale_fill_gradient2(midpoint = median(d))
#'
#' # Use manipulate scale using package scales
#' library("scales")
#'
#' # Threshold (using limit and na.value) and cubic scale to highlight differences
#' cubic_dist_trans <- trans_new(
#'   name = "cubic",
#'   # note that we have to do the inverse transformation for distances
#'   trans = function(x) x^(1/3),
#'   inverse = function(x) x^3
#' )
#'
#' ggdissplot(d, l, method = list(inter = "Spectral", intra = "MDS")) +
#'   labs(title = "K-Means + Seriation", subtitle = "cubic + biased color scale") +
#'   scale_fill_gradient(low = "black", high = "white",
#'     limit = c(0,2), na.value = "white",
#'     trans = cubic_dist_trans)
#'
#' # Use gray scale with logistic transformation
#' logis_2_.5_dist_trans <- trans_new(
#'   name = "Logistic transform (location, scale)",
#'   # note that we have to do the inverse transformation for distances
#'   trans = function(x) plogis(x, location = 2, scale = .5, log = FALSE),
#'   inverse = function(x) qlogis(x, location = 2, scale = .5, log = FALSE),
#' )
#'
#' ggdissplot(d, l, method = list(inter = "Spectral", intra = "MDS")) +
#'   labs(title = "K-Means + Seriation", subtitle = "logistic color scale") +
#'   scale_fill_gradient(low = "black", high = "white",
#'     trans = logis_2_.5_dist_trans,
#'     breaks = c(0, 1, 2, 3, 4))
#' }
#' @export
dissplot <- function(x,
  labels = NULL,
  method = "Spectral",
  control = NULL,
  lower_tri = TRUE,
  upper_tri = "average",
  diag = TRUE,
  cluster_labels = TRUE,
  cluster_lines = TRUE,
  reverse_columns = FALSE,
  options = NULL,
  ...) {
  ## add ... to options
  options <- c(options, list(...))
  options$cluster_labels <- cluster_labels
  options$cluster_lines <- cluster_lines
  options$reverse_columns <- reverse_columns

  ## make x dist
  if (!inherits(x, "dist")) {
    if (is.matrix(x) && isSymmetric(x))
      x <- as.dist(x)
    else
      stop("Argument 'x' cannot safely be coerced to class 'dist'.")
  }

  a <- .arrange_dissimilarity_matrix(x,
    labels = labels,
    method = method,
    control = control)

  if (is.null(options$plot) || options$plot)
    plot(a, lower_tri, upper_tri, diag, options)

  invisible(a)
}


## work horse
.arrange_dissimilarity_matrix <-
  function(x,
    labels = NULL,
    method = NULL,
    control = NULL) {
    ## x is already of class dist
    dim <- attr(x, "Size")
    diss_measure <- attr(x, "method")

    ## check labels
    if (!is.null(labels) && length(labels) != dim)
      stop("Number of labels in 'labels' does not match dimensions of 'x'.")

    m <- method


    ## set everything to NULL first
    order                   <- NULL
    k                       <- NULL             # number of clusters
    sil                     <- NULL
    avgSil                  <- NULL
    labels_unique           <- NULL
    cluster_dissimilarities <- NULL
    ## method$a means method$ aggregation (default is avg)
    aggregation	<- "avg"
    if (is.list(method) &&
        !is.null(method$a))
      aggregation <- method$a

    if (!is.list(method))
      method <-
      list(inter_cluster = m, intra_cluster = m)
    m <-
      pmatch(names(method),
        c("inter_cluster", "intra_cluster", "aggregation"))
    if (any(is.na(m)))
      stop("Unknown method component. Use 'inter_cluster', 'intra_cluster' and 'aggregation'.")
    names(method) <-
      c("inter_cluster", "intra_cluster", "aggregation")[m]

    if (!is.list(control[[1]])) {
      control <- list(inter_cluster = control, intra_cluster = control)
    }

    if (!is.null(method$inter_cluster)
      && is.na(method$inter_cluster)) {
      ## no setiation
      if (!is.null(labels)) {
        ## do coarse seriation
        order <- order(labels)
        k <- length(unique(labels))
        ## calculate cluster_dissimilarities for later
        cluster_dissimilarities <- .cluster_dissimilarity(x, labels,
          aggregation)
        aggregation <- attr(cluster_dissimilarities, "method")
        ## calculate silhouette values for later use
        sil <- cluster::silhouette(labels, x)

      }
      ## else keep the matrix as is -- do not reorder

    } else if (is.null(labels)) {
      ## reorder whole matrix if no labels are given
      order <- seriate(x,
        method = method$inter_cluster,
        control = control$inter)[[1]]

      method$inter_cluster <- if (!is.null(attr(order, "method")))
        attr(order, "method")
      else
        method$inter_cluster

      order <- get_order(order)

    } else{
      ## reorder clusters for given labels
      ## get number of clusters k
      k <- length(unique(labels))

      ## reorder with average pairwise dissimilarites between clusters
      cluster_dissimilarities <- .cluster_dissimilarity(x, labels,
        aggregation)
      aggregation <- attr(cluster_dissimilarities, "method")

      if (k > 2) {
        cluster_order <- seriate(
          as.dist(cluster_dissimilarities),
          method = method$inter_cluster,
          control = control$inter
        )[[1]]

        method$inter_cluster <-
          if (!is.null(attr(cluster_order, "method")))
            attr(cluster_order, "method")
        else
          method$inter_cluster

        cluster_order <- get_order(cluster_order)
      } else{
        cluster_order <- 1:k
      }

      ## calculate silhouette values for later use
      sil <- cluster::silhouette(labels, x)

      ## determine order for matrix from cluster order
      order <- c()

      if (!is.null(method$intra_cluster) &&
          is.na(method$intra_cluster)) {
        ## no intra cluster ordering
        for (i in 1:k) {
          order <- c(order, which(labels == cluster_order[i]))
        }
        ##method$intra_cluster <- NA

      } else{
        ## intra cluster order

        for (i in 1:k) {
          take <- which(labels == cluster_order[i])

          ## only reorder for >1 elements
          if (length(take) > 1) {
            if (is.character(method$intra_cluster) &&
                match(
                  tolower(method$intra_cluster),
                  c("sil", "silhouette", "silhouette width"),
                  nomatch = 0
                ) > 0) {
              intra_order <-  order(sil[take, "sil_width"],
                decreasing = TRUE)

              method$intra_cluster <- "silhouette width"
            } else{
              ## we use .rearrange_dist instead of permute
              ## since we take only a subset!
              block <- .rearrange_dist(x, take)

              intra_order <-
                seriate(block,
                  method = method$intra_cluster,
                  control = control$intra)[[1]]

              method$intra_cluster <-
                if (!is.null(attr(intra_order, "method")))
                  attr(intra_order, "method")
              else
                method$intra_cluster

              intra_order <- get_order(intra_order)
            }

            order <- c(order, take[intra_order])

          } else{
            order <- c(order, take)
          }

        }
      }


      ## reorder cluster_dissimilarities for later
      cluster_dissimilarities  <-
        cluster_dissimilarities[cluster_order, cluster_order]

    }

    ## reorder matrix
    if (!is.null(order)) {
      x_reordered <- permute(x, order)
      labels <- labels[order]
    }
    else
      x_reordered <- x

    ## prepare for return value
    cluster_description <- NULL

    if (!is.null(labels)) {
      labels_unique   <-  unique(labels)

      ## reorder silhouettes
      sil <- sil[order,]

      ## calculate avg silhouettes
      avgSil <- sapply(labels_unique, function(x)
        mean(sil[sil[, "cluster"] == x, "sil_width"]))

      ## generate description
      cluster_description = data.frame(
        position        = c(1:k),
        label           = labels_unique,
        size            = tabulate(labels)[labels_unique],
        ## FIXME: this is not the average anymore!
        aggregated_dissimilarity =
          diag(cluster_dissimilarities)[labels_unique],
        avg_silhouette_width = avgSil
      )
    }

    ## clean order from names, etc.
    attributes(order) <- NULL

    structure(
      list(
        x_reordered        = x_reordered,
        labels             = labels,
        seriation_methods  = method,
        aggregation_method = aggregation,
        k                  = k,
        cluster_dissimilarities =  cluster_dissimilarities,
        sil                = sil,
        order              = order,
        cluster_order      = labels_unique,
        diss_measure       = diss_measure,
        description        =  cluster_description
      ),
      class = "reordered_cluster_dissimilarity_matrix"
    )
  }


## create panels with avg. dissimilarity
## a is an arrangement
.average_tri <- function(a,
  lower_tri = "average",
  upper_tri = TRUE,
  diag = TRUE) {
  if (!inherits(a, "reordered_cluster_dissimilarity_matrix"))
    stop("a needs to be a reordered_cluster_dissimilarity_matrix")

  upper_avg <- !is.na(pmatch(tolower(upper_tri), "average"))
  lower_avg <- !is.na(pmatch(tolower(lower_tri), "average"))

  k <- a$k
  labels <- a$labels
  labels_unique <- a$cluster_order
  cluster_dissimilarities <- a$cluster_dissimilarities
  m <- as.matrix(a$x_reordered)

  ## blank out if FALSE or NA
  if (is.na(upper_tri) || (is.logical(upper_tri) && !upper_tri)) {
    m[upper.tri(m)] <- NA
    upper_tri <- FALSE
  }
  if (is.na(lower_tri) || (is.logical(lower_tri) && !lower_tri)) {
    m[lower.tri(m)] <- NA
    lower_tri <- FALSE
  }

  ## do off-diagonal averages by cluster
  if (!is.null(cluster_dissimilarities) &&
      !is.null(labels) && (upper_avg || lower_avg)) {
    for (i in seq(2, k)) {
      for (j in seq(i - 1)) {
        ## check empty clusters
        if (is.na(labels_unique[i]))
          next
        if (is.na(labels_unique[j]))
          next

        ## lower panels
        if (lower_avg) {
          m[labels == labels_unique[i], labels == labels_unique[j]] <-
            cluster_dissimilarities[i, j]
        }

        ## upper panels
        if (upper_avg) {
          m[labels == labels_unique[j], labels == labels_unique[i]] <-
            cluster_dissimilarities[i, j]
        }
      }
    }


    ## do diagonal
    for (i in seq(1, k)) {
      block <- m[labels == labels_unique[i],
        labels == labels_unique[i]]

      if (upper_avg) {
        block[upper.tri(block, diag = TRUE)] <-
          cluster_dissimilarities[i, i]

        m[labels == labels_unique[i],
          labels == labels_unique[i]] <- block
      }

      if (lower_avg) {
        block[lower.tri(block, diag = TRUE)] <-
          cluster_dissimilarities[i, i]

        m[labels == labels_unique[i],
          labels == labels_unique[i]] <- block
      }
    }
  }

  if (!diag)
    diag(m) <- NA

  m
}


## plot for reordered_cluster_dissimilarity_matrix
#' @rdname dissplot
#' @export
plot.reordered_cluster_dissimilarity_matrix <-
  function(x,
    lower_tri = TRUE,
    upper_tri = "average",
    diag = TRUE,
    options = NULL,
    ...) {
    ## add ... to options
    options <- c(options, list(...))

    k       <- x$k
    dim     <- attr(x$x_reordered, "Size")
    labels  <- x$labels
    #labels_unique <- unique(labels)
    labels_unique <- x$cluster_order

    m  <- .average_tri(x,
      lower_tri = lower_tri,
      upper_tri = upper_tri,
      diag = diag)

    ## default plot options
    options <- .get_parameters(
      options,
      list(
        cluster_labels = TRUE,
        cluster_lines  = TRUE,
        reverse_columns	= FALSE,
        silhouettes    = FALSE,
        col            = NULL,
        threshold      = NULL,
        zlim           = NULL,
        key            = TRUE,
        main           = NULL,
        axes           = "auto",
        gp             = gpar(),
        gp_lines       = gpar(),
        gp_labels      = gpar(),
        newpage        = TRUE,
        pop            = TRUE
      )
    )

    if (is.null(options$col))
      options$col <- rev(.sequential_pal())
    else
      options$col <- rev(options$col)

    i <- pmatch(options$axes, c("auto", "x", "y", "both", "none"))
    if (is.na(i))
      stop("Illegal vaule for axes. Use: 'auto', 'x', 'y', 'both' or 'none'!")
    options$axes <- c("auto", "x", "y", "both", "none")[i]

    ## clear page
    if (options$newpage)
      grid.newpage()

    ## do we have silhouettes?
    if (is.null(x$sil))
      options$silhouettes <- FALSE

    if (options$reverse_columns)
      m <- m[, ncol(m):1]

    if (!options$silhouettes) {
      pushViewport(viewport(
        layout = grid.layout(
          6,
          3,
          widths = unit.c(
            unit(2, "lines"),
            # space
            unit(1, "snpc") - unit(7, "lines"),
            # image
            unit(2, "lines")                        # space
          ),
          heights = unit.c(
            unit(2, "lines"),
            # title
            unit(1, "lines"),
            # space
            unit(1, "snpc") - unit(7, "lines"),
            # image
            unit(1, "lines"),
            # space
            unit(1, "lines"),
            # colorkey
            unit(2, "lines")                        # space
          )
        ),
        gp = options$gp
      ))

      main_vp     <- viewport(
        layout.pos.col = 2,
        layout.pos.row = 1,
        name = "main"
      )
      image_vp    <-
        viewport(layout.pos.col = 2, layout.pos.row = 3)
      colorkey_vp <- viewport(
        layout.pos.col = 2,
        layout.pos.row = 5,
        name = "colorkey"
      )

    } else{
      ## with silhouettes
      pushViewport(viewport(
        layout = grid.layout(
          6,
          5,
          widths = unit.c(
            unit(2, "lines"),
            # space
            unit(0.7, "snpc") - unit(2.5, "lines"),
            # image
            unit(1, "lines"),
            # space
            unit(0.3, "snpc") - unit(2.5, "lines"),
            # sil
            unit(2, "lines")                        # space
          ),
          heights = unit.c(
            unit(2, "lines"),
            # title
            unit(2, "lines"),
            # space
            unit(0.7, "snpc") - unit(2.5, "lines"),
            # image
            unit(1, "lines"),
            # space
            unit(1, "lines"),
            # colorkey
            unit(2, "lines")                        # space
          )
        ),
        gp = options$gp
      ))

      main_vp     <-
        viewport(
          layout.pos.col = 2:4,
          layout.pos.row = 1,
          name = "main"
        )
      image_vp    <-
        viewport(layout.pos.col = 2,   layout.pos.row = 3)
      sil_vp  <- viewport(
        layout.pos.col = 4,
        layout.pos.row = 3,
        name = "sil"
      )
      colorkey_vp <-
        viewport(
          layout.pos.col = 2,
          layout.pos.row = 5,
          name = "colorkey"
        )

    }

    ## main
    pushViewport(main_vp)
    grid.text(options$main, gp = gpar(cex = 1.3, fontface = "bold"))
    upViewport(1)

    ## silhouette
    if (options$silhouettes) {
      ## get and reorder silhouettes
      s <- x$sil[, "sil_width"]

      pushViewport(sil_vp)
      .grid_barplot_horiz(s,
        xlab = "Silhouette width",
        gp_bars = gpar(fill = "lightgrey", col = 0))
      upViewport(1)

    }

    ## image
    if (is.null(options$zlim))
      options$zlim <- range(m, na.rm = TRUE)
    if (!is.null(options$threshold))
      m[m > options$threshold] <- NA

    pushViewport(image_vp)
    .grid_image(m, col = options$col, zlim = options$zlim)

    ## add labels?
    if (options$axes == "auto" && nrow(m) > 25)
      options$axes <- "none"
    if (options$axes != "none") {
      downViewport("image")
      #grid.text(colnames(m), y = unit(-1, "lines"),
      #  x=unit(1:ncol(m), "native"), rot=90, just="right")

      grid.text(
        rownames(m),
        x = unit(1, "npc") + unit(1, "lines"),
        y = unit(1:nrow(m), "native"),
        just = "left",
        gp = options$gp_labels
      )
      upViewport(1)
    }

    upViewport(1)

    ## color key?
    if (options$key) {
      pushViewport(colorkey_vp)
      .grid_colorkey(options$zlim,
        col = options$col,
        threshold = options$threshold)
      upViewport(1)
    }

    ## plot cluster borders if we have labels and order
    if (!is.null(labels)) {
      labels_unique_y		<- labels_unique
      cluster_width_y		<- (tabulate(labels)[labels_unique])
      cluster_cuts_y		<- cumsum(cluster_width_y)
      cluster_center_y	<- cluster_cuts_y - cluster_width_y / 2

      if (options$reverse_columns) {
        labels_unique_x	<- rev(labels_unique)
        cluster_width_x <- (tabulate(labels)[labels_unique_x])
        cluster_cuts_x  <- cumsum(cluster_width_x)
        cluster_center_x <- cluster_cuts_x - cluster_width_x / 2
      } else{
        labels_unique_x <- labels_unique_y
        cluster_width_x <- cluster_width_y
        cluster_cuts_x <- cluster_cuts_y
        cluster_center_x <- cluster_center_y
      }

      if (options$cluster_labels) {
        seekViewport("image")

        ## above the plot
        grid.text(
          labels_unique_x,
          x = cluster_center_x,
          y = unit(1, "npc") + unit(1, "lines"),
          default.units = "native",
          gp = options$gp_labels
        )
        ## left of the plot
        grid.text(
          labels_unique_y,
          x = unit(-1, "lines"),
          y = cluster_center_y,
          default.units = "native",
          gp = options$gp_labels
        )
        upViewport(2)
      }

      if (options$cluster_lines) {
        ## draw lines separating the clusters
        #cluster_cuts <- cluster_cuts[-length(cluster_cuts)]
        ## remove last line

        seekViewport("image")
        for (i in 1:(k - 1)) {
          grid.lines(
            #x = c(0, dim),
            x = c(0.5, dim + 0.5),
            y = cluster_cuts_y[i] + .5,
            default.units = "native",
            gp = options$gp_lines
          )

          grid.lines(
            x = cluster_cuts_x[i] + .5,
            #y = c(0, dim),
            y = c(0.5, dim + 0.5),
            default.units = "native",
            gp = options$gp_lines
          )

        }
        upViewport(2)

      }
    }

    if (options$pop)
      popViewport(1)
    else
      upViewport(1)
  }


## print for reordered_cluster_dissimilarity_matrix
#' @rdname dissplot
#' @export
print.reordered_cluster_dissimilarity_matrix <-
  function(x, ...)
  {
    d <- attr(x$x_reordered, "Size")
    k <- if (!is.null(x$k))
      x$k
    else
      NA

    cat(gettextf("object of class '%s'\n", class(x)))
    cat("matrix dimensions:", d, "x", d, "\n")
    cat(gettextf("dissimilarity measure: '%s'\n", x$diss_measure))
    cat("number of clusters k:", k, "\n")
    if (!is.null(x$k)) {
      cat("\ncluster description\n")
      print(x$description)
    }

    cat("\n")
    cat("used seriation methods\n")
    cat(gettextf("inter-cluster: '%s'\n", x$seriation_methods$inter))
    cat(gettextf("intra-cluster: '%s'\n", x$seriation_methods$intra))

    cat("\n")
    cat(gettextf(
      "dissimilarity aggregation method: '%s'\n",
      x$aggregation_method
    ))

    invisible(x)
  }

## inter and intra cluster dissimilarity matrix from
## a dissimilarity matrix plus labels
.cluster_dissimilarity <-
  function(x,
    labels,
    method = c("avg", "min", "max",
      "Hausdorff")) {
    method <- match.arg(method)
    ## FIXME: Implement Hausdorff

    linkage <- if (method == "avg")
      mean
    else if (method == "min")
      min
    else if (method == "max")
      max
    else if (method == "Hausdorff")
      .hausdorff
    else
      stop("Unknown method.")

    if (!is.matrix(x))
      x <- as.matrix(x)


    ## kill self-dissimilarities (which are always 0)
    diag(x) <- NA

    k           <- length(unique(labels))
    diss_matrix <- matrix(nrow = k, ncol = k)

    ## calculate avg. dissimilarity between clusters
    for (i in 1:k) {
      slice <- x[labels == i, , drop = FALSE]
      for (j in 1:i) {
        block <- slice[, labels == j, drop = FALSE]

        val <- linkage(block, na.rm = TRUE)

        ## fix for clusters of size 1
        if (is.nan(val))
          val <- 0

        diss_matrix[i, j] <- val
        diss_matrix[j, i] <- val
      }
    }

    attr(diss_matrix, "method") <- method
    diss_matrix
  }

## implement Hausdorff distance between two sets from a dissimilarity matrix
##d_H = max{sup_x\inX inf_y\inY d(x,y), sup_y\inY inf_x\inX d(x,y)}
.hausdorff <- function(block, na.rm = TRUE)
  max(apply(block, MARGIN = 1, min, na.rm = na.rm),
    apply(block, MARGIN = 2, min, na.rm = na.rm))

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


## SPIN (Tsafrir et al. 2005)

## Weight matrix
## pimage(create_x(n=150, sigma=20, verbose=TRUE))
create_W <- function(n, sigma, verbose=FALSE) {
  w <- function(i, j, n, sigma) exp(-1*(i-j)^2/n/sigma)
  W <- outer(1:n, 1:n, FUN = w, n=n, sigma=sigma)

  ## make doubly stochastic
  for(i in 1:1000) {
    #cat(i, ".")
    W <- sweep(W, MARGIN = 1, STATS = rowSums(W), "/")
    W <- sweep(W, MARGIN = 2, STATS = colSums(W), "/")
    if(all(round(rowSums(W), 5) == 1) && all(round(colSums(W), 5) == 1)) break
  }

  if(verbose) cat("It took", i, "iterations to make W doubly stochastic!\n")
  if(i >999) warning("Weight matrix did not converge to doubly stochastic in 1000 itermation!")
  W
}

.spin_contr <- list(
  sigma = seq(20,1, length.out = 10),
  step = 5,
  W_function = NULL,
  verbose = FALSE
)

## SPIN: Neighborhood algorithms
seriate_dist_SPIN <- function(x, control = NULL) {
  param <- .get_parameters(control, .spin_contr)

  W_function <- if(is.null(param$W_function)) create_W else param$W_function
  sigma <- param$sigma
  step <- param$step
  verbose <- param$verbose

  D <- as.matrix(x)
  n <- nrow(D)

  ## weight matrix
  W <- W_orig <- W_function(n, sigma[1], verbose)

  energy_best <- Inf

  for(i in 1:(length(sigma)*step)) {
    if(verbose) cat("Iteration", i, "... ")

    M <- D %*% W

    ## heuristic for the linear assignment problem
    ## (second argument to order breakes ties randomly)
    P <- permutation_vector2matrix(
      order(apply(M, MARGIN = 1, which.min), sample(1:n)))
    #if(verbose) print(table(apply(M, MARGIN = 1, which.min)))

    energy_new <- sum(diag(P %*% M))
    if(verbose) cat("best energy:", energy_best,
      "new energy: ", energy_new, "\n")

    ## was energy improved?
    if(energy_new < energy_best) {
      energy_best <- energy_new
      P_best <- P
    }

    ## adapt sigma
    if(!(i %% step) && i != length(sigma)*step) {
      s <- sigma[i/step+1]
      if(verbose) cat("\nReducing sigma to:", s, "\n")

      W_orig <- W_function(n, s, verbose)


      ## recalculate best energy
      W <- crossprod(P, W_orig) ### t(P) %*% W
      M <- D %*% W
      energy_best <- sum(diag(P %*% M))
      if(verbose) cat("best energy is now:", energy_best, "\n\n")
    }else {
      W <- crossprod(P, W_orig) ### t(P) %*% W
    }
  }

  if(verbose) cat("Best Energy:", energy_best, "\n")
  o <- permutation_matrix2vector(P_best)
  names(o) <- names(x)[o]
  o
}

## SPIN: Side-to-Side algorithm

## this is the weight: pimage(tcrossprod(1:n - (n+1)/2))
.spin_sts_contr <- list(
  step = 25,
  nstart = 10,
  X = function(n) 1:n - (n+1)/2,
  verbose = FALSE
)

seriate_dist_SPIN_STS <- function(x, control = NULL) {
  param <- .get_parameters(control, .spin_sts_contr)

  step <- param$step
  verbose <- param$verbose
  nstart <- param$nstart
  X <- param$X

  D <- as.matrix(x)
  n <- nrow(D)

  ## X for weights W = X %*% t(X) (colunm vector)
  if(is.function(X)) X <- X(n)
  if(!is.numeric(X) || length(X) != n) stop("Invalid weight vector X.")
  W <- tcrossprod(X) ## X %*% t(X)

  .STS_run <- function() {
    if(verbose) cat("\nStarting new run\n")

    ## start with random permutation
    o_best <- o <- sample(1:n)
    #P_best <- P <- permutation_vector2matrix(o)
    #X_current <- crossprod(P, X)
    X_current <- X[o]
    #energy_best <- sum(diag(P %*% D %*% t(P) %*% W))
    energy_best <- sum(diag(D[o,o] %*% W))

    for(i in 1:step) {
      if(verbose) cat("Iteration", i, "... ")

      ## permutation matrix that orders S in descending order (break ties)
      S <- D %*% X_current
      o <- order(S, sample(1:n), decreasing = TRUE)
      #P <- permutation_vector2matrix(o)
      #X_current <- crossprod(P, X) ## t(P) %*% X
      X_current <- X[o] ## t(P) %*% X

      ## calculate energy F(P)
      #energy_new <- sum(diag(P %*% D %*% t(P) %*% W))
      energy_new <- sum(diag(D[o,o] %*% W))
      if(verbose) cat("best energy:", energy_best,
        "new energy: ", energy_new)

      ## was energy improved?
      if(energy_new < energy_best) {
        energy_best <- energy_new
        #P_best <- P
        o_best <- o
        if(verbose) cat(" - update")
      }

      if(verbose) cat("\n")

    }

    if(verbose) cat("Best Energy:", energy_best, "\n")

    #o <- permutation_matrix2vector(P_best)
    o <- o_best
    attr(o, "energy") <- energy_best
    o
  }

  res <- replicate(nstart, .STS_run(), simplify = FALSE)
  energy <- sapply(res, attr, "energy")

  if(verbose) cat("Overall best Energy:", min(energy), "\n")
  o <- res[[which.min(energy)]]
  names(o) <- names(x)[o]
  o
}

set_seriation_method("dist", "SPIN_NH", seriate_dist_SPIN,
  "Sorting Points Into Neighborhoods (SPIN) (Tsafrir 2005). Nighborhood algorithm to concentrate low distance values around the diagonal with a Gaussian weight matrix W_{ij} = exp(-(i-j)^2/(n*sigma)), where n is the size of the dissimilarity matrix and sigma is the variance around the diagonal that control the influence of global (large sigma) or local (small sigma) structure.", .spin_contr)
set_seriation_method("dist", "SPIN_STS", seriate_dist_SPIN_STS,
  "Sorting Points Into Neighborhoods (SPIN) (Tsafrir 2005). Side-to-Side algorithm which tries to push out large distance values.", .spin_sts_contr)

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


## setup registries

## seriate
registry_seriate <- registry(registry_class="seriation_registry",
  entry_class="seriation_method")

registry_seriate$set_field("kind", type = "character",
  is_key = TRUE, index_FUN = match_partial_ignorecase)
registry_seriate$set_field("name", type = "character",
  is_key = TRUE, index_FUN = match_partial_ignorecase)
registry_seriate$set_field("fun", type = "function",
  is_key = FALSE)
registry_seriate$set_field("description", type = "character",
  is_key = FALSE)
registry_seriate$set_field("control", type = "list",
  is_key = FALSE)


print.seriation_method <- function(x, ...) {
  writeLines(c(
    gettextf("name:        %s", x$name),
    gettextf("kind:        %s", x$kind),
    gettextf("description: %s", x$description)))

  if(length(x$control)>0) {
    writeLines("control (default values):")

    contr <- lapply(x$control, FUN =
        function(p) utils::capture.output(dput(p, control = list()))[1])

    print(as.data.frame(contr))
  } else writeLines("control: no parameters registered.")

  invisible(x)
}


## criterion
registry_criterion <- registry(registry_class="criterion_registry",
  entry_class="criterion_method")

registry_criterion$set_field("kind", type = "character",
  is_key = TRUE, index_FUN = match_partial_ignorecase)
registry_criterion$set_field("name", type = "character",
  is_key = TRUE, index_FUN = match_partial_ignorecase)
registry_criterion$set_field("fun", type = "function",
  is_key = FALSE)
registry_criterion$set_field("description", type = "character",
  is_key = FALSE)
registry_criterion$set_field("merit", type = "logical",
  is_key = FALSE)


print.criterion_method <-function(x, ...) {
  writeLines(c(
    gettextf("name:        %s", x$name),
    gettextf("kind:        %s", x$kind),
    gettextf("description: %s", x$description),
    gettextf("merit:       %s", x$merit)))
  invisible(x)
}

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

## Criterion for the quality of a permutation of a dissimilarity
## matrix

criterion.dist <- function(x, order = NULL, method = NULL,
  force_loss = FALSE, ...) {

    ## check order and x
    if(!is.null(order)) {
        if(!inherits(order, "ser_permutation"))
            order <- ser_permutation(order)
        .check_dist_perm(x, order)
    }

    ## check dist (most C code only works with lower-triangle version)
    if(attr(x, "Diag") || attr(x, "Upper"))
        x <- as.dist(x, diag = FALSE, upper = FALSE)
    if(!is.double(x)) mode(x) <- "double"

    ## get methods
    if(is.null(method)) method <- list_criterion_methods("dist")
    method <- lapply(method, function(m) get_criterion_method("dist", m))

    crit <- sapply(method,
        function(m) structure(m$fun(x, order, ...), names=m$name))

    if(force_loss) crit <- crit * sapply(method, FUN =
          function(m) ifelse(m$merit, -1, 1))

    crit
}

criterion.default <- criterion.dist

## Wrapper to computing the length of the order under a distance matrix,
## e.g. a tour where the leg between the first and last city is omitted.
## that this is a (Hamilton) path.
##
## Note that this corresponds to the sum of distances along the first
## off diagonal of the ordered distance matrix.

criterion_path_length <- function(x, order = NULL, ...) {
    if (is.null(order)) order <- 1:attr(x, "Size")
    else order <- get_order(order)
    .Call("order_length", x, order, PACKAGE = "seriation")
}

criterion_lazy_path_length <- function(x, order = NULL, ...) {
  if (is.null(order)) order <- 1:attr(x, "Size")
  else order <- get_order(order)
  .Call("lazy_path_length", x, order, PACKAGE = "seriation")
}

## Least squares criterion. measures the difference between the
## dissimilarities between two elements and the rank distance
## (PermutMatrix).

criterion_least_squares <- function(x, order = NULL, ...) {
    if(is.null(order)) order <- 1:attr(x, "Size")
    else order <- get_order(order)
    .Call("least_squares_criterion", x, order, PACKAGE = "seriation")
}

## inertia around the diagonal (see PermutMatrix)
criterion_inertia <- function(x, order = NULL, ...) {
    if(is.null(order)) order <- 1:attr(x, "Size")
    else order <- get_order(order)
    .Call("inertia_criterion", x, order, PACKAGE = "seriation")
}

## anti-Robinson loss functions (Streng and Schoenfelder 1978, Chen
## 2002)
## method: 1...i, 2...s, 3...w
.ar <- function(x, order = NULL, method = 1L) {
    if(is.null(order)) order <- 1:attr(x, "Size")
    else order <- get_order(order)
    .Call("ar", x, order, as.integer(method), PACKAGE = "seriation")
}

criterion_ar_events <- function(x, order, ...) .ar(x, order, 1L)
criterion_ar_deviations <- function(x, order, ...) .ar(x, order, 2L)
#criterion_ar_weighted <- function(x, order, ...) .ar(x, order, 3L)

## w \in [2,n-1]
## or pct \in [0 and 100%]; 0 -> 2 and 100 -> n-1
criterion_rgar <- function(x, order, w=NULL, pct=NULL, relative=TRUE, ...) {
  if(is.null(order)) order <- 1:attr(x, "Size")
  else order <- get_order(order)

  ### default is to take all
  if(is.null(w) && is.null(pct)) w <- length(order)-1L
  else if(is.null(w)) {
      w <- floor((length(order)-3L)*pct/100)+2L
      if(w<1) w <- 1
  }

  if(w<2 || w>=length(order)) stop("Window w needs to be 2<=w<length(order)!")
  .Call("rgar", x, order, as.integer(w), as.integer(relative),
    PACKAGE = "seriation")
}

criterion_bar <- function(x, order, b=NULL, ...) {
    if(is.null(order)) order <- 1:attr(x, "Size")
    else order <- get_order(order)

    ### we default to 1/5
    if(is.null(b)) b <- max(1,floor(length(order)/5))

    if(b<1 || b >=length(order))
      stop("Band size needs to be 1 <= b < length(order)!")
    .Call("bar", x, order, as.integer(b), PACKAGE = "seriation")
}

criterion_gradient_raw <- function(x, order, ...) {
    if(is.null(order)) order <- 1:attr(x, "Size")
    else order <- get_order(order)
    .Call("gradient", x, order, 1L, PACKAGE = "seriation")
}

criterion_gradient_weighted <- function(x, order, ...) {
    if(is.null(order)) order <- 1:attr(x, "Size")
    else order <- get_order(order)
    .Call("gradient", x, order, 2L, PACKAGE = "seriation")
}

.A_2SUM <- function(n)
  outer(1:n,1:n, FUN = function(i,j) (i-j)^2)

criterion_2SUM <- function(x, order, ...) {
    if(is.null(order)) order <- 1:attr(x, "Size")
    else order <- get_order(order)

    # this is sum(diag(A%*%B[o,o]))
    qap::qap.obj(.A_2SUM(attr(x, "Size")), 1/(1+as.matrix(x)), order)
}

### Note: We use n-abs(1-j) since QAP needs positive entires in A!
.A_LS <- function(n)
  outer(1:n,1:n, FUN = function(i,j) n-abs(i-j))

criterion_LS <- function(x, order, ...) {
    if(is.null(order)) order <- 1:attr(x, "Size")
    else order <- get_order(order)

    # this is sum(diag(A%*%B[o,o]))
    qap::qap.obj(.A_LS(attr(x, "Size")), as.matrix(x), order)
}


### these measures are calculated on similarity matrices
criterion_ME_dist <- function(x, order, ...)
    criterion(1/(1+as.matrix(x)), c(order, order), "ME")
criterion_Moore_stress_dist  <- function(x, order, ...)
    criterion(1/(1+as.matrix(x)), c(order, order),
      "Moore_stress")
criterion_Neumann_stress_dist  <- function(x, order, ...)
    criterion(1/(1+as.matrix(x)), c(order, order),
      "Neumann_stress")
criterion_R_dist  <- function(x, order, ...)
    criterion(1/(1+as.matrix(x)), c(order, order), "Cor_R")


### register methods
set_criterion_method("dist", "AR_events" , criterion_ar_events,
    "Anti-Robinson events", FALSE)
set_criterion_method("dist", "AR_deviations", criterion_ar_deviations,
    "Anti-Robinson deviations", FALSE)
## set_criterion_method("dist", "AR_weighted", criterion_ar_weighted)

set_criterion_method("dist", "RGAR", criterion_rgar,
  "Relative generalized anti-Robinson events", FALSE)
set_criterion_method("dist", "BAR", criterion_bar,
  "Banded Anti-Robinson Form", FALSE)

set_criterion_method("dist", "Gradient_raw" , criterion_gradient_raw,
    "Gradient measure", TRUE)
set_criterion_method("dist", "Gradient_weighted", criterion_gradient_weighted,
    "Gradient measure (weighted)", TRUE)

set_criterion_method("dist", "Path_length", criterion_path_length,
    "Hamiltonian path length", FALSE)
set_criterion_method("dist", "Lazy_path_length", criterion_lazy_path_length,
    "Lazy path length", FALSE)

set_criterion_method("dist", "Inertia", criterion_inertia,
    "Inertia criterion", TRUE)
set_criterion_method("dist", "Least_squares", criterion_least_squares,
    "Least squares criterion", FALSE)
set_criterion_method("dist", "ME", criterion_ME_dist,
    "Measure of effectiveness", TRUE)
set_criterion_method("dist", "Cor_R", criterion_R_dist,
    "Correlation coefficient R", TRUE)

set_criterion_method("dist", "Moore_stress", criterion_Moore_stress_dist,
    "Stress (Moore neighborhood)", FALSE)
set_criterion_method("dist", "Neumann_stress", criterion_Neumann_stress_dist,
    "Stress (Neumann neighborhood)", FALSE)

set_criterion_method("dist", "2SUM", criterion_2SUM,
    "2-SUM objective value (QAP)", FALSE)
set_criterion_method("dist", "LS", criterion_LS,
    "Linear seriation objective value (QAP)", FALSE)

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



## seriate dist objects

seriate.dist <- function(x, method = "Spectral", control = NULL, ...) {
    if(!all(x>=0)) stop("Negative distances not supported!")

    ## add ... to control
    control <- c(control, list(...))

    ## check x
    if(any(is.na(x))) stop("NAs not allowed in x!")
    if(any(x<0)) stop("No negative values allowed in x!")

    if(!is.character(method) || (length(method) != 1L))
      stop("Argument 'method' must be a character string.")
    method <- get_seriation_method("dist", method)

    if(!is.null(control$verbose) && control$verbose) cat(method$name, ": ",
      method$description, "\n\n", sep="")

    order <- method$fun(x, control = control)

    ser_permutation(ser_permutation_vector(order, method = method$name))
  }

seriate_dist_identity <- function(x, control = NULL) {
  #param <- .get_parameters(control, NULL)
  .get_parameters(control, NULL)

  o <- 1:attr(x, "Size")
  names(o) <- labels(x)
  o
}

seriate_dist_random <- function(x, control = NULL) {
  #param <- .get_parameters(control, NULL)
  .get_parameters(control, NULL)

  o <- 1:attr(x, "Size")
  names(o) <- labels(x)
  sample(o)
}


set_seriation_method("dist", "Identity", seriate_dist_identity,
  "Identity permutation")
set_seriation_method("dist", "Random", seriate_dist_random,
  "Random permutation")

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


# helper
ndim <- function(x)
  length(dim(x))

#' Permute the Order in Various Objects
#'
#' Provides the generic function and methods for permuting the order of various
#' objects including vectors, lists, dendrograms (also \code{hclust} objects),
#' the order of observations in a \code{dist} object, the rows and columns of a
#' matrix or data.frame, and all dimensions of an array given a suitable
#' [ser_permutation] object.
#'
#' The permutation vectors in [ser_permutation] are suitable if the number
#' of permutation vectors matches the number of dimensions of \code{x} and if
#' the length of each permutation vector has the same length as the
#' corresponding dimension of \code{x}.
#'
#' For 1-dimensional/1-mode data (list, vector, \code{dist}), \code{order} can
#' also be a single permutation vector of class [ser_permutation_vector]
#' or data which can be automatically coerced to this class (e.g. a numeric
#' vector).
#'
#' For \code{dendrogram} and \code{hclust}, subtrees are rotated to represent
#' the order best possible. If the order is not achieved perfectly then the
#' user is warned. This behavior can be changed with the extra parameter
#' \code{incompatible} which can take the values \code{"warn"} (default),
#' \code{"stop"} or \code{"ignore"}.
#'
#' @family permutation
#'
#' @param x an object (a list, a vector, a \code{dist} object, a matrix, an
#' array or any other object which provides \code{dim} and standard subsetting
#' with \code{"["}).
#' @param order an object of class [ser_permutation] which contains
#' suitable permutation vectors for \code{x}. Alternatively, a character string with the
#' name of a seriation method appropriate for `x` can be specified (see [seriate()]).
#' This will perform seriation and permute `x`.
#' @param margin specifies the dimensions to be permuted as a vector with dimension indices.
#' If `NULL`, \code{order} needs to contain a permutation for all dimensions.
#' If a single margin is specified, then \code{order} can also contain
#' a single permutation vector.
#' \code{margin} are ignored.
#' @param ...  if `order` is the name of a seriation method, then additional arguments are
#' passed on to [seriate()].
#' @returns A permuted object of the same class as `x`.
#' @author Michael Hahsler
#' @keywords manip
#' @examples
#' # List data types for permute
#' methods("permute")
#'
#' # Permute matrix
#' m <- matrix(rnorm(10), 5, 2, dimnames = list(1:5, LETTERS[1:2]))
#' m
#'
#' # Permute rows and columns
#' o <- ser_permutation(5:1, 2:1)
#' o
#'
#' permute(m, o)
#'
#' ## permute only columns
#' permute(m, o, margin = 2)
#'
#' ## permute using PCA seriation
#' permute(m, "PCA")
#'
#' ## permute only rows using PCA
#' permute(m, o, margin = 1)
#'
#' # Permute data.frames
#' df <- as.data.frame(m)
#' permute(df, o)
#'
#' # Permute objects in a dist object
#' d <- dist(m)
#' d
#'
#' permute(d, c(3, 2, 1, 4, 5))
#'
#' permute(d, "Spectral")
#'
#' # Permute a list
#' l <- list(a = 1:5, b = letters[1:3], c = 0)
#' l
#'
#' permute(l, c(2, 3, 1))
#'
#' # Permute a dendrogram
#' hc <- hclust(d)
#' plot(hc)
#' plot(permute(hc, 5:1))
#' @export
permute <- function(x, order, ...)
  UseMethod("permute")

permute.default <- function(x, order, ...)
  .permute_kd(x, order, ...)

#' @rdname permute
#' @export
permute.array <- function(x, order, margin = NULL, ...)
  .permute_kd(x, order, margin = margin, ...)

#' @rdname permute
#' @export
permute.matrix <- function(x, order, margin = NULL, ...)
  .permute_kd(x, order, margin = margin, ...)

#' @rdname permute
#' @export
permute.data.frame <- function(x, order, margin = NULL, ...)
  .permute_kd(x, order, margin = margin, ...)

#' @rdname permute
#' @export
permute.table <- function(x, order, margin = NULL, ...)
  .permute_kd(x, order, margin = margin, ...)

#' @rdname permute
#' @export
permute.numeric <- function(x, order, ...)
  .permute_1d(x, order, ...)

#' @rdname permute
#' @export
permute.character <- function(x, order, ...)
  .permute_1d(x, order, ...)

#' @rdname permute
#' @export
permute.list <- function(x, order, ...)
  .permute_1d(x, order, ...)

# special cases
#' @rdname permute
#' @export
permute.dist <- function(x, order, ...) {
  if (is.character(order))
    order <- seriate(x, method = order, ...)

  .nodots(...)

  if (!inherits(order, "ser_permutation"))
    order <- ser_permutation(order)

  if (.is_identity_permutation(order[[1]]))
    return(x)

  .check_dist_perm(x, order)

  .rearrange_dist(x, get_order(order, 1))
}

#' @rdname permute
#' @export
permute.dendrogram <- function(x, order, ...) {
  .nodots(...)

  if (length(get_order(order)) != stats::nobs(x))
    stop("Length of order and number of leaves in dendrogram do not agree!")


  # modeled after rotate in dendextend. Copied here to reduce the heavy dependency count of dendextend.
  #  x <- dendextend::rotate(x, order = match(get_order(order), get_order(x)))
  rot <- function (x, order, ...)
  {
    if (missing(order)) {
      warning("'order' parameter is missing, returning the tree as it was.")
      return(x)
    }
    labels_x <- labels(x)
    order_x <- order.dendrogram(x)
    number_of_leaves <- length(order_x)
    if (!is.numeric(order)) {
      order <- as.character(order)
      if (length(intersect(order, labels_x)) != number_of_leaves) {
        stop(
          "'order' is neither numeric nor a vector with ALL of the labels (in the order you want them to be)"
        )
      }
      order <- match(order, labels_x)
    }
    weights <- seq_len(number_of_leaves)
    weights_for_order <- numeric(number_of_leaves)
    weights_for_order[order_x[order]] <- weights
    reorder(x, weights_for_order, mean, ...)
  }

  x <- rot(x, order = match(get_order(order), get_order(x)))

  if (any(get_order(x) != get_order(order)))
    warning("Dendrogram cannot be perfectly reordered! Using best approximation.")

  x
}

#' @rdname permute
#' @export
permute.hclust <- function(x, order, ...) {
  nd <- stats::as.hclust(permute(stats::as.dendrogram(x), order, ...))
  x$merge <- nd$merge
  x$height <- nd$height
  x$order <- nd$order

  x
}

# helper
.check_dist_perm <- function(x, order) {
  if (length(order) != 1L)
    stop("dimensions do not match")

  if (attr(x, "Size") != length(get_order(order, 1)))
    stop("some permutation vectors do not fit dimension of data")

  # check dist
  if (isTRUE(attr(x, "Diag")) || isTRUE(attr(x, "Upper")))
    stop("'dist' with diagonal or upper triangle matrix not implemented")
}

.check_matrix_perm <- function(x, order) {
  if (ndim(x) != length(order))
    stop("dimensions do not match")
  if (any(dim(x) != sapply(order, length)))
    stop("some permutation vectors do not fit dimension of data")
}

.permute_kd <- function(x, order, margin = NULL, ...) {

  # DEPRECATED: Compatibility with old permutation for data.frame
  if (is.data.frame(x) && is.null(margin) && length(order) == 1 && !is.character(order)) {
    message(
      "permute for data.frames with a single seriation order is now deprecated. Specify the margin as follows: 'permute(x, order, margin = 1)'"
    )
    margin <- 1
  }

  if (is.null(margin))
    margin <- seq(ndim(x))
  else {
    margin <- as.integer(margin)
    if (!all(margin %in% seq(ndim(x))))
      stop("all margins need to specify a valid dimension in x")
  }


  if (is.character(order))
    order <- seriate(x, method = order, margin = margin, ...)

  .nodots(...)

  if (!inherits(order, "ser_permutation"))
    order <- ser_permutation(order)

  if (length(order) != ndim(x) && length(order) != length(margin))
    stop(
      "order needs to contain either orders for all dimensions of x or just orders for the selected margin."
    )

  # set margins not to be permuted to identity and copy the rest
  o <- seriate(x, method = "identity")
  if (length(order) <  ndim(x)) ### we only have order for specified margins
    for(i in seq(length(order)))
      o[[margin[i]]] <- order[[i]]
  else
    for (i in margin)
      o[[i]] <- order[[i]]

  # expand identity manual permutations (if any)
  for (i in which(sapply(o, .is_identity_permutation)))
    o[[i]] <- ser_permutation_vector(seq(dim(x)[i]))

  # check
  .check_matrix_perm(x, o)

  perm <- lapply(o, get_order)
  do.call("[", c(list(x), perm, drop = FALSE))
}

.permute_1d <- function(x, order, ...) {
  .nodots(...)

  if (!inherits(order, "ser_permutation"))
    order <- ser_permutation(order)

  if (length(order) != 1)
    stop("dimensions do not match!")

  if (.is_identity_permutation(order[[1]]))
    return(x)

  perm <- get_order(order, 1)
  if (length(x) != length(perm))
    stop("some permutation vectors do not fit dimension of data!")

  x[perm]
}


# if we used proxy we would say:
#.rearrange_dist <- function (x, order) x[[order]]

.rearrange_dist <- function (x, order) {
  # make C call
  mode(x) <- "double"
  # as.dist seems to make Size numeric and not integer!
  attr(x, "Size") <- as.integer(attr(x, "Size"))
  mode(order) <- "integer"

  d <- .Call("reorder_dist", x, order)

  labels <- if (is.null(labels(x)))
    NULL
  else
    labels(x)[order]

  structure(
    d,
    class   = "dist",
    Size    = length(order),
    Labels  = labels,
    Diag    = FALSE,
    Upper   = FALSE,
    method  = attr(x, "method")
  )
}

#' Gene Expression Data for Wood Formation in Poplar Trees
#'
#' A data matrix containing a sample of the normalized gene expression data for
#' 6 locations in the stem of Popla trees published in the study by Herzberg et
#' al (2001). The sample of 136 genes selected by Caraux and Pinloche (2005).
#'
#' @name Wood
#' @docType data
#' @format The format is a 136 x 6 matrix.
#' @references Hertzberg M., H. Aspeborg, J. Schrader, A. Andersson,
#' R.Erlandsson, K. Blomqvist, R. Bhalerao, M. Uhlen, T. T. Teeri, J.
#' Lundeberg, Bjoern Sundberg, P. Nilsson and Goeran Sandberg (2001): A
#' transcriptional roadmap to wood formation, \emph{PNAS,} \bold{98}(25),
#' 14732--14737.
#'
#' Caraux G. and Pinloche S. (2005): PermutMatrix: a graphical environment to
#' arrange gene expression profiles in optimal linear order,
#' \emph{Bioinformatics,} \bold{21}(7) 1280--1281.
#' @source The data was obtained from
#' \url{http://www.atgc-montpellier.fr/permutmatrix/manual/Exemples/Wood/Wood.htm}.
#' @keywords datasets
#' @examples
#'
#' data(Wood)
#' head(Wood)
#'
NULL

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' @rdname dissplot
#' @export
ggdissplot <- function(x,
  labels = NULL,
  method = "Spectral",
  control = NULL,
  lower_tri = TRUE,
  upper_tri = "average",
  diag = TRUE,
  cluster_labels = TRUE,
  cluster_lines = TRUE,
  reverse_columns = FALSE,
  ...) {
  check_installed("ggplot2")

  # make x dist
  if (!inherits(x, "dist")) {
    if (is.matrix(x) && isSymmetric(x))
      x <- as.dist(x)
    else
      stop("Argument 'x' cannot safely be coerced to class 'dist'.")
  }

  x <- .arrange_dissimilarity_matrix(x,
    labels = labels,
    method = method,
    control = control)

  m  <- .average_tri(x,
    lower_tri = lower_tri,
    upper_tri = upper_tri,
    diag = diag)

  k       <- x$k
  dim     <- attr(x$x_reordered, "Size")
  labels  <- x$labels
  labels_unique <- unique(labels)

  # So we can add cluster labels later
  if (cluster_labels)
    colnames(m) <- seq(ncol(m))

  g <- ggpimage(m, reverse_columns = reverse_columns, ...)

  # add cluster lines and labels
  if (!is.null(labels)) {
    cluster_width		<- tabulate(labels)[labels_unique]
    cluster_cuts		<- cumsum(cluster_width)
    cluster_center	<- cluster_cuts - cluster_width / 2

    clusters <-
      data.frame(
        center = cluster_center,
        cut = cluster_cuts,
        width = cluster_width,
        label = labels_unique
      )

    ### NULLIFY for CRAN check
    center <- label <- cut <- NULL

    if (cluster_labels) {
      # Place cluster labels along diagonal
      # if (!flip) {
      #   g <- g + ggplot2::geom_label(data = clusters,
      #     ggplot2::aes(
      #       x = center,
      #       y = nrow(m) - center,
      #       label = label
      #     ))
      # } else{
      #   g <- g + ggplot2::geom_label(data = clusters,
      #     ggplot2::aes(
      #       x = ncol(m) - center,
      #       y = nrow(m) - center,
      #       label = label
      #     ))
      # }

      # Place cluster labels on top as x-axis (needs the colnames set as a sequence)
      if (reverse_columns) {
        suppressMessages(
          g <-
            g + ggplot2::scale_x_discrete(
              breaks = ncol(m) - clusters$center,
              label = clusters$label,
              expand = c(0, 0),
              position = "top"
            ) +
            ggplot2::theme(
              axis.text.x = ggplot2::element_text(
                angle = 0,
                vjust = 0.5,
                hjust = .5
              )
            ) +
            ggplot2::labs(x = "Cluster")
        )
      } else{
        suppressMessages(
          g <- g + ggplot2::scale_x_discrete(
            breaks = clusters$center,
            label = clusters$label,
            expand = c(0, 0),
            position = "top"
          ) +
            ggplot2::theme(
              axis.text.x = ggplot2::element_text(
                angle = 0,
                vjust = 0.5,
                hjust = .5
              )
            ) +
            ggplot2::labs(x = "Cluster")
        )
      }

      if (cluster_lines) {
        ## draw lines separating the clusters

        if (reverse_columns) {
          g <-
            g + ggplot2::geom_hline(data = clusters, ggplot2::aes(yintercept = nrow(m) - cut + .5)) +
            ggplot2::geom_vline(data = clusters, ggplot2::aes(xintercept = ncol(m) - cut + .5))
        } else{
          g <-
            g + ggplot2::geom_hline(data = clusters, ggplot2::aes(yintercept = nrow(m) - cut + .5)) +
            ggplot2::geom_vline(data = clusters, ggplot2::aes(xintercept = cut + .5))
        }
      }
    }
  }

  # reverse color
  suppressMessages(g <-
      g + .gg_sequential_pal(dist = TRUE))

  g
}

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
    
greenred <- function(n, bias = 1)
  grDevices::colorRampPalette(c("green", "black", "red"), bias = bias, space = "Lab")(n)

### Lab looks a little purple here!
bluered <- function(n, bias = 1) 
  grDevices::colorRampPalette(c("blue", "white", "red"), bias = bias)(n)

grays <- function(n, power = 1)
  colorspace::sequential_hcl(n, c.=c(0), l=c(95, 40), power = power)

greys <- grays


## define default colors
.sequential_pal <- function(n=100, power=1) greys(n, power)
.diverge_pal <- function(n=100, bias=1) bluered(n, bias)

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
    
### lines data set from iVAT paper

create_lines_data <- function(n=250) {
  n1 <- n/5*2
  n2 <- n/5
  n3 <- n/5*2
  
  x1 <- data.frame(x = runif(n1, -5, 5), y = rnorm(n1, mean = 2, sd = .1))
  x2 <- data.frame(x = runif(n2, -3, 3), y = rnorm(n2, mean = 0, sd = .1))
  x3 <- data.frame(x = runif(n3, -5, 5), y = rnorm(n3, mean = -2, sd = .1))
  id <- c(rep(1, times=n1), rep(2, times=n2), rep(3, times=n3))
  
  
  x <- rbind(x1, x2, x3)
  o <- sample(nrow(x))
  x <- x[o,]
  id <- id[o]
  
  rownames(x) <- 1:nrow(x)
  attr(x, "id") <- id
  
  x
}

### ordered data by Michael Hahsler (cite this package)
create_ordered_data <- function(n = 250, 
  k=2, size = NULL, spacing = 6, path = "linear",
  sd1=1, sd2=0) {
  
  if(k>n) stop("k needs to be less than n!")
  path <- match.arg(path, c("linear", "circular"))
  
  ## size
  if(is.null(size)) size <- rep(1, k)
  else if(length(size) != k) stop("length of size vector and k do not agree!")
  size <- round(size/sum(size) * n)
  size[1] <- n - sum(size[-1])
  
  ## create data
  ids <- rep(1:k, times = size)
  
  x <- data.frame(
    x = rnorm(n, mean = ids*spacing, sd = sd1),
    y = rnorm(n, mean = 0, sd = sd2)
  )

  ## transform
  if(path == "circular"){ 
    p <- k*spacing
    theta <- x[,1]/p * 2*pi
    r <-  p/(2*pi)+ x[,2]
    x <- cbind(x=r*sin(theta), y=r*cos(theta))
  }
  
  ## randomize order
  o <- sample(nrow(x))
  x <- x[o , , drop=FALSE]
  ids <- ids[o]
  attr(x, "id") <- ids
  
  x
}

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

## Bridge to package tsp
.tsp_control <- list(
  method="arbitrary insertion",
  rep = 10,
  two_opt = TRUE
)

seriate_dist_tsp <- function(x, control = NULL){
  ## add a dummy city for cutting
  tsp <- insert_dummy(TSP(x), n = 1, label = "cut_here")

  if(is.null(control)) control <- .tsp_control

  tour <- solve_TSP(tsp, method = control$method,
    control = control)

  o <- cut_tour(tour, cut = "cut_here", exclude_cut = TRUE)
  names(o) <- labels(x)[o]
  o
}

set_seriation_method("dist", "TSP", seriate_dist_tsp,
  "Minimize Hamiltonian path length with a TSP solver (see solve_TSP in package TSP for available methods).", .tsp_control)

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2017 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


## Simulated annealing reimplimentation folowing arsa.f by Brusco et al.
## can use any criterion function


### neighborhood functions
LS_swap <- function(o, pos = sample.int(length(o), 2)) {
  tmp <- o[pos[1]]
  o[pos[1]] <- o[pos[2]]
  o[pos[2]] <- tmp
  o
}

### insert pos[1] in pos[2]
LS_insert <- function(o, pos = sample.int(length(o), 2)) {
  append(o[-pos[1]], o[pos[1]], after = pos[2]-1)
}

LS_reverse <- function(o, pos = sample.int(length(o), 2)) {
  o[pos[1]:pos[2]] <- o[pos[2]:pos[1]]
  o
}

LS_mixed <- function(o, pos = sample.int(length(o), 2)) {
  switch(sample.int(3, 1),
    LS_swap(o, pos),
    LS_insert(o, pos),
    LS_reverse(o, pos)
  )
}

.sa_contr <- list(
  criterion = "Gradient_raw",
  init = "Spectral",    ## use "Random" for random init.
  localsearch = LS_insert,
  cool = 0.5,
  tmin = 0.0001,
  nlocal = 10,      ## try nlocal x n local search steps
  verbose = FALSE
)

seriate_sa <- function(x, control = NULL) {
  param <- .get_parameters(control, .sa_contr)
  n <- attr(x, "Size")

  if(is.numeric(param$init)) {
    .check_dist_perm(x, order = param$init)
  }else{
    if(param$verbose) cat("\nObtaining initial solution via:",
      param$init, "\n")
    o <- get_order(seriate(x, method = param$init))
  }

  z <- criterion(x, o, method = param$criterion, force_loss = TRUE)
  if(param$verbose) cat("Initial z =", z,
    "(converted into loss if necessary)\n")

  iloop <- param$nlocal*n

  # find tmax (largest change for a move)
  znew <- replicate(iloop, expr = {
    criterion(x, param$localsearch(o), method = param$criterion,
      force_loss = TRUE)
  })

  tmax <- max(z-znew)
  if(tmax < 0) nloop <- 1L
  else nloop <- as.integer((log(param$tmin)-log(tmax))/log(param$cool))

  if(param$verbose) cat("Found tmax = ", tmax, "using", nloop, "iterations\n")

  zbest <- z
  temp <- tmax

  for(i in 1:nloop) {
    m <- 0L

    for(j in 1:iloop) {

      onew <- param$localsearch(o)
      znew <- criterion(x, onew, method = param$criterion, force_loss = TRUE)
      delta <- z-znew

      if(delta > 0 || runif(1) < exp(delta/temp)) {
        o <- onew
        z <- znew
        m <- m+1L
      }
    }

    if(param$verbose) {
      cat("temp = ", round(temp, 4), "\tz =", z,
        "\t performed moves = ", m, "/", iloop, "\n")
    }

    temp <- temp * param$cool
  }

  o
}

set_seriation_method("dist", "SA", seriate_sa,
  paste0("Minimize a specified seriation measure (criterion) using simulated annealing.\n",
    "Control parameters:\n",
    " - criterion to optimize\n",
    " - init (initial order; use \"Random\" for no warm start\n",
    " - localsearch (neighborhood function; Built-in functions are LS_insert, LS_swap, LS_reverse, and LS_mix (1/3 insertion, 1/3 swap and 1/3 reverse)\n",
    " - cool (cooling rate)\n",
    " - tmin (minimum temperature)\n",
    " - swap_to_inversion (proportion of swaps to inversions)\n",
    " - nlocal (number of objects times nlocal is the number of search tries per temperature\n"), .sa_contr)

C   ANTI-ROBINSON SERIATION
C   branch-and-bound
C   by Brusco, and Stahl, S.
C   R Interface by Michael Hahsler


C      PROGRAM DYNAMIC
C      SUBROUTINE dynamic(N, A, EPS, X)
      SUBROUTINE bbwrcg(N, A, EPS, X, Q, D, DD, S, UNSEL, IVERB)
      IMPLICIT INTEGER(A-Z)
C      DOUBLE PRECISION TIMEA,TIMEB,TIMTOT,A(50,50),EPS
      DOUBLE PRECISION EPS, A(N,N), D(N,N,N),
     1    DD(N,N,N),ZBEST,Z,ACT,DELTA,ZBD,IDX1,IDX2
      REAL S1
      INTEGER X(N),Q(N),S(N),UNSEL(N)

C     EPS is unused this is to supress the warning.
      EPS = 1.0d-07

C       Initialize R RNG
      CALL getrngstate()

      OLDM=0
      CHECKS=0


C
C #################################################################
C 10/13/01 This program fits an "weighted" row gradient criterion
C         to a symmetric proximity matrix.  Count +1 if the anti-
C         Robinson triple is satisfied, -1 if its not, and 0 for
C         ties.  Only look at upper half of matrix
C 07/20/02: Improved symmetry test implemented.
C 07/26/03: Fixed the incorrect symmetry test, added an interchange test
C          avoid use of so many "IF" statements using F & D matrices
C 12/24/03: Add insertion test to interchange test.
C 07/09/15: Fixed memory issue (MFH)
C #################################################################
C
C      OPEN(1,FILE='AMAT.DAT')          ! Dissimilarity matrix
C      OPEN(2,FILE='SEQ.OUT')           ! Output file
C      EPS = 1.0d-07
C      READ(1,*) N                      ! Read number of objects
C      WRITE(*,*) 'TYPE 1 FOR HALF MATRIX OR TYPE 2 FOR FULL MATRIX'
C      READ(*,*) ITYPE
C      ITYPE = 2
C      IF(ITYPE.EQ.2) THEN
C        READ(1,*) ((A(I,J),J=1,N),I=1,N)
C      ELSE
C        DO J = 2,N
C          READ(1,*) (A(I,J),I=1,J-1)
C        END DO
C        DO J = 2,N
C          DO I = 1,J-1
C            A(J,I) = A(I,J)
C          END DO
C        END DO
C      END IF
C      CALL GETTIM (IHR, IMIN, ISEC, I100)
C      CALL GETDAT (IYR, IMON, IDAY)
C      TIMEA=DFLOAT(86400*IDAY+3600*IHR+60*IMIN+ISEC)+DFLOAT(I100)/100.
      DO I = 1,N
        A(I,I) = 0.0D0
      END DO
C
      DO 848 I = 1,N
        DO 849 J = 1,N
          IF(I.EQ.J) GO TO 849
          DO 850 K = 1,N
            IF(I.EQ.K.OR.J.EQ.K) GO TO 850
C bbwrg
C            D(I,J,K) = A(I,K) - A(I,J)
            D(I,J,K) = 2.*A(I,K) - A(I,J) - A(J,K)


 850      CONTINUE
 849    CONTINUE
 848  CONTINUE
C
      DO 851 I = 1,N
        DO 852 J = 1,N
          IF(I.EQ.J) GO TO 852
          DO 853 K = 1,N
            IF(I.EQ.K.OR.J.EQ.K) GO TO 853
            ACT=D(I,J,K)
            IF(D(I,K,J).GT.ACT) ACT = D(I,K,J)
            IF(D(J,I,K).GT.ACT) ACT = D(J,I,K)
            DD(I,J,K) = ACT
 853      CONTINUE
 852    CONTINUE
 851  CONTINUE
C
      ZBEST = 0.0D0
C      DO 3500 JJJ = 1,100
      DO 3500 JJJ = 1,20
        DO I = 1,N
          UNSEL(I) = I
          Q(I) = 0
        END DO
        NNSEL = N
C 3501   CALL RANDOM(S1)
C 3501   S1 = rand()
 3501   CALL unifrand(S1)
        ISEL = INT(1. + S1*FLOAT(NNSEL))
        IF(ISEL.GT.NNSEL) ISEL = NNSEL
        Q(NNSEL) = UNSEL(ISEL)
        DO J = ISEL,NNSEL-1
          UNSEL(J) = UNSEL(J+1)
        END DO
        NNSEL = NNSEL - 1
        IF(NNSEL.GT.0) GO TO 3501
C        WRITE(*,72) (Q(J),J=1,N)
C 72     FORMAT(20I3)
        Z = 0.0D0
        DO I = 1,N-2
          R1 = Q(I)
          DO J = I+1,N-1
            R2 = Q(J)
            DO K = J+1,N
              R3 = Q(K)
              Z = Z + D(R1,R2,R3)
            END DO
          END DO
        END DO
 3502   ITRIG = 0
        DO II = 1,N-1
          DO JJ = II+1,N
C   R interrupt
            CALL rchkusr()
C
            R3 = Q(JJ)
            R2 = Q(II)
            DELTA=0.0D0
            DO I = 1,II-1
              R1 = Q(I)
              DELTA = DELTA + D(R1,R3,R2) - D(R1,R2,R3)
              DO J = II+1,JJ-1
                R4 = Q(J)
                DELTA = DELTA + D(R1,R3,R4) - D(R1,R2,R4)
                DELTA = DELTA + D(R1,R4,R2) - D(R1,R4,R3)
              END DO
            END DO
            DO J = II+1,JJ-1
              R4 = Q(J)
              DELTA = DELTA + D(R3,R4,R2) - D(R2,R4,R3)
              DO K = JJ+1,N
                R5 = Q(K)
                DELTA = DELTA + D(R4,R2,R5) - D(R4,R3,R5)
                DELTA = DELTA + D(R3,R4,R5) - D(R2,R4,R5)
              END DO
            END DO
            DO K = JJ + 1,N
              R5 = Q(K)
              DELTA = DELTA + D(R3,R2,R5) - D(R2,R3,R5)
            END DO
            DO I = II+1,JJ-2
              DO J = I+1,JJ-1
                R4A = Q(I)
                R4B = Q(J)
                DELTA = DELTA + D(R4A,R4B,R2) - D(R4A,R4B,R3)
                DELTA = DELTA + D(R3,R4A,R4B) - D(R2,R4A,R4B)
              END DO
            END DO
            IF(DELTA.GT.0) THEN
              Z = Z + DELTA
              Q(II) = R3
              Q(JJ) = R2
              ITRIG = 1
            END IF
          END DO
        END DO
        IF(ITRIG.EQ.1) GO TO 3502
        IF(Z.GT.ZBEST) ZBEST = Z
 3500 CONTINUE
C      WRITE(2,3505) ZBEST
      IF (IVERB == 1) THEN
C          WRITE(*,3505) ZBEST
          CALL dblepr('HEURISTIC OBJ VALUE', -1, DBLE(ZBEST), 1)
      ENDIF
C 3505 FORMAT(' HEURISTIC OBJ VALUE ',F20.4)
      Z = ZBEST-1
      DO I = 1,N
        Q(I) = 0
      END DO
C
      M=1
      Q(M)=1
      S(1)=1
      trig=1
      DO K = 2,N
        Q(K)=0
      END DO
C
  1   M = M + 1
C
C
      CHECKS=CHECKS+1
      IF (IVERB == 1 .AND. M .GT. OLDM) THEN
C          WRITE (*,6000) M+1, CHECKS
          CALL intpr('reached position', -1, M+1, 1)
          CALL intpr('with following number of checks', -1, CHECKS, 1)
C 6000 FORMAT('reached position ', I5, ' with ', I9, ' checks')
          OLDM=M
      ENDIF

C
C   R interrupt
      CALL rchkusr()
C
C   main loop
C
  2   Q(M)=Q(M)+1
C
C MFH: Make sure to not get out of bounds with S(Q(M)) - 7/9/15
      IF(Q(M).GT.N) GO TO 222
      IF(S(Q(M)).EQ.1) GO TO 2               ! REDUNDANCY
  222 IF(M.EQ.1.AND.Q(M).GT.N) GO TO 9       ! TERMINATE
      IF(M.GT.1.AND.Q(M).GT.N) GO TO 7       ! GO TO RETRACTION
C only for bbwrcg
      IF(TRIG.EQ.0.AND.Q(M).EQ.2) GO TO 2    ! SYMMETRY FATHOM
C
      S(Q(M))=1
      IF(M.EQ.1) GO TO 1
      IF(M.EQ.N-1) THEN
        CALL EVALBBWRCG(ZBD,Q,N,D)
        IF(ZBD.GT.Z) THEN
          Z=ZBD
          IF (IVERB == 1) THEN
C              WRITE(*,*) 'Eval =',z
              CALL dblepr('Eval', -1, DBLE(z), 1)
          ENDIF
          DO I = 1,N
            X(I)=Q(I)
          END DO
        END IF
        Q(N)=0
        S(Q(M))=0
        GO TO 2
      ELSE
        DO 251 MM = M-1,1,-1    ! Insertion Test
          R3=Q(M)
          IDX1=0
          IDX2=0
          DO I = 1,MM-1
            R1=Q(I)
            DO J = MM,M-1
              R4=Q(J)
              IDX1=IDX1+D(R1,R4,R3)
              IDX2=IDX2+D(R1,R3,R4)
C
            END DO
C
          END DO
C
          DO 250 I = 1,N
            IF(S(I).EQ.1) GO TO 250
            R5=I
C
            DO J = MM,M-1
              R4=Q(J)
              IDX1=IDX1+D(R4,R3,R5)
              IDX2=IDX2+D(R3,R4,R5)
            END DO
C
  250     CONTINUE
C
          DO J = MM, M-2
            DO JJ = J+1, M-1
              R4A=Q(J)
              R4B=Q(JJ)
              IDX1=IDX1+D(R4A,R4B,R3)
              IDX2=IDX2+D(R3,R4A,R4B)
            END DO
          END DO
          IF(IDX1.LT.IDX2) THEN
            S(Q(M))=0
C            ism2 = ism2 + 1
            GO TO 2
          END IF
  251   CONTINUE
C        go to 253
C
        DO 151 MM = M-2,1,-1    ! Interchange Test
          R3=Q(M)
          R2=Q(MM)
          IDX1=0
          IDX2=0
          DO J = MM+1,M-1
            R4 = Q(J)
            IDX1=IDX1+D(R2,R4,R3)
            IDX2=IDX2+D(R3,R4,R2)
          END DO
          DO I = 1,MM-1
            R1=Q(I)
            IDX1=IDX1+D(R1,R2,R3)
            IDX2=IDX2+D(R1,R3,R2)
            DO J = MM+1,M-1
              R4=Q(J)
              IDX1=IDX1+D(R1,R2,R4)
              IDX2=IDX2+D(R1,R3,R4)
C
              IDX1=IDX1+D(R1,R4,R3)
              IDX2=IDX2+D(R1,R4,R2)
C
            END DO
C
          END DO
C
          DO 150 I = 1,N
            IF(S(I).EQ.1) GO TO 150
            R5=I
            IDX1=IDX1+D(R2,R3,R5)
            IDX2=IDX2+D(R3,R2,R5)
C
            DO J = MM+1,M-1
              R4=Q(J)
              IDX1=IDX1+D(R2,R4,R5)
              IDX2=IDX2+D(R3,R4,R5)
C
              IDX1=IDX1+D(R4,R3,R5)
              IDX2=IDX2+D(R4,R2,R5)
            END DO
C
  150     CONTINUE
C
          DO J = MM+1, M-2
            DO JJ = J+1, M-1
              R4A=Q(J)
              R4B=Q(JJ)
              IDX1=IDX1+D(R4A,R4B,R3)
              IDX2=IDX2+D(R4A,R4B,R2)
              IDX1=IDX1+D(R2,R4A,R4B)
              IDX2=IDX2+D(R3,R4A,R4B)
            END DO
          END DO
          IF(IDX1.LT.IDX2) THEN
C            ism = ism + 1
            S(Q(M))=0
            GO TO 2
          END IF
  151   CONTINUE
C
        CALL BOUND2BBWRCG(ZBD,N,Q,M,D,S,DD)
        IF(ZBD.LE.Z) THEN
          S(Q(M))=0
C          ism3 = ism3 + 1
          GO TO 2
        END IF
        IF(Q(M).EQ.1) TRIG=1
        GO TO 1
      END IF
C
   7  IF(Q(M).EQ.1) TRIG=0
C MFH: Make sure to not get out of bounds with S(Q(M)) - 6/9/15
      IF(Q(M).GT.N) GO TO 777
      S(Q(M))=0
  777 Q(M)=0
      M=M-1
      IF(Q(M).EQ.1) TRIG=0
      S(Q(M))=0
C
C      WRITE(*,*) 'X',(X(J),J=1,N)
C      WRITE(*,*) 'Q',(Q(J),J=1,N)
C

      GO TO 2
C   9  CALL GETTIM (IHR, IMIN, ISEC, I100)
C      CALL GETDAT (IYR, IMON, IDAY)
C      TIMEB=DFLOAT(86400*IDAY+3600*IHR+60*IMIN+ISEC)+DFLOAT(I100)/100.
C      TIMTOT=TIMEB-TIMEA
C      write(*,*) ism,ism2,ism3
C      WRITE(*,69) Z,TIMTOT
C  9   WRITE(*,69) Z
C      WRITE(2,69) Z,TIMTOT
C      WRITE(2,70) (X(I),I=1,N)
C 69   FORMAT(' MAXIMUM UNWEIGHTED ROW GRADIENT INDEX ',I20)
C 69   FORMAT(' MAXIMUM UNWEIGHTED ROW GRADIENT INDEX ',I7,' CPU TIME ',
C     1        F8.2)
C 70   FORMAT(30I3)
C
  9   IF (IVERB == 1) THEN
C          PRINT *, 'total number of checks: ', CHECKS
          CALL intpr('total number of checks', -1, CHECKS, 1)
      ENDIF

C    Return R RNG
      CALL Putrngstate()


      RETURN
      END
C
      SUBROUTINE BOUND2BBWRCG(ZBD,N,Q,M,D,S,DD)
      IMPLICIT INTEGER(A-Z)
      DOUBLE PRECISION D(N,N,N),ZBD,DD(N,N,N),Z1,Z2,Z3,ZA,ZB,
     1   ZCT,N4
C       ACT is now unused

      INTEGER Q(N),S(N)
      Z1=0
      DO I = 1,M-2
        R1=Q(I)
        DO J = I+1,M-1
          R2=Q(J)
          DO K = J+1,M
            R3=Q(K)
            Z1=Z1+D(R1,R2,R3)
          END DO
        END DO
      END DO
C
      Z2=0
      DO I = 1,M-1
        R1=Q(I)
        DO J = I+1,M
          R2=Q(J)
          DO 60 K = 1,N
            IF(S(K).EQ.1) GO TO 60
            R3=K
            Z2=Z2+D(R1,R2,R3)
 60       CONTINUE
        END DO
      END DO
C
      Z3=0
      DO 90 I = 1,N-1
        IF(S(I).EQ.1) GO TO 90
        R2=I
        DO 91 J = I+1,N
          IF(S(J).EQ.1) GO TO 91
          R3=J
          ZA=0
          ZB=0
          DO 92 K = 1,M
            R1=Q(K)
            ZA=ZA+D(R1,R2,R3)
            ZB=ZB+D(R1,R3,R2)
 92       CONTINUE
          ZCT=ZA
          IF(ZB.GT.ZCT) ZCT=ZB
          Z3=Z3+ZCT
 91     CONTINUE
 90   CONTINUE
C
      N4=0
      DO 93 I = 1,N-2
        IF(S(I).EQ.1) GO TO 93
        R1=I
        DO 94 J = I+1,N-1
          IF(S(J).EQ.1) GO TO 94
          DO 95 K = J+1,N
            IF(S(K).EQ.1) GO TO 95
C            ACT=D(I,J,K)
C            IF(D(I,K,J).GT.ACT) ACT=D(I,K,J)
C            IF(D(J,I,K).GT.ACT) ACT=D(J,I,K)
C            N4=N4+ACT
             N4 = N4 + DD(I,J,K)
  95      CONTINUE
  94    CONTINUE
  93  CONTINUE
C      N1=N*(N-1)*(N-2)/3    ! This bound is OK!  The N1 is total
C      N2=M*(M-1)*(M-2)/3    ! and N2 and N3 are truly computed terms.
C      N3=(N-M)*(M*(M-1))  ! So N1-N2-N3 assumes +1 for rest, which
      ZBD=Z1+Z2+Z3+n4   ! (N-M)*(N-M-1)*(N-M-2)/3      +n4
C      WRITE(*,98) N,M,N1,N2,N3,Z1,Z2,N1-N2-N3,ZBD
C 98   FORMAT(9I7)
      RETURN
      END
C
      SUBROUTINE EVALBBWRCG(ZBD,Q,N,D)
      IMPLICIT INTEGER(A-Z)
      DOUBLE PRECISION D(N,N,N),ZBD
      INTEGER Q(N)
      ZBD=0
      DO 85 I = 1,N
        DO J = 1,N-1
          IF(Q(J).EQ.I) GO TO 85
        END DO
        Q(N)=I
 85   CONTINUE
      DO I = 1,N-2
        R1=Q(I)
        DO J = I+1,N-1
          R2=Q(J)
          DO K = J+1,N
            R3=Q(K)
            ZBD=ZBD+D(R1,R2,R3)
          END DO
        END DO
      END DO
      RETURN
      END

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


## BEA FORTRAN code by Fionn Murtagh

bea <- function(a, istart=0, jstart=0)
{
    ## Permute rows and colums, using "bond energy algorithm".
    if(!is.matrix(a))
        stop("First input argument must be a matrix.\n")

    n <- nrow(a)
    m <- ncol(a)
    b    <- matrix(0.0, n, m)
    mode(a) <- "single"
    mode(b) <- "single"
    ib   <- integer(n)
    jb   <- integer(m)
    ifin <- integer(n)
    jfin <- integer(m)
    ener <- 0.0

    if (istart == 0) istart <- floor(runif(1,1,n))
    if (jstart == 0) jstart <- floor(runif(1,1,m))

    bea1  <- .Fortran("rbea",
                      n = as.integer(n),
                      m = as.integer(m),
                      a = as.matrix(a),             # input data
                      istart = as.integer(istart),  # 1st row placement
                      b = as.matrix(b),             # permuted array
                      ib = as.integer(ib),          # permuted order of rows
		      ifin = as.integer(ifin),      # for book-keeping
		      PACKAGE="seriation")
    a <- bea1$b
    
    bea2  <- .Fortran("cbea",
                      n = as.integer(n),
                      m = as.integer(m),
                      a = as.matrix(a),             # input data
                      jstart = as.integer(jstart),  # 1st col. placement
                      b = as.matrix(b),             # permuted array
                      jb = as.integer(jb),          # permuted order of cols.
		      jfin = as.integer(jfin),      # for book-keeping
		      PACKAGE="seriation")

    energy  <- .Fortran("energy",
                       n = as.integer(n),
                       m = as.integer(m),
                       b = as.matrix(bea2$b),
                       ener = as.single(ener),
		       PACKAGE="seriation")

    list(b = bea2$b, ib = bea1$ib, jb = bea2$jb, e = energy$ener)
}

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



## S3 permutation and permutations classes
## permutations consists of instances of permutation

## permutation_vector

## constructor (NA is identity vector)
ser_permutation_vector <- function(x, method = NULL) {
  if(inherits(x, "ser_permutation_vector")) return(x)

  ## make sure it's an integer vector
  if(is.vector(x) && !is.integer(x)) x <- as.integer(x)

  if(.is_identity_permutation(x)) attr(x, "method") <- "identity permutation"
  if(!is.null(method)) attr(x, "method") <- method

  class(x) <- c("ser_permutation_vector", class(x))
  .valid_permutation_vector(x)
  x
}

## accessors
get_order <- function(x, ...) UseMethod("get_order")

get_order.ser_permutation_vector <- function(x, ...) NextMethod()

get_order.hclust <- function(x, ...) x$order

get_order.dendrogram <- function(x, ...) order.dendrogram(x)


get_order.integer <- function(x, ...) {
  o <- as.integer(x)
  if(.is_identity_permutation(x)) stop("Cannot get order from identity permutation.")
  #names(o) <- names(x)[o]
  o
}

## returns the order of objects (index of first, second, etc. object)
get_order.default <- function(x, ...)
  stop(gettextf("No permutation accessor implemented for class '%s'. ",
    class(x)))

## returns for each object its rank (rank of first, second, etc. object)
get_rank <- function(x, ...) order(get_order(x, ...))

get_permutation_matrix <- function(x, ...)
  permutation_vector2matrix(get_order(x, ...))

## c will create a ser_permutation!
c.ser_permutation_vector <- function(..., recursive = FALSE)
  do.call("ser_permutation", list(...))

## convert to permuation matrix
permutation_vector2matrix <- function(x) {
  x <- get_order(x)
  .valid_permutation_vector(x)

  n <- length(x)
  pm <- matrix(0, nrow = n, ncol = n)
  for(i in 1:n) pm[i, x[i]] <- 1
  pm
}

permutation_matrix2vector <- function(x) {
  .valid_permutation_matrix(x)
  o <- apply(x, MARGIN = 1, FUN = function(r) which(r==1))
  o
}

## reverse
rev.ser_permutation_vector <- function(x) {
  if(inherits(x, "hclust")) { x$order <- rev(x$order); x }
  else ser_permutation_vector(rev(get_order(x)), method=get_method(x))
}


## currently method is an attribute of permutation
get_method <- function(x, printable = FALSE) {
  method <- attr(x, "method")

  if(printable && is.null(method)) method <- "unknown"
  method
}


## print et al
length.ser_permutation_vector <- function(x)
  if(!.is_identity_permutation(x)) length(get_order(x)) else 0L

print.ser_permutation_vector <-
  function(x, ...)
  {
    writeLines(
      c(gettextf("object of class %s",
        paste(sQuote(class(x)), collapse = ", ")),
        gettextf("contains a permutation vector of length %d", length(x)),
        gettextf("used seriation method: '%s'",
          get_method(x, printable = TRUE))))
    invisible(x)
  }

## fake summary (we dont really provide a summary,
## but summary produces now a reasonable result --- same as print)
summary.ser_permutation_vector <- function(object, ...) {
  object
}


## helpers
.is_identity_permutation <- function (x)
  if(is.integer(x) && length(as.vector(x))==1 && is.na(x[1])) TRUE else FALSE

.valid_permutation_vector <- function(x) {
  ## identity vector
  if(.is_identity_permutation(x)) return()

  perm <- get_order(x)
  valid <- TRUE

  tab <- table(perm)
  if(any(tab != 1)) valid <- FALSE
  if(length(tab) != length(perm)
    || any(names(tab) != sequence(length(perm)))) valid <- FALSE

  if(!valid) stop("Invalid permutation vector!\nVector: ",
    paste(perm, collapse=", "))
}

.valid_permutation_matrix <- function(x) {
  if(any(rowSums(x)!=1) || any(colSums(x)!=1) || any(x!=1 & x!=0))
    stop("Not a valid permutation matrix")
}

library(seriation)
set.seed(0)

context("ser_permutation_vector")

p <- sample(1:10)
sp <- ser_permutation_vector(p, method="valid")

expect_identical(length(sp), 10L)
expect_identical(get_order(sp), p)
expect_identical(get_order(rev(sp)), rev(p))


expect_error(ser_permutation_vector(c(1:10, 12L), method="invalid"), "Invalid permutation vector!")
expect_error(ser_permutation_vector(c(1:10, 3L), method="invalid"), "Invalid permutation vector!")

context("ser_permutation")

expect_identical(length(ser_permutation(sp)), 1L)
expect_identical(length(ser_permutation(sp, sp)), 2L)

hc <- hclust(dist(runif(10)))
expect_identical(length(ser_permutation(sp, hc)), 2L)
hc <- ser_permutation_vector(hc, method="hc")
expect_identical(length(ser_permutation(sp, hc, sp)), 3L)
expect_identical(length(ser_permutation(ser_permutation(sp), 1:10)), 2L)

context("permute")

## vector
expect_identical(permute(1:10, ser_permutation(1:10)), 1:10)
expect_identical(permute(LETTERS[1:10], ser_permutation(1:10)), LETTERS[1:10])
expect_identical(permute(1:10, ser_permutation(10:1)), 10:1)
expect_identical(permute(LETTERS[1:10], ser_permutation(10:1)), LETTERS[10:1])

expect_error(permute(1:10, ser_permutation(1:11)))

## matrix
m <- matrix(runif(9), ncol=3)
expect_identical(permute(m, ser_permutation(1:3, 3:1)), m[,3:1])
expect_identical(permute(m, ser_permutation(3:1, 3:1)), m[3:1,3:1])

expect_error(permute(m, ser_permutation(1:10, 1:9)))
expect_error(permute(m, ser_permutation(1:9, 1:11)))

## dist
d <- dist(matrix(runif(25), ncol=5))
attr(d, "call") <- NULL   ### permute removes the call attribute
expect_identical(permute(d, ser_permutation(1:5)), d)
### is_equivalent_to ignores attributes
expect_equivalent(permute(d, ser_permutation(5:1)), as.dist(as.matrix(d)[5:1,5:1]))

expect_error(permute(d, ser_permutation(1:8)))

## list
l <- list(a = 1:10, b = letters[1:5], 25)
expect_identical(permute(l, 3:1), rev(l))

## dendrogram
dend <- as.dendrogram(hclust(d))
expect_equal(dend, permute(dend, order.dendrogram(dend)))
expect_equal(rev(dend), permute(dend, rev(order.dendrogram(dend))))

# chances are that a random order will not be perfect
o <- sample(5)
expect_warning(permute(dend, o))

## hclust
hc <- hclust(d)
expect_equal(hc, permute(hc, get_order(hc)))

### Note: rev for hclust adds labels! (So we only compare merge, height and order)
#expect_equal(rev(hc), permute(hc, rev(get_order(hc))))
expect_equal(rev(hc)[1:3], permute(hc, rev(get_order(hc)))[1:3])

expect_warning(permute(hc, o))

context("permutation_matrix2vector")
pv <- 1:5
pm <- permutation_vector2matrix(pv)
expect_true(all(diag(pm) == 1))

pv <- sample(1:100)

## convert into a permutation matrix
pm <- permutation_vector2matrix(pv)

## convert back
expect_identical(permutation_matrix2vector(pm), pv)

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2015 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


#' Register Seriation Based on OPTICS
#'
#' Use ordering points to identify the clustering structure (OPTICS) for [seriate()].
#'
#' Registers the method \code{"optics"} for [seriate()]. This method applies
#' the OPTICS ordering algorithm to create an ordering.
#'
#' \bold{Note:} Package \pkg{dbscan} needs to be installed.
#'
#' @aliases register_optics optics OPTICS
#' @seealso [dbscan::optics()] in \pkg{dbscan}.
#' @family seriation
#' @returns Nothing.
#'
#' @references Mihael Ankerst, Markus M. Breunig, Hans-Peter Kriegel, Joerg
#' Sander (1999). OPTICS: Ordering Points To Identify the Clustering Structure.
#' ACM SIGMOD international conference on Management of data. ACM Press. pp.
#' 49-60. \doi{10.1145/304181.304187}
#' @keywords optimize cluster
#' @examples
#'
#' \dontrun{
#' register_optics()
#' get_seriation_method("dist", "optics")
#'
#' d <- dist(random.robinson(50, pre=TRUE, noise=.1))
#'
#' o <- seriate(d, method = "optics")
#' pimage(d, o)
#' }
#'
#' @export
register_optics <- function() {
  check_installed("dbscan")

  .contr <- list(
    eps = NULL,
    minPts = 5
  )

  optics_order <- function(x, control) {
    control <- .get_parameters(control, .contr)

    dbscan::optics(x, eps = control$eps, minPts = control$minPts)$order
  }

  set_seriation_method(
    "dist",
    "optics",
    optics_order,
    "Use ordering points to identify the clustering structure (OPTICS) to create an order",
    .contr
  )
}

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


.mds_control <- list(
  method = "cmdscale"
)

## Multidimensional scaling
seriate_dist_mds <- function(x, control = NULL){
  control <- .get_parameters(control, .mds_control)

  if(control$method == "cmdscale" ) {
    sc <- cmdscale(x, k=1)
    return(order(sc[,1]))

  }else if(control$method == "isoMDS"){
    sc <- MASS::isoMDS(x+1e-6, trace = FALSE, k=1)
    return(order(sc$points[,1]))

  }else if(control$method == "sammon") {
    sc <- MASS::sammon(x+1e-6, trace = FALSE, k=1)
    return(order(sc$points[,1]))

  }else stop("unknown method")

}

seriate_dist_mds_metric <- function(x, control = NULL)
  seriate_dist_mds(x, control=list(method="cmdscale"))

seriate_dist_mds_nonmetric <- function(x, control = NULL)
  seriate_dist_mds(x, control=list(method="isoMDS"))

## Angle between the first 2 PCS. Fiendly (2002)
seriate_dist_angle <- function(x, control = NULL) {
  .get_parameters(control, NULL)

  sc <- cmdscale(x, k=2)
  .order_angle(sc)
}


set_seriation_method("dist", "MDS", seriate_dist_mds,
  "Order using the first component found by multidimensional scaling. Element method in control can be \"cmdscale\", \"isoMDS\" or \"sammon\".", .mds_control)
set_seriation_method("dist", "MDS_metric", seriate_dist_mds_metric,
  "Order using the first component found by metric multidimensional scaling (cmdscsale).")
set_seriation_method("dist", "MDS_nonmetric", seriate_dist_mds_nonmetric,
  "Order using the first component found by non-metric multidimensional scaling (isoMDS).")
set_seriation_method("dist", "MDS_angle", seriate_dist_angle,
  "Order by the angle in this space given by the first two components found by MDS (Friendly, 2002).")

#######################################################################
# seriation - Infrastructure for seriation
# Copyrigth (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


## use the projection on the first pricipal component to determine the
## order
.pca_contr <- list(
  center = TRUE,
  scale. = FALSE,
  tol = NULL,
  verbose = FALSE
)

seriate_matrix_fpc <- function(x, control = NULL) {
  control <- .get_parameters(control, .pca_contr)

  center  <- control$center
  scale.  <- control$scale.
  tol     <- control$tol
  verbose <- control$verbose

  pr <- prcomp(x, center = center, scale. = scale., tol = tol)
  scores <- pr$x[,1]
  row <- order(scores)
  if(verbose) cat("row: first principal component explains",
    pr$sdev[1] / sum(pr$sdev)* 100,"%\n")

  pr <- prcomp(t(x), center = center, scale. = scale., tol = tol)
  scores <- pr$x[,1]
  col <- order(scores)
  if(verbose) cat("col: first principal component explains",
    pr$sdev[1] / sum(pr$sdev)* 100,"%\n")

  names(row) <- rownames(x)[row]
  names(col) <- colnames(x)[col]

  list(row = row, col = col)
}

## Angle between the first 2 PCS. Fiendly (2002)
.order_angle <- function(x) {

  alpha <- atan2(x[,1], x[,2])
  o <- order(alpha)
  cut <- which.max(abs(diff(c(alpha[o], alpha[o[1]]+2*pi))))
  if(cut==length(o)) o
  else o[c((cut+1):length(o), 1:(cut))]

}

.angle_contr <- list(
  center = TRUE,
  scale. = FALSE,
  tol = NULL
)

seriate_matrix_angle <- function(x, control = NULL) {
  control <- .get_parameters(control, .angle_contr)

  center  <- control$center
  scale.  <- control$scale.
  tol     <- control$tol

  pr <- prcomp(x, center = center, scale. = scale., tol = tol)
  row <- .order_angle(pr$x[,1:2])

  pr <- prcomp(t(x), center = center, scale. = scale., tol = tol)
  col <- .order_angle(pr$x[,1:2])

  names(row) <- rownames(x)[row]
  names(col) <- colnames(x)[col]

  list(row = row, col = col)
}

set_seriation_method("matrix", "PCA", seriate_matrix_fpc,
  "Uses the projection of the data on its first principal component to determine the order. Note that for a distance matrix calculated from x with Euclidean distance, this methods minimizes the least square criterion.", .pca_contr)
set_seriation_method("matrix", "PCA_angle", seriate_matrix_angle,
  "Projects the data on the first two principal components and then orders by the angle in this space. The order is split by the larges gap between adjacent angles.", .angle_contr)

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' Class ser_permutation -- A Collection of Permutation Vectors for Seriation
#'
#' The class `ser_permutation` is a collection of permutation vectors
#' (see class [ser_permutation_vector]), one for each dimension (mode)
#' of the data to be permuted.
#'
#' @family permutation
#'
#' @param x,object an object of class `ser_permutation_vector` or
#'     any object which can be converted into
#'     a object of class `ser_permutation` (e.g. an integer
#'       vector).
#' @param ... vectors for further dimensions.
#'
#' @returns An object of class `ser_permutation`.
#'
#' @author Michael Hahsler
#' @examples
#' o <- ser_permutation(1:5, 10:1)
#' o
#'
#' ## length (number of dimensions)
#' length(o)
#'
#' ## get permutation vector for 2nd dimension
#' get_order(o, 2)
#'
#' ## reverse dimensions
#' o[2:1]
#'
#' ## combine
#' o <- c(o, ser_permutation(1:15))
#' o
#'
#' ## get an individual permutation
#' o[[2]]
#'
#' ## reverse the order of a permutation
#' o[[2]] <- rev(o[[2]])
#' get_order(o,2)
#' @keywords classes
#' @export
ser_permutation <- function(x, ...) {
  x <- c(list(x), list(...))

  x <- lapply(
    x,
    FUN = function(obj) {
      if (inherits(obj, "ser_permutation"))
        return(obj)
      if (inherits(obj, "ser_permutation_vector"))
        return(list(obj))
      return(list(ser_permutation_vector(obj)))
    }
  )

  x <- unlist(x, recursive = FALSE)
  class(x) <- c("ser_permutation", "list")
  x
}

#' @rdname ser_permutation
#' @export
print.ser_permutation <- function(x, ...) {
  writeLines(c(
    gettextf("object of class %s",
      paste(sQuote(class(
        x
      )), collapse = ", ")),
    gettextf("contains permutation vectors for %d-mode data\n",
      length(x))
  ))

  print(
    data.frame(
      "vector length" = sapply(
        x,
        FUN = function(o)
          if (.is_identity_permutation(o))
            NA_integer_
        else
          length(o)
      ),
      "seriation method" =
        sapply(x, get_method, printable = TRUE),
      check.names = FALSE
    )
  )

  invisible(x)
}

## fake summary (we don't really provide a summary,
## but summary produces now a reasonable result --- same as print)
#' @rdname ser_permutation
#' @export
summary.ser_permutation <- function(object, ...)
  object

#' @rdname ser_permutation
#' @param recursive ignored.
#' @export
c.ser_permutation <- function(..., recursive = FALSE)
  do.call("ser_permutation", list(...))

## fixme [[<- needs to check for ser_permutation_vector

#' @rdname ser_permutation
#' @param i index of the dimension(s) to extract.
#' @export
"[.ser_permutation" <- function(object, i, ...)
  do.call("ser_permutation", unclass(object)[i])

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


## use the projection on the first principal component to determine the
## order

## use the projection on the first principal component to determine the
## order
.ca_contr <- list(
  dim = 1L,
  ca_param = NULL
)

seriate_matrix_ca <- function(x, control = NULL) {
  control <- .get_parameters(control, .ca_contr)

  mat.ca <- do.call(ca::ca, c(list(obj = x), control$ca_param))
  rcoord <- mat.ca$rowcoord    # row coordinates
  row <- order(rcoord[, control$dim])
  ccoord <- mat.ca$colcoord    # col coordinates
  col <- order(ccoord[, control$dim])

  #names(row) <- rownames(x)[row]
  #names(col) <- colnames(x)[col]

  list(row = row, col = col)
}


set_seriation_method(
  "matrix",
  "CA",
  seriate_matrix_ca,
  "This method calculates a correspondence analysis of the matrix and computes an order according to the scores on a correspondence analysis dimension.",
  .ca_contr
)

#######################################################################
# Code to map between ranges for continuous variables
# Copyright (C) 2011 Michael Hahsler
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

## mapping helper

map <- function(x,
  range = c(0, 1),
  from.range = NA) {
  ## deal with infinite values
  infs <- is.infinite(x)
  if (any(infs)) {
    warning(
      "x contains infinite values. +Inf will be mapped to be mapped to largest value + range and -Inf to smallest value - range."
    )
    min_max <- range(x[!infs], na.rm = TRUE)
    pos_inf_val <- min_max[2] + (min_max[2] - min_max[1])
    neg_inf_val <- min_max[1] - (min_max[2] - min_max[1])

    x[infs] <-
      ifelse(sign(x[infs] > 0), pos_inf_val, neg_inf_val)
  }

  ## set from range
  if (any(is.na(from.range)))
    from.range <- range(x, na.rm = TRUE)

  if (length(from.range) != 2L ||
      from.range[1] > from.range[2])
    stop('from.range needs to contain 2 numbers (upper <= lower bound).')
  from.range_width <- from.range[2] - from.range[1]

  if (length(range) != 2L)
    stop('range needs to contain 2 numbers (upper and lower bound).')
  range_width <- range[2] - range[1]

  ## if all values are the same and no from.range is given, then return the average range
  if (from.range_width == 0) {
    x[] <- mean(range)
    return(x)
  }

  ## map to [0,1]
  x <- (x - from.range[1]) / from.range_width

  ## map from [0,1] to [range]
  x <- x * range_width + range[1]

  x
}

map_int <- function(x,
  range = c(1L, 100L),
  from.range = NA) {
  if (length(range) == 1L)
    range <- c(1L, range)
  as.integer(map(x, c(range[1], range[2]), from.range))
}

library(seriation)
library(testthat)

context("map")
map <- seriation:::map
v <- 0:10

expect_equal(map(v), seq(0, 1, length.out = length(v)))
expect_equal(map(v, range = c(100,200)), seq(100, 200, length.out = length(v)))
expect_equal(map(v, range = c(200,100)), seq(200, 100, length.out = length(v)))


expect_error(map(v, from.range = c(200,100)))
expect_error(map(v, from.range = c(0, 5, 10)))

expect_equal(map(rep.int(1, 10)), rep(.5, 10))

m <- outer(0:10, 0:10, "+")
expect_equal(map(m), outer(seq(0, 1, length.out = 11), seq(0, 1, length.out = 11), "+") / 2)


context("map_int")
map_int <- seriation:::map_int

expect_identical(map_int(v, range = c(-100, 100)), as.integer(seq(-100, 100, length.out = length(v))))

#' Voting Patterns in the Second Rehnquist U.S. Supreme Court
#'
#' Contains a (a subset of the) decisions for the stable 8-yr
#' period 1995-2002 of the second Rehnquist Supreme Court.
#' Decisions are aggregated to
#' the joint probability for disagreement between judges.
#'
#' @name SupremeCourt
#' @aliases SupremeCourt
#' @docType data
#' @format
#'   A square, symmetric 9-by-9 matrix with the joint probability for disagreement.
#' @references
#'     Sirovich, L. (2003). A pattern analysis of the second Rehnquist
#'     U.S. Supreme Court. \emph{Proceedings of the National Academy of Sciences of the United
#'     States of America,} 100, 7432-7437. \doi{10.1073/pnas.1132164100}
#' @author Michael Hahsler
#' @examples
#' data("SupremeCourt")
#'
#' # joint probability of disagreement
#' SupremeCourt
#'
#' d <- as.dist(SupremeCourt)
#' o <- seriate(d)
#' o
#'
#' # judges in original alphabetical order
#' pimage(d, diag = TRUE, upper = TRUE)
#'
#' # judges reordered by seriation based on similar decisions
#' pimage(d, o, diag = TRUE, upper = TRUE)
#'
#' # Use optimal leaf ordering (hierarchical clustering with reordering)
#' # which uses a dendrogram
#' o <- seriate(d, method = "OLO")
#' o
#'
#' plot(o[[1]])
#' @keywords datasets
NULL

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' Plot Heat Map Reordered Using Seriation
#'
#' Provides heatmaps reordered using several different seriation methods. This
#' includes dendrogram based reordering with optimal leaf order and matrix
#' seriation-based heat maps.
#'
#' For dendrogram based heat maps, the arguments are passed on to
#' [stats::heatmap()] in \pkg{stats}. The following arguments for `heatmap()`
#' cannot be used: `margins`, `Rowv`, `Colv`, `hclustfun`, `reorderfun`.
#'
#' For seriation-based heat maps further arguments include:
#' - `gp` an object of class `gpar` containing graphical
#'   parameters (see [gpar()] in package \pkg{grid}).
#' - `newpage` a logical indicating whether to start plot on a new
#'   page
#' - `prop` a logical indicating whether the height and width of `x` should
#'   be plotted proportional to its dimensions.
#' - `showdist` Display seriated dissimilarity matrices? Values are
#'   `"none"`, `"both"`, `"rows"` or `"columns"`.
#' - `key` logical; show a colorkey?
#' - `key.lab` Label plotted next to the color key.
#' - `margins` bottom and right-hand-side margins are calculated
#'   automatically or can be specifies as a vector of two numbers (in lines).
#' - `zlim` range of values displayed.
#' - `col`, `col_dist` color palettes used.
#'
#' @family plots
#'
#' @param x a matrix or a dissimilarity matrix of class dist. If a
#' dissimilarity matrix is used, then the `distfun` is ignored.
#' @param distfun function used to compute the distance (dissimilarity) between
#' both rows and columns. For `gghmap()`, this
#' parameter is passed on in `control`.
#' @param method a character strings indicating the used seriation algorithm
#' (see [seriate.dist()]). If the method results in a dendrogram then
#' [stats::heatmap()] is used to show the dendrograms, otherwise
#' reordered distance matrices are shown instead.
#' @param control a list of control options passed on to the seriation
#' algorithm specified in `method`.
#' @param scale character indicating if the values should be centered and
#' scaled in either the row direction or the column direction, or none. Default
#' is none.
#' @param showDend Show dendrograms in the margin?
#' @param col a list of colors used.
#' @param row_labels,col_labels a logical indicating if row and column labels
#' in `x` should be displayed.  If `NULL` then labels are displayed
#' if the `x` contains the appropriate dimname and the number of labels is
#' 25 or less. A character vector of the appropriate length with labels can
#' also be supplied.
#' @param prop logical; change the aspect ratio so cells in the image have a
#' equal width and height.
#' @param \dots further arguments passed on to [stats::heatmap()].
#' @return An invisible list with elements:
#' \item{rowInd, colInd}{index permutation vectors.}
#' \item{reorder_method}{name of the method used to reorder the matrix.}
#'
#' The list may contain additional elements (dendrograms, colors, etc).
#'
#' @author Michael Hahsler
#' @keywords hplot
#' @examples
#' data("Wood")
#'
#' # regular heatmap from package stats
#' heatmap(Wood, main = "Wood (standard heatmap)")
#'
#' # Default heatmap does Euclidean distance, hierarchical clustering with
#' # average-link and optimal leaf ordering
#' hmap(Wood, main = "Wood (opt. leaf ordering)")
#'
#' # Heatmap shown without dendrograms (used pimage)
#' hmap(Wood, main = "Wood (opt. leaf ordering)",  showDend = FALSE)
#'
#' # Heatmap with correlation-based distance, green-red color (greenred is
#' # predefined) and optimal leaf ordering and no row label
#' dist_cor <- function(x) as.dist(1 - cor(t(x)))
#' hmap(Wood, distfun = dist_cor, col = greenred(100), row_labels = FALSE,
#'   main = "Wood (reorded by corr. between obs.)")
#'
#' # Heatmap with order based on the angle in two-dimensional MDS space.
#' hmap(Wood, method = "MDS_angle", col = greenred(100), row_labels = FALSE,
#'   main = "Wood (reorderd using ange in MDS space)")
#'
#' # Heatmap for distances
#' d <- dist(Wood)
#' hmap(d, method = "OLO", main = "Wood (Euclidean distances)")
#'
#' # order-based with dissimilarity matrices
#' hmap(Wood, method = "MDS_angle", showdist = "both",
#'   col = greenred(100), col_dist = greens(100),
#'   keylab = "norm. Expression", main = "Wood (reporderd with distances)")
#'
#' # Manually seriate and plot as pimage.
#' o <- seriate(Wood, method = "heatmap", control = list(dist_fun = dist, seriation_method = "OLO"))
#' o
#'
#' pimage(Wood, o, prop = FALSE)
#'
#' # Note: method heatmap calculates reorderd hclust objects which can be used for many heatmap
#' # implementations.
#' heatmap(Wood, Rowv = as.dendrogram(o[[1]]), Colv = as.dendrogram(o[[2]]))
#'
#' # ggplot 2 version does not support dendrograms in the margin (for now)
#' if (require("ggplot2")) {
#'   library("ggplot2")
#'
#'   gghmap(Wood) + labs(title = "Wood", subtitle = "Optimal leaf ordering")
#'
#'   # More parameters (see ? ggpimage): reverse column order and flip axes, make a proportional plot
#'   gghmap(Wood, reverse_columns = TRUE) +
#'     labs(title = "Wood", subtitle = "Optimal leaf ordering")
#'
#'   gghmap(Wood, flip_axes = TRUE) +
#'     labs(title = "Wood", subtitle = "Optimal leaf ordering")
#'
#'   gghmap(Wood, flip_axes = TRUE, prop = TRUE) +
#'     labs(title = "Wood", subtitle = "Optimal leaf ordering")
#'
#'   gghmap(Wood, distfun = dist_cor) +
#'     labs(title = "Wood", subtitle = "Reorded by correlation between observations") +
#'     scale_fill_gradient2(low = "darkgreen", high = "red")
#'
#'   gghmap(d, prop = TRUE) +
#'     labs(title = "Wood", subtitle = "Euclidean distances, reordered)")
#'
#'   # Note: the ggplot2-based version cannot show distance matrices in the same plot.
#'
#'   # Manually seriate and plot as pimage.
#'   o <- seriate(Wood, method = "heatmap", control = list(dist_fun = dist,
#'     seriation_method = "OLO"))
#'   o
#'
#'   ggpimage(Wood, o, prop = FALSE)
#' }
#' @export
hmap <- function(x,
  distfun = stats::dist,
  method = "OLO",
  control = NULL,
  scale = c("none", "row", "column"),
  showDend = TRUE,
  col = NULL,
  row_labels = NULL,
  col_labels = NULL,
  ...) {
  scale <- match.arg(scale)

  if (is.null(col)) {
    if (any(x < 0, na.rm = TRUE))
      col <- .diverge_pal()
    else
      col <- .sequential_pal()
  }

  # dist or matrix?
  if (inherits(x, "dist")) {
    dist_row <- dist_col <- x
    o_col <- o_row <- seriate(x,
      method = method, control = control)[[1]]
    x <- as.matrix(x)

    # dist uses reversed colors!
    col <- rev(col)
  } else {
    if (!is.matrix(x))
      x <- as.matrix(x)

    if (!is.null(scale)) {
      if (scale == "row")
        x <- t(scale(t(x)))
      if (scale == "col")
        x <- scale(x)
    }

    dist_row <- distfun(x)
    o_row <- seriate(dist_row,
      method = method, control = control)[[1]]

    #o_row <- ser_align(list(ser_permutation_vector(order(rowMeans(x, na.rm = TRUE), decreasing = TRUE)), o_row))[[2]]

    dist_col <- distfun(t(x))
    o_col <- seriate(dist_col,
      method = method, control = control)[[1]]

    #o_col <- ser_align(list(ser_permutation_vector(order(colMeans(x, na.rm = TRUE), decreasing = FALSE)), o_col))[[2]]
  }


  # is hierarchical? then let's do a heatmap from stats
  if (inherits(o_col, "hclust") && showDend) {
    # heatmap by default scales rows: we don't want that!
    # options are ignored for now: we use ...

    stats::heatmap(
      x,
      Rowv = stats::as.dendrogram(o_row),
      Colv = stats::as.dendrogram(o_col),
      scale = "none",
      col = col,
      labRow = row_labels,
      labCol = col_labels,
      ...
    )

  } else {
    ### we plot seriated distance matrices
    .hmap_dist(x, method, dist_row, dist_col, o_row, o_col, col = col,
      row_labels = row_labels, col_labels = col_labels, ...)
  }

  ## return permutation indices
  return(invisible(list(
    rowInd = o_row,
    colInd = o_col,
    seriation_method = method
  )))

}

## grid-based dissimilarity plot with seriation
.hmap_dist <-
  function(x,
    method,
    dist_row,
    dist_col,
    o_row,
    o_col,
    ...) {

    ## options
    options <- list(...)
    options <- .get_parameters(
      options,
      list(
        col       = if (any(x < 0))
          .diverge_pal()
        else
          .sequential_pal(),
        col_dist  = .sequential_pal(),
        prop      = FALSE,
        main      = NULL,
        key       = TRUE,
        keylab   = "",
        row_labels    = NULL,
        col_labels    = NULL,
        showdist  = "none",
        symm      = FALSE,
        margins   = NULL,
        zlim      = if (any(x < 0, na.rm = TRUE))
          max(abs(range(x, na.rm = TRUE))) * c(-1, 1)
        else
          range(x, na.rm = TRUE),
        newpage   = TRUE,
        gp        = gpar()
      )
    )

    options$col_dist <- rev(options$col_dist)
    .showdist_options <- c("none", "row", "column", "both")
    options$showdist <- .showdist_options[pmatch(options$showdist,
      .showdist_options)]
    if (is.na(options$showdist))
      stop("Unknown value for showdist. Use one of: ",
        paste(dQuote(.showdist_options), collapse = ", "))

    ## if symmetric then we only use o_row and dist_row
    if (length(o_row) == length(o_col) && options$symm == TRUE) {
      o_col <- o_row
      dist_col <- dist_row
    }

    x <- permute(x, ser_permutation(o_row, o_col))

    if (options$showdist == "none") {
      pimage(
        x,
        col = options$col,
        main = options$main,
        zlim = options$zlim,
        row_labels = options$row_labels,
        col_labels = options$col_labels,
        prop = options$prop,
        key = options$key,
        newpage = options$newpage,
        gp = options$gp
      )
      return()
    }

    dist_row <- permute(dist_row, o_row)
    dist_col <- permute(dist_col, o_col)

    # deal with row/col labels
    row_labels <- options$row_labels
    col_labels <- options$col_labels
    if (!is.null(row_labels) && !is.logical(row_labels)) {
      if (length(row_labels) != nrow(x))
        stop("Length of row_labels does not match the number of rows of x.")
      rownames(x) <- row_labels
      row_labels <- TRUE
    }

    if (!is.null(col_labels) && !is.logical(col_labels)) {
      if (length(col_labels) != ncol(x))
        stop("Length of col_labels does not match the number of columns of x.")
      colnames(x) <- col_labels
      col_labels <- TRUE
    }

    if (is.null(row_labels))
      if (!is.null(rownames(x)) &&
          nrow(x) < 25) {
        row_labels <- TRUE
      } else{
        row_labels <- FALSE
      }
    if (is.null(col_labels))
      if (!is.null(colnames(x)) &&
          ncol(x) < 25) {
        col_labels <- TRUE
      } else{
        col_labels <- FALSE
      }

    if (is.null(rownames(x)))
      rownames(x) <- seq(nrow(x))
    if (is.null(colnames(x)))
      colnames(x) <- seq(ncol(x))

    ## Note: we need a list to store units!
    if (is.null(options$margins)) {
      options$margins <- list(unit(1, "lines"), unit(1, "lines"))
      if (col_labels)
        options$margins[[1]] <-
          max(stringWidth(colnames(x))) + unit(2, "lines")
      if (row_labels)
        options$margins[[2]] <-
          max(stringWidth(rownames(x))) + unit(2, "lines")
      all_names <-
        c("", if (col_labels)
          colnames(x), if (row_labels)
            rownames(x))
      options$margins[[3]] <-
        max(stringWidth(all_names)) + unit(2, "lines")
    } else
      options$margins <-
      list(
        unit(options$margins[1], "lines"),
        unit(options$margins[2], "lines"),
        unit(max(options$margins), "lines")
      )


    ## plot
    if (options$newpage)
      grid.newpage()

    ## surrounding viewport
    pushViewport(viewport(
      layout = grid.layout(
        nrow = 3 ,
        ncol = 3,
        widths = unit.c(
          unit(1, "lines"),
          unit(1, "snpc") - options$margins[[3]] - unit(3, "lines"),
          options$margins[[2]]
        ),
        heights = unit.c(
          unit(3, "lines"),
          # main
          unit(1, "snpc") - options$margins[[3]] - unit(3, "lines"),
          options$margins[[1]]
        )
      ),
      width = unit(1, "snpc"),
      height = unit(1, "snpc"),
      gp = options$gp
    ))


    ## main title
    if (!is.null(options$main)) {
      pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 2))
      grid.text(options$main, gp = gpar(cex = 1.3))
      upViewport(1)
    }


    ## plots
    if (options$prop) {
      widths <- unit.c(
        unit(1 - ncol(x) / sum(ncol(x), nrow(x)), "npc") - unit(.25, "lines"),
        unit(.5, "lines"),
        unit(ncol(x) / sum(ncol(x), nrow(x)), "npc") - unit(.25, "lines")
      )

      heights <- unit.c(
        unit(1 - nrow(x) / sum(ncol(x), nrow(x)), "npc") - unit(.25, "lines"),
        unit(.5, "lines"),
        #space
        unit(nrow(x) / sum(ncol(x), nrow(x)), "npc") - unit(.25, "lines")
      )
    } else{
      heights <- widths <- unit.c(unit(1, "null"),
        unit(.5, "lines"),   # space
        unit(1, "null"))
    }

    pushViewport(
      viewport(
        layout = grid.layout(
          nrow = 3,
          ncol = 3,
          widths = widths,
          heights = heights
        ),
        width = unit(1, "snpc"),
        height = unit(1, "snpc"),
        layout.pos.row = 2,
        layout.pos.col = 2
      )
    )

    # data
    pushViewport(viewport(layout.pos.row = 3, layout.pos.col = 3))

    .grid_image(x,
      col = options$col,
      gp = options$gp,
      zlim = options$zlim)

    downViewport("image")
    if (col_labels)
      grid.text(
        colnames(x),
        y = unit(-1, "lines"),
        x = unit(1:ncol(x), "native"),
        rot = 90,
        just = "right"
      ) # , gp=options$gp)
    if (row_labels)
      grid.text(
        rownames(x),
        x = unit(1, "npc") + unit(1, "lines"),
        y = unit(1:nrow(x), "native"),
        just = "left"
      ) #, gp=options$gp)
    popViewport(1)

    popViewport(1)

    # rows
    if (options$showdist %in% c("row", "both")) {
      pushViewport(viewport(layout.pos.row = 3, layout.pos.col = 1))
      .grid_image(as.matrix(dist_row),
        col = options$col_dist,
        gp = options$gp)
      popViewport(1)
    }

    # cols
    if (options$showdist %in% c("column", "both")) {
      pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 3))
      .grid_image(as.matrix(dist_col),
        col = options$col_dist,
        gp = options$gp)
      popViewport(1)
    }

    # colorkey
    if (options$key) {
      pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 1))

      pushViewport(viewport(
        width = unit(0.5, "npc"),
        height = unit(1, "lines")
      ))
      .grid_colorkey(
        options$zlim,
        col = options$col,
        lab = options$keylab,
        gp = options$gp
      )
      popViewport(2)
    }

    popViewport(2)

  }

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

## ser_permutation_vector represents a single permutation represented as an
## integer vector or a hclust object.

## Constructor
## x can be
##  * an integer vector
##  * a hclust or dendrogram object (leaf order)
##  * NA represents the identity permutation
##  * a ser_permutation (list) of length 1

#' Class ser_permutation_vector -- A Single Permutation Vector for Seriation
#'
#' The class `ser_permutation_vector`
#' represents a single permutation vector.
#'
#' A permutation vector
#' maps a set of \eqn{n} objects \eqn{\{O_1, O_2, ..., O_n\}}{{O_1, O_2, ..., O_n}} onto itself.
#'
#' __Ordering Representation:__
#' In \pkg{seriation} we represent a permutation \eqn{\pi}{\pi}
#' as a vector which lists the objects' indices in their permuted order. This can
#' be seen as replacing the object in position \eqn{i} with the object
#' in position \eqn{\pi(i)}.
#' For example, the permutation vector \eqn{\langle3, 1, 2\rangle}{<3, 1, 2>} indicates that in
#' first position is the object with index 3 then the object with index 1 and finally
#' the object with index 2. This representation is often called a (re)arrangement or ordering.
#' The ordering can be extracted from a permutation vector object
#' via [get_order()]. Such an ordering can be directly used
#' to subset the list of original objects with `"["` to apply the permutation.
#'
#' __Rank Representation:__
#' An alternative way to specify a permutation is via a list of the ranks
#' of the objects after permutation. This representation is often called
#' a map or substitution. Ranks can be extracted from a permutation vector using [get_rank()].
#'
#' __Permutation Matrix:__
#' Another popular representation is a permutation matrix which performs
#' permutations using matrix multiplication. A permutation matrix can be obtained
#' using [get_permutation_matrix()].
#'
#' `ser_permutation_vector` objects are usually packed into
#' a [ser_permutation] object
#' which is a collection (a `list`) of \eqn{k} permutation vectors for \eqn{k}-mode data.
#'
#' The constructor `ser_permutation_vector()`
#' checks if the permutation vector is valid
#' (i.e. if all integers occur exactly once).
#'
#' @family permutation
#'
#' @param x,object an object which contains a permutation vector (currently an
#'     integer vector or an object of class [hclust]). The value `NA`
#'     creates an identity permutation.
#' @param method a string representing the method used to obtain the
#'     permutation vector.
#' @param ... further arguments.
#'
#' @returns  An object of class `ser_permutation_vector`.
#' @author Michael Hahsler
#'
#' @examples
#' p <- ser_permutation_vector(sample(10), "random")
#' p
#'
#' ## some methods
#' length(p)
#' get_method(p)
#' get_order(p)
#' get_rank(p)
#' get_permutation_matrix(p)
#'
#' r <- rev(p)
#' r
#' get_order(r)
#'
#' ## create a identity permutation vector (with unknown length)
#' ip <- ser_permutation_vector(NA)
#' ip
#'
#' @keywords classes
#' @export
ser_permutation_vector <- function(x, method = NULL) {
  if (inherits(x, "ser_permutation_vector"))
    return(x)

  if (inherits(x, "hclust") || inherits(x, "dendrogram")) {
    # nothing to do
  } else if (length(x) == 1 && is.na(x)) {
    x <- NA_integer_
    attr(x, "method") <- "identity permutation"
  } else if (is.numeric(x)) {
    nm <- names(x)
    x <- as.integer(x)
    names(x) <- nm
  } else if (inherits(x, "ser_permutation") && length(x) == 1) {
    x <- x[[1]]
  } else {
    stop("x does not contain a supported permutation.")
  }

  if (!is.null(method))
    attr(x, "method") <- method

  class(x) <- c("ser_permutation_vector", class(x))
  .valid_permutation_vector(x)
  x
}

#' @rdname ser_permutation_vector
#' @param recursive ignored
#' @export
c.ser_permutation_vector <- function(..., recursive = FALSE)
  do.call("ser_permutation", list(...))




## reverse
#' @rdname ser_permutation_vector
#' @export
rev.ser_permutation_vector <- function(x) {
  if (inherits(x, "hclust")) {
    x$order <- rev(x$order)
    x
  }
  else
    ser_permutation_vector(rev(get_order(x)), method = get_method(x))
}


## currently method is an attribute of permutation
#' @rdname ser_permutation_vector
#' @param printable a logical; prints "unknown" instead of `NULL` for non-existing methods.
#' @export
get_method <- function(x, printable = FALSE) {
  method <- attr(x, "method")

  if (printable && is.null(method))
    method <- "unknown"
  method
}


## print et al
#' @rdname ser_permutation_vector
#' @export
length.ser_permutation_vector <- function(x) {
  if (!.is_identity_permutation(x))
    length(get_order(x))
  else
    0L
}

#' @rdname ser_permutation_vector
#' @export
print.ser_permutation_vector <-
  function(x, ...)
  {
    writeLines(c(
      gettextf("object of class %s",
        paste(sQuote(class(
          x
        )), collapse = ", ")),
      gettextf("contains a permutation vector of length %d", length(x)),
      gettextf("used seriation method: '%s'",
        get_method(x, printable = TRUE))
    ))
    invisible(x)
  }

## fake summary (we don't really provide a summary,
## but summary produces now a reasonable result --- same as print)
#' @rdname ser_permutation_vector
#' @export
summary.ser_permutation_vector <- function(object, ...) {
  object
}


## helpers

## an identity permutation is a single NA.
.is_identity_permutation <- function(x) is.na(x[1])

## calls stop if the vector is not valid
.valid_permutation_vector <- function(x) {

  ## identity vector is always valid
  if (.is_identity_permutation(x))
    return(invisible(TRUE))

  ## valid permutations have a get_order function implemented
  perm <- get_order(x)
  valid <- TRUE

  tab <- table(perm)
  if (any(tab != 1))
    valid <- FALSE
  if (length(tab) != length(perm)
    || any(names(tab) != sequence(length(perm))))
    valid <- FALSE

  if (!valid)
    stop("Invalid permutation vector!\nVector: ",
      paste(perm, collapse = ", "))

  invisible(valid)
}

.valid_permutation_matrix <- function(x) {
  if (any(rowSums(x) != 1) || any(colSums(x) != 1) ||
      any(x != 1 & x != 0))
    stop("Not a valid permutation matrix")

  invisible(TRUE)
}

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2015 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


#' Register Seriation Based on 1D UMAP
#'
#' Use uniform manifold approximation and projection (UMAP) to embed the data
#' on the number line and create a order for [seriate()].
#'
#' Registers the method \code{"umap"} for [seriate()]. This method applies
#' 1D UMAP to data represented by a distance matrix and extracts the order from
#' the 1D embedding.
#'
#' \bold{Note:} Package \pkg{umap} needs to be installed.
#'
#' @aliases register_umap umap
#' @seealso [umap::umap()] in \pkg{umap}.
#' @family seriation
#' @returns Nothing.
#'
#' @references McInnes, L, Healy, J, UMAP: Uniform Manifold Approximation and
#' Projection for Dimension Reduction, ArXiv e-prints 1802.03426, 2018
#' @keywords optimize cluster
#' @examples
#'
#' \dontrun{
#' register_umap()
#' get_seriation_method("dist", "umap")
#'
#' d <- dist(random.robinson(50, pre=TRUE, noise=.1))
#'
#' o <- seriate(d, method = "umap")
#' pimage(d, o)
#' }
#'
#' @export
register_umap <- function() {
  check_installed("umap")

  .contr <- unclass(umap::umap.defaults)
  .contr$n_components <- 1
  .contr$input <- "dist"

  umap_order <- function(x, control) {
    control <- .get_parameters(control, .contr)

    class(control) <- class(umap::umap.defaults)

    embedding <- umap::umap(as.matrix(x), config = control)
    order(embedding$layout)
  }

  set_seriation_method(
    "dist",
    "umap",
    umap_order,
    "Use 1D Uniform manifold approximation and projection (UMAP) embedding to create an order",
    .contr
  )
}

#' @title `r packageDescription("seriation")$Package`: `r packageDescription("seriation")$Title`
#'
#' @description Infrastructure for ordering objects with an implementation of several seriation/sequencing/ordination techniques to reorder matrices, dissimilarity matrices, and dendrograms. Also provides (optimally) reordered heatmaps, color images and clustering visualizations like dissimilarity plots, and visual assessment of cluster tendency plots (VAT and iVAT).
#'
#' @references Michael Hahsler, Kurt Hornik, and Christian Buchta. Getting things in order: An introduction to the R package seriation. Journal of Statistical Software, 25(3):1--34, March 2008. \doi{10.18637/jss.v025.i03}
#'
#'
#' @section Key functions:
#' - Seriation: [seriate()], [criterion()], [get_order()], [permute()]
#' - Visualization: [pimage()], [bertinplot()], [hmap()], [dissplot()], [VAT()]
#'
#' @author Michael Hahsler
#' @docType package
#' @name seriation-package
#'
#' @importFrom graphics plot text title
#' @importFrom ca ca
#' @importFrom stats reorder as.dist hclust runif rnorm dist order.dendrogram prcomp
#' @useDynLib seriation, .registration=TRUE
NULL

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

## image method that makes a proper image plot of a matrix.
## the rows and columns are swapped and the order of the
## columns (original rows) is reversed.

#' @rdname pimage
#' @include pimage.R
#' @export
ggpimage <- function(x,
  order = NULL,
  zlim = NULL,
  upper_tri = TRUE,
  lower_tri = TRUE,
  diag = TRUE,
  row_labels = NULL,
  col_labels = NULL,
  prop = TRUE,
  flip_axes = FALSE,
  reverse_columns = FALSE)
  UseMethod("ggpimage")


### Note for matrix large values are dark, for dist large values are light!
#' @rdname pimage
#' @export
ggpimage.matrix <- function(x,
  order = NULL,
  zlim = NULL,
  upper_tri = TRUE,
  lower_tri = TRUE,
  diag = TRUE,
  row_labels = NULL,
  col_labels = NULL,
  prop = TRUE,
  flip_axes = FALSE,
  reverse_columns = FALSE)
{
  check_installed("ggplot2")

  x <- as.matrix(x)

  # check data
  if (all(is.na(x)))
    stop("all data missing in x.")
  if (any(is.infinite(x)))
    stop("x contains infinite entries.")

  # reorder
  if (!is.null(order))
    x <- permute(x, order)

  # mask triangles
  if (any(!upper_tri ||
      !lower_tri ||
      !diag) &&
      nrow(x) != ncol(x))
    stop("Upper triangle, lower triangle or diag can only be suppressed for square matrices!")
  if (!upper_tri)
    x[upper.tri(x)] <- NA
  if (!lower_tri)
    x[lower.tri(x)] <- NA
  if (!diag)
    diag(x) <- NA


  # reverse order of columns
  if (reverse_columns)
    x <- x[, seq(ncol(x), 1)]

  # change x and y?
  if (flip_axes) {
    x <- t(x)
    tmp <- row_labels
    row_labels <- col_labels
    col_labels <- tmp
  }


  # plot
  g <-
    .ggpimage_empty(
      x,
      zlim = zlim,
      row_labels = row_labels,
      col_labels = col_labels,
      prop = prop,
      expand = FALSE
    )

  g <- g + ggplot2::geom_raster(ggplot2::aes(fill = x))
  g
}

ggpimage.default <- ggpimage.matrix

## small values are dark
#' @rdname pimage
#' @export
ggpimage.dist <-
  function(x,
    order = NULL,
    zlim = NULL,
    upper_tri = FALSE,
    lower_tri = TRUE,
    diag = FALSE,
    row_labels = NULL,
    col_labels = NULL,
    prop = TRUE,
    flip_axes = FALSE,
    reverse_columns = FALSE) {
    check_installed("ggplot2")

    # reorder specific for dist (we have only a single permutation)
    if (!is.null(order))
      x <- permute(x, order)

    if (flip_axes)
      warning("flipping axes has no effect for distance matrices.")

    g <- ggpimage.matrix(
      as.matrix(x),
      order = NULL,
      zlim = zlim,
      upper_tri,
      lower_tri,
      diag,
      row_labels,
      col_labels,
      prop = prop,
      flip_axes = FALSE,
      reverse_columns = reverse_columns
    )

    # reverse color for dist
    suppressMessages(g <-
        g + .gg_sequential_pal(dist = TRUE, limits = zlim))

    g
  }

### Note for matrix large values are dark, for dist large values are light!
.ggpimage_empty <- function(x,
  zlim = NULL,
  row_labels = NULL,
  col_labels = NULL,
  prop = TRUE,
  expand = TRUE) {
  check_installed("ggplot2")

  x <- as.matrix(x)

  # check data
  if (all(is.na(x)))
    stop("all data missing in x.")
  if (any(is.infinite(x)))
    stop("x contains infinite entries.")

  # deal with row/col labels
  if (!is.null(row_labels) && !is.logical(row_labels)) {
    if (length(row_labels) != nrow(x))
      stop("Length of row_labels does not match the number of rows of x.")
    rownames(x) <- row_labels
    row_labels <- TRUE
  }

  if (!is.null(col_labels) && !is.logical(col_labels)) {
    if (length(col_labels) != ncol(x))
      stop("Length of col_labels does not match the number of columns of x.")
    colnames(x) <- col_labels
    col_labels <- TRUE
  }

  if (is.null(row_labels))
    if (!is.null(rownames(x)) &&
        nrow(x) < 25) {
      row_labels <- TRUE
    } else{
      row_labels <- FALSE
    }

  if (is.null(col_labels))
    if (!is.null(colnames(x)) &&
        ncol(x) < 25) {
      col_labels <- TRUE
    } else{
      col_labels <- FALSE
    }

  if (is.null(rownames(x)))
    rownames(x) <- seq(nrow(x))
  if (is.null(colnames(x)))
    colnames(x) <- seq(ncol(x))

  # convert to data.frame with row, col and x
  x_df <- data.frame(
    row = factor(rep(seq(nrow(
      x
    )), times = ncol(x)), levels = seq(nrow(x), 1)),
    col = factor(rep(seq(ncol(
      x
    )), each = nrow(x)), levels = seq(ncol(x))),
    x = as.vector(x)
  )

  if (!is.null(rownames(x)))
    levels(x_df[["row"]]) <- rev(rownames(x))
  if (!is.null(colnames(x)))
    levels(x_df[["col"]]) <- colnames(x)

  # plot
  g <- ggplot2::ggplot(x_df,
    ggplot2::aes(y = row, x = col))

  # axes (row and col labels)
  if (expand)
    expand <- ggplot2::waiver()
  else
    expand <- c(0, 0)

  if (col_labels)
    breaksCol <- ggplot2::waiver()
  else
    breaksCol <- NULL
  if (row_labels)
    breaksRow <- ggplot2::waiver()
  else
    breaksRow <- NULL

  g <- g +
    ggplot2::scale_x_discrete(breaks = breaksCol, expand = expand) +
    ggplot2::scale_y_discrete(breaks = breaksRow, expand = expand)

  # no axis or legend labels
  g <- g +
    ggplot2::labs(x = NULL, y = NULL, fill = NULL)

  g <-
    g + ggplot2::theme(axis.text.x = ggplot2::element_text(
      angle = 90,
      hjust = 1,
      vjust = .5
    ))

  if (prop)
    g <- g + ggplot2::theme(aspect.ratio = nrow(x) / ncol(x))

  # colors scalesi
  if (is.logical(x)) {
    g <-
      g + .gg_logical_pal()

    # colors for diverging
  } else if (any(x < 0, na.rm = TRUE) && any(x > 0, na.rm = TRUE)) {
    g <-
      g + .gg_diverge_pal(limits = zlim)

  } else {
    g <-
      g + .gg_sequential_pal(limits = zlim)
  }

  g
}

#' Bertin's Characteristics of Townships
#'
#' This data contains nine characteristics for 16 townships. The data
#' set was used by Bertin (1981) to illustrate that the conciseness
#' of presentation can be improved by seriating the rows and columns.
#'
#' @name Townships
#' @aliases Townships
#' @docType data
#' @format
#'   A matrix with 16 0-1 variables (columns) indicating the presence
#'   (`1`) or absence (`0`) of characteristics of townships
#'   (rows).
#' @references
#' Bertin, J. (1981): _Graphics and Graphic Information Processing_. Berlin, Walter de Gruyter.
#' @author Michael Hahsler
#' @examples
#' data("Townships")
#'
#' ## original data
#' pimage(Townships)
#' criterion(Townships)
#'
#' ## seriated data
#' order <- seriate(Townships, method = "BEA", control = list(rep = 5))
#' pimage(Townships, order)
#' criterion(Townships, order)
#' @keywords datasets
NULL

#' Conversion Between Permutation Vector and Permutation Matrix
#'
#' Converts between permutation vectors and matrices.
#'
#' @family permutation
#'
#' @param x A permutation vector (any object that can be converted into a
#' permutation vector, e.g., a integer vector or a `hclust` object) or a
#' matrix representing a permutation. Arguments are checked.
#' @returns
#' - `permutation_vector2matrix()`: returns a permutation matrix.
#' - `permutation_matrix2vector()`: returns the permutation as a integer vector.
#'
#' @author Michael Hahsler
#' @keywords manip
#' @examples
#' ## create a random permutation vector
#' pv <- sample(1:5)
#' pv
#'
#' ## convert into a permutation matrix
#' pm <- permutation_vector2matrix(pv)
#' pm
#'
#' ## convert back
#' permutation_matrix2vector(pm)
#' @export
permutation_vector2matrix <- function(x) {
  x <- get_order(x)
  .valid_permutation_vector(x)

  n <- length(x)
  pm <- matrix(0, nrow = n, ncol = n)
  for (i in 1:n)
    pm[i, x[i]] <- 1
  pm
}

#' @rdname permutation_vector2matrix
#' @export
permutation_matrix2vector <- function(x) {
  .valid_permutation_matrix(x)
  o <- apply(
    x,
    MARGIN = 1,
    FUN = function(r)
      which(r == 1)
  )
  o
}

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' Visual Analysis for Cluster Tendency Assessment (VAT/iVAT)
#'
#' Implements Visual Analysis for Cluster Tendency Assessment (VAT; Bezdek and
#' Hathaway, 2002) and Improved Visual Analysis for Cluster Tendency Assessment
#' (iVAT; Wang et al, 2010).
#'
#' `path_dist()` redefines the distance between two objects as the minimum
#' over the largest distances in all possible paths between the objects as used
#' for iVAT.
#'
#' @family plots
#'
#' @param x a \code{dist} object.
#' @param upper_tri,lower_tri a logical indicating whether to show the upper or
#' lower triangle of the VAT matrix.
#' @param ... further arguments are passed on to \code{\link{pimage}} for the
#' regular plots and \code{\link{ggpimage}} for the ggplot2 plots.
#' @returns Nothing.
#'
#' @author Michael Hahsler
#' @references Bezdek, J.C. and Hathaway, R.J. (2002): VAT: a tool for visual
#' assessment of (cluster) tendency. \emph{Proceedings of the 2002
#' International Joint Conference on Neural Networks (IJCNN '02)}, Volume: 3,
#' 2225--2230.
#'
#' Havens, T.C. and Bezdek, J.C. (2012): An Efficient Formulation of the
#' Improved Visual Assessment of Cluster Tendency (iVAT) Algorithm, \emph{IEEE
#' Transactions on Knowledge and Data Engineering,} \bold{24}(5), 813--822.
#'
#' Wang L., U.T.V. Nguyen, J.C. Bezdek, C.A. Leckie and K. Ramamohanarao
#' (2010): iVAT and aVAT: Enhanced Visual Analysis for Cluster Tendency
#' Assessment, \emph{Proceedings of the PAKDD 2010, Part I, LNAI 6118,} 16--27.
#' @keywords cluster manip
#' @examples
#' ## lines data set from Havens and Bezdek (2011)
#' x <- create_lines_data(250)
#' plot(x, xlim=c(-5,5), ylim=c(-3,3), cex=.2)
#' d <- dist(x)
#'
#' ## create regular VAT
#' VAT(d, main = "VAT for Lines")
#' ## same as: pimage(d, seriate(d, "VAT"))
#'
#' ## ggplot2 version
#' if (require("ggplot2")) {
#'   ggVAT(d) + labs(title = "VAT")
#' }
#'
#' ## create iVAT which shows visually the three lines
#' iVAT(d, main = "iVAT for Lines")
#' ## same as:
#' ## d_path <- path_dist(d)
#' ## pimage(d_path, seriate(d_path, "VAT for Lines"))
#'
#' ## ggplot2 version
#' if (require("ggplot2")) {
#'   ggiVAT(d) + labs(title = "iVAT for Lines")
#' }
#'
#' ## compare with dissplot (shows banded structures and relationship between
#' ## center line and the two outer lines)
#' dissplot(d, method = "OLO_single", main = "Dissplot for Lines", col = bluered(100, bias = .5))
#'
#' ## compare with optimally reordered heatmap
#' hmap(d, method = "OLO_single", main = "Heatmap for Lines (opt. leaf ordering)",
#'   col = bluered(100, bias = .5))
#' @export
VAT <- function(x,
  upper_tri = TRUE,
  lower_tri = TRUE,
  ...) {
  if (!inherits(x, "dist"))
    stop("x needs to be of class 'dist'!")
  pimage(x,
    seriate(x, "VAT"),
    upper_tri = upper_tri,
    lower_tri = lower_tri,
    ...)
}

#' @rdname VAT
#' @export
iVAT <- function(x,
  upper_tri = TRUE,
  lower_tri = TRUE,
  ...) {
  if (!inherits(x, "dist"))
    stop("x needs to be of class 'dist'!")
  x <- path_dist(x)
  pimage(x,
    seriate(x, "VAT"),
    upper_tri = upper_tri,
    lower_tri = lower_tri,
    ...)
}


## calculate path distance from iVAT using a modified version fo Floyd's alg.
## d_ij = smallest value of the largest values of all possible paths between i and j

#' @rdname VAT
#' @export
path_dist <- function(x) {
  #A <- as.matrix(x)
  #n <- nrow(A)
  #for(k in 1:n)
  # for(i in 1:n)
  #   for(j in 1:n)
  #      if(max(A[i,k], A[k,j]) < A[i,j]) A[i,j] <- max(A[i,k], A[k,j])
  #d <- as.dist(A)

  ## make C call
  m <- as.matrix(x)

  if (any(is.na(m)))
    stop("NAs not allowed in x.")
  if (any(m < 0))
    stop("Negative values not allowed in x.")
  mode(m) <- "double"

  ## replace Inf with large number
  m[is.infinite(m)] <- .Machine$double.xmax

  if (any(m < 0))
    stop("Negative values not allowed in x.")

  m <- .Call("pathdist_floyd", m, PACKAGE = "seriation")
  as.dist(m)
}

#' Zoo Data Set
#'
#' A database containing characteristics of different animals. The database was
#' created and donated by Richard S. Forsyth and is available from the UCI
#' Machine Learning Repository (Newman et al, 1998).
#'
#'
#' @name Zoo
#' @docType data
#' @format
#'   A data frame with 101 observations on the following 17 variables.
#' \describe{
#'   \item{\code{hair}}{a numeric vector}
#'   \item{\code{feathers}}{a numeric vector}
#'   \item{\code{eggs}}{a numeric vector}
#'   \item{\code{milk}}{a numeric vector}
#'   \item{\code{airborne}}{a numeric vector}
#'   \item{\code{aquatic}}{a numeric vector}
#'   \item{\code{predator}}{a numeric vector}
#'   \item{\code{toothed}}{a numeric vector}
#'   \item{\code{backbone}}{a numeric vector}
#'   \item{\code{breathes}}{a numeric vector}
#'   \item{\code{venomous}}{a numeric vector}
#'   \item{\code{fins}}{a numeric vector}
#'   \item{\code{legs}}{a numeric vector}
#'   \item{\code{tail}}{a numeric vector}
#'   \item{\code{domestic}}{a numeric vector}
#'   \item{\code{catsize}}{a numeric vector}
#'   \item{\code{class}}{a factor with levels \code{amphibian} \code{bird} \code{fish} \code{insect} \code{invertebrate} \code{mammal} \code{reptile}}
#' }
#' @source D.J. Newman, S. Hettich, C.L. Blake and C.J. Merz (1998): UCI
#' Repository of machine learning databases,
#' \url{https://www.ics.uci.edu/~mlearn/MLRepository.html}, University of
#' California, Irvine, Dept. of Information and Computer Sciences.
#' @keywords datasets
#' @examples
#'
#' data("Zoo")
#' x <- scale(Zoo[, -17])
#'
#'
#' d <- dist(x)
#' pimage(d)
#'
#' order <- seriate(d, method = "tsp")
#' pimage(d, order)
#'
NULL

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' @rdname bertinplot
#' @export
ggbertinplot <- function(x,
  order = NULL,
  geom = "bar",
  highlight = TRUE,
  row_labels = TRUE,
  col_labels = TRUE,
  flip_axes = TRUE,
  prop = FALSE,
  ...) {
  check_installed("ggplot2")

  if (!is.matrix(x))
    stop("Argument 'x' must be a matrix.")

  geom <-
    match.arg(tolower(geom),
      choices = c("tile", "rectangle", "circle", "line", "bar", "none"))

  # reorder
  if (!is.null(order))
    x <- permute(x, order)

  # change x and y?
  if (flip_axes) {
    x <- t(x)
    tmp <- row_labels
    row_labels <- col_labels
    col_labels <- tmp
  }

  if (is.logical(highlight) && highlight)
    highlight <- mean(x, na.rm = TRUE)

  g <-
    .ggpimage_empty(
      x,
      row_labels = row_labels,
      col_labels = col_labels,
      prop = prop,
      expand = geom != "raster"
    )

  if (col_labels)
    breaksCol <- ggplot2::waiver()
  else
    breaksCol <- NULL
  if (row_labels)
    breaksRow <- ggplot2::waiver()
  else
    breaksRow <- NULL

  # put col labels on top (message about replacing scale for x)
  suppressMessages(
    g <- g +
      ggplot2::scale_x_discrete(
        breaks = breaksRow,
        position = "top",
        expand = if (geom != "raster")
          ggplot2::waiver()
        else
          c(0, 0)
      ) +
      ggplot2::scale_y_discrete(
        breaks = breaksCol,
        position = "right",
        expand = if (geom != "raster")
          ggplot2::waiver()
        else
          c(0, 0)
      ) +
      ggplot2::theme(axis.text.x = ggplot2::element_text(hjust = 0, vjust = .5)) +
      ggplot2::theme(legend.position = "bottom")
  )

  # add geom

  # raster does not use highlight
  if (geom == "tile")
    g <- g + ggplot2::geom_raster(ggplot2::aes(fill = x))

  if (geom == "circle")
    if (highlight) {
      suppressMessages(
        g <-
          g + ggplot2::geom_point(
            ggplot2::aes(size = x,
              fill = x > highlight),
            color = "black",
            pch = 21
          ) + .gg_logical_pal() +
          ggplot2::guides(fill = "none", size = "none")
      )
    } else{
      g <- g + ggplot2::geom_point(ggplot2::aes(size = x))
    }

  if (geom == "rectangle")
    if (highlight) {
      suppressMessages(
        g <-
          g + ggplot2::geom_tile(
            ggplot2::aes(
              x = col,
              y = row,
              height = x / max(x, na.rm = TRUE) * .8,
              width = x / max(x, na.rm = TRUE) * .8,
              fill = x > highlight
            ),
            color = "black"
          ) +
          .gg_logical_pal() +
          ggplot2::guides(fill = "none")
      )
    } else{
      g <- g +
        ggplot2::geom_tile(ggplot2::aes(height = x / max(x) * .9), width = .8)
    }

  # TODO: do not display facet labels when row_labels == FALSE

  # no highlight for line
  if (geom == "line")
    g <- g +
    ggplot2::geom_line(ggplot2::aes(x = col, y = x, group = row)) +
    # Note: facets display the lowest level first so we need to reverse them
    ggplot2::facet_grid(rows = ggplot2::vars(stats::reorder(row, rev(as.integer(
      row
    ))))) +
    ggplot2::theme(
      strip.text.y.right = ggplot2::element_text(angle = 0, color = "black"),
      strip.background = ggplot2::element_blank()
    )

  if (geom == "bar")
    if (highlight) {
      suppressMessages(
        g <- g +
          ggplot2::geom_bar(
            ggplot2::aes(
              x = col,
              y = x,
              group = row,
              fill = x > highlight
            ),
            stat = "identity",
            color = "black",
            width = .8
          ) +
          # Note: facets display the lowest level first so we need to reverse them
          ggplot2::facet_grid(rows = ggplot2::vars(stats::reorder(
            row, rev(as.integer(row))
          ))) +
          ggplot2::theme(
            strip.text.y.right = ggplot2::element_text(angle = 0, color = "black"),
            strip.background = ggplot2::element_blank()
          ) +
          .gg_logical_pal() +
          ggplot2::guides(fill = "none")
      )
    } else{
      g <- g +
        ggplot2::geom_bar(ggplot2::aes(x = col,
          y = x,
          group = row),
          stat = "identity",
          width = .8) +
        # Note: facets display the lowest level first so we need to reverse them
        ggplot2::facet_grid(rows = ggplot2::vars(stats::reorder(row, rev(
          as.integer(row)
        )))) +
        ggplot2::theme(
          strip.text.y.right = ggplot2::element_text(angle = 0, color = "black"),
          strip.background = ggplot2::element_blank()
        )
    }

  g
}

#' Extracting Order Information from a Permutation Object
#'
#' Method to get the order information from an object of class
#' [ser_permutation] or [ser_permutation_vector]. Order information
#' can be extracted as an integer permutation vector, a vector containing the
#' object ranks or a permutation matrix.
#'
#' `get_order()` returns the seriation as an integer vector containing the
#' order of the objects after permutation. That is, the index of the first,
#' second, \eqn{..., n}-th object. These permutation vectors can directly be
#' used to reorder objects using subsetting with `"["`.  \emph{Note:} In
#' \pkg{seriation} we usually use these order-based permutation vectors.
#'
#' `get_rank()` returns the seriation as an integer vector containing the
#' rank/position for each objects in the permutation. That is, position of the
#' first, second, \eqn{..., n}-th object after permutation.  Note: Use
#' `order()` to convert ranks back to an order.
#'
#' `get_permutation_matrix()` returns a \eqn{n \times n}{n x n} permutation
#' matrix.
#'
#' @family permutation
#'
#' @param x an object of class [ser_permutation] or
#' [ser_permutation_vector].
#' @param dim order information for which dimension should be returned?
#' @param ... further arguments are ignored for `get_order()`.  For
#' `get_rank()` and for `get_permutation_matrix()` the additional
#' arguments are passed on to `get_order()` (e.g., as `dim`).
#' @return Returns an integer permutation vector/a permutation matrix.
#'
#' @author Michael Hahsler
#' @keywords manip
#' @examples
#' ## ser_permutation_vector is a single permutation vector
#' o <- ser_permutation_vector(sample(10))
#' o
#'
#' get_order(o)
#' get_rank(o)
#' get_permutation_matrix(o)
#'
#' ## ser_permutation contains one permutation vector for each dimension
#' o2 <- ser_permutation(o, sample(5))
#' o2
#'
#' get_order(o2, dim = 2)
#' get_rank(o2, dim = 2)
#' get_permutation_matrix(o2, dim = 2)
#' @export
get_order <- function(x, ...)
  UseMethod("get_order")

get_order.default <- function(x, ...)
  stop(gettextf("No permutation accessor implemented for class '%s'. ",
    class(x)))

#' @rdname get_order
#' @export
get_order.ser_permutation_vector <- function(x, ...)
  NextMethod()


#' @rdname get_order
#' @export
get_order.ser_permutation <-
  function(x, dim = 1, ...)
    get_order(x[[dim]])


#' @rdname get_order
#' @export
get_order.hclust <- function(x, ...)
  structure(.Data = x$order, names = x$labels[x$order])


#' @rdname get_order
#' @export
get_order.dendrogram <- function(x, ...)
  order.dendrogram(x)

#' @rdname get_order
#' @export
get_order.integer <- function(x, ...) {
  if (.is_identity_permutation(x))
    stop("Cannot get order vector from symbolic identity permutation (undefined length).")
  structure(as.integer(x), names = names(x))
}


## returns for each object its rank (rank of first, second, etc. object)
#' @rdname get_order
#' @export
get_rank <- function(x, ...)
  order(get_order(x, ...))

#' @rdname get_order
#' @export
get_permutation_matrix <- function(x, ...)
  permutation_vector2matrix(get_order(x, ...))

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' Plot a Bertin Matrix
#'
#' Plot a data matrix of cases and variables. Each value is represented by a
#' symbol. Large values are highlighted. Note that Bertin arranges the cases
#' horizontally and the variables as rows. The matrix can be rearranged using
#' seriation techniques to make structure in the data visible (see Falguerolles
#' et al 1997).
#'
#' The plot is organized as a matrix of symbols. The symbols are drawn by a
#' panel function, where all symbols of a row are drawn by one call of the
#' function (using vectorization). The interface for the panel function is
#' `panel.myfunction(value, spacing, hl)`. `value` is the vector of
#' values for a row scaled between 0 and 1, `spacing` contains the
#' relative space between symbols and `hl` is a logical vector indicating
#' which symbol should be highlighted.
#'
#' Cut lines can be added to an existing Bertin plot using
#' `bertin_cut_line(x = NULL, y = NULL)`. `x`/`y` is can be a
#' number indicating where to draw the cut line between two columns/rows. If
#' both `x` and `y` is specified then one can select a row/column and
#' the other can select a range to draw a line which does only span a part of
#' the row/column. It is important to call `bertinplot()` with the option
#' `pop = FALSE`.
#'
#' `ggbertinplot()` calls [ggpimage()] and all additional parameters are
#' passed on.
#'
#' @family plots
#' @param x a data matrix. Note that following Bertin, columns are variables
#' and rows are cases. This behavior can be reversed using `reverse = TRUE`
#' in `options`.
#' @param order an object of class `ser_permutation` to rearrange `x`
#' before plotting.  If `NULL`, no rearrangement is performed.
#' @param panel.function a function to produce the symbols. Currently available
#' functions are `panel.bars` (default), `panel.circles`,
#' `panel.rectangles`, `panel.tiles` and `panel.lines`. For
#' circles and squares neg. values are represented by a dashed border. For
#' blocks all blocks are the same size (can be used with `shading = TRUE`).
#' @param geom visualization type. Available ggplot2 geometries are: `"tile"`,
#' `"rectangle"`, `"circle"`, `"line"`, `"bar"`, `"none"`.
#' @param highlight a logical scalar indicating whether to use highlighting.
#' If `TRUE`, all variables with values greater than the variable-wise
#' mean are highlighted. To control highlighting, also a logical matrix or a
#' matrix with colors with the same dimensions as `x` can be supplied.
#' @param row_labels,col_labels a logical indicating if row and column labels
#' in `x` should be displayed.  If `NULL` then labels are displayed
#' if the `x` contains the appropriate dimname and the number of labels is
#' 25 or less. A character vector of the appropriate length with labels can
#' also be supplied.
#' @param flip_axes logical indicating whether to swap cases and variables in
#' the plot. The default (`TRUE`) is to plot cases as columns and
#' variables as rows.
#' @param prop logical; change the aspect ratio so cells in the image have a
#' equal width and height.
#' @param col,y and x in `bertin_cut_line()` are for adding a line to a `bertinplot()` (not ggplot2-based).
#' @param value,spacing,hl are used internally for the panel functions.
#' @param ...
#'   `ggbertinplot()`: further parameters are passed on to [ggpimage()].
#'
#'   `bertinplot()`: further parameters can include:
#'  - `xlab, ylab` labels (default: use labels from `x`).
#'  - `spacing` relative space between symbols (default: 0.2).
#'  - `shading` use gray shades to encode value instead of
#'    highlighting (default: `FALSE`).
#'  - `shading.function` a function that accepts a single argument in range \eqn{[.1, .8]}
#'    and returns a valid corresponding color (e.g., using [rgb()]).
#'  - `frame` plot a grid to separate symbols (default: `FALSE`).
#'  - `mar` margins (see [par()]).
#'  - `gp_labels` `gpar` object for labels (see [gpar()])
#'  - `gp_panels` `gpar` object for panels (see [gpar()]).
#'  - `newpage` a logical indicating whether to start
#'     the plot on a new page (see [grid.newpage()]).
#'  -  `pop` a logical indicating whether to pop the created viewports
#'     (see [pop.viewport()])?
#'
#' @returns Nothing.
#'
#' @author Michael Hahsler
#' @references de Falguerolles, A., Friedrich, F., Sawitzki, G. (1997): A
#' Tribute to J. Bertin's Graphical Data Analysis. In: Proceedings of the
#' SoftStat '97 (Advances in Statistical Software 6), 11--20.
#' @keywords hplot cluster
#' @examples
#' data("Irish")
#' scale_by_rank <- function(x) apply(x, 2, rank)
#' x <- scale_by_rank(Irish[,-6])
#'
#' # Use the the sum of absolute rank differences
#' order <- c(
#'   seriate(dist(x, "minkowski", p = 1)),
#'   seriate(dist(t(x), "minkowski", p = 1))
#' )
#'
#' # Plot
#' bertinplot(x, order)
#'
#' # Some alternative displays
#' bertinplot(x, order, panel = panel.tiles, shading_col = bluered(100), highlight = FALSE)
#' bertinplot(x, order, panel = panel.circles, spacing = -.2)
#' bertinplot(x, order, panel = panel.rectangles)
#' bertinplot(x, order, panel = panel.lines)
#'
#' # Plot with cut lines (we manually set the order here)
#' order <- ser_permutation(c(21, 16, 19, 18, 14, 12, 20, 15,
#'     17, 26, 13, 41,  7, 11, 5, 23, 28, 34, 31, 1, 38, 40,
#'     3, 39,  4, 27, 24,  8, 37, 36, 25, 30, 33, 35,  2,
#'     22, 32, 29, 10,  6,  9),
#'     c(4, 2, 1, 6, 8, 7, 5, 3))
#'
#' bertinplot(x, order, pop=FALSE)
#' bertin_cut_line(, 4) ## horizontal line between rows 4 and 5
#' bertin_cut_line(, 7) ## separate "Right to Life" from the rest
#' bertin_cut_line(14, c(0, 4)) ## separate a block of large values (vertically)
#'
#' # ggplot2-based plots
#' if (require("ggplot2")) {
#'   library(ggplot2)
#'
#'   # Default plot uses bars and highlighting values larger than the mean
#'   ggbertinplot(x, order)
#'
#'   # highlight values in the 4th quartile
#'   ggbertinplot(x, order, highlight = quantile(x, probs = .75))
#'
#'   # Use different geoms. "none" lets the user specify their own geom.
#'   # Variables set are row, col and x (for the value).
#'
#'   ggbertinplot(x, order, geom = "tile", prop = TRUE)
#'   ggbertinplot(x, order, geom = "rectangle")
#'   ggbertinplot(x, order, geom = "rectangle", prop = TRUE)
#'   ggbertinplot(x, order, geom = "circle")
#'   ggbertinplot(x, order, geom = "line")
#'
#'   # Tiles with diverging color scale
#'   ggbertinplot(x, order, geom = "tile", prop = TRUE) +
#'     scale_fill_gradient2(midpoint = mean(x))
#'
#'   # Custom geom (geom = "none"). Defined variables are row, col, and x for the value
#'   ggbertinplot(x, order, geom = "none", prop = FALSE) +
#'     geom_point(aes(x = col, y = row, size = x, color = x > 30), pch = 15) +
#'     scale_size(range = c(1, 10))
#'
#'   # Use a ggplot2 theme with theme_set()
#'   old_theme <- theme_set(theme_minimal() +
#'       theme(panel.grid = element_blank())
#'     )
#'   ggbertinplot(x, order, geom = "bar")
#'   theme_set(old_theme)
#' }
#' @export
bertinplot <- function(x,
  order = NULL,
  panel.function = panel.bars,
  highlight = TRUE,
  row_labels = TRUE,
  col_labels = TRUE,
  flip_axes = TRUE,
  ...) {
  if (!is.matrix(x))
    stop("Argument 'x' must be a matrix.")

  # add ... to options
  options <- list(...)
  options$panel.function <- panel.function

  options <- .get_parameters(
    options,
    list(
      panel.function = panel.bars,
      flip_axes   = TRUE,
      frame       = FALSE,
      spacing     = 0.2,
      margins     = c(5, 4, 8, 8),
      gp_labels   = gpar(),
      gp_panels   = gpar(),
      shading	    = NULL,
      shading_col = .sequential_pal(100),
      newpage     = TRUE,
      pop         = TRUE
    )
  )

  ## panel.blocks has no spacing!
  if (identical(options$panel.function, panel.blocks))
    options$spacing <- 0

  if (is.null(options$shading))
    if (identical(options$panel.function, panel.blocks)) {
      options$shading <- TRUE
    } else {
      options$shading <- FALSE
    }

  ## order
  if (!is.null(order))
    x <- permute(x, order)

  ## note: Bertin switched cols and rows for his display!
  # change x and y?
  if (flip_axes) {
    x <- t(x)
    tmp <- row_labels
    row_labels <- col_labels
    col_labels <- tmp
  }

  ## highlight
  if (is.logical(highlight) && highlight)
    highlight <- mean(x, na.rm = TRUE)

  ## clear page
  if (options$newpage)
    grid.newpage()

  ## create outer viewport
  xlim <- c(options$spacing, ncol(x) + 1 - options$spacing)
  pushViewport(
    plotViewport(
      margins = options$mar,
      layout = grid.layout(nrow(x), 1),
      xscale = xlim,
      yscale = c(0, nrow(x)),
      default.units = "native",
      name = "bertin"
    )
  )

  # shading and highlighting
  if (options$shading)
    col <- .map_color(x, options$shading_col)
  else
    col <- matrix(1, nrow = nrow(x), ncol = ncol(x))

  if (highlight)
    col[x < highlight] <- NA

  # map to [0, 1]
  x <- map(x)

  for (variable in seq(nrow(x))) {
    value <- x[variable, ]
    hl <- col[variable, ]

    ## handle neg. values
    if (identical(options$panel.function, panel.bars) ||
        identical(options$panel.function, panel.lines)) {
      ylim <- c(min(value, 0, na.rm = TRUE),
        max(value, 0, na.rm = TRUE) + options$spacing)
    } else{
      ylim <- c(0,
        max(abs(value), 0.1, na.rm = TRUE))
    }

    pushViewport(
      viewport(
        layout.pos.col = 1,
        layout.pos.row = variable,
        xscale = xlim,
        yscale = ylim,
        default.units = "native",
        gp = options$gp_panels
      )
    )

    ## call panel function
    options$panel.function(value, options$spacing, hl)

    ## do frame
    if (options$frame)
      grid.rect(
        x = seq(length(value)),
        width = 1,
        default.units = "native",
        gp = gpar(fill = NA)
      )

    upViewport(1)
  }

  spacing_corr <-
    if (options$spacing <= 0)
      - options$spacing + 0.2
  else
    0

  grid.text(
    colnames(x),
    x = seq(ncol(x)),
    y = nrow(x) + spacing_corr,
    rot = 90,
    just = "left",
    default.units = "native",
    gp = options$gp_labels
  )

  grid.text(
    rev(rownames(x)),
    x = 1 + spacing_corr / ncol(x) / 4,
    y = 0.5:(nrow(x) - 0.5) / nrow(x),
    just = "left",
    default.units = "npc",
    gp = options$gp_labels
  )

  if (options$pop)
    popViewport(1)
  else
    upViewport(1)
}

#' @rdname bertinplot
#' @export
panel.bars <- function(value, spacing, hl) {
  grid.rect(
    x = seq(length(value)),
    y = spacing / 2,
    width = 1 - spacing,
    height = value * (1 - spacing),
    just = c("centre", "bottom"),
    default.units = "native",
    gp = gpar(fill = hl)
  )
}

#' @rdname bertinplot
#' @export
panel.circles <- function(value, spacing, hl) {
  ## neg. values are dashed
  lty <- as.integer(value < 0) + 1L
  lty[!is.finite(lty)] <- 0L

  value <- abs(value)

  value[value == 0] <- NA ### hide empty squares

  grid.circle(
    x = seq(length(value)),
    y = unit(.5, "npc"),
    r = value / 2 * (1 - spacing),
    default.units = "native",
    gp = gpar(fill = hl, lty = lty)
  )
}

#' @rdname bertinplot
#' @export
panel.rectangles <- function(value, spacing, hl) {
  ## neg. values are dashed
  lty <- as.integer(value < 0) + 1L
  lty[!is.finite(lty)] <- 0L

  value[value == 0] <- NA ### hide emply squares

  grid.rect(
    x = seq(length(value)),
    width = value * (1 - spacing),
    height = value * (1 - spacing),
    default.units = "native",
    just = c("centre", "center"),
    gp = gpar(fill = hl, lty = lty)
  )
}

#' @rdname bertinplot
#' @export
panel.squares <- panel.rectangles

#' @rdname bertinplot
#' @export
panel.tiles <- function(value, spacing, hl) {
  grid.rect(
    x = seq(length(value)),
    width = 1,
    height = unit(1, "npc"),
    default.units = "native",
    just = c("centre", "center"),
    gp = gpar(fill = hl)
  )
}

#' @rdname bertinplot
#' @export
panel.blocks <- panel.tiles

### hl is ignored
#' @rdname bertinplot
#' @export
panel.lines <- function(value, spacing, hl) {
  grid.lines(
    x = seq(length(value)),
    y = value * (1 - spacing),
    default.units = "native"
  )
}


## add cut lines manually to a bertin plot
#' @rdname bertinplot
#' @export
bertin_cut_line <- function(x = NULL,
  y = NULL,
  col = "red") {
  if (length(x) < 2)
    x <- rep(x, 2)
  if (length(y) < 2)
    y <- rep(y, 2)

  ## find the bertin Viewport
  if (inherits(try(seekViewport("bertin"), silent = TRUE)
    , "try-error")) {
    stop("bertinplot() needs to be called with options = list(pop = FALSE) first!")
  }

  if (is.null(x))
    x <- unit(c(0, 1), units = "npc")
  else
    x <- x + .5

  if (is.null(y))
    y <- unit(c(0, 1), units = "npc")
  else
    y <- y

  grid.lines(
    x = x,
    y = y,
    default.units = "native",
    gp = gpar(col = col, lwd = 2)
  )
}

#######################################################################
# seriation - Infrastructure for seriation
# Copyright (C) 2011 Michael Hahsler, Christian Buchta and Kurt Hornik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#' Registry for Criterion Methods
#'
#' A registry to manage methods used by [criterion()] to calculate a criterion value given data and a
#' permutation.
#'
#' All methods below are convenience methods for the registry named
#' `registry_criterion`.
#'
#' `list_criterion_method()` lists all available methods for a given data
#' type (`kind`). The result is a vector of character strings with the
#' short names of the methods. If `kind` is missing, then a list of
#' methods is returned.
#'
#' `get_criterion_method()` returns information (including the
#' implementing function) about a given method in form of an object of class
#' `"criterion_method"`.
#'
#' With `set_criterion_method()` new criterion methods can be added by the
#' user. The implementing function (`fun`) needs to have the formal
#' arguments `x, order, ...`, where `x` is the data object, order is
#' an object of class [ser_permutation_vector] and `...` can contain
#' additional information for the method passed on from [criterion()]. The
#' implementation has to return the criterion value as a scalar.
#'
#' @family criterion
#'
#' @param kind the data type the method works on. For example, `"dist"`,
#' `"matrix"` or `"array"`.
#' @param name the name for the method used to refer to the method in the
#' function [criterion()].
#' @param fun a function containing the method's code.
#' @param description a description of the method. For example, a long name.
#' @param merit a boolean indicating if the criterion measure is a merit
#' (`TRUE`) or a loss (`FALSE`) measure.
#' @param x an object of class "criterion_method" to be printed.
#' @param ... further information that is stored for the method in the
#' registry.
#' @returns
#' - `list_criterion_method()` results is a vector of character strings with the
#'   names of the methods used for `criterion()`.
#' - `get_criterion_method()` returns a given method in form of an object of class
#'   `"criterion_method"`.
#' @author Michael Hahsler
#' @seealso This registry uses [registry()] in package \pkg{registry}.
#' @keywords misc
#' @examples
#' ## the registry
#' registry_criterion
#'
#' # List all criterion calculation methods by type
#' list_criterion_methods()
#'
#' # List methods for matrix
#' list_criterion_methods("matrix")
#'
#' get_criterion_method("dist", "AR_d")
#'
#' # Define a new method (sum of the diagonal elements)
#'
#' ## 1. implement a function to calculate the measure
#' criterion_method_matrix_foo <- function(x, order, ...) {
#' if(!is.null(order)) x <- permute(x,order)
#'     sum(diag(x))
#' }
#'
#' ## 2. Register new method
#' set_criterion_method("matrix", "DiagSum", criterion_method_matrix_foo,
#'     description = "Calculated the sum of all diagonal entries", merit = FALSE)
#'
#' list_criterion_methods("matrix")
#' get_criterion_method("matrix", "DiagSum")
#'
#' ## 3. use all criterion methods (including the new one)
#' criterion(matrix(1:9, ncol = 3))
#' @export
registry_criterion <-
  registry(registry_class = "criterion_registry",
    entry_class = "criterion_method")

registry_criterion$set_field("kind",
  type = "character",
  is_key = TRUE,
  index_FUN = match_partial_ignorecase)
registry_criterion$set_field("name",
  type = "character",
  is_key = TRUE,
  index_FUN = match_partial_ignorecase)
registry_criterion$set_field("fun", type = "function",
  is_key = FALSE)
registry_criterion$set_field("description", type = "character",
  is_key = FALSE)
registry_criterion$set_field("merit", type = "logical",
  is_key = FALSE)

#' @rdname registry_criterion
#' @export
list_criterion_methods <- function(kind) {
  if (missing(kind)) {
    kinds <- unique(sort(as.vector(
      sapply(registry_criterion$get_entries(), "[[", "kind")
    )))

    sapply(
      kinds,
      FUN = function(k)
        list_criterion_methods(k)
    )

  } else{
    sort(as.vector(sapply(
      registry_criterion$get_entries(kind = kind), "[[", "name"
    )))
  }
}

#' @rdname registry_criterion
#' @export
get_criterion_method <- function(kind, name) {
  method <- registry_criterion$get_entry(kind = kind, name = name)
  if (is.null(method))
    stop("Unknown criterion. Check list_criterion_methods(\"",
      kind,
      "\")")

  method
}

## <NOTE>
## For criterion() methods, argument 'method' really allows selecting
## *several* methods ... should perhaps be called 'methods'?
## We thus have a getter which returns a named list of methods from the
## registry, and a setter for single methods.
## </NOTE>

#' @rdname registry_criterion
#' @export
set_criterion_method <- function(kind,
  name,
  fun,
  description = NULL,
  merit = NA,
  ...) {
  ## check formals
  ##if(!identical(names(formals(definition)),
  ##              c("x", "order", "...")))
  ##    stop("Criterion methods must have formals 'x', 'order', and '...'.")

  ## check if criterion is already in registry
  r <- registry_criterion$get_entry(kind = kind, name = name)
  if (!is.null(r) && r$name == name) {
    warning("Entry with name ", name, " already exists! Modifying entry.")
    registry_criterion$modify_entry(
      kind = kind,
      name = name,
      fun = fun,
      description = description,
      merit = merit
    )
  } else {
    registry_criterion$set_entry(
      kind = kind,
      name = name,
      fun = fun,
      description = description,
      merit = merit
    )
  }

}

#' @rdname registry_criterion
#' @export
print.criterion_method <- function(x, ...) {
  writeLines(c(
    gettextf("name:        %s", x$name),
    gettextf("kind:        %s", x$kind),
    gettextf("description: %s", x$description),
    gettextf("merit:       %s", x$merit)
  ))
  invisible(x)
}

\documentclass[nojss]{jss}

\usepackage[english]{babel}

%\documentclass[fleqn, a4paper]{article}
%\usepackage{a4wide}
%\usepackage[round,longnamesfirst]{natbib}
%\usepackage{graphicx,keyval,thumbpdf,url}
%\usepackage{hyperref}
%\usepackage{Sweave}
\SweaveOpts{strip.white=true}
\AtBeginDocument{\setkeys{Gin}{width=0.6\textwidth}}

\usepackage[utf8]{inputenc}

%% end of declarations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\usepackage{amsmath}
\usepackage{amsfonts}


%\newcommand{\strong}[1]{{\normalfont\fontseries{b}\selectfont #1}}
\newcommand{\class}[1]{\mbox{\textsf{#1}}}
\newcommand{\func}[1]{\mbox{\texttt{#1()}}}
%\newcommand{\code}[1]{\mbox{\texttt{#1}}}
%\newcommand{\pkg}[1]{\strong{#1}}
\newcommand{\samp}[1]{`\mbox{\texttt{#1}}'}
%\newcommand{\proglang}[1]{\textsf{#1}}
\newcommand{\set}[1]{\mathcal{#1}}
\newcommand{\sQuote}[1]{`{#1}'}
\newcommand{\dQuote}[1]{``{#1}''}
\newcommand\R{{\mathbb{R}}}

\DeclareMathOperator*{\argmin}{argmin}
\DeclareMathOperator*{\argmax}{argmax}

%% almost as usual
\author{Michael Hahsler\\Southern Methodist University \And
Kurt Hornik\\Wirtschaftsuniversit\"at Wien \AND
Christian Buchta\\Wirtschaftsuniversit\"at Wien}
\title{Getting Things in Order:\\
An Introduction to the \proglang{R}~Package~\pkg{seriation}}

%% for pretty printing and a nice hypersummary also set:
\Plainauthor{Michael Hahsler, Kurt Hornik, Christian Buchta} %% comma-separated
\Plaintitle{Getting Things in Order: An Introduction to the R Package seriation} %% without formatting
\Shorttitle{Getting Things in Order} %% a short title (if necessary)

%% an abstract and keywords
\Abstract{Seriation, i.e., finding a suitable linear order for a set of
objects given data and a loss or merit function, is a basic problem in
data analysis.  Caused by the problem's combinatorial nature, it is
hard to solve for all but very small sets.  Nevertheless, both exact
solution methods and heuristics are available.  In this paper we
present the package~\pkg{seriation} which provides an infrastructure
for seriation with \proglang{R}.  The infrastructure comprises data
structures to represent linear orders as permutation vectors, a wide
array of seriation methods using a consistent interface, a method to
calculate the value of various loss and merit functions, and several
visualization techniques which build on seriation. To illustrate how
easily the package can be applied for a variety of applications, a
comprehensive collection of examples is presented.}

\Keywords{combinatorial data analysis, seriation, permutation, \proglang{R}}
\Plainkeywords{combinatorial data analysis, seriation, permutation, R} %% without formatting

\Address{
Michael Hahsler\\
    Engineering Management, Information, and Systems\\
    Lyle School of Engineering\\
    Southern Methodist University\\
    P.O. Box 750123 \\
    Dallas, TX 75275-0123\\
    E-mail: \email{mhahsler@lyle.smu.edu}\\
    URL: \url{http://lyle.smu.edu/~mhahsler}

Kurt Hornik\\
Department f\"ur Statistik \& Mathematik\\
Wirtschaftsuniversit\"at Wien\\
1090 Wien, Austria\\
E-mail: \email{kurt.hornik@wu.ac.at}\\
URL: \url{http://statmath.wu.ac.at/~hornik/}

Christian Buchta\\
Department f\"ur Welthandel\\
Wirtschaftsuniversit\"at Wien\\
1090 Wien, Austria\\
E-mail: \email{christian.buchta@wu.ac.at}\\
URL: \url{http://www.wu.ac.at/itf/institute/staff/buchta}
			    }

\hyphenation{Brusco}
\sloppy

%% \VignetteIndexEntry{An Introduction to the R package seriation}

\begin{document}

%\title{Getting Things in Order: An introduction to the
%R~package~\pkg{seriation}}
%\author{Michael Hahsler, Kurt Hornik and Christian Buchta}
\maketitle


%\abstract{Seriation, i.e., finding a suitable linear order for a set of
%  objects given data and a loss or merit function, is a basic problem in
%  data analysis.  Caused by the problem's combinatorial nature, it is
%  hard to solve for all but very small sets.  Nevertheless, both exact
%  solution methods and heuristics are available.  In this paper we
%  present the package~\pkg{seriation} which provides an infrastructure
%  for seriation with \proglang{R}.  The infrastructure comprises data
%  structures to represent linear orders as permutation vectors, a wide
%  array of seriation methods using a consistent interface, a method to
%  calculate the value of various loss and merit functions, and several
%  visualization techniques which build on seriation. To illustrate how
%  easily the package can be applied for a variety of applications, a
%  comprehensive collection of examples is presented.}
%


<<echo=FALSE>>=
options(scipen=3, digits=4)
### for sampling
set.seed(1234)
@

\section{Introduction}

A basic problem in data analysis, called \emph{seriation} or sometimes
\emph{sequencing}, is to arrange all objects in a set in a linear order
given available data and some loss or merit function in order to
reveal structural information.  Together with
cluster analysis and variable selection, seriation is an important
problem in the field of \emph{combinatorial data
  analysis}~\citep{seriation:Arabie:1996}.  Solving problems in
combinatorial data analysis requires the solution of discrete
optimization problems which, in the most general case, involves
evaluating all feasible solutions.  Due to the combinatorial nature, the
number of possible solutions grows with problem size (number of objects, $n$)
by the order~$O(n!)$.  This makes a brute-force enumerative approach
infeasible for all but very small problems.  To solve larger problems
(currently with up to 40 objects), partial enumeration methods can be used.  For
example, \cite{seriation:Hubert:2001} propose dynamic programming and
\cite{seriation:Brusco:2005} use a branch-and-bound strategy.  For even
larger problems only heuristics can be employed.

It has to be noted that seriation has a rich history in archaeology.
\cite{seriation:Petrie:1899} was the first to use seriation as a formal method.
He applied it to find a chronological order for graves discovered in the Nile
area given objects found there. He used a cross-tabulation of grave sites and
objects and rearranged the table using row and column permutations till all
large values were close to the diagonal. In the rearranged table graves with
similar objects are closer to each other. Together with the assumption that
different objects continuously come into and go out of fashion, the order of
graves in the rearranged table suggests a chronological order.
Initially, the rearrangement of rows and columns of this
contingency table was done manually and the adequacy was only judged
subjectively by the researcher.  Later, \cite{seriation:Robinson:1951},
\cite{seriation:Kendall:1971} and others proposed measures of agreement
between rows to quantify optimality of the resulting table.  A
comprehensive description of the development of seriation in archaeology
is presented by \cite{seriation:Ihm:2005}.

Techniques related to seriation are also popular in several other
fields.  Especially in ecology scaling techniques are used under the
name \emph{ordination}.  For these applications several \proglang{R} packages
already exist (e.g.,
\pkg{ade4}~\citep{seriation:Chessel:2007,seriation:Dray:2007} and
\pkg{vegan}~\citep{seriation:Oksanen:2007}). This paper describes the new
package \pkg{seriation} which differs from existing packages in the
following ways:
\begin{itemize}
 \item \pkg{seriation} provides a flexible infrastructure for seriation;
 \item \pkg{seriation} focuses on seriation as a combinatorial
  optimization problem.
\end{itemize}

This paper starts with a formal introduction of the seriation problem as
a combinatorial optimization problem in Section~\ref{sec:seriation}.  In
Section~\ref{sec:methods} we give an overview of seriation methods.  In
Section~\ref{sec:infrastructure} we present the infrastructure provided
by the package~\pkg{seriation}.  Several examples and applications for
seriation are given in Section~\ref{sec:example}.
Section~\ref{sec:conclusion} concludes.

A previous version of this manuscript was published in the \emph{Journal
  of Statistical Software} \citep{seriation:Hahsler+Hornik:2008}.


\section{Seriation as a combinatorial optimization problem}
\label{sec:seriation}

To seriate a set of $n$ objects $\{O_1,\dots,O_n\}$ one typically starts
with an $n \times n$ symmetric dissimilarity matrix~$\mathbf{D} =
(d_{ij})$ where $d_{ij}$ for $1 \le i,j \le n$ represents the
dissimilarity between objects $O_i$ and $O_j$, and $d_{ii} = 0$ for
all~$i$.  We define a permutation function $\Psi$ as a function which
reorders the objects in $\mathbf{D}$ by simultaneously permuting rows
and columns.  The seriation problem is to find a permutation function
$\Psi^*$
%$\{1,\dots,n\} \rightarrow \{1,\dots,n\}$, i.e. a
%bijection that maps the set of indices of the objects (and equally of rows and
%columns of $\mathbf{D}$) onto itself,
which optimizes the value of a given loss function~$L$ or merit function~$M$.
This results in the optimization problems
\begin{equation}
    \Psi^* = \argmin_\Psi L(\Psi(\mathbf{D})) \quad \text{or} \quad
    \Psi^* = \argmax_\Psi M(\Psi(\mathbf{D})),
\end{equation}
respectively.
%This is clearly a hard discrete optimization problem since the number of
%possible permutations is $n!$ which makes an exhaustive
%search for sets with a medium to large number of objects infeasible.
%Partial enumeration methods and heuristics can be used. Such methods are
%presented in Section~\ref{sec:methods}.
%But first, we review commonly used loss functions in the following section.
%\marginpar{two-mode data missing}

A symmetric dissimilarity matrix is known as \emph{two-way one-mode}
data since it has columns and rows (two-way) but only represents one set
of objects (one-mode).  Seriation is also possible for two-way two-mode
data which are represented by a general nonnegative matrix.  In such data
columns and rows represent two sets of objects which are reordered
simultaneously.  For loss/merit functions for two-way two-mode data the
optimal order of columns can depend of the order of rows and vice
versa or it can be independent allowing for breaking the optimization
down into two separate problems, one for the columns and one for the
rows.  Another way to deal with the seriation for two-way two-mode data
is to calculate two dissimilarity matrices, one for each mode, and then
solve two seriation problems for two-way one-mode data.  Furthermore,
seriation can be generalized to $k$-way $k$-mode data in the form of a
$k$-dimensional array by defining suitable loss/merit functions for such data
or by breaking the problem down into several lower dimensional
independent problems.

To assess the complexity of seriation of $k$-way $k$-mode data, let us
assume the data is a $k$-dimensional array with the dimensions
containing $n_1, n_2, \ldots, n_k$ objects. If the loss/merit function allows
for separating the problem into $k$ independent problems, the problem
size is just the sum of the individual problems. By using complete
enumeration the size is $O(\sum_{i=1}^k{n_i!})$. If the problem is not
separable and the optimal seriation of each dimension depends on the
order of the objects of the other dimensions, the problem size is
$O((\sum_{i=1}^k{n_i})!)$. For example for $k=5$ and all dimensions
containing 5 objects, the search space for separable dimensions is only
600 while without separability it is larger than $10^{25}$ clearly too
big to be solvable in reasonable time. This shows that for data with
even only a few dimensions and a few objects each, finding the optimal
solution is infeasible and loss/merit functions which allow for separating the
problem are highly desirable.

In the following subsections, we review some commonly employed loss/merit
functions.  Most functions are used for two-way one-mode data but the
measure of effectiveness and stress can be also used for two-way
two-mode data. For the implementation of various loss or merit measures
see function~\func{criterion} in Section~\ref{sec:infrastructure}.



%\section{Loss functions}
%\label{sec:criteria}
%In the literature several loss functions are suggested.
%We review the most commonly used functions.

\subsection{Column/row gradient measures}

A symmetric dissimilarity matrix where the values in all rows and
columns only increase when moving away from the main diagonal is called
a perfect \emph{anti-Robinson matrix} after the statistician
\cite{seriation:Robinson:1951}. Formally, an $n \times n$ dissimilarity
matrix $\mathbf{D}$ is in anti-Robinson form if and only if the
following two gradient conditions hold~\citep{seriation:Hubert:2001}:
\begin{align}
    \text{within rows:} & \quad  d_{ik} \le d_{ij} \quad \text{for}
        \quad 1 \le i < k < j \le n; \\
    \text{within columns:} & \quad  d_{kj} \le d_{ij} \quad \text{for}
        \quad 1 \le i < k < j \le n.
\end{align}

In an anti-Robinson matrix the smallest dissimilarity values appear
close to the main diagonal, therefore, the closer objects are together
in the order of the matrix, the higher their similarity.  This provides
a natural objective for seriation.

It has to be noted that $\mathbf{D}$ can be brought into a perfect
anti-Robinson form by row and column permutation whenever $\mathbf{D}$  is an
ultrametric or $\mathbf{D}$ has an exact Euclidean representation in a single
dimension~\citep{seriation:Hubert:2001}. However, for most data only an
approximation to the anti-Robinson form is possible.

A suitable merit measure which quantifies the divergence of a matrix from the
anti-Robinson form was given by \cite{seriation:Hubert:2001} as
\begin{equation}
    M(\mathbf{D}) =
    \sum_{i<k<j}f(d_{ik}, d_{ij}) + \sum_{i<k<j}f(d_{kj}, d_{ij})
    \label{equ:gradient}
\end{equation}
where $f(\cdot,\cdot)$ is a function which defines how a violation or
satisfaction of a gradient condition for an object triple ($O_i$, $O_k$ and
$O_j$) is counted.  \cite{seriation:Hubert:2001} suggest two functions. The
first function is given by:
\begin{equation}
    f(z,y) = \mathrm{sign}(y-z) =
    \begin{cases}
        +1 \quad \text{if} \quad z < y; \\
        \phantom{+}0 \quad \text{if} \quad z = y; \\
        -1 \quad \text{if} \quad z > y.
    \end{cases}
\end{equation}
It results in the raw number of triples satisfying the gradient
constraints minus triples which violate the constraints.

The second function is defined as:
\begin{equation}
    f(z,y) = |y-z|\mathrm{sign}(y-z) = y-z
\end{equation}
It weighs each satisfaction or violation by its
magnitude given by the absolute difference between the values.

\subsection{Anti-Robinson events}
An even simpler loss function can be created in the same way as the gradient
measures above by concentrating on violations only.
\begin{equation}
    L(\mathbf{D}) =
    \sum_{i<k<j}f(d_{ik}, d_{ij}) + \sum_{i<k<j}f(d_{kj}, d_{ij})
\end{equation}

To only count the violations we use
\begin{equation}
    f(z, y) = I(z, y) =
    \begin{cases}
        1 \quad \text{if} \quad z > y \quad \text{and} \\
        0 \quad \text{otherwise.}
    \end{cases}
\end{equation}
$I(\cdot)$ is an indicator function returning $1$ only for violations.
\cite{seriation:Chen:2002} presented a formulation for an equivalent
loss function and called the violations \emph{anti-Robinson events}.
\cite{seriation:Chen:2002} also introduced a weighted versions of the loss
function resulting in
\begin{equation}
    f(z, y) = |y-z|I(z, y)
\end{equation}
using the absolute deviations as weights.

\subsection{Hamiltonian path length}
The dissimilarity matrix $\mathbf{D}$ can be represented as a finite weighted
graph $G = (\Omega,E)$ where the set of objects~$\Omega$ constitute the
vertices and each edge~$e_{ij} \in E$ between the objects $O_i, O_j \in \Omega$
has a weight~$w_{ij}$ associated which represents the dissimilarity~$d_{ij}$.

Such a graph can be used for seriation~\citep[see,
e.g.,][]{seriation:Hubert:1974,seriation:Caraux:2005}.  An order~$\Psi$
of the objects can be seen as a path through the graph where each node
is visited exactly once, i.e., a Hamiltonian path.  Minimizing the
Hamiltonian path length results in a seriation optimal with respect to
dissimilarities between neighboring objects.  The loss function based on
the Hamiltonian path length is:
\begin{equation}
    L(\mathbf{D}) = \sum_{i=1}^{n-1} d_{i,i+1}.
\end{equation}

Note that the length of the Hamiltonian path is equal to the value of the
\emph{minimal span loss function} \citep[as used by][]{seriation:Chen:2002},
and both notions are related to the \emph{traveling salesperson
problem}~\citep{seriation:Gutin:2002}.

\subsection{Inertia criterion}
Another way to look at the seriation problem is not to focus on placing small
dissimilarity values close to the diagonal, but to push large values away from
it. A function to quantify this is the moment of inertia of dissimilarity
values around the diagonal \citep{seriation:Caraux:2005} defined as
\begin{equation}
    M(\mathbf{D}) = \sum_{i=1}^n \sum_{j=1}^n d_{ij}|i-j|^2.
\end{equation}
$|i-j|^2$ is used as a measure for the distance to the diagonal and $d_{ij}$
gives the weight.  This is a merit function since the sum increases when higher
dissimilarity values are placed farther away from the diagonal.

\subsection{Least squares criterion}
Another natural loss function for seriation is to quantify the deviations
between the dissimilarities in $\mathbf{D}$ and the rank differences of the
objects.  Such deviations can be measured, e.g, by the sum of squares of
deviations \citep{seriation:Caraux:2005} defined by
\begin{equation}
    L(\mathbf{D}) = \sum_{i=1}^n \sum_{j=1}^n (d_{ij} - |i-j|)^2,
\end{equation}
where $|i-j|$ is the rank difference or gap between $O_i$ and $O_j$.

The least squares criterion defined here is related to uni-dimensional
scaling~\citep{seriation:Leeuw:2005}, where the objective is to place all
$n$ objects on a straight line using a position vector~$\mathbf{z} =
z_1,z_2,\ldots,z_n$ such that the dissimilarities
in $\mathbf{D}$ are
preserved by the relative positions in the best possible way. The optimization
problem of uni-dimensional scaling is to find the
position vector~$\mathbf{z^*}$ which minimizes $\sum_{i=1}^n \sum_{j=1}^n
(d_{ij} - |z_i-z_j|)^2$.
This is close to the seriation problem, but in
addition to the ranking of the objects also takes the distances between objects
on the resulting scale into account.

Note that if Euclidean distance is used to calculate $\mathbf{D}$ from a data
matrix~$\mathbf{X}$, using the order of the elements in $\mathbf{X}$ as they
occur projected on the first principal component of $\mathbf{X}$
minimizes the loss function of uni-dimensional scaling (using squared
distances). Using this order, also provides a good solution
for the least square seriation criterion.

\subsection{Linear Seriation Criterion}
The Linear Seriation Criterion (Hubert and Schultz 1976)
weights the distances with the absolute rank differences.

$$L(\mathbf{D}) \sum_{i=1}^n \sum_{j=1}^n d_{ij} (-|i-j|)$$

\subsection{2-Sum Problem}
The 2-Sum loss criterion \citep{seriation:Barnard:1993}
multiplies the similarity between objects
with the squared rank differences.

$$L(\mathbf{D}) \sum_{i,j=1}^p \frac{1}{1+d_{ij}} (i-j)^2,$$

where $s_{ij} = \frac{1}{1+d_{ij}}$ represents the similarity between
objects $i$ and $j$.


\subsection{Measure of effectiveness}
\label{sec:ME}

\cite{seriation:McCormick:1972} defined the
\emph{measure of effectiveness (ME)} for an $n \times m$ matrix~$\mathbf{X} =
(x_{ij})$ as
\begin{equation}
    M(\mathbf{X}) =
    \frac{1}{2}
    \sum_{i=1}^{n} \sum_{j=1}^{m} x_{ij}[x_{i,j+1}+x_{i,j-1}+
        x_{i+1,j}+x_{i-1,j}]
    \label{equ:ME}
\end{equation}
with, by convention $x_{0,j}=x_{n+1,j}=x_{i,0}=x_{i,m+1}=0$.  ME is maximized
if each element is as closely related numerically to its four neighboring
elements as possible.

ME was developed for two-way two-mode data, however, ME can also be used for a
symmetric  matrix (one-mode data) and gets maximal only if all large values are
grouped together around the main diagonal.

Note that the definition in equation~(\ref{equ:ME})
can be rewritten as
\begin{equation}
    M(\mathbf{X}) =
    \frac{1}{2}
    \sum_{i=1}^{n} \sum_{j=1}^{m} x_{ij}[x_{i,j+1}+x_{i,j-1}] +
    \sum_{i=1}^{n} \sum_{j=1}^{m} x_{ij}[x_{i+1,j}+x_{i-1,j}]
\end{equation}
showing that the contributions of column and row order to the merit function
are independent.

\subsection{Stress}
\label{sec:stress}

Stress measures the conciseness of the presentation of a matrix
(two-mode data) and can be seen as a purity function which compares the
values in a matrix with their neighbors. The stress measures used here
are computed as the sum of squared distances of each matrix entry from
its adjacent entries.  \cite{seriation:Niermann:2005} defined for an $n
\times m$ matrix~$\mathbf{X} = (x_{ij})$ two types of neighborhoods:

\begin{itemize}
    \item The Moore neighborhood comprises the (at most) eight adjacent entries.
        The local stress measure for element~$x_{ij}$ is defined as
        \begin{equation}
            \sigma_{ij} = \sum_{k=\max(1,i-1)}^{\min(n,i+1)}
                \sum_{l=\max(1,j-1)}^{\min(m,j+1)}
                (x_{ij} - x_{kl})^2
        \end{equation}

    \item The Neumann neighborhood comprises the (at most) four adjacent entries
        resulting in the local stress of $x_{ij}$ of
        \begin{equation}
            \sigma_{ij} =
            \sum_{k=\max(1,i-1)}^{\min(n,i+1)} (x_{ij} - x_{kj})^2 +
            \sum_{l=\max(1,j-1)}^{\min(m,j+1)} (x_{ij} - x_{il})^2
            %(x_{ij} - x(i-1,j))^2 + (x_{ij} - x(i+1,j))^2 +
            %(x_{ij} - x(i,j-1))^2 + (x_{ij} - x(i,j+1))^2
        \end{equation}
\end{itemize}

Both local stress measures can be used to construct a global measure for the
whole matrix by summing over all entries which can be used as a loss function:
\begin{equation}
    L(\mathbf{X}) =
    \sum_{i=1}^n \sum_{j=1}^m \sigma_{ij}
\end{equation}

The major difference between the Moore and the Neumann neighborhood is
that for the later the contributions of row and column order to
stress are independent.

Stress can be also used as a loss function for
symmetric proximity matrices (one-mode data).
%,
%since it can only be optimal, if large values are
%concentrated around the main diagonal.
Note also, that stress with Neumann
neighborhood is related to the measure of effectiveness defined above (in
Section~\ref{sec:ME}) since both measures are optimal if for each cell the cell
and its four neighbors are numerically as similar as possible.

\section{Seriation methods}
\label{sec:methods}

Solving the discrete optimization problem for seriation with most loss/merit
functions is clearly very hard. The number of possible permutations for $n$
objects is $n!$ which makes an exhaustive search for sets with a medium to
large number of objects infeasible.  In this section, we describe some methods
(partial enumeration, heuristics and other methods) which are typically used
for seriation. For each method we state for which type of loss/merit functions
it is suitable and whether it finds the optimum or is a heuristic.  For the
implementation of various seriation methods see function~\func{seriate} in
Section~\ref{sec:infrastructure}.


\subsection{Partial enumeration methods}

Partial enumeration methods search for the exact solution of a
combinatorial optimization problem.  Exploiting properties of the search
space, only a subset of the enormous number of possible combinations has
to be evaluated.  Popular partial enumeration methods which are used for
seriation are \emph{dynamic programming}~\citep{seriation:Hubert:2001}
and \emph{branch-and-bound}~\citep{seriation:Brusco:2005}.

Dynamic programming recursively searches for the optimal solution checking and
storing $2^n-1$ results. Although $2^n-1$ grows at a lower rate than $n!$ and
is for $n \gg 3$ considerably smaller, the storage requirements of $2^n-1$
results still grow fast, limiting the maximal problem size severely. For
example, for $n=30$ more than one billion results have to be calculated and
stored, clearly a number too large for the main memory capacity of most current
computers.

Branch-and-bound has only very moderate storage requirements. The
forward-branching procedure~\citep{seriation:Brusco:2005} starts to build
partial permutations from left (first position) to right. At each step, it is
checked if the permutation is valid and several fathoming tests are performed
to check if the algorithm should continue with the partial permutation.  The
most important fathoming test is the boundary test, which checks if the partial
permutation can possibly lead to a complete permutation with a better solution
than the currently best one.  In this way large parts of the search space can
be omitted. However, in contrast to the dynamic programming approach, the
reduction of search space is strongly data dependent and poorly structured
data can lead to very poor performance.  With branch-and-bound slightly larger
problems can be solved than with dynamic programming in reasonable time.
\cite{seriation:Brusco:2005} state that depending on the data, in some cases
proximity matrices with 40 or more objects can be handled with current
hardware.

Partial enumeration methods can be used to find the exact solution
independently of the loss/merit function. However, partial enumeration is
limited to only relatively small problems.

\subsection{Traveling salesperson problem solver}

Seriation by minimizing the length of a Hamiltonian path through a graph
is equal to solving a traveling salesperson problem.  The traveling
salesperson or salesman problem (TSP) is a well known and well
researched combinatorial optimization
problem~\citep[see, e.g.,][]{seriation:Gutin:2002}. The goal is to find the
shortest tour that, starting from a given city, visits each city in a
given list exactly once and then returns to the starting city.
In graph theory a TSP tour is called
a \emph{Hamiltonian cycle.} But for the seriation problem, we are
looking for a Hamiltonian path. \cite{seriation:Garfinkel:1985}
described a simple transformation of the TSP to find the shortest
Hamiltonian path.  An additional row and column of 0's is added
(sometimes this is referred to as a \emph{dummy city}) to the original
$n \times n$ dissimilarity matrix~$\mathbf{D}$.  The solution of this
$(n+1)$-city TSP, gives the shortest path where the city representing
the added row/column cuts the cycle into a linear path.

As the general seriation problem, solving the TSP is difficult.  In the
seriation case with $n+1$ cities, $n!$ tours have to be checked. However,
despite this vast searching space, small instances can be solved efficiently
using dynamic programming \citep{seriation:Held:1962} and larger instances of
several hundred objects can be solved using \emph{branch-and-cut}
algorithms~\citep{seriation:Padberg:1990}. For even larger instances or if
running time is critical, a wide array of heuristics are available, ranging
from simple nearest neighbor approaches to construct a
tour~\citep{seriation:Rosenkrantz:1977} to complex heuristics like the
Lin-Kernighan heuristic~\citep{seriation:Lin:1973}.  A comprehensive overview
of heuristics and exact methods can be found in \cite{seriation:Gutin:2002}.


\subsection{Bond energy algorithm}

The \emph{bond energy algorithm}~\citep[BEA;][]{seriation:McCormick:1972} is a
simple heuristic to rearrange columns and rows of a matrix
(two-way two-mode data) such that each entry
is as closely numerically related to its four neighbors as possible.  To
achieve this, BEA tries to maximize the measure of effectiveness (ME) defined
in Section~\ref{sec:ME}.  For optimizing the ME, columns and rows can be
treated separately since changing the order of rows does not influence the ME
contributions of the columns and vice versa. BEA consists of the
following three steps:
\begin{enumerate}
 \item Place one randomly chosen column.
 \item Try to place each remaining column at each possible position
  left, right and between the already placed columns and calculate every
  time the increase in ME.  Choose the column and position which gives
  the largest increase in ME and place the column.  Repeat till all
  columns are placed.
 \item Repeat procedure with rows.
\end{enumerate}

This greedy algorithm works fast and only depends on the choice of the
first column/row. This dependence can be reduced by repeating the
procedure several times with different choices and returning the solution
with the highest ME.

Although \cite{seriation:McCormick:1972} use BEA also for non-binary data,
\cite{seriation:Arabie:1990} argue that the measure of effectiveness only
serves its intended purpose of finding an arrangement which is
close to Robinson form for binary data and should therefore only be
used for binary data.

\cite{seriation:Lenstra:1974} notes that the optimization problem of BEA
can be stated as two independent traveling salesperson problems (TSPs).
For example, the row TSP for an $n \times m$ matrix~$\mathbf{X}$
consists of $n$ cities with an $n \times n$ distance matrix~$\mathbf{D}$
where the distances are
\begin{displaymath}
    d_{ij} = -\sum_{k=1}^m x_{ik}x_{jk}.
\end{displaymath}
BEA is in fact a simple suboptimal TSP heuristic using this distances
and instead of BEA any TSP solver can be used to obtain an order.
With an exact TSP solver, the optimal solution can be found.

\subsection{Hierarchical clustering}
\label{sec:hierarchical_clustering}

Hierarchical clustering produces a series of nested clusterings which
can be visualized by a dendrogram, a tree where each internal node
represents a split into subtrees and has a measure of
similarity/dissimilarity attached to it.  As a simple heuristic to find
a linear order of objects, the order of the leaf nodes in a dendrogram
structure can be used. This idea is used, e.g., by heat maps to reorder
rows and columns with the aim to place more similar objects and
variables closer together.

%For hierarchical clustering several methods are available (e.g.,
%single linkage, average linkage, complete linkage, ward method) resulting in
%different dendrograms.
%However,
The order of leaf nodes in a dendrogram is not unique.  A binary
(two-way splits only) dendrogram for $n$ objects has $2^{n-1}$ internal
nodes and at each internal node the left and right subtree (or leaves)
can be swapped resulting in $2^{n-1}$ distinct leaf orderings.  To find
a unique or optimal order, an additional criterion has to be defined.
\cite{seriation:Gruvaeus:1972} suggest to obtain a unique order by
requiring to order the leaf nodes such that at each level the objects at
the edge of each cluster are adjacent to that object outside the cluster
to which it is nearest.

\cite{seriation:Bar-Joseph:2001} suggest to rearrange the dendrogram such that
the Hamiltonian path connecting the leaves is minimized and called this the
optimal leaf order. The authors also present a fast algorithm with time
complexity $O(n^4)$ to solve this optimization problem. Note that this problem
is related to the TSP described above, however, the given dendrogram structure
significantly reduces the number of permissible permutations making the problem
easier.

Although hierarchical clustering solves an optimization problem different to
the seriation problem discussed in this paper, hierarchical clustering still
can produce useful orderings, e.g., for visualization.

\subsection{Rank-two ellipse seriation}

\cite{seriation:Chen:2002} proposes to
generate a sequence of correlation matrices
$R^1, R^2, \ldots$. $R^1$ is the correlation matrix
of the original distance matrix $\mathbf{D}$ and
\begin{equation}
R^{n+1} = \phi R^n,
\end{equation}
where $\phi(\cdot)$ calculates a correlation matrix.

\cite{seriation:Chen:2002} shows that the
rank of the matrix $R^n$ falls with increasing $n$ and that if the sequence
is continued till the first matrix in the sequence has a rank of 2,
projecting all points in this matrix on its first two eigenvectors,
all points will fall on an ellipse.
\cite{seriation:Chen:2002} suggests to use the order of the points on
this ellipse as a seriation where the ellipse can be cut at any of the
two interception points (top or bottom) with the vertical axis.

Although the rank-two ellipse seriation procedure does not try to solve a
combinatorial optimization problem, it still provides for some cases a useful
ordering.

\subsection{Spectral Seriation}
Spectral seriation uses a relaxation to minimize the 2-Sum Problem
\citep{seriation:Barnard:1993}.
Rewriting the minimization problem using a permutation vector $\pi$,
its inverse, rescaling to $\mathrm{q}$ and using a Lagrangian
multiplier for the constraint on the permutation yields \citep{seriation:Ding:2004} the following equivalent optimization problem:

$$\mathrm{min}_\mathbf{q} \frac{\mathbf{q}^T L_\mathbf{S}\mathbf{q}}{\mathbf{q}^T\mathbf{q}}$$

where $L_\mathbf{S}$ is the Laplacian of $\mathbf{S}$.

The optimal order can be recovered by the sorting order of
the Fiedler vector (i.e., the second smallest eigenvector of the Laplacian
of the similarity matrix).

\subsection{Quadratic Assignment Problem}
Both, the linear seriation criterion and the 2-Sum problem formulation
can be written as a Quadratic Assignment Problem (QAP). However,
the QAP is in general NP-hard. Methods include
QIP, linearization, branch and bound and cutting planes as well as
heuristics including Tabu search, simulated annealing, genetic algorithms, and
ant systems \citep{seriation:Burkard:1998}.

\section{The package infrastructure}
\label{sec:infrastructure}

The \pkg{seriation} package provides the data structures and some algorithms
to efficiently handle seriation with \proglang{R}. As the
input data for seriation
\proglang{R}
already provides

\begin{itemize}
    \item for two-way one-mode data the class \code{dist},
    \item for two-way two-mode data the class \code{matrix}, and
    \item for $k$-way $k$-mode data the class \code{array}.
\end{itemize}


\begin{figure}[tp]
    \centerline{
    %\includegraphics[width=12cm]{infrastructure}}
    \includegraphics[width=10cm]{classes}}

    \caption{UML class diagram of the data structures for permutations provided by \pkg{seriation}}
    \label{fig:infrastructure}
\end{figure}

However, \proglang{R} provides no classes for representing permutation vectors.
\pkg{seriation} adds the necessary data structure (using the S3 class
system) as depicted in the UML class diagram \citep{seriation:Fowler:2004} in
Figure~\ref{fig:infrastructure}. In this diagram classes are represented by
rectangles and different symbols are used to state the type of
relationship between the classes. The class
\code{ser\_permutation}
in Figure~\ref{fig:infrastructure}
represents the permutation information for $k$-mode
data (including the cases of $k=1$ and $k=2$). It consists of $k$ permutation
vectors (class \code{ser\_permutation\_vector}). This relationship is
represented by the solid diamond and the star above the connection between the
two classes. Class \code{ser\_permutation\_vector} is defined \emph{abstract}
and only its concrete implementations (classes connected with the triangle
symbol) are used to store a permutation vector.
This design with an abstract class was chosen to allow to use
different representations for the permutation vectors.
Currently, the permutation vector can be stored as a simple
integer vector or as an object of class \code{hclust} (defined in
package \pkg{stats}). \code{hclust} describes a hierarchical clustering
tree (dendrogram) including an ordering for the tree's node leaves which
provides a permutation for all objects (see
Section~\ref{sec:hierarchical_clustering}).

Class \code{ser\_permutation\_vector} has a constructor
\func{ser\_permutation\_vector} which converts data into the correct concrete
subclass of \code{ser\_permutation\_vector} and checks if it contains a proper
permutation vector.  For \code{ser\_permutation\_vector} the methods
\func{print}, \func{length} for the length of the permutation vector,
\func{get\_method} to get the method used to generate the permutation, and
\func{get\_order} to access the raw (integer) permutation vector are available.
To use an additional class to represent permutations as a concrete subclass of
\code{ser\_permutation\_vector} only an appropriate accessor method
\func{get\_order} has to be implemented for the new class.

For \code{ser\_permutation} a constructor is provided which can bind $k$
\code{ser\_permutation\_vector} objects together into an object for $k$-mode
data. \code{ser\_permutation} is implemented as a list of length~$k$ and each
element contains a \code{ser\_permutation\_vector} object.  Methods like
\func{length}, accessing elements with \code{[[},
% ]]
\code{[[<-},
% ]]
subsetting with \code{[}, and combining with \func{c} work as expected.
Also a \func{print} method is provided. Finally, direct access to the
raw permutation vectors is available using \func{get\_order}.  Here a
second argument (which defaults to $1$) specifies the dimension (mode)
for which the order vector is requested.

All seriation algorithms are available via the function \func{seriate}
defined as:
\begin{quotation}
\code{seriate(x,  method = NULL, control = NULL, ...)}
\end{quotation}
where \code{x} is the input data, \code{method} is a string defining the
seriation method to be used and \code{control} can contain a list with
additional information for the algorithm.  \func{seriate} returns an object
of class \code{ser\_permutation} with a length conforming to the number of
dimensions of~\code{x}.
Typical input data are a dissimilarity matrix (class~\code{dist};
see package \pkg{stats} for more information) for one-mode two-way data,
\code{matrix} for two-mode two-way data and \code{array} for $k$-mode $k$-way
data.
For \code{matrix} and \code{array} the additional argument
\code{margin} can be used to restrict the dimensions which should be seriated
(e.g., with \code{margin = 1} only the first dimension,
i.e., the columns of a matrix, are seriated).

%\begin{landscape}
\begin{table}[tp]
\centering
    \begin{tabular}{p{5cm}p{3cm}p{4cm}l}
        \hline
Algorithm & \code{method} & Optimizes & Input data \\
        \hline
Simulated annealing & \code{"ARSA"} & Linear seriation crit.&\code{dist} \\
Branch-and-bound & \code{"BBURCG"} & Gradient measure &\code{dist} \\
Branch-and-bound & \code{"BBWRCG"} & Gradient measure (weighted)& \code{dist} \\
TSP solver & \code{"TSP"} & Hamiltonian path length& \code{dist} \\
Optimal leaf ordering  & \code{"OLO"}
\code{"OLO_single"}
\code{"OLO_average"}
\code{"OLO_complete"}
& Hamiltonian path length (restricted)& \code{dist} \\
Gruvaeus and Wainer & \code{"GW"}
\code{"GW_single"}
\code{"GW_average"}
\code{"GW_complete"}
& Hamiltonian path length (restricted) & \code{dist} \\
MDS & \code{"MDS"}
\code{"MDS_metric"}
\code{"MDS_nonmetric"}
\code{"MDS_angle"}
& Least square crit.& \code{dist} \\

Spectral seriation & \code{"Spectral"}
\code{"Spectral_norm"}
& 2-Sum crit. & \code{dist} \\

QAP & \code{"QAP_2SUM"}
& 2-Sum crit. & \code{dist} \\

 & \code{"QAP_LS"}
& Linear seriation crit. & \code{dist} \\

 & \code{"QAP_BAR"}
& Banded AR form  & \code{dist} \\

 & \code{"QAP_Inertia"}
& Inertia crit. & \code{dist} \\

	Genetic Algorithm & \code{"GA"}*
& various & \code{dist} \\

	DendSer & \code{"DendSer"}*
& various & \code{dist} \\

Hierarchical clustering & \code{"HC"}
\code{"HC_single"}
\code{"HC_average"}
\code{"HC_complete"}
& Other& \code{dist} \\
Rank-two ellipse seriation  & \code{"R2E"} & Other& \code{dist} \\
Sorting Points Into Neighborhoods & \code{"SPIN_NH"}
\code{"SPIN_STS"} & Other& \code{dist} \\
Visual Assessment of (Clustering) Tendency & \code{"VAT"}& Other& \code{dist} \\
        \hline
Bond Energy Algorithm & \code{"BEA"} &
    Measure of effectiveness & \code{matrix} \\
TSP to optimize ME & \code{"BEA\_TSP"} &
    Measure of effectiveness& \code{matrix} \\
Principal component analysis& \code{"PCA"}
\code{"PCA_angle"}&
    Least square crit.& \code{matrix} \\
        \hline
    \end{tabular}
    \caption{Currently implemented methods for \func{seriation} (* methods need to be registered).}
\label{tab:methods}
\end{table}
%\end{landscape}

Various seriation methods were already introduced in this paper in
Section~\ref{sec:methods}. In Table~\ref{tab:methods} we summarize the
methods currently available in the package for seriation.  The code for
the simulated annealing heuristic~\citep{seriation:Brusco:2007} and the
two branch-and-bound implementations~\citep{seriation:Brusco:2005} was
obtained from the authors.  The TSP solvers (exact solvers and a variety
of heuristics) is provided by package
\pkg{TSP}~\citep{seriation:Hahsler:2007, seriation:Hahsler:2007b}.  For
optimal leaf ordering we implemented the algorithm
by~\cite{seriation:Bar-Joseph:2001}. The BEA code was kindly provided by
Fionn Murtagh.  For the Gruvaeus and Wainer algorithm, the
implementation in package \pkg{gclus}~\citep{seriation:Hurley:2007} is
used. For the rank-two ellipse seriation we implemented the algorithm
by~\cite{seriation:Chen:2002}.
Spectral seriation is described by~\cite{seriation:Ding:2004}.
Note that some methods implemented
(e.g., the rank-two ellipse seriation) do not fall within the
combinatorial optimization framework of this paper and thus are not
dealt with here in detail.  They are included in the package since they
can be useful for various applications.
A detailed empirical comparison of seriation methods and criteria can be found
in the study by \cite{hahsler:Hahsler2016d}.
%Over time more methods will be
%added to the package.

To calculate the value of a loss/merit function for data and
a certain permutation, the function
\begin{quotation}
\code{criterion(x, order = NULL, method = NULL, ...)}
\end{quotation}
is provided. \code{x} is the data object, \code{order} contains a suitable
object of class \code{ser\_permutation} (if omitted no permutation is
performed) and \code{method} specifies the type of loss/merit function. A
vector of several methods can be used resulting in a named vector
with the values of the requested functions.  If \code{method} is omitted
(\code{method = NULL}), the values for all applicable loss/merit functions are
calculated and returned.  We already defined different loss/merit functions for
seriation in Section~\ref{sec:seriation}. In Table~\ref{tab:criteria} we
indicate the loss/merit functions currently available in the package.

\begin{table}[t]
\centering
\begin{tabular}{llll}
        \hline
        Name & \code{method} & merit/loss & Input data \\
        \hline
        Anti-Robinson events& \code{"AR\_events"} &
loss & \code{dist} \\
        Anti-Robinson deviations& \code{"AR\_deviations"} &
loss & \code{dist} \\
        Banded Anti-Robinson& \code{"BAR"} &
loss & \code{dist} \\
        Gradient measure& \code{"Gradient\_raw"} &
merit & \code{dist} \\
        Gradient measure (weighted)& \code{"Gradient\_weighted"} &
merit & \code{dist} \\
        Hamiltonian path length & \code{"Path\_length"} & loss & \code{dist} \\
        Inertia criterion& \code{"Inertia"} & merit & \code{dist} \\
        Least squares criterion& \code{"Least\_squares"} & loss & \code{dist} \\

        Linear Seriation criterion& \code{"LS"} & loss & \code{dist} \\
        2-Sum criterion& \code{"2SUM"} & loss & \code{dist} \\

        \hline
        Measure of effectiveness& \code{"ME"} &
merit & \code{matrix} \\
        Stress (Moore neighborhood)& \code{"Moore\_stress"} &
loss & \code{matrix} \\
        Stress (Neumann neighborhood)& \code{"Neumann\_stress"} &
loss & \code{matrix} \\
        \hline
    \end{tabular}
    \caption{Implemented loss/merit functions in function \func{criterion}.}
\label{tab:criteria}
\end{table}

All methods for \func{seriate} and \func{criterion} are managed by a
registry mechanism which makes the seriation framework easily extensible
for users.  For example, a new seriation method can be registered using
\func{set\_seriation\_method} and then used in the same way as the
built-in methods with \func{seriate}. All available methods in the
registry can be viewed using \func{list\_seriation\_methods} and
\func{show\_seriation\_methods}.  For criterion methods, the same
interface is available by just substituting `seriation' by `criterion'
in the function names. An example for how to add new methods can be
found in section~\ref{sec:registering} of this paper.

In addition the package offers the (generic) function
\begin{quotation}
\code{permute(x, order)}
\end{quotation}
where \code{x} is the data (a \code{dist} object, a matrix, an
array, a list or a numeric vector) to be reordered and \code{order} is a
\code{ser\_permutation} object of suitable length.
%The permutation for
%\code{dist} objects uses package \pkg{proxy}~\citep{seriation:Meyer:2007}.

For visualization, the package offers several options:
\begin{itemize}
\item Matrix shading with \func{pimage}. In contrast to the
        standard \func{image} in package~\pkg{graphics}, \func{pimage}
        displays the matrix as is with the first element in the top
        left-hand corner and using a gamma-corrected gray scale.

\item Different heat maps (e.g., with optimally reordered
    dendrograms) with \func{hmap}.

\item Visualization of data matrices in the spirit
of~\cite{seriation:Bertin:1981} with \func{bertinplot}.

\item \emph{Dissimilarity plot}, a new visualization to judge the
    quality of a clustering using matrix shading
    and seriation with \func{dissplot}.
\end{itemize}

We will introduce the package usage and the visualization options
in the examples in the next section.

\section{Examples and applications}
\label{sec:example}


We start this section with a simple first session to demonstrate the basic
usage of the package. Then we present and discuss several seriation
applications.

\subsection{A first session using seriation}
In the following example, we use the well known iris data set
(from \proglang{R}'s \pkg{datasets} package) which gives the
measurements in centimeters of the variables sepal length and width and petal
length and width, respectively, for 50 flowers from each of 3 species of the
iris family (Iris Setosa, Versicolor and Virginica).

First, we load the package \pkg{seriation} and the iris data set.  We
remove the species classification and reorder the objects randomly since
they are already sorted by species in the data set. Then we calculate
the Euclidean distances between objects.

<<echo=FALSE>>=
set.seed(1234)
@

<<>>=
library("seriation")

data("iris")
x <- as.matrix(iris[-5])
x <- x[sample(seq_len(nrow(x))),]
d <- dist(x)
@

To seriate the objects given the dissimilarities, we just call
\func{seriate} with the default settings.

<<>>=
o <- seriate(d)
o
@

The result is an object of class \code{ser\_permutation} for
one-mode data. The permutation vector length is $150$ for the
$150$ objects in the iris data set and the used seriation method is
\code{"ARSA"}, a simulated annealing heuristic
(see~Table~\ref{tab:methods}). The actual order can be accessed
using \func{get\_order}. In the following we show the first
15 elements in the permutation vector.

<<>>=
head(get_order(o), 15)
@


To visually inspect the effect of seriation on the distance matrix, we use
matrix shading with \func{pimage} (the result is shown in
Figure~\ref{fig:pimage1}).

<<label=pimage1, fig=TRUE, include=FALSE, width=7.5>>=
pimage(d, main = "Random")
@
<<label=pimage1-2, fig=TRUE, include=FALSE, width=7.5>>=
pimage(d, o, main = "Reordered")
@
\begin{figure}
    \centering
    \includegraphics[width=7.5cm]{seriation-pimage1}
    \includegraphics[width=7.5cm]{seriation-pimage1-2}
    \caption{Matrix shading of the  distance matrix for the iris data.}
    \label{fig:pimage1}
\end{figure}

We can also compare the improvement for different loss/merit functions
using \func{criterion}.

<<>>=
cbind(random = criterion(d), reordered = criterion(d, o))
@

Naturally, the reordered dissimilarity matrix achieves better values for all
criteria. Note that the gradient measures, inertia and the measure of
effectiveness are merit functions and for these measures larger values are
better (use \code{show\_criterion\_methods("dist")} to find out which measures
are loss and merit functions).

To visually compare the original data matrix and the
result of seriation, we can also use \func{pimage}.
We standardize the data using scale such that the visualized value
is the number of standard deviations an object differs from the
variable mean. For matrices containing negative values, \code{pimage}
uses automatically a divergent palette.
After using \func{pimage} for the original random data matrix,
we create a suitable \code{ser\_permutation} object for the original
two-mode data. Since the seriation above only produced an order for the rows
of the data, we add an identity permutation vector
for the columns (represented by \code{NA})
to the permutations object using the combine
function \func{c}. This new permutation object for $2$-mode data is used
for displaying the reordered scaled data. The two plots are shown in
Figure~\ref{fig:pimage2}.

<<label=pimage2, fig=TRUE, include=FALSE, width=7.5>>=
pimage(scale(x), main = "Random", prop = FALSE)
@

<<label=pimage2-2, fig=TRUE, include=FALSE, width=7.5>>=
o_2mode <- c(o, NA)
pimage(scale(x), o_2mode, main = "Reordered", prop = FALSE)
@

\begin{figure}
    \centering
    \includegraphics[width=7.5cm]{seriation-pimage2}
    \includegraphics[width=7.5cm]{seriation-pimage2-2}
    \caption{Matrix shading of the iris data matrix.}
    \label{fig:pimage2}
\end{figure}

\subsection{Comparing different seriation methods}

To compare different seriation methods we use again the randomized iris data
set and the distance matrix \code{d} from the previous example.  We include in
the comparison several seriation methods for dissimilarity matrices described
in Section~\ref{sec:methods}.

<<>>=
methods <- c("TSP","R2E", "ARSA", "HC", "GW", "OLO")
o <- sapply(methods, FUN = function(m) seriate(d, m))
@

<<echo=FALSE>>=
timing <- sapply(methods, FUN = function(m) system.time(seriate(d, m)),
    simplify = FALSE)
@


\begin{table}
    \centering
     \begin{tabular}{lcccccc}
     \hline
     Seriation Method &
     \Sexpr{methods[1]}&
     \Sexpr{methods[2]}&
     \Sexpr{methods[3]}&
     \Sexpr{methods[4]}&
     \Sexpr{methods[5]}&
     \Sexpr{methods[6]} \\
     \hline
     Execution time [sec] &
     \Sexpr{round(timing[[methods[1]]][1],4)}&
     \Sexpr{round(timing[[methods[2]]][1],4)}&
     \Sexpr{round(timing[[methods[3]]][1],4)}&
     \Sexpr{round(timing[[methods[4]]][1],4)}&
     \Sexpr{round(timing[[methods[5]]][1],4)}&
     \Sexpr{round(timing[[methods[6]]][1],4)}\\
     \hline
     \end{tabular}
%%% fix me: for the vignette we need something else
\caption{Execution time of seriation of the iris data set for different
    methods.}
    \label{tab:timings}
\end{table}

Table~\ref{tab:timings} contains the execution times for running
seriation with the different methods. Except for the simulated annealing
method (ARSA) the seriation only takes a fraction of a second.
The direction of the resulting orderings is first normalized (aligned)
and then the orderings are displayed using matrix shading
(see Figure~\ref{fig:pimage3}).


<<label=pimage3-pre, eval=FALSE>>=
o <- ser_align(o)
for(s in o) pimage(d, s, main = get_method(s), key = FALSE)
@

<<label=pimage3, echo=FALSE, fig=TRUE, include=FALSE>>=
o <- ser_align(o)
for(i in 1:length(o)) {
    pdf(file=paste("seriation-pimage_comp_", i , ".pdf", sep=""))
    pimage(d, o[[i]], main = get_method(o[[i]]), key = FALSE)
    dev.off()
}
@

\begin{figure}
    \centering
    \includegraphics[width=.3\linewidth]{seriation-pimage_comp_1.pdf}
    \includegraphics[width=.3\linewidth]{seriation-pimage_comp_2.pdf}
    \includegraphics[width=.3\linewidth]{seriation-pimage_comp_3.pdf}\\
    \includegraphics[width=.3\linewidth]{seriation-pimage_comp_4.pdf}
    \includegraphics[width=.3\linewidth]{seriation-pimage_comp_5.pdf}
    \includegraphics[width=.3\linewidth]{seriation-pimage_comp_6.pdf}
    \caption{Image plot of the distance matrix for the iris data
    using rearrangement by different seriation methods.}
    \label{fig:pimage3}
\end{figure}


The first row of matrices in Figure~\ref{fig:pimage3} contains the
orders obtained by a TSP solver the rank-two ellipse seriation by Chen
and using the simulated annealing method (ARSA). The results of Chen and
ARSA are very similar (except that the order is reversed). The TSP
solver produces a smoother image with some lighter lines visible.  The
reason for these lines is that the TSP only optimizes distances locally
between two neighboring objects. Therefore it is possible that in a
quite homogeneous block several objects are enclosed gradually getting
more different and then getting more similar again (see, e.g., the light
line close to the upper left corner of the TSP image in
Figure~\ref{fig:pimage3}).

The second row of Figure~\ref{fig:pimage3} contains three images based on
hierarchical clustering. The visual impression gets better from left (just
hierarchical clustering) to right (first using the Gruvaeus Wainer heuristic
and then optimal leaf ordering to rearrange the branches of the dendrogram
obtained by hierarchical clustering). The most striking feature in the
image for hierarchical clustering (HC in Figure~\ref{fig:pimage3}) is the
distinct cross going right through the center of the plot. This indicates that
several relatively dissimilar objects are caught in an otherwise homogeneous
block. This effect vanishes after rearranging the dendrogram branches
(see GW and OLO in Figure~\ref{fig:pimage3}).

%' To investigate this effect,
%' we plot the dendrogram obtained by hierarchical clustering which is used
%' to order the objects and compare it to the dendrogram rearranged
%' using the Gruvaeus Wainer heuristic.
%'
%' <<label=pimage3_dend, eval=FALSE>>=
%' plot(o[["HC"]], labels = FALSE, main = "Dendrogram HC")
%' plot(o[["GW"]], labels = FALSE, main = "Dendrogram GW")
%' @
%' <<echo=FALSE, fig=FALSE, include=FALSE, width=9>>=
%' def.par <- par(no.readonly = TRUE)
%' pdf(file="seriation-pimage3_dendrogram.pdf", width=9, height=4)
%' layout(t(1:2))
%' plot(o[["HC"]], labels = FALSE, main = "Dendrogram HC")
%' symbols(74.7,.5, rect = matrix(c(4, 3), ncol=2), add= TRUE,
%'     inches = FALSE, lwd =2)
%'
%' plot(o[["GW"]], labels = FALSE, main = "Dendrogram GW")
%' symbols(98.7,.5, rect = matrix(c(4, 3), ncol=2), add= TRUE,
%'     inches = FALSE, lwd =2)
%' par(def.par)
%' tmp <- dev.off()
%' @
%'
%' \begin{figure}
%'     \centering
%'     \includegraphics[width=\linewidth, trim=0 80 0 0, clip=TRUE]{seriation-pimage3_dendrogram}
%'     \caption{Dendrograms for the seriation with HC and GW.}
%'     \label{fig:pimage3_dendrogram}
%' \end{figure}
%'
%' Comparing the two dendrograms in Figure~\ref{fig:pimage3_dendrogram}, we see
%' that the branch left from the top is almost unchanged. The branch which is
%' responsible for the light cross in the shaded image is highlighted by a box.
%' The Gruvaeus Wainer heuristic rotates the highlighted branch towards the right
%' since the objects in it are more similar to the objects in there.

Finally, we compare the values of the loss/merit functions
for the different seriation methods.
<<>>=
crit <- sapply(o, FUN = function(x) criterion(d, x))
t(crit)
@

<<echo=FALSE, fig=TRUE, include=FALSE, label=crit1, width=6, height=8>>=
def.par <- par(no.readonly = TRUE)
m <- c("Path_length", "AR_events", "Moore_stress")
layout(matrix(seq_along(m), ncol=1))
#tmp <- apply(crit[m,], 1, dotchart, sub = m)
tmp <- lapply(m, FUN = function(i) dotchart(crit[i,], sub = i))
par(def.par)
@

\begin{figure}
    \centering
    \includegraphics[width=14cm]{seriation-crit1}
    \caption{Comparison of different methods and seriation criteria}
    \label{fig:crit1}
\end{figure}

For easier comparison, Figure~\ref{fig:crit1} contains a plot of the criteria
Hamiltonian path length, anti-Robinson events (\code{AR\_events}) and stress
using the Moore neighborhood. Clearly, the methods which directly try to
minimize the Hamiltonian path length (hierarchical clustering with optimal leaf
ordering (\code{OLO}) and the TSP heuristic) provide the best results
concerning the path length. For the number of anti-Robinson events, using the
simulated annealing heuristic (\code{ARSA}) provides the best result.  Regarding stress, the
simulated annealing heuristic also provides the best result although, it does
not directly minimize this loss function.

\subsection{Registering new methods}
\label{sec:registering}
New methods to calculate criterion values and to compute a seriation can
be easily added by the user via the method registry mechanism provided
in \pkg{seriation}. Here we give a simple example of how to implement and
register a new seriation method.

In the registry we distinguish between methods for different types of
input data.  With the following two commands we produce a list
of the available seriation methods for input data of class \code{dist}
and \code{matrix}.

<<>>=
list_seriation_methods("dist")
list_seriation_methods("matrix")
@

To get detailed information on a seriation method use the following.
<<>>=
get_seriation_method("dist", name = "ARSA")
@

To add a new seriation method, we first have to implement the seriation code as
a function with the two formal arguments \code{x} and \code{control}, and for
arrays also an additional argument \code{margin}.
\code{x} is the data
object and \code{control} contains a list with additional information for the
method passed on from \func{seriate}. The function has to return a list
of objects which can be coerced into \code{ser\_permutation\_vector}
objects (e.g., a list of integer vectors). The elements in the list
have to be in order corresponding to the dimensions of \code{x}.

In this example we just create a method to return a permutation
which reverses the original order of the objects, i.e., which returns
the reverse identity order.

<<>>=
seriation_method_reverse <- function(x, control = NULL,
                                     margin = seq_along(dim(x))) {
          lapply(seq_along(dim(x)), function(i)
            if (i %in% margin) rev(seq(dim(x)[i]))
            else NA)
}
@

The function produces integer sequences of the correct lengths,
one for each dimension of \code{x} (\code{control} is not used).
Since the function works for \code{matrix} and \code{array} we can register
it for both data types under the short name `Reverse'.

<<>>=
set_seriation_method("matrix", "New_Reverse", seriation_method_reverse,
    "Reverse identity order")

set_seriation_method("array", "New_Reverse", seriation_method_reverse,
    "Reverse identity order")
@

Now the new seriation method is registered and can be found by the user
and applied to data.
<<>>=
list_seriation_methods("matrix")

o <- seriate(matrix(1, ncol = 3, nrow = 4), "New_Reverse")
o

get_order(o, 1)
get_order(o, 2)
@


Criterion methods can be added in the same way. We refer the interested reader
to the documentation accompanying the package for detailed information and
an example.

If you have implemented a new criterion or seriation method, please consider
submitting the code to one of the maintainers of \pkg{seriation} for
inclusion in a future release of the package.

\subsection{Heat maps}

A heat map is a shaded/color coded data matrix with a dendrogram added to one
side and to the top to indicate the order of rows and columns. Typically,
reordering is done according to row or column means within the restrictions
imposed by the dendrogram. Heat maps recently became popular for visualizing
large scale genome expression data obtained via DNA microarray technology
\citep[see, e.g.,][]{seriation:Eisen:1998}.

From Section~\ref{sec:hierarchical_clustering} we know that it is possible to
find the optimal ordering of the leaf nodes of a dendrogram which minimizes
the distances between adjacent objects in reasonable time.  Such an order might
provide an improvement over using simple reordering such as the row or column
means with respect to presentation. In \pkg{seriation} we provide
the function \func{hmap} which uses optimal ordering and can also use
seriation directly on distance matrices without using hierarchical
clustering to produce dendrograms first.

For the following example, we use again the randomly reordered iris data set
\code{x} from the examples above. To make the variables (columns) comparable,
we use standard scaling.

<<>>=
x <- scale(x, center = FALSE)
@

To produce a heat map with optimally reordered dendrograms (using
by default Optimal Leaf Ordering), the function
\func{hmap} can be used with its default settings.

<<eval=FALSE>>=
hmap(x, margin = c(7, 4), cexCol = 1, row_labels = FALSE)
@

With these settings, the
Euclidean distances between rows and between columns are calculated (with
\func{dist}), hierarchical clustering (\func{hclust}) is performed, the
resulting dendrograms are optimally reordered, and \func{heatmap.2} in package
\pkg{gplots} is used for plotting
(see Figure~\ref{fig:heatmap}(a) for the resulting plot).

<<eval=FALSE>>=
hmap(x, method = "MDS")
@

If a seriation method is used that does not depend on dendrograms, instead of hierarchical clustering,
seriation on the dissimilarity matrices for rows and columns is
performed and the reordered matrix
with the reordered dissimilarity matrices to the left and on top is
displayed (see Figure~\ref{fig:heatmap}(b)). A \code{method} argument can be used to choose different
seriation methods.

<<echo=FALSE, fig=FALSE, include=FALSE>>=
#bitmap(file = "seriation-heatmap1.png", type = "pnggray",
#    height = 6, width = 6, res = 300, pointsize=14)
pdf(file = "seriation-heatmap1.pdf")
hmap(x, margin = c(7, 4), row_labels = FALSE, cexCol = 1)
tmp <- dev.off()
@
<<echo=FALSE, fig=FALSE, include=FALSE>>=
pdf(file = "seriation-heatmap2.pdf")
hmap(x, method="MDS")
tmp <- dev.off()
@

\begin{figure}
    \begin{minipage}[b]{.48\linewidth}
        \centering
        \includegraphics[width=\linewidth]{seriation-heatmap1} \\
            (a)
    \end{minipage}
    \begin{minipage}[b]{.48\linewidth}
        \centering
        \includegraphics[width=\linewidth]{seriation-heatmap2} \\
            (b)
    \end{minipage}
    \caption{Two presentations of the rearranged iris data matrix. (a) as an
optimally reordered heat map and (b) as a seriated data matrix with reordered
dissimilarity matrices to the left and on top.}
    \label{fig:heatmap}
\end{figure}


\subsection{Bertin's permutation matrix}

\cite{seriation:Bertin:1981,seriation:Bertin:1999}
introduced permutation matrices to analyze
multivariate data with medium to low sample size.  The idea is to reveal a more
homogeneous structure in a data matrix~$\mathbf{X}$ by simultaneously
rearranging rows and columns. The rearranged matrix is displayed and cases and
variables can be grouped manually to gain a better understanding of the data.

%To quantify homogeneity, a purity function
%\begin{displaymath}
%  \phi = \Phi(\mathbf{X})
%\end{displaymath}
%is defined. Let $\Pi$ be the set of all permutation functions
%$\pi$ for matrix $\mathbf{X}$.
%Note that function $\pi$ performs row and column permutations on a matrix.
%The optimal permutation with respect to
%purity can be found by
%\begin{displaymath}
%  \pi^* = \argmax\nolimits_{\pi \in \Pi} \Phi(\pi(\mathbf{X})).
%\end{displaymath}
%Since, depending on the purity function, finding the optimal
%solution can be hard, often a near optimal solution is also acceptable
%for visualization.
%
%A possible purity function $\Phi$ is:
%Given distances between rows and columns of the data matrix, define purity as
%the sum of distances of adjacent rows/columns.  Using this purity function,
%finding the optimal permutation $\pi^*$ means solving two (independent) TSPs,
%one for the columns and one for the rows.

To find a rearrangement of columns and rows which reveals structure a
purity function is used. A possible purity function is: Given distances between rows and columns of the data matrix, define purity as
the sum of distances of adjacent rows/columns. Using this purity function,
finding the optimal permutation means solving two (independent) TSPs,
one for the columns and one for the rows which can be done very conveniently
using the infrastructure provided by \pkg{seriation}.

As an example, we use the results of $8$ constitutional referenda for $41$
Irish communities~\citep{seriation:Falguerolles:1997}\footnote{The Irish data
set is included in this package. The original data and the text of the
referenda can be obtained from~\url{http://www.electionsireland.org/}}.  To
make values comparable across columns (variables), the ranks of the values for
each variable are used instead of the original values.

<<>>=
data("Irish")
orig_matrix <- apply(Irish[,-6], 2, rank)
@

For seriation, we calculate distances between rows and between columns using
the sum of absolute rank differences (this is equal to the Minkowski distance
with power $1$). Then we apply seriation (using a TSP heuristic) to both
distance matrices and combine the two resulting \code{ser\_permutation} objects
into one object for two-mode data. The original and the reordered matrix are
plotted using \func{bertinplot}.

<<>>=
o <- c(
    seriate(dist(orig_matrix, "minkowski", p = 1), method = "TSP"),
    seriate(dist(t(orig_matrix), "minkowski", p = 1), method = "TSP")
)
o
@

In a newer version of the package this can be also done with the new heatmap seriation method for matrices.
<<>>=
get_seriation_method("matrix", name = "heatmap")

o <- seriate(orig_matrix, method = "heatmap", dist_fun = function(d) dist(d, "minkowski", p = 1),
             seriation_method = "TSP")
o
@



<<eval=FALSE>>=
bertinplot(orig_matrix)
bertinplot(orig_matrix, o)
@
<<echo=FALSE, fig=TRUE, include=FALSE, label=bertin1, width=10>>=
bertinplot(orig_matrix)
@
<<echo=FALSE, fig=TRUE, include=FALSE, label=bertin2, width=10>>=
bertinplot(orig_matrix, o)
@
\begin{figure}
    \centering
    \includegraphics[width=15cm, trim=60 60 0 0]{seriation-bertin1} \\
    (a)

    \includegraphics[width=15cm, trim=60 60 0 0]{seriation-bertin2} \\
    (b)
    \caption{Bertin plot for the (a) original arrangement and the (b)
    reordered Irish data set.}
    \label{fig:bertin}
\end{figure}



The original matrix and the rearranged matrix are shown in
Figure~\ref{fig:bertin} as a matrix of bars where high values are highlighted
(filled blocks).  Note that following Bertin, the cases (communities) are
displayed as the columns and the variables (referenda) as rows.  Depending on
the number of cases and variables, columns and rows can be exchanged to obtain
a better visualization.

Although the columns are already ordered (communities in the same city
appear consecutively) in the original data matrix in
Figure~\ref{fig:bertin}(a), it takes some effort to find structure in
the data.  For example, it seems that the variables `Marriage',
`Divorce', `Right to Travel' and `Right to Information' are correlated
since the values are all high in the block made up by the columns of the
communities in Dublin.  The reordered matrix confirms this but makes the
structure much more apparent. Especially the contribution of low values
(which are not highlighted) to the overall structure becomes only
visible after rearrangement.

\subsection{Binary data matrices}

Binary or $0$-$1$ data matrices are quite common.  Often such matrices are
called \emph{incidence matrices} since a $1$ in a cell indicates the incidence
of an event. In archaeology such an event could be that a special type of
artifact was found at a certain archaeological site.
This can be seen as a simplification of a so-called \emph{abundance matrix}
which codes in each cell the (relative) frequency or quantity of an artifact
type at a site.  See \cite{seriation:Ihm:2005} for a comparison of
incidence and abundance matrices in archaeology.

Here we are interested in binary data.
For the example we use an artificial data set from~\cite{seriation:Bertin:1981}
called \emph{Townships}.  The data set contains $9$ binary characteristics
(e.g., has a veterinary or has a high school) for $16$ townships. The idea of
the data set is that townships evolve from a rural to an urban environment over
time.

After loading the data set (which comes with the package), we use
\func{bertinplot} to visualize the data (\func{pimage} could also be used
but \func{bertinplot} allows for a nicer visualization).
Bars, the standard visualization of \func{bertinplot}, do not
make much sense for binary data. We therefore use the
panel function \func{panel.squares} without spacing
to plot black squares.

<<fig=TRUE, include=FALSE, label=binary1, width=9>>=
data("Townships")

bertinplot(Townships, panel = panel.tiles)
@

The original data in Figure~\ref{fig:binary}(a) does not reveal structure in
the data. To improve the display, we run the  bond energy algorithm (BEA) for
columns and rows $10$ times with random starting points and report the best
solution.

<<echo=FALSE>>=
## to get consistent results
set.seed(10)
@

<<fig=TRUE, include=FALSE, label=binary2, width=9>>=
o <- seriate_rep(Townships, method = "BEA", criterion = "ME", rep = 10)
bertinplot(Townships, o, panel = panel.tiles)
@

The reordered matrix is displayed in Figure~\ref{fig:binary}(b).  A
clear structure is visible.  The variables (rows in a Bertin plot) can be
split into the three categories describing different evolution states of
townships:
\begin{enumerate}
    \item Rural: No doctor, one-room school and possibly also
        no water supply
    \item Intermediate: Land reallocation, veterinary and agricultural
        cooperative
    \item Urban: Railway station, high school and police station
\end{enumerate}

The townships also clearly fall into these three groups which
tentatively can be called villages (first~$7$), towns (next~5) and
cities (final~2).  The townships B and C are on the transition to the
next higher group.

\begin{figure}
    \centering
    \includegraphics[width=12cm, trim=0 40 0 30]{seriation-binary1} \\
    (a)

    \includegraphics[width=12cm, trim=0 40 0 30]{seriation-binary2} \\
    (b)

    \caption{The townships data set in original order (a) and
    reordered using BEA (b).}
    \label{fig:binary}
\end{figure}


<<>>=
rbind(
  original = criterion(Townships),
  reordered = criterion(Townships, o)
)
@

BEA tries to maximize the measure of effectiveness which is
much higher in the reordered matrix (in fact, 65 is the maximum for
the data set). Also the two types of stress are improved
significantly.

\subsection{Dissimilarity plot}

Assessing the quality of an obtained cluster solution has been a
research topic since the invention of cluster analysis. This is
especially important since all popular cluster algorithms produce a
clustering even for data without a ``cluster'' structure.

%A method to judge the quality of a cluster solution is by inspecting a
%visualization. For hierarchical clustering
%dendrogramms~\cite{seriation:Hartigan:1967} are available which show the
%hierarchical structure of the clustering as a binary tree and cluster quality
%can be judged by looking at the dissimilarities between objects in a cluster
%and objects in other clusters. However, such a visualization is
%only possible for heirarchical/nested clusterings.
%
%\marginpar{Cite Pison et al 1999 and Kaufmann and Rousseeuw}
%For the an arbitrary partitional clustering, the original objects can
%be displayed in a 2 dimensional scatter plot
%after using dimensionality reduction (e.g., PCA, MDS).
%Objects belonging to the same cluster can be marked and thus, if the
%dimensionality reduction preserves a large proportion of the
%variavility in the original data, the separation between clusters can be
%visually judged.
%
%Silhouettes

Matrix shading is an old technique to visualize clusterings by
displaying the rearranged matrices~\citep[see,
e.g.,][]{seriation:Sneath:1973,seriation:Ling:1973,seriation:Gale:1984}.
Initially matrix shading was used in connection with hierarchical
clustering, where the order of the dendrogram leaf nodes was used to
arrange the matrix.  However, with some extensions, matrix shading can
also be used with any partitional clustering method.

\cite{seriation:Strehl:2003} suggest a matrix shading visualization called
\emph{CLUSION} where the dissimilarity matrix is arranged such that all objects
pertaining to a single cluster appear in consecutive order in the matrix. The
authors call this \emph{coarse seriation}. The result of a ``good'' clustering
should be a matrix with low dissimilarity values forming blocks around the main
diagonal. However, using coarse seriation, the order of the clusters has to be
predefined and the objects within each cluster are unordered.

The dissimilarity plots implemented by the function \func{dissplot} in
\pkg{seriation} improve \emph{CLUSION} using seriation methods.  It aims
at visualizing global structure (similarity between different clusters
is reflected by their position relative to each other) as well as the
micro structure within each cluster (position of objects).

To position the clusters in the dissimilarity plot, an inter-cluster
dissimilarity matrix is calculated using the average between cluster
dissimilarities. \func{seriate} is used on this inter-cluster dissimilarity
matrix to arrange the clusters relative to each other resulting in on average
more similar clusters to appear closer together in the plot.  Within each
cluster, \func{seriate} is used again on the sub-matrix of the dissimilarity
matrix concerning only the objects in the cluster.

For the example, we use again Euclidean distance between the objects in the
iris data set.

<<>>=
data("iris")
iris <- iris[sample(seq_len(nrow(iris))), ]
x_iris <- iris[, -5]
d_iris <- dist(x_iris, method = "euclidean")
@

First, we use \func{dissplot} without a clustering. We set \code{method} to
\code{NA} to prevent reordering and display the original matrix (see
Figure~\ref{fig:dissplot1}(a)). Then we omit the method argument which results
in using the default seriation technique from \func{seriate}. Since we
did not provide a clustering, the whole matrix is reordered
in one piece.  From the result shown in Figure~\ref{fig:dissplot1}(b) it seems
that there is a clear structure in the data which suggests a two cluster
solution.

<<eval=FALSE, label=dissplot1>>=
## plot original matrix
dissplot(d_iris, method = NA)
@

<<eval=FALSE, label=dissplot2>>=
## plot reordered matrix
dissplot(d_iris, main = "Dissimilarity plot with seriation")
@

<<echo=FALSE, fig=FALSE, include=FALSE>>=
pdf(file = "seriation-dissplot1.pdf")
<<dissplot1>>
tmp <- dev.off()
pdf(file = "seriation-dissplot2.pdf")
<<dissplot2>>
tmp <- dev.off()
@


\begin{figure}
    \begin{minipage}[b]{.48\linewidth}
    \centering
    \includegraphics[width=\linewidth]{seriation-dissplot1} \\
    (a)
    \end{minipage}
    \begin{minipage}[b]{.48\linewidth}
    \centering
    \includegraphics[width=\linewidth]{seriation-dissplot2} \\
    (b)
    \end{minipage}
    \caption{Two dissimilarity plots.
    (a) the original dissimilarity matrix and
    (b) the seriated dissimilarity matrix.}
    \label{fig:dissplot1}
\end{figure}

Next, we create a cluster solution using the $k$-means algorithm.
Although we know
that the data set should contain $3$ groups representing the three species
of iris, we let $k$-means produce a $10$ cluster solution to study how such a
misspecification can be spotted using \func{dissplot}.

<<echo=FALSE>>=
set.seed(1234)
@
<<>>=
l <- kmeans(x_iris, 10)$cluster
#$
@

We create a standard dissimilarity plot by providing the cluster
solution as a vector of labels. The function rearranges the matrix and
plots the result. Since rearrangement can be a time consuming procedure for
large matrices, the rearranged matrix and all
information needed for plotting is returned as the result.

<<eval=FALSE, label=dissplot3>>=
res <- dissplot(d_iris, labels = l,
    main = "Dissimilarity plot - standard")
@
<<echo=FALSE, fig=FALSE, include=FALSE>>=
pdf(file = "seriation-dissplot3.pdf")

## visualize the clustering
<<dissplot3>>
tmp <- dev.off()


pdf(file = "seriation-dissplot4.pdf")
## threshold
plot(res, main = "Dissimilarity plot - threshold",
    threshold = 3)

tmp <- dev.off()
@

\begin{figure}
    \centering
    \includegraphics[width=10cm]{seriation-dissplot3}\\
    (a)

    \includegraphics[width=10cm]{seriation-dissplot4}\\
    (b)
    \caption{Dissimilarity plot for $k$-means solution with 10 clusters.
    (a) standard plot and (b) plot with threshold.}
    \label{fig:dissplot3}
\end{figure}

<<>>=
res
@

The resulting plot is shown in Figure~\ref{fig:dissplot3}(a).  The
inter-cluster dissimilarities are shown as solid
gray blocks and the average object
dissimilarity within each cluster as gray triangles below the main diagonal of
the matrix. Since the clusters are arranged such that more similar clusters
are closer together, it is easy to see in Figure~\ref{fig:dissplot3}(a)
that clusters 6, 3 and 1 as well as clusters 10, 9, 5, 7, 8, 4 and 2
are very similar and form two blocks.
This suggests again that a two cluster solution would
be reasonable.

Since slight variations of gray values are hard to distinguish,
we plot the matrix again (using \func{plot} on the result above) and
use a threshold on the dissimilarity to suppress high dissimilarity
values in the plot.

<<eval=FALSE>>=
plot(res, options = list(main = "Seriation - threshold",
    threshold = 3))
@

In the resulting plot in Figure~\ref{fig:dissplot3}(b), we see that the
block containing 10, 9, 5, 7, 8, 4 and 2 is very well defined and
cleanly separated from the other block. This suggests that these clusters
should form together a cluster in a solution with less clusters.
The other block is less well defined. There is considerable overlap between
clusters 6 and 3, but also cluster 3 and 1 share similar objects.

Using the information stored in the result of \func{dissplot} and
the class information available for the iris data set, we can analyze
the cluster solution and the interpretations of the dissimilarity plot.

<<>>=
#names(res)
table(iris[res$order, 5], res$label)[,res$cluster_order]
#$
@

As the plot in Figure~\ref{fig:dissplot3} indicated, the clusters 10, 9, 5, 7,
8, 4 and 2 should be a single cluster containing only flowers of the species
Iris Setosa. The clusters 6, 3 and 1 are more problematic since they contain a
mixture of Iris Versicolor and Virginica.

To illustrate the results of the dissimilarity plot in case a clustering
with a $k$ smaller than the actual number of groups in the data is used,
we use the Ruspini data set which consists of 75 points in four groups and
is also often used to illustrate clustering techniques. We load the data set,
calculate distances, perform $k$-means clustering with $k=3$ (although the
real number of groups is 4) and produce
a dissimilarity plot.

<<label=ruspini, fig=TRUE, include=FALSE>>=
data("ruspini", package = "cluster")
d <- dist(ruspini)
l <- kmeans(ruspini, 3)$cluster
dissplot(d, labels = l)
@

\begin{figure}
    \centering
    \includegraphics[width=10cm]{seriation-ruspini}\\
    \caption{Dissimilarity plot for $k$-means solution with 3 clusters
    for the Ruspini data set with 4 groups.}
    \label{fig:ruspini}
\end{figure}

The dissimilarity plot in Figure~\ref{fig:ruspini} shows that
cluster 3 actually should be two separate clusters represented
by the two clearly visible darker triangles next to the main diagonal.

The dissimilarity plot using seriation is a useful tool to inspect the result
of clustering. It is especially useful to spot
misspecifications of the number of clusters employed.
A more detailed treatment of dissimilarity plots as a tool for exploring
partitional clustering can be found in
\cite{seriation:Hahsler+Kornik:2011}.

\section{Conclusion}
\label{sec:conclusion}

In this paper we presented the infrastructure provided by the
package~\pkg{seriation}.  The infrastructure contains the necessary data
structures to store the linear order for one-, two- and $k$-mode data.
It also provides a wide array of seriation methods for different input
data, e.g., dissimilarities, binary and general data matrices focusing
on combinatorial optimization.  New seriation methods can be easily
incorporated into the \pkg{seriation} framework by the user with the
method registry mechanism provided.

Based on seriation, \pkg{seriation} features several visualization
techniques.  In particular, the optimally reordered heat map, the
Bertin plot and the dissimilarity plot present clear improvements over
standard plots.

A natural extension to \pkg{seriation} is the synthesis of ensembles of
seriations into a ``consensus'' one.  Such ensembles do not only arise
when using different seriation methods, but also when varying data or
control parameters to obtain more robust solutions (see
e.g.~\cite{seriation:Jurman:2008} for a recent application of such ideas
in a molecular profiling context).  The \proglang{R}~extension package
\pkg{relations}~\citep{seriation:Hornik+Meyer:2008} contains a variety
of methods for obtaining consensus \emph{relations}, covering consensus
seriation (where the relations are linear orders on the objects) as a
special case.

Future work on \pkg{seriation} will focus on adding further seriation
methods, such as for example methods for higher dimensional arrays and
methods for block seriation which aim at finding simultaneous partitions
of rows and columns in a data matrix~\citep[see,
e.g.,][]{seriation:Marcotorchino:1987}.

\section*{Acknowledgments}

The authors would like to thank Michael Brusco, Hans-Friedrich K{\"o}hn
and Stephanie Stahl for their seriation code, Fionn Murtagh for his BEA
implementation and the anonymous reviewers for their valuable comments
and suggestions.

%
%\bibliographystyle{abbrvnat}
\bibliography{seriation}
%
\end{document}

\documentclass[nojss]{jss}

\usepackage[english]{babel}

%\documentclass[fleqn, a4paper]{article}
%\usepackage{a4wide}
%\usepackage[round,longnamesfirst]{natbib}
%\usepackage{graphicx,keyval,thumbpdf,url}
%\usepackage{hyperref}
%\usepackage{Sweave}
\SweaveOpts{strip.white=true}
\AtBeginDocument{\setkeys{Gin}{width=0.6\textwidth}}

\usepackage[utf8]{inputenc}

%% end of declarations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\usepackage{amsmath}
\usepackage{amsfonts}


%\newcommand{\strong}[1]{{\normalfont\fontseries{b}\selectfont #1}}
\newcommand{\class}[1]{\mbox{\textsf{#1}}}
\newcommand{\func}[1]{\mbox{\texttt{#1()}}}
%\newcommand{\code}[1]{\mbox{\texttt{#1}}}
%\newcommand{\pkg}[1]{\strong{#1}}
\newcommand{\samp}[1]{`\mbox{\texttt{#1}}'}
%\newcommand{\proglang}[1]{\textsf{#1}}
\newcommand{\set}[1]{\mathcal{#1}}
\newcommand{\sQuote}[1]{`{#1}'}
\newcommand{\dQuote}[1]{``{#1}''}
\newcommand\R{{\mathbb{R}}}

\DeclareMathOperator*{\argmin}{argmin}
\DeclareMathOperator*{\argmax}{argmax}

%% almost as usual
\author{Michael Hahsler\\Southern Methodist University \And
Kurt Hornik\\Wirtschaftsuniversit\"at Wien \AND
Christian Buchta\\Wirtschaftsuniversit\"at Wien}
\title{Getting Things in Order:\\
An Introduction to the \proglang{R}~Package~\pkg{seriation}}

%% for pretty printing and a nice hypersummary also set:
\Plainauthor{Michael Hahsler, Kurt Hornik, Christian Buchta} %% comma-separated
\Plaintitle{Getting Things in Order: An Introduction to the R Package seriation} %% without formatting
\Shorttitle{Getting Things in Order} %% a short title (if necessary)

%% an abstract and keywords
\Abstract{Seriation, i.e., finding a suitable linear order for a set of
objects given data and a loss or merit function, is a basic problem in
data analysis.  Caused by the problem's combinatorial nature, it is
hard to solve for all but very small sets.  Nevertheless, both exact
solution methods and heuristics are available.  In this paper we
present the package~\pkg{seriation} which provides an infrastructure
for seriation with \proglang{R}.  The infrastructure comprises data
structures to represent linear orders as permutation vectors, a wide
array of seriation methods using a consistent interface, a method to
calculate the value of various loss and merit functions, and several
visualization techniques which build on seriation. To illustrate how
easily the package can be applied for a variety of applications, a
comprehensive collection of examples is presented.}

\Keywords{combinatorial data analysis, seriation, permutation, \proglang{R}}
\Plainkeywords{combinatorial data analysis, seriation, permutation, R} %% without formatting

\Address{
Michael Hahsler\\
    Engineering Management, Information, and Systems\\
    Lyle School of Engineering\\
    Southern Methodist University\\
    P.O. Box 750123 \\
    Dallas, TX 75275-0123\\
    E-mail: \email{mhahsler@lyle.smu.edu}\\
    URL: \url{http://lyle.smu.edu/~mhahsler}

Kurt Hornik\\
Department f\"ur Statistik \& Mathematik\\
Wirtschaftsuniversit\"at Wien\\
1090 Wien, Austria\\
E-mail: \email{kurt.hornik@wu.ac.at}\\
URL: \url{http://statmath.wu.ac.at/~hornik/}

Christian Buchta\\
Department f\"ur Welthandel\\
Wirtschaftsuniversit\"at Wien\\
1090 Wien, Austria\\
E-mail: \email{christian.buchta@wu.ac.at}\\
URL: \url{http://www.wu.ac.at/itf/institute/staff/buchta}
			    }

\hyphenation{Brusco}
\sloppy

%% \VignetteIndexEntry{An Introduction to the R package seriation}

\begin{document}

%\title{Getting Things in Order: An introduction to the
%R~package~\pkg{seriation}}
%\author{Michael Hahsler, Kurt Hornik and Christian Buchta}
\maketitle


%\abstract{Seriation, i.e., finding a suitable linear order for a set of
%  objects given data and a loss or merit function, is a basic problem in
%  data analysis.  Caused by the problem's combinatorial nature, it is
%  hard to solve for all but very small sets.  Nevertheless, both exact
%  solution methods and heuristics are available.  In this paper we
%  present the package~\pkg{seriation} which provides an infrastructure
%  for seriation with \proglang{R}.  The infrastructure comprises data
%  structures to represent linear orders as permutation vectors, a wide
%  array of seriation methods using a consistent interface, a method to
%  calculate the value of various loss and merit functions, and several
%  visualization techniques which build on seriation. To illustrate how
%  easily the package can be applied for a variety of applications, a
%  comprehensive collection of examples is presented.}
%


<<echo=FALSE>>=
options(scipen=3, digits=4)
### for sampling
set.seed(1234)
@

\section{Introduction}

A basic problem in data analysis, called \emph{seriation} or sometimes
\emph{sequencing}, is to arrange all objects in a set in a linear order
given available data and some loss or merit function in order to
reveal structural information.  Together with
cluster analysis and variable selection, seriation is an important
problem in the field of \emph{combinatorial data
  analysis}~\citep{seriation:Arabie:1996}.  Solving problems in
combinatorial data analysis requires the solution of discrete
optimization problems which, in the most general case, involves
evaluating all feasible solutions.  Due to the combinatorial nature, the
number of possible solutions grows with problem size (number of objects, $n$)
by the order~$O(n!)$.  This makes a brute-force enumerative approach
infeasible for all but very small problems.  To solve larger problems
(currently with up to 40 objects), partial enumeration methods can be used.  For
example, \cite{seriation:Hubert:2001} propose dynamic programming and
\cite{seriation:Brusco:2005} use a branch-and-bound strategy.  For even
larger problems only heuristics can be employed.

It has to be noted that seriation has a rich history in archeology.
\cite{seriation:Petrie:1899} was the first to use seriation as a formal method.
He applied it to find a chronological order for graves discovered in the Nile
area given objects found there. He used a cross-tabulation of grave sites and
objects and rearranged the table using row and column permutations till all
large values were close to the diagonal. In the rearranged table graves with
similar objects are closer to each other. Together with the assumption that
different objects continuously come into and go out of fashion, the order of
graves in the rearranged table suggests a chronological order.
Initially, the rearrangement of rows and columns of this
contingency table was done manually and the adequacy was only judged
subjectively by the researcher.  Later, \cite{seriation:Robinson:1951},
\cite{seriation:Kendall:1971} and others proposed measures of agreement
between rows to quantify optimality of the resulting table.  A
comprehensive description of the development of seriation in archeology
is presented by \cite{seriation:Ihm:2005}.

Techniques related to seriation are also popular in several other
fields.  Especially in ecology scaling techniques are used under the
name \emph{ordination}.  For these applications several \proglang{R} packages
already exist (e.g.,
\pkg{ade4}~\citep{seriation:Chessel:2007,seriation:Dray:2007} and
\pkg{vegan}~\citep{seriation:Oksanen:2007}). This paper describes the new
package \pkg{seriation} which differs from existing packages in the
following ways:
\begin{itemize}
 \item \pkg{seriation} provides a flexible infrastructure for seriation;
 \item \pkg{seriation} focuses on seriation as a combinatorial
  optimization problem.
\end{itemize}

This paper starts with a formal introduction of the seriation problem as
a combinatorial optimization problem in Section~\ref{sec:seriation}.  In
Section~\ref{sec:methods} we give an overview of seriation methods.  In
Section~\ref{sec:infrastructure} we present the infrastructure provided
by the package~\pkg{seriation}.  Several examples and applications for
seriation are given in Section~\ref{sec:example}.
Section~\ref{sec:conclusion} concludes.

A previous version of this manuscript was published in the \emph{Journal
  of Statistical Software} \citep{seriation:Hahsler+Hornik:2008}.


\section{Seriation as a combinatorial optimization problem}
\label{sec:seriation}

To seriate a set of $n$ objects $\{O_1,\dots,O_n\}$ one typically starts
with an $n \times n$ symmetric dissimilarity matrix~$\mathbf{D} =
(d_{ij})$ where $d_{ij}$ for $1 \le i,j \le n$ represents the
dissimilarity between objects $O_i$ and $O_j$, and $d_{ii} = 0$ for
all~$i$.  We define a permutation function $\Psi$ as a function which
reorders the objects in $\mathbf{D}$ by simultaneously permuting rows
and columns.  The seriation problem is to find a permutation function
$\Psi^*$
%$\{1,\dots,n\} \rightarrow \{1,\dots,n\}$, i.e. a
%bijection that maps the set of indices of the objects (and equally of rows and
%columns of $\mathbf{D}$) onto itself,
which optimizes the value of a given loss function~$L$ or merit function~$M$.
This results in the optimization problems
\begin{equation}
    \Psi^* = \argmin_\Psi L(\Psi(\mathbf{D})) \quad \text{or} \quad
    \Psi^* = \argmax_\Psi M(\Psi(\mathbf{D})),
\end{equation}
respectively.
%This is clearly a hard discrete optimization problem since the number of
%possible permutations is $n!$ which makes an exhaustive
%search for sets with a medium to large number of objects infeasible.
%Partial enumeration methods and heuristics can be used. Such methods are
%presented in Section~\ref{sec:methods}.
%But first, we review commonly used loss functions in the following section.
%\marginpar{two-mode data missing}

A symmetric dissimilarity matrix is known as \emph{two-way one-mode}
data since it has columns and rows (two-way) but only represents one set
of objects (one-mode).  Seriation is also possible for two-way two-mode
data which are represented by a general nonnegative matrix.  In such data
columns and rows represent two sets of objects which are reordered
simultaneously.  For loss/merit functions for two-way two-mode data the
optimal order of columns can depend of the order of rows and vice
versa or it can be independent allowing for breaking the optimization
down into two separate problems, one for the columns and one for the
rows.  Another way to deal with the seriation for two-way two-mode data
is to calculate two dissimilarity matrices, one for each mode, and then
solve two seriation problems for two-way one-mode data.  Furthermore,
seriation can be generalized to $k$-way $k$-mode data in the form of a
$k$-dimensional array by defining suitable loss/merit functions for such data
or by breaking the problem down into several lower dimensional
independent problems.

To assess the complexity of seriation of $k$-way $k$-mode data, let us
assume the data is a $k$-dimensional array with the dimensions
containing $n_1, n_2, \ldots, n_k$ objects. If the loss/merit function allows
for separating the problem into $k$ independent problems, the problem
size is just the sum of the individual problems. By using complete
enumeration the size is $O(\sum_{i=1}^k{n_i!})$. If the problem is not
separable and the optimal seriation of each dimension depends on the
order of the objects of the other dimensions, the problem size is
$O((\sum_{i=1}^k{n_i})!)$. For example for $k=5$ and all dimensions
containing 5 objects, the search space for separable dimensions is only
600 while without separability it is larger than $10^{25}$ clearly too
big to be solvable in reasonable time. This shows that for data with
even only a few dimensions and a few objects each, finding the optimal
solution is infeasible and loss/merit functions which allow for separating the
problem are highly desirable.

In the following subsections, we review some commonly employed loss/merit
functions.  Most functions are used for two-way one-mode data but the
measure of effectiveness and stress can be also used for two-way
two-mode data. For the implementation of various loss or merit measures
see function~\func{criterion} in Section~\ref{sec:infrastructure}.



%\section{Loss functions}
%\label{sec:criteria}
%In the literature several loss functions are suggested.
%We review the most commonly used functions.

\subsection{Column/row gradient measures}

A symmetric dissimilarity matrix where the values in all rows and
columns only increase when moving away from the main diagonal is called
a perfect \emph{anti-Robinson matrix} after the statistician
\cite{seriation:Robinson:1951}. Formally, an $n \times n$ dissimilarity
matrix $\mathbf{D}$ is in anti-Robinson form if and only if the
following two gradient conditions hold~\citep{seriation:Hubert:2001}:
\begin{align}
    \text{within rows:} & \quad  d_{ik} \le d_{ij} \quad \text{for}
        \quad 1 \le i < k < j \le n; \\
    \text{within columns:} & \quad  d_{kj} \le d_{ij} \quad \text{for}
        \quad 1 \le i < k < j \le n.
\end{align}

In an anti-Robinson matrix the smallest dissimilarity values appear
close to the main diagonal, therefore, the closer objects are together
in the order of the matrix, the higher their similarity.  This provides
a natural objective for seriation.

It has to be noted that $\mathbf{D}$ can be brought into a perfect
anti-Robinson form by row and column permutation whenever $\mathbf{D}$  is an
ultrametric or $\mathbf{D}$ has an exact Euclidean representation in a single
dimension~\citep{seriation:Hubert:2001}. However, for most data only an
approximation to the anti-Robinson form is possible.

A suitable merit measure which quantifies the divergence of a matrix from the
anti-Robinson form was given by \cite{seriation:Hubert:2001} as
\begin{equation}
    M(\mathbf{D}) =
    \sum_{i<k<j}f(d_{ik}, d_{ij}) + \sum_{i<k<j}f(d_{kj}, d_{ij})
    \label{equ:gradient}
\end{equation}
where $f(\cdot,\cdot)$ is a function which defines how a violation or
satisfaction of a gradient condition for an object triple ($O_i$, $O_k$ and
$O_j$) is counted.  \cite{seriation:Hubert:2001} suggest two functions. The
first function is given by:
\begin{equation}
    f(z,y) = \mathrm{sign}(y-z) =
    \begin{cases}
        +1 \quad \text{if} \quad z < y; \\
        \phantom{+}0 \quad \text{if} \quad z = y; \\
        -1 \quad \text{if} \quad z > y.
    \end{cases}
\end{equation}
It results in the raw number of triples satisfying the gradient
constraints minus triples which violate the constraints.

The second function is defined as:
\begin{equation}
    f(z,y) = |y-z|\mathrm{sign}(y-z) = y-z
\end{equation}
It weighs each satisfaction or violation by its
magnitude given by the absolute difference between the values.

\subsection{Anti-Robinson events}
An even simpler loss function can be created in the same way as the gradient
measures above by concentrating on violations only.
\begin{equation}
    L(\mathbf{D}) =
    \sum_{i<k<j}f(d_{ik}, d_{ij}) + \sum_{i<k<j}f(d_{kj}, d_{ij})
\end{equation}

To only count the violations we use
\begin{equation}
    f(z, y) = I(z, y) =
    \begin{cases}
        1 \quad \text{if} \quad z > y \quad \text{and} \\
        0 \quad \text{otherwise.}
    \end{cases}
\end{equation}
$I(\cdot)$ is an indicator function returning $1$ only for violations.
\cite{seriation:Chen:2002} presented a formulation for an equivalent
loss function and called the violations \emph{anti-Robinson events}.
\cite{seriation:Chen:2002} also introduced a weighted versions of the loss
function resulting in
\begin{equation}
    f(z, y) = |y-z|I(z, y)
\end{equation}
using the absolute deviations as weights.

\subsection{Hamiltonian path length}
The dissimilarity matrix $\mathbf{D}$ can be represented as a finite weighted
graph $G = (\Omega,E)$ where the set of objects~$\Omega$ constitute the
vertices and each edge~$e_{ij} \in E$ between the objects $O_i, O_j \in \Omega$
has a weight~$w_{ij}$ associated which represents the dissimilarity~$d_{ij}$.

Such a graph can be used for seriation~\citep[see,
e.g.,][]{seriation:Hubert:1974,seriation:Caraux:2005}.  An order~$\Psi$
of the objects can be seen as a path through the graph where each node
is visited exactly once, i.e., a Hamiltonian path.  Minimizing the
Hamiltonian path length results in a seriation optimal with respect to
dissimilarities between neighboring objects.  The loss function based on
the Hamiltonian path length is:
\begin{equation}
    L(\mathbf{D}) = \sum_{i=1}^{n-1} d_{i,i+1}.
\end{equation}

Note that the length of the Hamiltonian path is equal to the value of the
\emph{minimal span loss function} \citep[as used by][]{seriation:Chen:2002},
and both notions are related to the \emph{traveling salesperson
problem}~\citep{seriation:Gutin:2002}.

\subsection{Inertia criterion}
Another way to look at the seriation problem is not to focus on placing small
dissimilarity values close to the diagonal, but to push large values away from
it. A function to quantify this is the moment of inertia of dissimilarity
values around the diagonal \citep{seriation:Caraux:2005} defined as
\begin{equation}
    M(\mathbf{D}) = \sum_{i=1}^n \sum_{j=1}^n d_{ij}|i-j|^2.
\end{equation}
$|i-j|^2$ is used as a measure for the distance to the diagonal and $d_{ij}$
gives the weight.  This is a merit function since the sum increases when higher
dissimilarity values are placed farther away from the diagonal.

\subsection{Least squares criterion}
Another natural loss function for seriation is to quantify the deviations
between the dissimilarities in $\mathbf{D}$ and the rank differences of the
objects.  Such deviations can be measured, e.g, by the sum of squares of
deviations \citep{seriation:Caraux:2005} defined by
\begin{equation}
    L(\mathbf{D}) = \sum_{i=1}^n \sum_{j=1}^n (d_{ij} - |i-j|)^2,
\end{equation}
where $|i-j|$ is the rank difference or gap between $O_i$ and $O_j$.

The least squares criterion defined here is related to uni-dimensional
scaling~\citep{seriation:Leeuw:2005}, where the objective is to place all
$n$ objects on a straight line using a position vector~$\mathbf{z} =
z_1,z_2,\ldots,z_n$ such that the dissimilarities
in $\mathbf{D}$ are
preserved by the relative positions in the best possible way. The optimization
problem of uni-dimensional scaling is to find the
position vector~$\mathbf{z^*}$ which minimizes $\sum_{i=1}^n \sum_{j=1}^n
(d_{ij} - |z_i-z_j|)^2$.
This is close to the seriation problem, but in
addition to the ranking of the objects also takes the distances between objects
on the resulting scale into account.

Note that if Euclidean distance is used to calculate $\mathbf{D}$ from a data
matrix~$\mathbf{X}$, using the order of the elements in $\mathbf{X}$ as they
occur projected on the first principal component of $\mathbf{X}$
minimizes the loss function of uni-dimensional scaling (using squared
distances). Using this order, also provides a good solution
for the least square seriation criterion.

\subsection{Linear Seriation Criterion}
The Linear Seriation Criterion (Hubert and Schultz 1976)
weights the distances with the absolute rank differences.

$$L(\mathbf{D}) \sum_{i=1}^n \sum_{j=1}^n d_{ij} (-|i-j|)$$

\subsection{2-Sum Problem}
The 2-Sum loss criterion \citep{seriation:Barnard:1993}
multiplies the similarity between objects
with the squared rank differences.

$$L(\mathbf{D}) \sum_{i,j=1}^p \frac{1}{1+d_{ij}} (i-j)^2,$$

where $s_{ij} = \frac{1}{1+d_{ij}}$ represents the similarity between
objects $i$ and $j$.


\subsection{Measure of effectiveness}
\label{sec:ME}

\cite{seriation:McCormick:1972} defined the
\emph{measure of effectiveness (ME)} for an $n \times m$ matrix~$\mathbf{X} =
(x_{ij})$ as
\begin{equation}
    M(\mathbf{X}) =
    \frac{1}{2}
    \sum_{i=1}^{n} \sum_{j=1}^{m} x_{ij}[x_{i,j+1}+x_{i,j-1}+
        x_{i+1,j}+x_{i-1,j}]
    \label{equ:ME}
\end{equation}
with, by convention $x_{0,j}=x_{n+1,j}=x_{i,0}=x_{i,m+1}=0$.  ME is maximized
if each element is as closely related numerically to its four neighboring
elements as possible.

ME was developed for two-way two-mode data, however, ME can also be used for a
symmetric  matrix (one-mode data) and gets maximal only if all large values are
grouped together around the main diagonal.

Note that the definition in equation~(\ref{equ:ME})
can be rewritten as
\begin{equation}
    M(\mathbf{X}) =
    \frac{1}{2}
    \sum_{i=1}^{n} \sum_{j=1}^{m} x_{ij}[x_{i,j+1}+x_{i,j-1}] +
    \sum_{i=1}^{n} \sum_{j=1}^{m} x_{ij}[x_{i+1,j}+x_{i-1,j}]
\end{equation}
showing that the contributions of column and row order to the merit function
are independent.

\subsection{Stress}
\label{sec:stress}

Stress measures the conciseness of the presentation of a matrix
(two-mode data) and can be seen as a purity function which compares the
values in a matrix with their neighbors. The stress measures used here
are computed as the sum of squared distances of each matrix entry from
its adjacent entries.  \cite{seriation:Niermann:2005} defined for an $n
\times m$ matrix~$\mathbf{X} = (x_{ij})$ two types of neighborhoods:

\begin{itemize}
    \item The Moore neighborhood comprises the (at most) eight adjacent entries.
        The local stress measure for element~$x_{ij}$ is defined as
        \begin{equation}
            \sigma_{ij} = \sum_{k=\max(1,i-1)}^{\min(n,i+1)}
                \sum_{l=\max(1,j-1)}^{\min(m,j+1)}
                (x_{ij} - x_{kl})^2
        \end{equation}

    \item The Neumann neighborhood comprises the (at most) four adjacent entries
        resulting in the local stress of $x_{ij}$ of
        \begin{equation}
            \sigma_{ij} =
            \sum_{k=\max(1,i-1)}^{\min(n,i+1)} (x_{ij} - x_{kj})^2 +
            \sum_{l=\max(1,j-1)}^{\min(m,j+1)} (x_{ij} - x_{il})^2
            %(x_{ij} - x(i-1,j))^2 + (x_{ij} - x(i+1,j))^2 +
            %(x_{ij} - x(i,j-1))^2 + (x_{ij} - x(i,j+1))^2
        \end{equation}
\end{itemize}

Both local stress measures can be used to construct a global measure for the
whole matrix by summing over all entries which can be used as a loss function:
\begin{equation}
    L(\mathbf{X}) =
    \sum_{i=1}^n \sum_{j=1}^m \sigma_{ij}
\end{equation}

The major difference between the Moore and the Neumann neighborhood is
that for the later the contributions of row and column order to
stress are independent.

Stress can be also used as a loss function for
symmetric proximity matrices (one-mode data).
%,
%since it can only be optimal, if large values are
%concentrated around the main diagonal.
Note also, that stress with Neumann
neighborhood is related to the measure of effectiveness defined above (in
Section~\ref{sec:ME}) since both measures are optimal if for each cell the cell
and its four neighbors are numerically as similar as possible.

\section{Seriation methods}
\label{sec:methods}

Solving the discrete optimization problem for seriation with most loss/merit
functions is clearly very hard. The number of possible permutations for $n$
objects is $n!$ which makes an exhaustive search for sets with a medium to
large number of objects infeasible.  In this section, we describe some methods
(partial enumeration, heuristics and other methods) which are typically used
for seriation. For each method we state for which type of loss/merit functions
it is suitable and whether it finds the optimum or is a heuristic.  For the
implementation of various seriation methods see function~\func{seriate} in
Section~\ref{sec:infrastructure}.


\subsection{Partial enumeration methods}

Partial enumeration methods search for the exact solution of a
combinatorial optimization problem.  Exploiting properties of the search
space, only a subset of the enormous number of possible combinations has
to be evaluated.  Popular partial enumeration methods which are used for
seriation are \emph{dynamic programming}~\citep{seriation:Hubert:2001}
and \emph{branch-and-bound}~\citep{seriation:Brusco:2005}.

Dynamic programming recursively searches for the optimal solution checking and
storing $2^n-1$ results. Although $2^n-1$ grows at a lower rate than $n!$ and
is for $n \gg 3$ considerably smaller, the storage requirements of $2^n-1$
results still grow fast, limiting the maximal problem size severely. For
example, for $n=30$ more than one billion results have to be calculated and
stored, clearly a number too large for the main memory capacity of most current
computers.

Branch-and-bound has only very moderate storage requirements. The
forward-branching procedure~\citep{seriation:Brusco:2005} starts to build
partial permutations from left (first position) to right. At each step, it is
checked if the permutation is valid and several fathoming tests are performed
to check if the algorithm should continue with the partial permutation.  The
most important fathoming test is the boundary test, which checks if the partial
permutation can possibly lead to a complete permutation with a better solution
than the currently best one.  In this way large parts of the search space can
be omitted. However, in contrast to the dynamic programming approach, the
reduction of search space is strongly data dependent and poorly structured
data can lead to very poor performance.  With branch-and-bound slightly larger
problems can be solved than with dynamic programming in reasonable time.
\cite{seriation:Brusco:2005} state that depending on the data, in some cases
proximity matrices with 40 or more objects can be handled with current
hardware.

Partial enumeration methods can be used to find the exact solution
independently of the loss/merit function. However, partial enumeration is
limited to only relatively small problems.

\subsection{Traveling salesperson problem solver}

Seriation by minimizing the length of a Hamiltonian path through a graph
is equal to solving a traveling salesperson problem.  The traveling
salesperson or salesman problem (TSP) is a well known and well
researched combinatorial optimization
problem~\citep[see, e.g.,][]{seriation:Gutin:2002}. The goal is to find the
shortest tour that, starting from a given city, visits each city in a
given list exactly once and then returns to the starting city.
In graph theory a TSP tour is called
a \emph{Hamiltonian cycle.} But for the seriation problem, we are
looking for a Hamiltonian path. \cite{seriation:Garfinkel:1985}
described a simple transformation of the TSP to find the shortest
Hamiltonian path.  An additional row and column of 0's is added
(sometimes this is referred to as a \emph{dummy city}) to the original
$n \times n$ dissimilarity matrix~$\mathbf{D}$.  The solution of this
$(n+1)$-city TSP, gives the shortest path where the city representing
the added row/column cuts the cycle into a linear path.

As the general seriation problem, solving the TSP is difficult.  In the
seriation case with $n+1$ cities, $n!$ tours have to be checked. However,
despite this vast searching space, small instances can be solved efficiently
using dynamic programming \citep{seriation:Held:1962} and larger instances of
several hundred objects can be solved using \emph{branch-and-cut}
algorithms~\citep{seriation:Padberg:1990}. For even larger instances or if
running time is critical, a wide array of heuristics are available, ranging
from simple nearest neighbor approaches to construct a
tour~\citep{seriation:Rosenkrantz:1977} to complex heuristics like the
Lin-Kernighan heuristic~\citep{seriation:Lin:1973}.  A comprehensive overview
of heuristics and exact methods can be found in \cite{seriation:Gutin:2002}.


\subsection{Bond energy algorithm}

The \emph{bond energy algorithm}~\citep[BEA;][]{seriation:McCormick:1972} is a
simple heuristic to rearrange columns and rows of a matrix
(two-way two-mode data) such that each entry
is as closely numerically related to its four neighbors as possible.  To
achieve this, BEA tries to maximize the measure of effectiveness (ME) defined
in Section~\ref{sec:ME}.  For optimizing the ME, columns and rows can be
treated separately since changing the order of rows does not influence the ME
contributions of the columns and vice versa. BEA consists of the
following three steps:
\begin{enumerate}
 \item Place one randomly chosen column.
 \item Try to place each remaining column at each possible position
  left, right and between the already placed columns and calculate every
  time the increase in ME.  Choose the column and position which gives
  the largest increase in ME and place the column.  Repeat till all
  columns are placed.
 \item Repeat procedure with rows.
\end{enumerate}

This greedy algorithm works fast and only depends on the choice of the
first column/row. This dependence can be reduced by repeating the
procedure several times with different choices and returning the solution
with the highest ME.

Although \cite{seriation:McCormick:1972} use BEA also for non-binary data,
\cite{seriation:Arabie:1990} argue that the measure of effectiveness only
serves its intended purpose of finding an arrangement which is
close to Robinson form for binary data and should therefore only be
used for binary data.

\cite{seriation:Lenstra:1974} notes that the optimization problem of BEA
can be stated as two independent traveling salesperson problems (TSPs).
For example, the row TSP for an $n \times m$ matrix~$\mathbf{X}$
consists of $n$ cities with an $n \times n$ distance matrix~$\mathbf{D}$
where the distances are
\begin{displaymath}
    d_{ij} = -\sum_{k=1}^m x_{ik}x_{jk}.
\end{displaymath}
BEA is in fact a simple suboptimal TSP heuristic using this distances
and instead of BEA any TSP solver can be used to obtain an order.
With an exact TSP solver, the optimal solution can be found.

\subsection{Hierarchical clustering}
\label{sec:hierarchical_clustering}

Hierarchical clustering produces a series of nested clusterings which
can be visualized by a dendrogram, a tree where each internal node
represents a split into subtrees and has a measure of
similarity/dissimilarity attached to it.  As a simple heuristic to find
a linear order of objects, the order of the leaf nodes in a dendrogram
structure can be used. This idea is used, e.g., by heat maps to reorder
rows and columns with the aim to place more similar objects and
variables closer together.

%For hierarchical clustering several methods are available (e.g.,
%single linkage, average linkage, complete linkage, ward method) resulting in
%different dendrograms.
%However,
The order of leaf nodes in a dendrogram is not unique.  A binary
(two-way splits only) dendrogram for $n$ objects has $2^{n-1}$ internal
nodes and at each internal node the left and right subtree (or leaves)
can be swapped resulting in $2^{n-1}$ distinct leaf orderings.  To find
a unique or optimal order, an additional criterion has to be defined.
\cite{seriation:Gruvaeus:1972} suggest to obtain a unique order by
requiring to order the leaf nodes such that at each level the objects at
the edge of each cluster are adjacent to that object outside the cluster
to which it is nearest.

\cite{seriation:Bar-Joseph:2001} suggest to rearrange the dendrogram such that
the Hamiltonian path connecting the leaves is minimized and called this the
optimal leaf order. The authors also present a fast algorithm with time
complexity $O(n^4)$ to solve this optimization problem. Note that this problem
is related to the TSP described above, however, the given dendrogram structure
significantly reduces the number of permissible permutations making the problem
easier.

Although hierarchical clustering solves an optimization problem different to
the seriation problem discussed in this paper, hierarchical clustering still
can produce useful orderings, e.g., for visualization.

\subsection{Rank-two ellipse seriation}

\cite{seriation:Chen:2002} proposes to
generate a sequence of correlation matrices
$R^1, R^2, \ldots$. $R^1$ is the correlation matrix
of the original distance matrix $\mathbf{D}$ and
\begin{equation}
R^{n+1} = \phi R^n,
\end{equation}
where $\phi(\cdot)$ calculates a correlation matrix.

\cite{seriation:Chen:2002} shows that the
rank of the matrix $R^n$ falls with increasing $n$ and that if the sequence
is continued till the first matrix in the sequence has a rank of 2,
projecting all points in this matrix on its first two eigenvectors,
all points will fall on an ellipse.
\cite{seriation:Chen:2002} suggests to use the order of the points on
this ellipse as a seriation where the ellipse can be cut at any of the
two interception points (top or bottom) with the vertical axis.

Although the rank-two ellipse seriation procedure does not try to solve a
combinatorial optimization problem, it still provides for some cases a useful
ordering.

\subsection{Spectral Seriation}
Spectral seriation uses a relaxation to minimize the 2-Sum Problem
\citep{seriation:Barnard:1993}.
Rewriting the minimization problem using a permutation vector $\pi$,
its inverse, rescaling to $\mathrm{q}$ and using a Lagrangian
multiplier for the constraint on the permutation yields \citep{seriation:Ding:2004} the following equivalent optimization problem:

$$\mathrm{min}_\mathbf{q} \frac{\mathbf{q}^T L_\mathbf{S}\mathbf{q}}{\mathbf{q}^T\mathbf{q}}$$

where $L_\mathbf{S}$ is the Laplacian of $\mathbf{S}$.

The optimal order can be recovered by the sorting order of
the Fiedler vector (i.e., the second smallest eigenvector of the Laplacian
of the similarity matrix).

\subsection{Quadratic Assignment Problem}
Both, the linear seriation criterion and the 2-Sum problem formulation
can be written as a Quadratic Assignment Problem (QAP). However,
the QAP is in general NP-hard. Methods include
QIP, linearization, branch and bound and cutting planes as well as
heuristics including Tabu search, simulated annealing, genetic algorithms, and
ant systems \citep{seriation:Burkard:1998}.

\section{The package infrastructure}
\label{sec:infrastructure}

The \pkg{seriation} package provides the data structures and some algorithms
to efficiently handle seriation with \proglang{R}. As the
input data for seriation
\proglang{R}
already provides

\begin{itemize}
    \item for two-way one-mode data the class \code{dist},
    \item for two-way two-mode data the class \code{matrix}, and
    \item for $k$-way $k$-mode data the class \code{array}.
\end{itemize}


\begin{figure}[tp]
    \centerline{
    %\includegraphics[width=12cm]{infrastructure}}
    \includegraphics[width=10cm]{classes}}

    \caption{UML class diagram of the data structures for permutations provided by \pkg{seriation}}
    \label{fig:infrastructure}
\end{figure}

However, \proglang{R} provides no classes for representing permutation vectors.
\pkg{seriation} adds the necessary data structure (using the S3 class
system) as depicted in the UML class diagram \citep{seriation:Fowler:2004} in
Figure~\ref{fig:infrastructure}. In this diagram classes are represented by
rectangles and different symbols are used to state the type of
relationship between the classes. The class
\code{ser\_permutation}
in Figure~\ref{fig:infrastructure}
represents the permutation information for $k$-mode
data (including the cases of $k=1$ and $k=2$). It consists of $k$ permutation
vectors (class \code{ser\_permutation\_vector}). This relationship is
represented by the solid diamond and the star above the connection between the
two classes. Class \code{ser\_permutation\_vector} is defined \emph{abstract}
and only its concrete implementations (classes connected with the triangle
symbol) are used to store a permutation vector.
This design with an abstract class was chosen to allow to use
different representations for the permutation vectors.
Currently, the permutation vector can be stored as a simple
integer vector or as an object of class \code{hclust} (defined in
package \pkg{stats}). \code{hclust} describes a hierarchical clustering
tree (dendrogram) including an ordering for the tree's node leaves which
provides a permutation for all objects (see
Section~\ref{sec:hierarchical_clustering}).

Class \code{ser\_permutation\_vector} has a constructor
\func{ser\_permutation\_vector} which converts data into the correct concrete
subclass of \code{ser\_permutation\_vector} and checks if it contains a proper
permutation vector.  For \code{ser\_permutation\_vector} the methods
\func{print}, \func{length} for the length of the permutation vector,
\func{get\_method} to get the method used to generate the permutation, and
\func{get\_order} to access the raw (integer) permutation vector are available.
To use an additional class to represent permutations as a concrete subclass of
\code{ser\_permutation\_vector} only an appropriate accessor method
\func{get\_order} has to be implemented for the new class.

For \code{ser\_permutation} a constructor is provided which can bind $k$
\code{ser\_permutation\_vector} objects together into an object for $k$-mode
data. \code{ser\_permutation} is implemented as a list of length~$k$ and each
element contains a \code{ser\_permutation\_vector} object.  Methods like
\func{length}, accessing elements with \code{[[},
% ]]
\code{[[<-},
% ]]
subsetting with \code{[}, and combining with \func{c} work as expected.
Also a \func{print} method is provided. Finally, direct access to the
raw permutation vectors is available using \func{get\_order}.  Here a
second argument (which defaults to $1$) specifies the dimension (mode)
for which the order vector is requested.

All seriation algorithms are available via the function \func{seriate}
defined as:
\begin{quotation}
\code{seriate(x,  method = NULL, control = NULL, ...)}
\end{quotation}
where \code{x} is the input data, \code{method} is a string defining the
seriation method to be used and \code{control} can contain a list with
additional information for the algorithm.  \func{seriate} returns an object
of class \code{ser\_permutation} with a length conforming to the number of
dimensions of~\code{x}.
Typical input data are a dissimilarity matrix (class~\code{dist};
see package \pkg{stats} for more information) for one-mode two-way data,
\code{matrix} for two-mode two-way data and \code{array} for $k$-mode $k$-way
data.
For \code{matrix} and \code{array} the additional argument
\code{margin} can be used to restrict the dimensions which should be seriated
(e.g., with \code{margin = 1} only the first dimension,
i.e., the columns of a matrix, are seriated).

%\begin{landscape}
\begin{table}[tp]
\centering
    \begin{tabular}{p{5cm}p{3cm}p{4cm}l}
        \hline
Algorithm & \code{method} & Optimizes & Input data \\
        \hline
Simulated annealing & \code{"ARSA"} & Linear seriation crit.&\code{dist} \\
Branch-and-bound & \code{"BBURCG"} & Gradient measure &\code{dist} \\
Branch-and-bound & \code{"BBWRCG"} & Gradient measure (weighted)& \code{dist} \\
TSP solver & \code{"TSP"} & Hamiltonian path length& \code{dist} \\
Optimal leaf ordering  & \code{"OLO"}
\code{"OLO_single"}
\code{"OLO_average"}
\code{"OLO_complete"}
& Hamiltonian path length (restricted)& \code{dist} \\
Gruvaeus and Wainer & \code{"GW"}
\code{"GW_single"}
\code{"GW_average"}
\code{"GW_complete"}
& Hamiltonian path length (restricted) & \code{dist} \\
MDS & \code{"MDS"}
\code{"MDS_metric"}
\code{"MDS_nonmetric"}
\code{"MDS_angle"}
& Least square crit.& \code{dist} \\

Spectral seriation & \code{"Spectral"}
\code{"Spectral_norm"}
& 2-Sum crit. & \code{dist} \\

QAP & \code{"QAP_2SUM"}
& 2-Sum crit. & \code{dist} \\

 & \code{"QAP_LS"}
& Linear seriation crit. & \code{dist} \\

 & \code{"QAP_BAR"}
& Banded AR form  & \code{dist} \\

 & \code{"QAP_Inertia"}
& Inertia crit. & \code{dist} \\

	Genetic Algorithm & \code{"GA"}*
& various & \code{dist} \\

	DendSer & \code{"DendSer"}*
& various & \code{dist} \\

Hierarchical clustering & \code{"HC"}
\code{"HC_single"}
\code{"HC_average"}
\code{"HC_complete"}
& Other& \code{dist} \\
Rank-two ellipse seriation  & \code{"R2E"} & Other& \code{dist} \\
Sorting Points Into Neighborhoods & \code{"SPIN_NH"}
\code{"SPIN_STS"} & Other& \code{dist} \\
Visual Assessment of (Clustering) Tendency & \code{"VAT"}& Other& \code{dist} \\
        \hline
Bond Energy Algorithm & \code{"BEA"} &
    Measure of effectiveness & \code{matrix} \\
TSP to optimize ME & \code{"BEA\_TSP"} &
    Measure of effectiveness& \code{matrix} \\
Principal component analysis& \code{"PCA"}
\code{"PCA_angle"}&
    Least square crit.& \code{matrix} \\
        \hline
    \end{tabular}
    \caption{Currently implemented methods for \func{seriation} (* methods need to be registered).}
\label{tab:methods}
\end{table}
%\end{landscape}

Various seriation methods were already introduced in this paper in
Section~\ref{sec:methods}. In Table~\ref{tab:methods} we summarize the
methods currently available in the package for seriation.  The code for
the simulated annealing heuristic~\citep{seriation:Brusco:2007} and the
two branch-and-bound implementations~\citep{seriation:Brusco:2005} was
obtained from the authors.  The TSP solvers (exact solvers and a variety
of heuristics) is provided by package
\pkg{TSP}~\citep{seriation:Hahsler:2007, seriation:Hahsler:2007b}.  For
optimal leaf ordering we implemented the algorithm
by~\cite{seriation:Bar-Joseph:2001}. The BEA code was kindly provided by
Fionn Murtagh.  For the Gruvaeus and Wainer algorithm, the
implementation in package \pkg{gclus}~\citep{seriation:Hurley:2007} is
used. For the rank-two ellipse seriation we implemented the algorithm
by~\cite{seriation:Chen:2002}.
Spectral seriation is described by~\cite{seriation:Ding:2004}.
Note that some methods implemented
(e.g., the rank-two ellipse seriation) do not fall within the
combinatorial optimization framework of this paper and thus are not
dealt with here in detail.  They are included in the package since they
can be useful for various applications.  
A detailed empirical comparison of seriation methods and crieria can be found
in the study by \cite{hahsler:Hahsler2016d}.
%Over time more methods will be
%added to the package.

To calculate the value of a loss/merit function for data and
a certain permutation, the function
\begin{quotation}
\code{criterion(x, order = NULL, method = NULL, ...)}
\end{quotation}
is provided. \code{x} is the data object, \code{order} contains a suitable
object of class \code{ser\_permutation} (if omitted no permutation is
performed) and \code{method} specifies the type of loss/merit function. A
vector of several methods can be used resulting in a named vector
with the values of the requested functions.  If \code{method} is omitted
(\code{method = NULL}), the values for all applicable loss/merit functions are
calculated and returned.  We already defined different loss/merit functions for
seriation in Section~\ref{sec:seriation}. In Table~\ref{tab:criteria} we
indicate the loss/merit functions currently available in the package.

\begin{table}[t]
\centering
\begin{tabular}{llll}
        \hline
        Name & \code{method} & merit/loss & Input data \\
        \hline
        Anti-Robinson events& \code{"AR\_events"} &
loss & \code{dist} \\
        Anti-Robinson deviations& \code{"AR\_deviations"} &
loss & \code{dist} \\
        Banded Anti-Robinson& \code{"BAR"} &
loss & \code{dist} \\
        Gradient measure& \code{"Gradient\_raw"} &
merit & \code{dist} \\
        Gradient measure (weighted)& \code{"Gradient\_weighted"} &
merit & \code{dist} \\
        Hamiltonian path length & \code{"Path\_length"} & loss & \code{dist} \\
        Inertia criterion& \code{"Inertia"} & merit & \code{dist} \\
        Least squares criterion& \code{"Least\_squares"} & loss & \code{dist} \\

        Linear Seriation criterion& \code{"LS"} & loss & \code{dist} \\
        2-Sum criterion& \code{"2SUM"} & loss & \code{dist} \\

        \hline
        Measure of effectiveness& \code{"ME"} &
merit & \code{matrix} \\
        Stress (Moore neighborhood)& \code{"Moore\_stress"} &
loss & \code{matrix} \\
        Stress (Neumann neighborhood)& \code{"Neumann\_stress"} &
loss & \code{matrix} \\
        \hline
    \end{tabular}
    \caption{Implemented loss/merit functions in function \func{criterion}.}
\label{tab:criteria}
\end{table}

All methods for \func{seriate} and \func{criterion} are managed by a
registry mechanism which makes the seriation framework easily extensible
for users.  For example, a new seriation method can be registered using
\func{set\_seriation\_method} and then used in the same way as the
built-in methods with \func{seriate}. All available methods in the
registry can be viewed using \func{list\_seriation\_methods} and
\func{show\_seriation\_methods}.  For criterion methods, the same
interface is available by just substituting `seriation' by `criterion'
in the function names. An example for how to add new methods can be
found in section~\ref{sec:registering} of this paper.

In addition the package offers the (generic) function
\begin{quotation}
\code{permute(x, order)}
\end{quotation}
where \code{x} is the data (a \code{dist} object, a matrix, an
array, a list or a numeric vector) to be reordered and \code{order} is a
\code{ser\_permutation} object of suitable length.
%The permutation for
%\code{dist} objects uses package \pkg{proxy}~\citep{seriation:Meyer:2007}.

For visualization, the package offers several options:
\begin{itemize}
\item Matrix shading with \func{pimage}. In contrast to the
        standard \func{image} in package~\pkg{graphics}, \func{pimage}
        displays the matrix as is with the first element in the top
        left-hand corner and using a gamma-corrected gray scale.

\item Different heat maps (e.g., with optimally reordered
    dendrograms) with \func{hmap}.

\item Visualization of data matrices in the spirit
of~\cite{seriation:Bertin:1981} with \func{bertinplot}.

\item \emph{Dissimilarity plot}, a new visualization to judge the
    quality of a clustering using matrix shading
    and seriation with \func{dissplot}.
\end{itemize}

We will introduce the package usage and the visualization options
in the examples in the next section.

\section{Examples and applications}
\label{sec:example}


We start this section with a simple first session to demonstrate the basic
usage of the package. Then we present and discuss several seriation
applications.

\subsection{A first session using seriation}
In the following example, we use the well known iris data set
(from \proglang{R}'s \pkg{datasets} package) which gives the
measurements in centimeters of the variables sepal length and width and petal
length and width, respectively, for 50 flowers from each of 3 species of the
iris family (Iris setosa, versicolor and virginica).

First, we load the package \pkg{seriation} and the iris data set.  We
remove the species classification and reorder the objects randomly since
they are already sorted by species in the data set. Then we calculate
the Euclidean distances between objects.

<<echo=FALSE>>=
set.seed(1234)
@

<<>>=
library("seriation")

data("iris")
x <- as.matrix(iris[-5])
x <- x[sample(seq_len(nrow(x))),]
d <- dist(x)
@

To seriate the objects given the dissimilarities, we just call
\func{seriate} with the default settings.

<<>>=
o <- seriate(d)
o
@

The result is an object of class \code{ser\_permutation} for
one-mode data. The permutation vector length is $150$ for the
$150$ objects in the iris data set and the used seriation method is
\code{"ARSA"}, a simulated annealing heuristic
(see~Table~\ref{tab:methods}). The actual order can be accessed
using \func{get\_order}. In the following we show the first
15 elements in the permutation vector.

<<>>=
head(get_order(o), 15)
@


To visually inspect the effect of seriation on the distance matrix, we use
matrix shading with \func{pimage} (the result is shown in
Figure~\ref{fig:pimage1}).

<<label=pimage1, fig=TRUE, include=FALSE, width=7.5>>=
pimage(d, main = "Random")
@
<<label=pimage1-2, fig=TRUE, include=FALSE, width=7.5>>=
pimage(d, o, main = "Reordered")
@
\begin{figure}
    \centering
    \includegraphics[width=7.5cm]{seriation-pimage1}
    \includegraphics[width=7.5cm]{seriation-pimage1-2}
    \caption{Matrix shading of the  distance matrix for the iris data.}
    \label{fig:pimage1}
\end{figure}

We can also compare the improvement for different loss/merit functions
using \func{criterion}.

<<>>=
cbind(random = criterion(d), reordered = criterion(d, o))
@

Naturally, the reordered dissimilarity matrix achieves better values for all
criteria. Note that the gradient measures, inertia and the measure of
effectiveness are merit functions and for these measures larger values are
better (use \code{show\_criterion\_methods("dist")} to find out which measures
are loss and merit functions).

To visually compare the original data matrix and the
result of seriation, we can also use \func{pimage}.
We standardize the data using scale such that the visualized value
is the number of standard diviations an object differs from the
variable mean. For matrices containing negative values, \code{pimage}
uses automatically a divergent palette.
After using \func{pimage} for the original random data matrix,
we create a suitable \code{ser\_permutation} object for the original
two-mode data. Since the seriation above only produced an order for the rows
of the data, we add an identity permutation vector
for the columns (represented by \code{NA})
to the permutations object using the combine
function \func{c}. This new permutation object for $2$-mode data is used
for displaying the reordered scaled data. The two plots are shown in
Figure~\ref{fig:pimage2}.

<<label=pimage2, fig=TRUE, include=FALSE, width=7.5>>=
pimage(scale(x), main = "Random")
@

<<label=pimage2-2, fig=TRUE, include=FALSE, width=7.5>>=
o_2mode <- c(o, NA)
pimage(scale(x), o_2mode, main = "Reordered")
@

\begin{figure}
    \centering
    \includegraphics[width=7.5cm]{seriation-pimage2}
    \includegraphics[width=7.5cm]{seriation-pimage2-2}
    \caption{Matrix shading of the iris data matrix.}
    \label{fig:pimage2}
\end{figure}

\subsection{Comparing different seriation methods}

To compare different seriation methods we use again the randomized iris data
set and the distance matrix \code{d} from the previous example.  We include in
the comparison several seriation methods for dissimilarity matrices described
in Section~\ref{sec:methods}.

<<>>=
methods <- c("TSP","R2E", "ARSA", "HC", "GW", "OLO")
o <- sapply(methods, FUN = function(m) seriate(d, m))
@

<<echo=FALSE>>=
timing <- sapply(methods, FUN = function(m) system.time(seriate(d, m)),
    simplify = FALSE)
@


\begin{table}
    \centering
     \begin{tabular}{lcccccc}
     \hline
     Seriation Method &
     \Sexpr{methods[1]}&
     \Sexpr{methods[2]}&
     \Sexpr{methods[3]}&
     \Sexpr{methods[4]}&
     \Sexpr{methods[5]}&
     \Sexpr{methods[6]} \\
     \hline
     Execution time [sec] &
     \Sexpr{round(timing[[methods[1]]][1],4)}&
     \Sexpr{round(timing[[methods[2]]][1],4)}&
     \Sexpr{round(timing[[methods[3]]][1],4)}&
     \Sexpr{round(timing[[methods[4]]][1],4)}&
     \Sexpr{round(timing[[methods[5]]][1],4)}&
     \Sexpr{round(timing[[methods[6]]][1],4)}\\
     \hline
     \end{tabular}
%%% fix me: for the vignette we need something else
\caption{Execution time of seriation of the iris data set for different
    methods.}
    \label{tab:timings}
\end{table}

Table~\ref{tab:timings} contains the execution times for running
seriation with the different methods. Except for the simulated annealing
method (ARSA) the seriation only takes a fraction of a second.
The direction of the resulting orderings is first normalized (aligned)
and then the orderings are displayed using matrix shading
(see Figure~\ref{fig:pimage3}).


<<label=pimage3-pre, eval=FALSE>>=
o <- ser_align(o)
for(s in o) pimage(d, s, main = get_method(s), key = FALSE)
@

<<label=pimage3, echo=FALSE, fig=TRUE, include=FALSE>>=
o <- ser_align(o)
for(i in 1:length(o)) {
    pdf(file=paste("seriation-pimage_comp_", i , ".pdf", sep=""))
    pimage(d, o[[i]], main = get_method(o[[i]]), key = FALSE)
    dev.off()
}
@

\begin{figure}
    \centering
    \includegraphics[width=.3\linewidth]{seriation-pimage_comp_1.pdf}
    \includegraphics[width=.3\linewidth]{seriation-pimage_comp_2.pdf}
    \includegraphics[width=.3\linewidth]{seriation-pimage_comp_3.pdf}\\
    \includegraphics[width=.3\linewidth]{seriation-pimage_comp_4.pdf}
    \includegraphics[width=.3\linewidth]{seriation-pimage_comp_5.pdf}
    \includegraphics[width=.3\linewidth]{seriation-pimage_comp_6.pdf}
    \caption{Image plot of the distance matrix for the iris data
    using rearrangement by different seriation methods.}
    \label{fig:pimage3}
\end{figure}


The first row of matrices in Figure~\ref{fig:pimage3} contains the
orders obtained by a TSP solver the rank-two ellipse seriation by Chen
and using the simulated annealing method (ARSA). The results of Chen and
ARSA are very similar (except that the order is reversed). The TSP
solver produces a smoother image with some lighter lines visible.  The
reason for these lines is that the TSP only optimizes distances locally
between two neighboring objects. Therefore it is possible that in a
quite homogeneous block several objects are enclosed gradually getting
more different and then getting more similar again (see, e.g., the light
line close to the upper left corner of the TSP image in
Figure~\ref{fig:pimage3}).

The second row of Figure~\ref{fig:pimage3} contains three images based on
hierarchical clustering. The visual impression gets better from left (just
hierarchical clustering) to right (first using the Gruvaeus Wainer heuristic
and then optimal leaf ordering to rearrange the branches of the dendrogram
obtained by hierarchical clustering). The most striking feature in the
image for hierarchical clustering (HC in Figure~\ref{fig:pimage3}) is the
distinct cross going right through the center of the plot. This indicates that
several relatively dissimilar objects are caught in an otherwise homogeneous
block. This effect vanishes after rearranging the dendrogram branches
(see GW and OLO in Figure~\ref{fig:pimage3}).

%' To investigate this effect,
%' we plot the dendrogram obtained by hierarchical clustering which is used
%' to order the objects and compare it to the dendrogram rearranged
%' using the Gruvaeus Wainer heuristic.
%'
%' <<label=pimage3_dend, eval=FALSE>>=
%' plot(o[["HC"]], labels = FALSE, main = "Dendrogram HC")
%' plot(o[["GW"]], labels = FALSE, main = "Dendrogram GW")
%' @
%' <<echo=FALSE, fig=FALSE, include=FALSE, width=9>>=
%' def.par <- par(no.readonly = TRUE)
%' pdf(file="seriation-pimage3_dendrogram.pdf", width=9, height=4)
%' layout(t(1:2))
%' plot(o[["HC"]], labels = FALSE, main = "Dendrogram HC")
%' symbols(74.7,.5, rect = matrix(c(4, 3), ncol=2), add= TRUE,
%'     inches = FALSE, lwd =2)
%'
%' plot(o[["GW"]], labels = FALSE, main = "Dendrogram GW")
%' symbols(98.7,.5, rect = matrix(c(4, 3), ncol=2), add= TRUE,
%'     inches = FALSE, lwd =2)
%' par(def.par)
%' tmp <- dev.off()
%' @
%'
%' \begin{figure}
%'     \centering
%'     \includegraphics[width=\linewidth, trim=0 80 0 0, clip=TRUE]{seriation-pimage3_dendrogram}
%'     \caption{Dendrograms for the seriation with HC and GW.}
%'     \label{fig:pimage3_dendrogram}
%' \end{figure}
%'
%' Comparing the two dendrograms in Figure~\ref{fig:pimage3_dendrogram}, we see
%' that the branch left from the top is almost unchanged. The branch which is
%' responsible for the light cross in the shaded image is highlighted by a box.
%' The Gruvaeus Wainer heuristic rotates the highlighted branch towards the right
%' since the objects in it are more similar to the objects in there.

Finally, we compare the values of the loss/merit functions
for the different seriation methods.
<<>>=
crit <- sapply(o, FUN = function(x) criterion(d, x))
t(crit)
@

<<echo=FALSE, fig=TRUE, include=FALSE, label=crit1, width=6, height=8>>=
def.par <- par(no.readonly = TRUE)
m <- c("Path_length", "AR_events", "Moore_stress")
layout(matrix(seq_along(m), ncol=1))
#tmp <- apply(crit[m,], 1, dotchart, sub = m)
tmp <- lapply(m, FUN = function(i) dotchart(crit[i,], sub = i))
par(def.par)
@

\begin{figure}
    \centering
    \includegraphics[width=14cm]{seriation-crit1}
    \caption{Comparison of different methods and seriation criteria}
    \label{fig:crit1}
\end{figure}

For easier comparison, Figure~\ref{fig:crit1} contains a plot of the criteria
Hamiltonian path length, anti-Robinson events (\code{AR\_events}) and stress
using the Moore neighborhood. Clearly, the methods which directly try to
minimize the Hamiltonian path length (hierarchical clustering with optimal leaf
ordering (\code{OLO}) and the TSP heuristic) provide the best results
concerning the path length. For the number of anti-Robinson events, using the
simulated annealing heuristic (\code{ARSA}) provides the best result.  Regarding stress, the
simulated annealing heuristic also provides the best result although, it does
not directly minimize this loss function.

\subsection{Registering new methods}
\label{sec:registering}
New methods to calculate criterion values and to compute a seriation can
be easily added by the user via the method registry mechanism provided
in \pkg{seriation}. Here we give a simple example of how to implement and
register a new seriation method.

In the registry we distinguish between methods for different types of
input data.  With the following two commands we produce a short overview
of the the first few available seriation methods for input data of class \code{dist}
and \code{matrix}.

<<>>=
show_seriation_methods("dist")[1:3]
show_seriation_methods("matrix")[1:3]
@

The overview is intended to make it convenient for the user to choose an
appropriate method.  It contains the name of the method used as the
\code{method} argument for \func{seriate} and a short description.  To
get just the names the following function is also available:
<<>>=
list_seriation_methods("dist")
@

To add a new seriation method, we first have to implement the seriation code as
a function with the two formal arguments \code{x} and \code{control}.
\code{x} is the data
object and \code{control} contains a list with additional information for the
method passed on from \func{seriate}. The function has to return a list
of objects which can be coerced into \code{ser\_permutation\_vector}
objects (e.g., a list of integer vectors). The elements in the list
have to be in order corresponding to the dimensions of \code{x}.

In this example we just create a method to return a permutation
which reverses the original order of the objects, i.e., which returns
the reverse identity order.

<<>>=
seriation_method_reverse <- function(x, control = NULL) {
           lapply(dim(x), function(n) rev(seq(n)))
}
@

The function produces integer sequences of the correct lengths,
one for each dimension of \code{x} (\code{control} is not used).
Since the function works for \code{matrix} and \code{array} we can register
it for both data types under the short name `Reverse'.

<<>>=
set_seriation_method("matrix", "Reverse", seriation_method_reverse,
    "Reverse identity order")

set_seriation_method("array", "Reverse", seriation_method_reverse,
    "Reverse identity order")
@

Now the new seriation method is registered and can be found by the user
and applied to data.
<<>>=
show_seriation_methods("matrix")

o <- seriate(matrix(1, ncol=3, nrow=4), "reverse")
o

get_order(o, 1)
get_order(o, 2)
@


Criterion methods can be added in the same way. We refer the interested reader
to the documentation accompanying the package for detailed information and
an example.

If you have implemented a new criterion or seriation method, please consider
submitting the code to one of the maintainers of \pkg{seriation} for
inclusion in a future release of the package.

\subsection{Heat maps}

A heat map is a shaded/color coded data matrix with a dendrogram added to one
side and to the top to indicate the order of rows and columns. Typically,
reordering is done according to row or column means within the restrictions
imposed by the dendrogram. Heat maps recently became popular for visualizing
large scale genome expression data obtained via DNA microarray technology
\citep[see, e.g.,][]{seriation:Eisen:1998}.

From Section~\ref{sec:hierarchical_clustering} we know that it is possible to
find the optimal ordering of the leaf nodes of a dendrogram which minimizes
the distances between adjacent objects in reasonable time.  Such an order might
provide an improvement over using simple reordering such as the row or column
means with respect to presentation. In \pkg{seriation} we provide
the function \func{hmap} which uses optimal ordering and can also use
seriation directly on distance matrices without using hierarchical
clustering to produce dendrograms first.

For the following example, we use again the randomly reordered iris data set
\code{x} from the examples above. To make the variables (columns) comparable,
we use standard scaling.

<<>>=
x <- scale(x, center = FALSE)
@

To produce a heat map with optimally reordered dendrograms (using
by default Optimal Leaf Ordering), the function
\func{hmap} can be used with its default settings.

<<eval=FALSE>>=
hmap(x, margin =c(7, 4), cexCol=1, labRow = FALSE)
@

With these settings, the
Euclidean distances between rows and between columns are calculated (with
\func{dist}), hierarchical clustering (\func{hclust}) is performed, the
resulting dendrograms are optimally reordered, and \func{heatmap.2} in package
\pkg{gplots} is used for plotting
(see Figure~\ref{fig:heatmap}(a) for the resulting plot).

<<eval=FALSE>>=
hmap(x, method = "MDS")
@

If a seriation method is used that does not depend on dendrograms, instead of hierarchical clustering,
seriation on the dissimilarity matrices for rows and columns is
performed and the reordered matrix
with the reordered dissimilarity matrices to the left and on top is
displayed (see Figure~\ref{fig:heatmap}(b)). A \code{method} argument can be used to choose different
seriation methods.

<<echo=FALSE, fig=FALSE, include=FALSE>>=
#bitmap(file = "seriation-heatmap1.png", type = "pnggray",
#    height = 6, width = 6, res = 300, pointsize=14)
pdf(file = "seriation-heatmap1.pdf")
hmap(x, margin = c(7, 4), labRow = FALSE, cexCol=1)
tmp <- dev.off()
@
<<echo=FALSE, fig=FALSE, include=FALSE>>=
pdf(file = "seriation-heatmap2.pdf")
hmap(x, method="MDS")
tmp <- dev.off()
@

\begin{figure}
    \begin{minipage}[b]{.48\linewidth}
        \centering
        \includegraphics[width=\linewidth]{seriation-heatmap1} \\
            (a)
    \end{minipage}
    \begin{minipage}[b]{.48\linewidth}
        \centering
        \includegraphics[width=\linewidth]{seriation-heatmap2} \\
            (b)
    \end{minipage}
    \caption{Two presentations of the rearranged iris data matrix. (a) as an
optimally reordered heat map and (b) as a seriated data matrix with reordered
dissimilarity matrices to the left and on top.}
    \label{fig:heatmap}
\end{figure}


\subsection{Bertin's permutation matrix}

\cite{seriation:Bertin:1981,seriation:Bertin:1999}
introduced permutation matrices to analyze
multivariate data with medium to low sample size.  The idea is to reveal a more
homogeneous structure in a data matrix~$\mathbf{X}$ by simultaneously
rearranging rows and columns. The rearranged matrix is displayed and cases and
variables can be grouped manually to gain a better understanding of the data.

%To quantify homogeneity, a purity function
%\begin{displaymath}
%  \phi = \Phi(\mathbf{X})
%\end{displaymath}
%is defined. Let $\Pi$ be the set of all permutation functions
%$\pi$ for matrix $\mathbf{X}$.
%Note that function $\pi$ performs row and column permutations on a matrix.
%The optimal permutation with respect to
%purity can be found by
%\begin{displaymath}
%  \pi^* = \argmax\nolimits_{\pi \in \Pi} \Phi(\pi(\mathbf{X})).
%\end{displaymath}
%Since, depending on the purity function, finding the optimal
%solution can be hard, often a near optimal solution is also acceptable
%for visualization.
%
%A possible purity function $\Phi$ is:
%Given distances between rows and columns of the data matrix, define purity as
%the sum of distances of adjacent rows/columns.  Using this purity function,
%finding the optimal permutation $\pi^*$ means solving two (independent) TSPs,
%one for the columns and one for the rows.

To find a rearrangement of columns and rows which reveals structure a
purity function is used. A possible purity function is: Given distances between rows and columns of the data matrix, define purity as
the sum of distances of adjacent rows/columns. Using this purity function,
finding the optimal permutation means solving two (independent) TSPs,
one for the columns and one for the rows which can be done very conveniently
using the infrastructure provided by \pkg{seriation}.

As an example, we use the results of $8$ constitutional referenda for $41$
Irish communities~\citep{seriation:Falguerolles:1997}\footnote{The Irish data
set is included in this package. The original data and the text of the
referenda can be obtained from~\url{http://www.electionsireland.org/}}.  To
make values comparable across columns (variables), the ranks of the values for
each variable are used instead of the original values.

<<>>=
data("Irish")
orig_matrix <- apply(Irish[,-6], 2, rank)
@

For seriation, we calculate distances between rows and between columns using
the sum of absolute rank differences (this is equal to the Minkowski distance
with power $1$). Then we apply seriation (using a TSP heuristic) to both
distance matrices and combine the two resulting \code{ser\_permutation} objects
into one object for two-mode data. The original and the reordered matrix are
plotted using \func{bertinplot}.

<<>>=
o <- c(
    seriate(dist(orig_matrix, "minkowski", p = 1), method ="TSP"),
    seriate(dist(t(orig_matrix), "minkowski", p = 1), method = "TSP")
)
o
@



<<eval=FALSE>>=
bertinplot(orig_matrix)
bertinplot(orig_matrix, o)
@
<<echo=FALSE, fig=TRUE, include=FALSE, label=bertin1, width=10>>=
bertinplot(orig_matrix)
@
<<echo=FALSE, fig=TRUE, include=FALSE, label=bertin2, width=10>>=
bertinplot(orig_matrix, o)
@
\begin{figure}
    \centering
    \includegraphics[width=15cm, trim=60 60 0 0]{seriation-bertin1} \\
    (a)

    \includegraphics[width=15cm, trim=60 60 0 0]{seriation-bertin2} \\
    (b)
    \caption{Bertin plot for the (a) original arrangement and the (b)
    reordered Irish data set.}
    \label{fig:bertin}
\end{figure}



The original matrix and the rearranged matrix are shown in
Figure~\ref{fig:bertin} as a matrix of bars where high values are highlighted
(filled blocks).  Note that following Bertin, the cases (communities) are
displayed as the columns and the variables (referenda) as rows.  Depending on
the number of cases and variables, columns and rows can be exchanged to obtain
a better visualization.

Although the columns are already ordered (communities in the same city
appear consecutively) in the original data matrix in
Figure~\ref{fig:bertin}(a), it takes some effort to find structure in
the data.  For example, it seems that the variables `Marriage',
`Divorce', `Right to Travel' and `Right to Information' are correlated
since the values are all high in the block made up by the columns of the
communities in Dublin.  The reordered matrix confirms this but makes the
structure much more apparent. Especially the contribution of low values
(which are not highlighted) to the overall structure becomes only
visible after rearrangement.

\subsection{Binary data matrices}

Binary or $0$-$1$ data matrices are quite common.  Often such matrices are
called \emph{incidence matrices} since a $1$ in a cell indicates the incidence
of an event. In archeology such an event could be that a special type of
artifact was found at a certain archaeological site.
This can be seen as a simplification of a so-called \emph{abundance matrix}
which codes in each cell the (relative) frequency or quantity of an artifact
type at a site.  See \cite{seriation:Ihm:2005} for a comparison of
incidence and abundance matrices in archeology.

Here we are interested in binary data.
For the example we use an artificial data set from~\cite{seriation:Bertin:1981}
called \emph{Townships}.  The data set contains $9$ binary characteristics
(e.g., has a veterinary or has a high school) for $16$ townships. The idea of
the data set is that townships evolve from a rural to an urban environment over
time.

After loading the data set (which comes with the package), we use
\func{bertinplot} to visualize the data (\func{pimage} could also be used
but \func{bertinplot} allows for a nicer visualization).
Bars, the standard visualization of \func{bertinplot}, do not
make much sense for binary data. We therefore use the
panel function \func{panel.squares} without spacing
to plot black squares.

<<fig=TRUE, include=FALSE, label=binary1, width=9>>=
data("Townships")

bertinplot(Townships, options = list(panel=panel.squares, spacing = 0,
    frame = TRUE))
@

The original data in Figure~\ref{fig:binary}(a) does not reveal structure in
the data. To improve the display, we run the  bond energy algorithm (BEA) for
columns and rows $10$ times with random starting points and report the best
solution.

<<echo=FALSE>>=
## to get consistent results
set.seed(5)
@

<<fig=TRUE, include=FALSE, label=binary2, width=9>>=
o <- seriate(Townships, method = "BEA", control = list(rep = 10))
bertinplot(Townships, o, options = list(panel=panel.squares, spacing = 0,
    frame = TRUE))
@

The reordered matrix is displayed in Figure~\ref{fig:binary}(b).  A
clear structure is visible.  The variables (rows in a Bertin plot) can be
split into the three categories describing different evolution states of
townships:
\begin{enumerate}
    \item Rural: No doctor, one-room school and possibly also
        no water supply
    \item Intermediate: Land reallocation, veterinary and agricultural
        cooperative
    \item Urban: Railway station, high school and police station
\end{enumerate}

The townships also clearly fall into these three groups which
tentatively can be called villages (first~$7$), towns (next~5) and
cities (final~2).  The townships B and C are on the transition to the
next higher group.

\begin{figure}
    \centering
    \includegraphics[width=12cm, trim=0 40 0 30]{seriation-binary1} \\
    (a)

    \includegraphics[width=12cm, trim=0 40 0 30]{seriation-binary2} \\
    (b)

    \caption{The townships data set in original order (a) and
    reordered using BEA (b).}
    \label{fig:binary}
\end{figure}


<<>>=
rbind(original = criterion(Townships), reordered = criterion(Townships, o))
@

BEA tries to maximize the measure of effectiveness which is
much higher in the reordered matrix (in fact, 65 is the maximum for
the data set). Also the two types of stress are improved
significantly.

\subsection{Dissimilarity plot}

Assessing the quality of an obtained cluster solution has been a
research topic since the invention of cluster analysis. This is
especially important since all popular cluster algorithms produce a
clustering even for data without a ``cluster'' structure.

%A method to judge the quality of a cluster solution is by inspecting a
%visualization. For hierarchical clustering
%dendrogramms~\cite{seriation:Hartigan:1967} are available which show the
%hierarchical structure of the clustering as a binary tree and cluster quality
%can be judged by looking at the dissimilarities between objects in a cluster
%and objects in other clusters. However, such a visualization is
%only possible for heirarchical/nested clusterings.
%
%\marginpar{Cite Pison et al 1999 and Kaufmann and Rousseeuw}
%For the an arbitrary partitional clustering, the original objects can
%be displayed in a 2 dimensional scatter plot
%after using dimensionality reduction (e.g., PCA, MDS).
%Objects belonging to the same cluster can be marked and thus, if the
%dimensionality reduction preserves a large proportion of the
%variavility in the original data, the separation between clusters can be
%visually judged.
%
%Silhouettes

Matrix shading is an old technique to visualize clusterings by
displaying the rearranged matrices~\citep[see,
e.g.,][]{seriation:Sneath:1973,seriation:Ling:1973,seriation:Gale:1984}.
Initially matrix shading was used in connection with hierarchical
clustering, where the order of the dendrogram leaf nodes was used to
arrange the matrix.  However, with some extensions, matrix shading can
also be used with any partitional clustering method.

\cite{seriation:Strehl:2003} suggest a matrix shading visualization called
\emph{CLUSION} where the dissimilarity matrix is arranged such that all objects
pertaining to a single cluster appear in consecutive order in the matrix. The
authors call this \emph{coarse seriation}. The result of a ``good'' clustering
should be a matrix with low dissimilarity values forming blocks around the main
diagonal. However, using coarse seriation, the order of the clusters has to be
predefined and the objects within each cluster are unordered.

The dissimilarity plots implemented by the function \func{dissplot} in
\pkg{seriation} improve \emph{CLUSION} using seriation methods.  It aims
at visualizing global structure (similarity between different clusters
is reflected by their position relative to each other) as well as the
micro structure within each cluster (position of objects).

To position the clusters in the dissimilarity plot, an inter-cluster
dissimilarity matrix is calculated using the average between cluster
dissimilarities. \func{seriate} is used on this inter-cluster dissimilarity
matrix to arrange the clusters relative to each other resulting in on average
more similar clusters to appear closer together in the plot.  Within each
cluster, \func{seriate} is used again on the sub-matrix of the dissimilarity
matrix concerning only the objects in the cluster.

For the example, we use again Euclidean distance between the objects in the
iris data set.

<<>>=
data("iris")
iris <- iris[sample(seq_len(nrow(iris))),]
d <- dist(as.matrix(iris[-5]), method = "euclidean")
@

First, we use \func{dissplot} without a clustering. We set \code{method} to
\code{NA} to prevent reordering and display the original matrix (see
Figure~\ref{fig:dissplot1}(a)). Then we omit the method argument which results
in using the default seriation technique from \func{seriate}. Since we
did not provide a clustering, the whole matrix is reordered
in one piece.  From the result shown in Figure~\ref{fig:dissplot1}(b) it seems
that there is a clear structure in the data which suggests a two cluster
solution.

<<eval=FALSE, label=dissplot1>>=
## plot original matrix
dissplot(d, method = NA)
@

<<eval=FALSE, label=dissplot2>>=
## plot reordered matrix
dissplot(d, options = list(main = "Dissimilarity plot with seriation"))
@

<<echo=FALSE, fig=FALSE, include=FALSE>>=
pdf(file = "seriation-dissplot1.pdf")
<<dissplot1>>
tmp <- dev.off()
pdf(file = "seriation-dissplot2.pdf")
<<dissplot2>>
tmp <- dev.off()
@


\begin{figure}
    \begin{minipage}[b]{.48\linewidth}
    \centering
    \includegraphics[width=\linewidth]{seriation-dissplot1} \\
    (a)
    \end{minipage}
    \begin{minipage}[b]{.48\linewidth}
    \centering
    \includegraphics[width=\linewidth]{seriation-dissplot2} \\
    (b)
    \end{minipage}
    \caption{Two dissimilarity plots.
    (a) the original dissimilarity matrix and
    (b) the seriated dissimilarity matrix.}
    \label{fig:dissplot1}
\end{figure}

Next, we create a cluster solution using the $k$-means algorithm.
Although we know
that the data set should contain $3$ groups representing the three species
of iris, we let $k$-means produce a $10$ cluster solution to study how such a
misspecification can be spotted using \func{dissplot}.

<<echo=FALSE>>=
set.seed(1234)
@
<<>>=
l <- kmeans(d, 10)$cluster
#$
@

We create a standard dissimilarity plot by providing the cluster
solution as a vector of labels. The function rearranges the matrix and
plots the result. Since rearrangement can be a time consuming procedure for
large matrices, the rearranged matrix and all
information needed for plotting is returned as the result.

<<eval=FALSE, label=dissplot3>>=
res <- dissplot(d, labels = l,
    options = list(main = "Dissimilarity plot - standard"))
@
<<echo=FALSE, fig=FALSE, include=FALSE>>=
pdf(file = "seriation-dissplot3.pdf")

## visualize the clustering
<<dissplot3>>
tmp <- dev.off()


pdf(file = "seriation-dissplot4.pdf")
## threshold
plot(res, options = list(main = "Dissimilarity plot - threshold",
    threshold = 1.5))

tmp <- dev.off()
@

\begin{figure}
    \centering
    \includegraphics[width=10cm]{seriation-dissplot3}\\
    (a)

    \includegraphics[width=10cm]{seriation-dissplot4}\\
    (b)
    \caption{Dissimilarity plot for $k$-means solution with 10 clusters.
    (a) standard plot and (b) plot with threshold.}
    \label{fig:dissplot3}
\end{figure}

<<>>=
res
@

The resulting plot is shown in Figure~\ref{fig:dissplot3}(a).  The
inter-cluster dissimilarities are shown as solid
gray blocks and the average object
dissimilarity within each cluster as gray triangles below the main diagonal of
the matrix. Since the clusters are arranged such that more similar clusters
are closer together, it is easy to see in Figure~\ref{fig:dissplot3}(a)
that clusters 6, 3 and 1 as well as clusters 10, 9, 5, 7, 8, 4 and 2
are very similar and form two blocks.
This suggests again that a two cluster solution would
be reasonable.

Since slight variations of gray values are hard to distinguish,
we plot the matrix again (using \func{plot} on the result above) and
use a threshold on the dissimilarity to suppress high dissimilarity
values in the plot.

<<eval=FALSE>>=
plot(res, options = list(main = "Seriation - threshold",
    threshold = 1.5))
@

In the resulting plot in Figure~\ref{fig:dissplot3}(b), we see that the
block containing 10, 9, 5, 7, 8, 4 and 2 is very well defined and
cleanly separated from the other block. This suggests that these clusters
should form together a cluster in a solution with less clusters.
The other block is less well defined. There is considerable overlap between
clusters 6 and 3, but also cluster 3 and 1 share similar objects.

Using the information stored in the result of \func{dissplot} and
the class information available for the iris data set, we can analyze
the cluster solution and the interpretations of the dissimilarity plot.

<<>>=
#names(res)
table(iris[res$order, 5], res$label)[,res$cluster_order]
#$
@

As the plot in Figure~\ref{fig:dissplot3} indicated, the clusters 10, 9, 5, 7,
8, 4 and 2 should be a single cluster containing only flowers of the species
Iris setosa. The clusters 6, 3 and 1 are more problematic since they contain a
mixture of Iris versicolor and virginica.

To illustrate the results of the dissimilarity plot in case a clustering
with a $k$ smaller than the actual number of groups in the data is used,
we use the Ruspini data set which consists of 75 points in four groups and
is also often used to illustrate clustering techniques. We load the data set,
calculate distances, perform $k$-means clustering with $k=3$ (although the
real number of groups is 4) and produce
a dissimilarity plot.

<<label=ruspini, fig=TRUE, include=FALSE>>=
data("ruspini", package="cluster")
d <- dist(ruspini)
l <- kmeans(d, 3)$cluster
dissplot(d, labels = l)
@

\begin{figure}
    \centering
    \includegraphics[width=10cm]{seriation-ruspini}\\
    \caption{Dissimilarity plot for $k$-means solution with 3 clusters
    for the Ruspini data set with 4 groups.}
    \label{fig:ruspini}
\end{figure}

The dissimilarity plot in Figure~\ref{fig:ruspini} shows that
cluster 3 actually should be two separate clusters represented
by the two clearly visible darker triangles next to the main diagonal.

The dissimilarity plot using seriation is a useful tool to inspect the result
of clustering. It is especially useful to spot
misspecifications of the number of clusters employed.
A more detailed treatment of dissimilarity plots as a tool for exploring
partitional clustering can be found in
\cite{seriation:Hahsler+Kornik:2011}.

\section{Conclusion}
\label{sec:conclusion}

In this paper we presented the infrastructure provided by the
package~\pkg{seriation}.  The infrastructure contains the necessary data
structures to store the linear order for one-, two- and $k$-mode data.
It also provides a wide array of seriation methods for different input
data, e.g., dissimilarities, binary and general data matrices focusing
on combinatorial optimization.  New seriation methods can be easily
incorporated into the \pkg{seriation} framework by the user with the
method registry mechanism provided.

Based on seriation, \pkg{seriation} features several visualization
techniques.  In particular, the optimally reordered heat map, the
Bertin plot and the dissimilarity plot present clear improvements over
standard plots.

A natural extension to \pkg{seriation} is the synthesis of ensembles of
seriations into a ``consensus'' one.  Such ensembles do not only arise
when using different seriation methods, but also when varying data or
control parameters to obtain more robust solutions (see
e.g.~\cite{seriation:Jurman:2008} for a recent application of such ideas
in a molecular profiling context).  The \proglang{R}~extension package
\pkg{relations}~\citep{seriation:Hornik+Meyer:2008} contains a variety
of methods for obtaining consensus \emph{relations}, covering consensus
seriation (where the relations are linear orders on the objects) as a
special case.

Future work on \pkg{seriation} will focus on adding further seriation
methods, such as for example methods for higher dimensional arrays and
methods for block seriation which aim at finding simultaneous partitions
of rows and columns in a data matrix~\citep[see,
e.g.,][]{seriation:Marcotorchino:1987}.

\section*{Acknowledgments}

The authors would like to thank Michael Brusco, Hans-Friedrich K{\"o}hn
and Stephanie Stahl for their seriation code, Fionn Murtagh for his BEA
implementation and the anonymous reviewers for their valuable comments
and suggestions.


%
%\bibliographystyle{abbrvnat}
\bibliography{seriation}
%
\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bertinplot.R, R/ggbertinplot.R
\name{bertinplot}
\alias{bertinplot}
\alias{panel.bars}
\alias{panel.circles}
\alias{panel.rectangles}
\alias{panel.squares}
\alias{panel.tiles}
\alias{panel.blocks}
\alias{panel.lines}
\alias{bertin_cut_line}
\alias{ggbertinplot}
\title{Plot a Bertin Matrix}
\usage{
bertinplot(
  x,
  order = NULL,
  panel.function = panel.bars,
  highlight = TRUE,
  row_labels = TRUE,
  col_labels = TRUE,
  flip_axes = TRUE,
  ...
)

panel.bars(value, spacing, hl)

panel.circles(value, spacing, hl)

panel.rectangles(value, spacing, hl)

panel.squares(value, spacing, hl)

panel.tiles(value, spacing, hl)

panel.blocks(value, spacing, hl)

panel.lines(value, spacing, hl)

bertin_cut_line(x = NULL, y = NULL, col = "red")

ggbertinplot(
  x,
  order = NULL,
  geom = "bar",
  highlight = TRUE,
  row_labels = TRUE,
  col_labels = TRUE,
  flip_axes = TRUE,
  prop = FALSE,
  ...
)
}
\arguments{
\item{x}{a data matrix. Note that following Bertin, columns are variables
and rows are cases. This behavior can be reversed using \code{reverse = TRUE}
in \code{options}.}

\item{order}{an object of class \code{ser_permutation} to rearrange \code{x}
before plotting.  If \code{NULL}, no rearrangement is performed.}

\item{panel.function}{a function to produce the symbols. Currently available
functions are \code{panel.bars} (default), \code{panel.circles},
\code{panel.rectangles}, \code{panel.tiles} and \code{panel.lines}. For
circles and squares neg. values are represented by a dashed border. For
blocks all blocks are the same size (can be used with \code{shading = TRUE}).}

\item{highlight}{a logical scalar indicating whether to use highlighting.
If \code{TRUE}, all variables with values greater than the variable-wise
mean are highlighted. To control highlighting, also a logical matrix or a
matrix with colors with the same dimensions as \code{x} can be supplied.}

\item{row_labels, col_labels}{a logical indicating if row and column labels
in \code{x} should be displayed.  If \code{NULL} then labels are displayed
if the \code{x} contains the appropriate dimname and the number of labels is
25 or less. A character vector of the appropriate length with labels can
also be supplied.}

\item{flip_axes}{logical indicating whether to swap cases and variables in
the plot. The default (\code{TRUE}) is to plot cases as columns and
variables as rows.}

\item{...}{\code{ggbertinplot()}: further parameters are passed on to \code{\link[=ggpimage]{ggpimage()}}.

\code{bertinplot()}: further parameters can include:
\itemize{
\item \verb{xlab, ylab} labels (default: use labels from \code{x}).
\item \code{spacing} relative space between symbols (default: 0.2).
\item \code{shading} use gray shades to encode value instead of
highlighting (default: \code{FALSE}).
\item \code{shading.function} a function that accepts a single argument in range \eqn{[.1, .8]}
and returns a valid corresponding color (e.g., using \code{\link[=rgb]{rgb()}}).
\item \code{frame} plot a grid to separate symbols (default: \code{FALSE}).
\item \code{mar} margins (see \code{\link[=par]{par()}}).
\item \code{gp_labels} \code{gpar} object for labels (see \code{\link[=gpar]{gpar()}})
\item \code{gp_panels} \code{gpar} object for panels (see \code{\link[=gpar]{gpar()}}).
\item \code{newpage} a logical indicating whether to start
the plot on a new page (see \code{\link[=grid.newpage]{grid.newpage()}}).
\item \code{pop} a logical indicating whether to pop the created viewports
(see \code{\link[=pop.viewport]{pop.viewport()}})?
}}

\item{value, spacing, hl}{are used internally for the panel functions.}

\item{col, y}{and x in \code{bertin_cut_line()} are for adding a line to a \code{bertinplot()} (not ggplot2-based).}

\item{geom}{visualization type. Available ggplot2 geometries are: \code{"tile"},
\code{"rectangle"}, \code{"circle"}, \code{"line"}, \code{"bar"}, \code{"none"}.}

\item{prop}{logical; change the aspect ratio so cells in the image have a
equal width and height.}
}
\value{
Nothing.
}
\description{
Plot a data matrix of cases and variables. Each value is represented by a
symbol. Large values are highlighted. Note that Bertin arranges the cases
horizontally and the variables as rows. The matrix can be rearranged using
seriation techniques to make structure in the data visible (see Falguerolles
et al 1997).
}
\details{
The plot is organized as a matrix of symbols. The symbols are drawn by a
panel function, where all symbols of a row are drawn by one call of the
function (using vectorization). The interface for the panel function is
\code{panel.myfunction(value, spacing, hl)}. \code{value} is the vector of
values for a row scaled between 0 and 1, \code{spacing} contains the
relative space between symbols and \code{hl} is a logical vector indicating
which symbol should be highlighted.

Cut lines can be added to an existing Bertin plot using
\code{bertin_cut_line(x = NULL, y = NULL)}. \code{x}/\code{y} is can be a
number indicating where to draw the cut line between two columns/rows. If
both \code{x} and \code{y} is specified then one can select a row/column and
the other can select a range to draw a line which does only span a part of
the row/column. It is important to call \code{bertinplot()} with the option
\code{pop = FALSE}.

\code{ggbertinplot()} calls \code{\link[=ggpimage]{ggpimage()}} and all additional parameters are
passed on.
}
\examples{
data("Irish")
scale_by_rank <- function(x) apply(x, 2, rank)
x <- scale_by_rank(Irish[,-6])

# Use the the sum of absolute rank differences
order <- c(
  seriate(dist(x, "minkowski", p = 1)),
  seriate(dist(t(x), "minkowski", p = 1))
)

# Plot
bertinplot(x, order)

# Some alternative displays
bertinplot(x, order, panel = panel.tiles, shading_col = bluered(100), highlight = FALSE)
bertinplot(x, order, panel = panel.circles, spacing = -.2)
bertinplot(x, order, panel = panel.rectangles)
bertinplot(x, order, panel = panel.lines)

# Plot with cut lines (we manually set the order here)
order <- ser_permutation(c(21, 16, 19, 18, 14, 12, 20, 15,
    17, 26, 13, 41,  7, 11, 5, 23, 28, 34, 31, 1, 38, 40,
    3, 39,  4, 27, 24,  8, 37, 36, 25, 30, 33, 35,  2,
    22, 32, 29, 10,  6,  9),
    c(4, 2, 1, 6, 8, 7, 5, 3))

bertinplot(x, order, pop=FALSE)
bertin_cut_line(, 4) ## horizontal line between rows 4 and 5
bertin_cut_line(, 7) ## separate "Right to Life" from the rest
bertin_cut_line(14, c(0, 4)) ## separate a block of large values (vertically)

# ggplot2-based plots
if (require("ggplot2")) {
  library(ggplot2)

  # Default plot uses bars and highlighting values larger than the mean
  ggbertinplot(x, order)

  # highlight values in the 4th quartile
  ggbertinplot(x, order, highlight = quantile(x, probs = .75))

  # Use different geoms. "none" lets the user specify their own geom.
  # Variables set are row, col and x (for the value).

  ggbertinplot(x, order, geom = "tile", prop = TRUE)
  ggbertinplot(x, order, geom = "rectangle")
  ggbertinplot(x, order, geom = "rectangle", prop = TRUE)
  ggbertinplot(x, order, geom = "circle")
  ggbertinplot(x, order, geom = "line")

  # Tiles with diverging color scale
  ggbertinplot(x, order, geom = "tile", prop = TRUE) +
    scale_fill_gradient2(midpoint = mean(x))

  # Custom geom (geom = "none"). Defined variables are row, col, and x for the value
  ggbertinplot(x, order, geom = "none", prop = FALSE) +
    geom_point(aes(x = col, y = row, size = x, color = x > 30), pch = 15) +
    scale_size(range = c(1, 10))

  # Use a ggplot2 theme with theme_set()
  old_theme <- theme_set(theme_minimal() +
      theme(panel.grid = element_blank())
    )
  ggbertinplot(x, order, geom = "bar")
  theme_set(old_theme)
}
}
\references{
de Falguerolles, A., Friedrich, F., Sawitzki, G. (1997): A
Tribute to J. Bertin's Graphical Data Analysis. In: Proceedings of the
SoftStat '97 (Advances in Statistical Software 6), 11--20.
}
\seealso{
Other plots: 
\code{\link{VAT}()},
\code{\link{dissplot}()},
\code{\link{hmap}()},
\code{\link{palette}()},
\code{\link{pimage}()}
}
\author{
Michael Hahsler
}
\concept{plots}
\keyword{cluster}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/VAT.R, R/ggVAT.R
\name{VAT}
\alias{VAT}
\alias{iVAT}
\alias{path_dist}
\alias{ggVAT}
\alias{ggiVAT}
\title{Visual Analysis for Cluster Tendency Assessment (VAT/iVAT)}
\usage{
VAT(x, upper_tri = TRUE, lower_tri = TRUE, ...)

iVAT(x, upper_tri = TRUE, lower_tri = TRUE, ...)

path_dist(x)

ggVAT(x, upper_tri = TRUE, lower_tri = TRUE, ...)

ggiVAT(x, upper_tri = TRUE, lower_tri = TRUE, ...)
}
\arguments{
\item{x}{a \code{dist} object.}

\item{upper_tri, lower_tri}{a logical indicating whether to show the upper or
lower triangle of the VAT matrix.}

\item{...}{further arguments are passed on to \code{\link{pimage}} for the
regular plots and \code{\link{ggpimage}} for the ggplot2 plots.}
}
\value{
Nothing.
}
\description{
Implements Visual Analysis for Cluster Tendency Assessment (VAT; Bezdek and
Hathaway, 2002) and Improved Visual Analysis for Cluster Tendency Assessment
(iVAT; Wang et al, 2010).
}
\details{
\code{path_dist()} redefines the distance between two objects as the minimum
over the largest distances in all possible paths between the objects as used
for iVAT.
}
\examples{
## lines data set from Havens and Bezdek (2011)
x <- create_lines_data(250)
plot(x, xlim=c(-5,5), ylim=c(-3,3), cex=.2)
d <- dist(x)

## create regular VAT
VAT(d, main = "VAT for Lines")
## same as: pimage(d, seriate(d, "VAT"))

## ggplot2 version
if (require("ggplot2")) {
  ggVAT(d) + labs(title = "VAT")
}

## create iVAT which shows visually the three lines
iVAT(d, main = "iVAT for Lines")
## same as:
## d_path <- path_dist(d)
## pimage(d_path, seriate(d_path, "VAT for Lines"))

## ggplot2 version
if (require("ggplot2")) {
  ggiVAT(d) + labs(title = "iVAT for Lines")
}

## compare with dissplot (shows banded structures and relationship between
## center line and the two outer lines)
dissplot(d, method = "OLO_single", main = "Dissplot for Lines", col = bluered(100, bias = .5))

## compare with optimally reordered heatmap
hmap(d, method = "OLO_single", main = "Heatmap for Lines (opt. leaf ordering)",
  col = bluered(100, bias = .5))
}
\references{
Bezdek, J.C. and Hathaway, R.J. (2002): VAT: a tool for visual
assessment of (cluster) tendency. \emph{Proceedings of the 2002
International Joint Conference on Neural Networks (IJCNN '02)}, Volume: 3,
2225--2230.

Havens, T.C. and Bezdek, J.C. (2012): An Efficient Formulation of the
Improved Visual Assessment of Cluster Tendency (iVAT) Algorithm, \emph{IEEE
Transactions on Knowledge and Data Engineering,} \bold{24}(5), 813--822.

Wang L., U.T.V. Nguyen, J.C. Bezdek, C.A. Leckie and K. Ramamohanarao
(2010): iVAT and aVAT: Enhanced Visual Analysis for Cluster Tendency
Assessment, \emph{Proceedings of the PAKDD 2010, Part I, LNAI 6118,} 16--27.
}
\seealso{
Other plots: 
\code{\link{bertinplot}()},
\code{\link{dissplot}()},
\code{\link{hmap}()},
\code{\link{palette}()},
\code{\link{pimage}()}
}
\author{
Michael Hahsler
}
\concept{plots}
\keyword{cluster}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AAA_registry_criterion.R
\docType{data}
\name{registry_criterion}
\alias{registry_criterion}
\alias{list_criterion_methods}
\alias{get_criterion_method}
\alias{set_criterion_method}
\alias{print.criterion_method}
\title{Registry for Criterion Methods}
\format{
An object of class \code{criterion_registry} (inherits from \code{registry}) of length 20.
}
\usage{
registry_criterion

list_criterion_methods(kind)

get_criterion_method(kind, name)

set_criterion_method(kind, name, fun, description = NULL, merit = NA, ...)

\method{print}{criterion_method}(x, ...)
}
\arguments{
\item{kind}{the data type the method works on. For example, \code{"dist"},
\code{"matrix"} or \code{"array"}.}

\item{name}{the name for the method used to refer to the method in the
function \code{\link[=criterion]{criterion()}}.}

\item{fun}{a function containing the method's code.}

\item{description}{a description of the method. For example, a long name.}

\item{merit}{a boolean indicating if the criterion measure is a merit
(\code{TRUE}) or a loss (\code{FALSE}) measure.}

\item{...}{further information that is stored for the method in the
registry.}

\item{x}{an object of class "criterion_method" to be printed.}
}
\value{
\itemize{
\item \code{list_criterion_method()} results is a vector of character strings with the
names of the methods used for \code{criterion()}.
\item \code{get_criterion_method()} returns a given method in form of an object of class
\code{"criterion_method"}.
}
}
\description{
A registry to manage methods used by \code{\link[=criterion]{criterion()}} to calculate a criterion value given data and a
permutation.
}
\details{
All methods below are convenience methods for the registry named
\code{registry_criterion}.

\code{list_criterion_method()} lists all available methods for a given data
type (\code{kind}). The result is a vector of character strings with the
short names of the methods. If \code{kind} is missing, then a list of
methods is returned.

\code{get_criterion_method()} returns information (including the
implementing function) about a given method in form of an object of class
\code{"criterion_method"}.

With \code{set_criterion_method()} new criterion methods can be added by the
user. The implementing function (\code{fun}) needs to have the formal
arguments \verb{x, order, ...}, where \code{x} is the data object, order is
an object of class \link{ser_permutation_vector} and \code{...} can contain
additional information for the method passed on from \code{\link[=criterion]{criterion()}}. The
implementation has to return the criterion value as a scalar.
}
\examples{
## the registry
registry_criterion

# List all criterion calculation methods by type
list_criterion_methods()

# List methods for matrix
list_criterion_methods("matrix")

get_criterion_method("dist", "AR_d")

# Define a new method (sum of the diagonal elements)

## 1. implement a function to calculate the measure
criterion_method_matrix_foo <- function(x, order, ...) {
if(!is.null(order)) x <- permute(x,order)
    sum(diag(x))
}

## 2. Register new method
set_criterion_method("matrix", "DiagSum", criterion_method_matrix_foo,
    description = "Calculated the sum of all diagonal entries", merit = FALSE)

list_criterion_methods("matrix")
get_criterion_method("matrix", "DiagSum")

## 3. use all criterion methods (including the new one)
criterion(matrix(1:9, ncol = 3))
}
\seealso{
This registry uses \code{\link[=registry]{registry()}} in package \pkg{registry}.

Other criterion: 
\code{\link{criterion}()}
}
\author{
Michael Hahsler
}
\concept{criterion}
\keyword{misc}

\name{permutation_vector}
\alias{ser_permutation_vector}
\alias{permutation_vector}
\alias{print.ser_permutation_vector}
\alias{rev.ser_permutation_vector}
\alias{get_method}
\alias{length.ser_permutation_vector}
\title{Class ser_permutation_vector -- 
A Single Permutation Vector for Seriation}
\description{
The class \code{ser_permutation_vector} 
represents a single permutation vector. 
}
\usage{
## constructor
ser_permutation_vector(x, method = NULL)
}
\arguments{
  \item{x}{ an object which contains a permutation vector (currently an
      integer vector or an object of class \code{hclust}). The value \code{NA}
      creates an identity permutation. }
  \item{method}{ a string representing the method used to obtain the
      permutation vector}
    }
\details{
A permutation vector 
maps a set of \eqn{n} objects \eqn{\{O_1, O_2, ..., O_n\}}{{O_1, O_2, ..., O_n}} onto itself.
In \pkg{seriation} we represent a permutation \eqn{\pi}{\pi} 
as a vector which lists the objects in their permuted order. 
For example, the permutation vector \eqn{\langle3, 1, 2\rangle}{<3, 1, 2>} indicates that in first position is the object with index 3 then the object with index 1 and finally 
the object with index 2. 
A permutation vector can be extracted from a permuation vector object 
via \code{get_order()}. Such a permutation vector can be directly used
to subset the list of original objects with \code{"["} to apply the permutation.
\emph{Note:} An alternative way to specify a permutation is via a list of the ranks of the objects after permutation (see \code{get_rank()}).


\code{ser_permutation_vector} objects are usually packed into 
a \code{ser_permutation} object
which is a collection of \eqn{k} permutation vectors for \eqn{k}-mode data.

The constructor \code{ser_permutation_vector} 
checks if the permutation vector is valid 
(i.e. if all integers occur exactly once).

The following functions are implemented: \code{print}, \code{rev},
\code{length}, \code{get_order}, \code{get_rank}, \code{get_method}.
}
\value{
An object of class \code{ser_permutation_vector}.
}
\seealso{
\code{\link{ser_permutation}},
\code{\link{get_order}},
\code{\link{get_rank}},
\code{\link{get_permutation_matrix}},
\code{\link{permutation_vector2matrix}}.
}
\author{Michael Hahsler}
\examples{
p <- ser_permutation_vector(sample(10), "random")
p

## some methods
length(p)
get_method(p)
get_order(p)
get_rank(p)
get_permutation_matrix(p)

r <- rev(p)
r
get_order(r)

## create a indentity permutation vector (with unknown length)
ip <- ser_permutation_vector(NA)
ip 
}
\keyword{classes}% at least one, from doc/KEYWORDS

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seriate_SA.R
\name{LS}
\alias{LS}
\alias{LS_swap}
\alias{LS_insert}
\alias{LS_reverse}
\alias{LS_mixed}
\title{Neighborhood functions for Seriation Method SA}
\usage{
LS_swap(o, pos = sample.int(length(o), 2))

LS_insert(o, pos = sample.int(length(o), 2))

LS_reverse(o, pos = sample.int(length(o), 2))

LS_mixed(o, pos = sample.int(length(o), 2))
}
\arguments{
\item{o}{an integer vector with the order}

\item{pos}{random positions used for the local move.}
}
\value{
returns the new order vector representing the random neighbor.
}
\description{
Definition of different local neighborhood functions for the method \code{"SA"} for \code{\link[=seriate]{seriate()}}.
}
\details{
Local neighborhood functions are \code{LS_insert}, \code{LS_swap}, \code{LS_reverse}, and \code{LS_mix}
(1/3 insertion, 1/3 swap and 1/3 reverse). Any neighborhood function can be defined.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hmap.R, R/gghmap.R
\name{hmap}
\alias{hmap}
\alias{gghmap}
\title{Plot Heat Map Reordered Using Seriation}
\usage{
hmap(
  x,
  distfun = stats::dist,
  method = "OLO",
  control = NULL,
  scale = c("none", "row", "column"),
  showDend = TRUE,
  col = NULL,
  row_labels = NULL,
  col_labels = NULL,
  ...
)

gghmap(
  x,
  distfun = stats::dist,
  method = "OLO",
  control = NULL,
  scale = c("none", "row", "column"),
  prop = FALSE,
  ...
)
}
\arguments{
\item{x}{a matrix or a dissimilarity matrix of class dist. If a
dissimilarity matrix is used, then the \code{distfun} is ignored.}

\item{distfun}{function used to compute the distance (dissimilarity) between
both rows and columns. For \code{gghmap()}, this
parameter is passed on in \code{control}.}

\item{method}{a character strings indicating the used seriation algorithm
(see \code{\link[=seriate.dist]{seriate.dist()}}). If the method results in a dendrogram then
\code{\link[stats:heatmap]{stats::heatmap()}} is used to show the dendrograms, otherwise
reordered distance matrices are shown instead.}

\item{control}{a list of control options passed on to the seriation
algorithm specified in \code{method}.}

\item{scale}{character indicating if the values should be centered and
scaled in either the row direction or the column direction, or none. Default
is none.}

\item{showDend}{Show dendrograms in the margin?}

\item{col}{a list of colors used.}

\item{row_labels, col_labels}{a logical indicating if row and column labels
in \code{x} should be displayed.  If \code{NULL} then labels are displayed
if the \code{x} contains the appropriate dimname and the number of labels is
25 or less. A character vector of the appropriate length with labels can
also be supplied.}

\item{\dots}{further arguments passed on to \code{\link[stats:heatmap]{stats::heatmap()}}.}

\item{prop}{logical; change the aspect ratio so cells in the image have a
equal width and height.}
}
\value{
An invisible list with elements:
\item{rowInd, colInd}{index permutation vectors.}
\item{reorder_method}{name of the method used to reorder the matrix.}

The list may contain additional elements (dendrograms, colors, etc).
}
\description{
Provides heatmaps reordered using several different seriation methods. This
includes dendrogram based reordering with optimal leaf order and matrix
seriation-based heat maps.
}
\details{
For dendrogram based heat maps, the arguments are passed on to
\code{\link[stats:heatmap]{stats::heatmap()}} in \pkg{stats}. The following arguments for \code{heatmap()}
cannot be used: \code{margins}, \code{Rowv}, \code{Colv}, \code{hclustfun}, \code{reorderfun}.

For seriation-based heat maps further arguments include:
\itemize{
\item \code{gp} an object of class \code{gpar} containing graphical
parameters (see \code{\link[=gpar]{gpar()}} in package \pkg{grid}).
\item \code{newpage} a logical indicating whether to start plot on a new
page
\item \code{prop} a logical indicating whether the height and width of \code{x} should
be plotted proportional to its dimensions.
\item \code{showdist} Display seriated dissimilarity matrices? Values are
\code{"none"}, \code{"both"}, \code{"rows"} or \code{"columns"}.
\item \code{key} logical; show a colorkey?
\item \code{key.lab} Label plotted next to the color key.
\item \code{margins} bottom and right-hand-side margins are calculated
automatically or can be specifies as a vector of two numbers (in lines).
\item \code{zlim} range of values displayed.
\item \code{col}, \code{col_dist} color palettes used.
}
}
\examples{
data("Wood")

# regular heatmap from package stats
heatmap(Wood, main = "Wood (standard heatmap)")

# Default heatmap does Euclidean distance, hierarchical clustering with
# average-link and optimal leaf ordering
hmap(Wood, main = "Wood (opt. leaf ordering)")

# Heatmap shown without dendrograms (used pimage)
hmap(Wood, main = "Wood (opt. leaf ordering)",  showDend = FALSE)

# Heatmap with correlation-based distance, green-red color (greenred is
# predefined) and optimal leaf ordering and no row label
dist_cor <- function(x) as.dist(1 - cor(t(x)))
hmap(Wood, distfun = dist_cor, col = greenred(100), row_labels = FALSE,
  main = "Wood (reorded by corr. between obs.)")

# Heatmap with order based on the angle in two-dimensional MDS space.
hmap(Wood, method = "MDS_angle", col = greenred(100), row_labels = FALSE,
  main = "Wood (reorderd using ange in MDS space)")

# Heatmap for distances
d <- dist(Wood)
hmap(d, method = "OLO", main = "Wood (Euclidean distances)")

# order-based with dissimilarity matrices
hmap(Wood, method = "MDS_angle", showdist = "both",
  col = greenred(100), col_dist = greens(100),
  keylab = "norm. Expression", main = "Wood (reporderd with distances)")

# Manually seriate and plot as pimage.
o <- seriate(Wood, method = "heatmap", control = list(dist_fun = dist, seriation_method = "OLO"))
o

pimage(Wood, o, prop = FALSE)

# Note: method heatmap calculates reorderd hclust objects which can be used for many heatmap
# implementations.
heatmap(Wood, Rowv = as.dendrogram(o[[1]]), Colv = as.dendrogram(o[[2]]))

# ggplot 2 version does not support dendrograms in the margin (for now)
if (require("ggplot2")) {
  library("ggplot2")

  gghmap(Wood) + labs(title = "Wood", subtitle = "Optimal leaf ordering")

  # More parameters (see ? ggpimage): reverse column order and flip axes, make a proportional plot
  gghmap(Wood, reverse_columns = TRUE) +
    labs(title = "Wood", subtitle = "Optimal leaf ordering")

  gghmap(Wood, flip_axes = TRUE) +
    labs(title = "Wood", subtitle = "Optimal leaf ordering")

  gghmap(Wood, flip_axes = TRUE, prop = TRUE) +
    labs(title = "Wood", subtitle = "Optimal leaf ordering")

  gghmap(Wood, distfun = dist_cor) +
    labs(title = "Wood", subtitle = "Reorded by correlation between observations") +
    scale_fill_gradient2(low = "darkgreen", high = "red")

  gghmap(d, prop = TRUE) +
    labs(title = "Wood", subtitle = "Euclidean distances, reordered)")

  # Note: the ggplot2-based version cannot show distance matrices in the same plot.

  # Manually seriate and plot as pimage.
  o <- seriate(Wood, method = "heatmap", control = list(dist_fun = dist,
    seriation_method = "OLO"))
  o

  ggpimage(Wood, o, prop = FALSE)
}
}
\seealso{
Other plots: 
\code{\link{VAT}()},
\code{\link{bertinplot}()},
\code{\link{dissplot}()},
\code{\link{palette}()},
\code{\link{pimage}()}
}
\author{
Michael Hahsler
}
\concept{plots}
\keyword{hplot}

\name{SupremeCourt}
\alias{SupremeCourt}
\docType{data}
\title{Voting Patterns in the Second Rehnquist U.S. Supreme Court}
\description{
    Contains a (a subset of the) decisions for the stable 8-yr
    period 1995-2002 of the second Rehnquist Supreme Court.
    Decisions are aggregated to
    the joint probability for disagreement between judges.
}
\usage{data("SupremeCourt")}
\format{
  A square, symmetric 9-by-9 matrix with the joint probability for disagreement.
}
\references{
    Sirovich, L. (2003). A pattern analysis of the second Rehnquist
    U.S. Supreme Court. \emph{Proceedings of the National Academy of Sciences of the United States of America,} 100, 7432-7437. doi:10.1073/pnas.1132164100.
}
\author{Michael Hahsler}
\examples{
data("SupremeCourt")

SupremeCourt

d <- as.dist(SupremeCourt)
o <- seriate(d)

pimage(d, o)

plot(hclust(as.dist(SupremeCourt)))
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AAA_registry_seriate.R
\docType{data}
\name{registry_seriate}
\alias{registry_seriate}
\alias{list_seriation_methods}
\alias{get_seriation_method}
\alias{set_seriation_method}
\alias{print.seriation_method}
\title{Registry for Seriation Methods}
\format{
An object of class \code{seriation_registry} (inherits from \code{registry}) of length 48.
}
\usage{
registry_seriate

list_seriation_methods(kind)

get_seriation_method(kind, name)

set_seriation_method(
  kind,
  name,
  definition,
  description = NULL,
  control = list(),
  ...
)

\method{print}{seriation_method}(x, ...)
}
\arguments{
\item{kind}{the data type the method works on. For example, \code{"dist"},
\code{"matrix"} or \code{"array"}. If missing, then methods for any type are
shown.}

\item{name}{the name for the method used to refer to the method in
\code{\link[=seriate]{seriate()}}.}

\item{definition}{a function containing the method's code.}

\item{description}{a description of the method. For example, a long name.}

\item{control}{a list with control arguments and default values.}

\item{...}{further information that is stored for the method in the
registry.}

\item{x}{an object of class  "seriation_method" to be printed.}
}
\value{
\itemize{
\item \code{list_seriation_method()} result is a vector of character strings with the
names of the methods. These names are used for methods in \code{seriate()}.
\item \code{get_seriation_method()} returns a given method in form of an object of class
\code{"seriation_method"}.
}
}
\description{
A registry to manage methods used by \code{\link[=seriate]{seriate()}}.
}
\details{
The functions below are convenience function for the registry
\code{registry_seriate}.

\code{list_seriation_method()} lists all available methods for a given data
type (\code{kind}) (e.g., "dist", "matrix").
The result is a vector of character strings with the
method names that can be used in function \code{seriate()}.
If \code{kind} is missing, then a list of
methods is returned.

\code{get_seriation_method()} returns detailed information for a given method in
form of an object of class \code{"seriation_method"}.
The information includes a description, parameters and the
implementing function.

With \code{set_seriation_method()} new seriation methods can be added by the
user. The implementing function (\code{definition}) needs to have the formal
arguments \code{x, control}, where \code{x} is the data object and
\code{control} contains a list with additional information for the method
passed on from \code{seriate()}.  The implementation has to return a list of
objects which can be coerced into \code{ser_permutation_vector} objects
(e.g., integer vectors). The elements in the list have to be in
corresponding order to the dimensions of \code{x}.
}
\examples{
# Registry
registry_seriate

# List all seriation methods by type
list_seriation_methods()

# List methods for matrix seriation
list_seriation_methods("matrix")

get_seriation_method(name = "BEA")

# Example for defining a new seriation method (reverse identity function for matrix)

# 1. Create the seriation method
seriation_method_reverse <- function(x, control) {
   # return a list of order vectors, one for each dimension
   list(seq(nrow(x), 1), seq(ncol(x), 1))
}

# 2. Register new method
set_seriation_method("matrix", "Reverse", seriation_method_reverse,
    description = "Reverse identity order", control = list())

list_seriation_methods("matrix")
get_seriation_method("matrix", "reverse")

# 3. Use the new seriation methods
seriate(matrix(1:12, ncol=3), "reverse")
}
\seealso{
This registry uses \code{\link[=registry]{registry()}} in package \pkg{registry}.

Other seriation: 
\code{\link{register_DendSer}()},
\code{\link{register_GA}()},
\code{\link{register_optics}()},
\code{\link{register_tsne}()},
\code{\link{register_umap}()},
\code{\link{seriate}()}
}
\author{
Michael Hahsler
}
\concept{seriation}
\keyword{misc}

\name{Psych24}
\alias{Psych24}
\docType{data}
\title{Results of 24 Psychological Test for 8th Grade Students}
\description{
A data set collected by Holzinger and Swineford (1939) which consists of the
results of 24 psychological tests given to 145 seventh and eighth grade
students in a Chicago suburb. This data set contains the correlation matrix
for the 24 test results.

The data set was also used as an example for visualization of cluster analysis
by Ling (1973).

}
\usage{data("Psych24")}
\format{
 A 24 x 24 correlation matrix.  
}
%\details{
%}
\references{
Holzinger, K. L., Swineford, F. (1939): 
A study in factor analysis: The stability of a bi-factor solution.
\emph{Supplementary Educational Monograph,} No. \bold{48}. 
Chicago: University of Chicago Press.

Ling, R. L. (1973): A computer generated aid for cluster analysis. 
\emph{Communications of the ACM}, 
\bold{16}(6), pp. 355--361.
}
\examples{
data("Psych24")

## create a dist object and also get rid of the one negative entry in the
## correlation matrix
d <- as.dist(1 - abs(Psych24))

pimage(d)

## do hclust as in Ling (1973)
hc <- hclust(d, method = "complete")
plot(hc)

pimage(d, hc)

## use seriation
order <- seriate(d, method = "tsp")
#order <- seriate(d, method = "tsp", control = list(method = "concorde"))
pimage(d, order)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/register_optics.R
\name{register_optics}
\alias{register_optics}
\alias{optics}
\alias{OPTICS}
\title{Register Seriation Based on OPTICS}
\usage{
register_optics()
}
\value{
Nothing.
}
\description{
Use ordering points to identify the clustering structure (OPTICS) for \code{\link[=seriate]{seriate()}}.
}
\details{
Registers the method \code{"optics"} for \code{\link[=seriate]{seriate()}}. This method applies
the OPTICS ordering algorithm to create an ordering.

\bold{Note:} Package \pkg{dbscan} needs to be installed.
}
\examples{

\dontrun{
register_optics()
get_seriation_method("dist", "optics")

d <- dist(random.robinson(50, pre=TRUE, noise=.1))

o <- seriate(d, method = "optics")
pimage(d, o)
}

}
\references{
Mihael Ankerst, Markus M. Breunig, Hans-Peter Kriegel, Joerg
Sander (1999). OPTICS: Ordering Points To Identify the Clustering Structure.
ACM SIGMOD international conference on Management of data. ACM Press. pp.
49-60. \doi{10.1145/304181.304187}
}
\seealso{
\code{\link[dbscan:optics]{dbscan::optics()}} in \pkg{dbscan}.

Other seriation: 
\code{\link{register_DendSer}()},
\code{\link{register_GA}()},
\code{\link{register_tsne}()},
\code{\link{register_umap}()},
\code{\link{registry_seriate}},
\code{\link{seriate}()}
}
\concept{seriation}
\keyword{cluster}
\keyword{optimize}

\name{uniscale}
\alias{uniscale}
\alias{orderplot}
\title{Unidimensional Scaling from Seriation Results}
\description{
Performs (approximate) unidimensional scaling by first performin seriation
to obtain a permutation and the using the permutation to calulate the
configuration.
}
\usage{
uniscale(d, order = NULL, method = "QAP_LS", rep = 10, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{d}{a dissimilarity matrix.}
  \item{order}{a precomputed permutation (configuration) order.
  If \code{NULL}, then seriation is performed using the method
  specified in \code{method}.
}
  \item{method}{seriation method used if \code{o} is \code{NULL}.
}
  \item{rep}{Number of repetitions of the seriation heuristic.}
  \item{\dots}{additional arguments are passed on to the seriation method.}
}
\details{
Uses the method describes in Maier and De Leeuw (2015) to calculate the
minimum stress configuration for either a given configuration/permutation/order
or for a permutation computed via a seriation method.

The code is similar to \code{uniscale} in \pkg{smacof}, but scales to larger
datasets since it does not check all permutations.
}
\value{
A vector with the fitted configuration.
}
\references{
Mair P., De Leeuw J. (2015). Unidimensional scaling. In \emph{Wiley StatsRef: Statistics Reference Online,} Wiley, New York.
\doi{10.1002/9781118445112.stat06462.pub2}
}
\author{
Michael Hahsler with code from Patrick Mair (from \pkg{smacof}).
}
\examples{
data(SupremeCourt)

d <- as.dist(SupremeCourt)

sc <- uniscale(d)
sc

orderplot(sc)
}
\keyword{optimize}% use one of  RShowDoc("KEYWORDS")

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seriate.R, R/seriate.array.R,
%   R/seriate.data.frame.R, R/seriate.dist.R, R/seriate.matrix.R,
%   R/seriate.table.R
\name{seriate}
\alias{seriate}
\alias{seriate.array}
\alias{seriate.data.frame}
\alias{seriate.dist}
\alias{seriate.matrix}
\alias{seriate.table}
\title{Seriate Dissimilarity Matrices, Matrices or Arrays}
\usage{
seriate(x, ...)

\method{seriate}{array}(x, method = "PCA", control = NULL, margin = seq(length(dim(x))), ...)

\method{seriate}{data.frame}(x, method = "Heatmap", control = NULL, margin = c(1, 2), ...)

\method{seriate}{dist}(x, method = "Spectral", control = NULL, ...)

\method{seriate}{matrix}(x, method = "PCA", control = NULL, margin = c(1, 2), ...)

\method{seriate}{table}(x, method = "CA", control = NULL, margin = c(1, 2), ...)
}
\arguments{
\item{x}{the data.}

\item{...}{further arguments are added to the \code{control} list.}

\item{method}{a character string with the name of the seriation method
(default: varies by data type).}

\item{control}{a list of control options passed on to the seriation
algorithm.}

\item{margin}{a vector giving the margin indices (dimensions) to be
seriated. For example, for a matrix, \code{1} indicates rows, \code{2}
indicates columns, \code{c(1,2)} indicates rows and columns. Unseriated margins return
a identity seriation order.}
}
\value{
Returns an object of class \link{ser_permutation}.
}
\description{
Tries to find an linear order for objects using data in form of a
dissimilarity matrix (two-way one mode data), a data matrix (two-way
two-mode data) or a data array (k-way k-mode data). The order can then be
used to reorder the dissimilarity matrix/data matrix using
\code{\link[=permute]{permute()}}.
}
\details{
Seriation methods are managed via a registry. See
\code{\link[=list_seriation_methods]{list_seriation_methods()}} for help. In the following, we discuss only the
built-in methods that are registered automatically by the package \pkg{seriation}.

Many seriation methods (heuristically) optimize (minimize or maximize) an
objective function. The value of the function for a given seriation can be
calculated using \code{\link[=criterion]{criterion()}}. In this manual page, we
include the measure which is optimized by each method using \strong{bold font}.
If no measure is mentioned, then the measure does not directly optimize a measure.
A definition of the measures can be found in the \code{\link[=criterion]{criterion()}} manual page.

\strong{Seriation methods for distance matrices (dist)}

One-mode two-way data has to be provided as a dist object (not
as a symmetric matrix). Similarities have to be transformed into
dissimilarities. Currently, the following methods are implemented (for a
more detailed description and an experimental comparison see
\href{https://michael.hahsler.net/research/paper/EJOR_seriation_2016.pdf}{Hahsler (2017)}:
\itemize{
\item "ARSA" Anti-Robinson seriation by simulated
annealing to minimize the \strong{linear seriation criterion} (simulated
annealing initialization used in Brusco et al 2008).
\item "BBURCG" Anti-Robinson seriation by branch-and-bound to
minimize the \strong{unweighted gradient measure} (Brusco and Stahl 2005).
This is only feasible for a relatively small number of objects.
\item "BBWRCG" Anti-Robinson seriation by branch-and-bound to
minimize the \strong{weighted gradient measure} (Brusco and Stahl 2005). This
is only feasible for a relatively small number of objects.
\item "TSP" Traveling salesperson problem solver to minimize the
\strong{Hamiltonian path length}. The solvers in \pkg{TSP} are used (see
\code{\link[TSP:solve_TSP]{TSP::solve_TSP()}}). The solver method can be passed on via the \code{control}
argument, e.g. \code{control = list(method = "two_opt")}. Default is the est
of 10 runs of arbitrary insertion heuristic with 2-opt improvement.

Since a tour returned by a TSP solver is a connected circle and we are
looking for a path representing a linear order, we need to find the best
cutting point.  Climer and Zhang (2006) suggest to add a dummy city with
equal distance to each other city before generating the tour. The place of
this dummy city in an optimal tour with minimal length is the best cutting
point (it lies between the most distant cities).
\item "R2E" Rank-two ellipse seriation (Chen 2002).

This method starts with generating a sequence of correlation matrices
\eqn{R^1, R^2, \ldots}. \eqn{R^1} is the correlation matrix of the original
distance matrix \eqn{D} (supplied to the function as \code{x}), and
\deqn{R^{n+1} = \phi R^n,} where \eqn{\phi} calculates the correlation
matrix.

The rank of the matrix \eqn{R^n} falls with increasing \eqn{n}. The first
\eqn{R^n} in the sequence which has a rank of 2 is found. Projecting all
points in this matrix on the first two eigenvectors, all points fall on an
ellipse. The order of the points on this ellipse is the resulting order.

The ellipse can be cut at the two interception points (top or bottom) of the
vertical axis with the ellipse. In this implementation the top most cutting
point is used.
\item "MDS", "MDS_metric", "MDS_nonmetric", "MDS_angle" Multidimensional scaling (MDS).

Use multidimensional scaling techniques to find an linear order by
minimizing \strong{stress}. Note MDS algorithms used for a single dimension
tend to end up in local optima and unidimensional scaling (see Maier and De
Leeuw, 2015) would be more appropriate. However, generally, ordering along
the first component of MDS provides good results.

\code{control} parameters:
\itemize{
\item \code{method}: One of \code{"cmdscale"}, \code{"isoMDS"} or \code{"sammon"}. \code{"cmdscale"} performs metric
MDS using \code{\link[stats:cmdscale]{stats::cmdscale()}}. Non-metric MDS methods \code{"isoMDS"} and \code{"sammon"}
are preformed using \code{\link[MASS:isoMDS]{MASS::isoMDS()}}.
}

By default, metric MDS is used (\code{\link[stats:cmdscale]{stats::cmdscale()}}). In case of
of general dissimilarities, non-metric MDS can be used. The method can be specified
as the element \code{method} (\code{"cmdscale"}, \code{"isoMDS"} or \code{"sammon"}) in \code{control}.

For convenience, seriation methods \code{"MDS_metric"} performs \code{\link[=cmdscale]{cmdscale()}} and
\code{"MDS_nonmetric"} performs \code{\link[MASS:isoMDS]{MASS::isoMDS()}}.

\code{"MDS_angle"} projects the data on the first two components found by
MDS and then orders by the angle in this space. The order is split by the
larges gap between adjacent angles. A similar method was used for ordering
correlation matrices by Friendly (2002).
\item "HC", "HC_single", "HC_complete", "HC_average", "HC_ward" Hierarchical clustering.

Using the order of the leaf nodes in a dendrogram obtained by hierarchical
clustering can be used as a very simple seriation technique. This method
applies hierarchical clustering (\code{\link[=hclust]{hclust()}}) to \code{x}. The clustering
method can be given using a \code{"method"} element in the \code{control}
list. If omitted, the default \code{"average"} is used.

For convenience the other methods are provided as shortcuts.
\item "GW" Hierarchical clustering (Gruvaeus and Wainer, 1972).

The methods start with a dendrogram created by \code{\link[=hclust]{hclust()}}. As the
\code{"method"} element in the \code{control} list a clustering method
(default \code{"average"}) can be specified. Alternatively, an \link{hclust}
object can be supplied using an element named \code{"hclust"}.

A dendrogram (binary tree) has \eqn{2^{n-1}} internal nodes (subtrees) and
the same number of leaf orderings. That is, at each internal node the left
and right subtree (or leaves) can be swapped, or, in terms of a dendrogram,
be flipped. The leaf-node reordering to minimize
\strong{Hamiltonian path length (restricted)}.

Method \code{"GW"} uses an algorithm developed by Gruvaeus and Wainer (1972)
as implemented \code{\link[gclus:hclust]{gclus::reorder.hclust()}} (Hurley 2004).  The clusters are
ordered at each level so that the objects at the edge of each cluster are
adjacent to that object outside the cluster to which it is nearest. The
method produces an unique order.

For convenience \code{"GW_single"}, \code{"GW_average"},
\code{"GW_complete"}, and \code{"GW_ward"} are provided.
\item "OLO" Optimal leaf ordering (Bar-Joseph et al., 2001).

Also starts with a dendrogram and
produces an optimal leaf ordering with respect to the minimizing the sum of
the distances along the (Hamiltonian) path connecting the leaves in the
given order. The time complexity of the algorithm is \eqn{O(n^3)}. Note that
non-finite distance values are not allowed.

For convenience \code{"OLO_single"},
\code{"OLO_average"}, \code{"OLO_complete"}, and \code{"OLO_ward"} are provided.
\item "VAT" Visual Assessment of (Clustering) Tendency (Bezdek and Hathaway (2002)).

Creates an order based on Prim's algorithm for finding a minimum spanning
tree (MST) in a weighted connected graph representing the distance matrix.
The order is given by the order in which the nodes (objects) are added to
the MST.
\item "SA" Simulated Annealing for diverse criterion measures.

Implement simulated annealing similar to the ARSA method, however, it works
for any criterion measure defined in \pkg{seriation}. By default the
algorithm optimizes for raw gradient measure and is warm started with the
result of spectral seriation (2-Sum problem) since Hahsler (2017) shows that
2-Sum solutions are similar to solutions for the gradient measure.

Several popular local neighborhood functions are
provided an new can be defined (see \link{LS}).

Note that this is an R implementation repeatedly calling criterion, and
therefore is relatively slow.
\item "Spectral", "Spectral_norm" Spectral seriation (Ding and He 2004).

Spectral seriation uses a relaxation to minimize the \strong{2-Sum Problem}
(Barnard, Pothen, and Simon, 1993). It uses the order of the Fiedler vector
of the similarity matrix's (normalized) Laplacian.

Spectral seriation gives a good trade-off between seriation quality, speed
and scalability (see Hahsler, 2017).
\item "SPIN_STS", "SPIN_NH" Sorting Points Into Neighborhoods (SPIN) (Tsafrir 2005).

Given a weight matrix \eqn{W}, the algorithms try to
minimize the energy for a permutation (matrix \eqn{P}) given by \deqn{F(P) =
  tr(PDP^TW),} where \eqn{tr} denotes the matrix trace.

\code{"SPIN_STS"} implements the Side-to-Side algorithm which tries to push
out large distance values. The default weight matrix suggested in the paper
with \eqn{W=XX^T} and \eqn{X_i=i-(n+1)/2} is used. We run the algorithm from
\code{step} (25) iteration and restart the algorithm \code{nstart} (10) with
random initial permutations (default values in parentheses). Via
\code{control} the parameters \code{step}, \code{nstart}, \code{X} and
\code{verbose}.

\code{"SPIN_NH"} implements the neighborhood algorithm (concentrate low
distance values around the diagonal) with a Gaussian weight matrix
\eqn{W_{ij} = exp(-(i-j)^2/n\sigma)}, where \eqn{n} is the size of the
dissimilarity matrix and \eqn{\sigma} is the variance around the diagonal
that control the influence of global (large \eqn{\sigma}) or local (small
\eqn{\sigma}) structure.

We use the heuristic suggested in the paper for the linear assignment
problem. We do not terminate as indicated in the algorithm, but run all the
iterations since the heuristic does not guarantee that the energy is
strictly decreasing. We also implement the heuristic "annealing" scheme
where \eqn{\sigma} is successively reduced. The parameters in \code{control}
are \code{sigma} which can be a single value or a decreasing sequence
(default: 20 to 1 in 10 steps) and \code{step} which defines how many update
steps are performed before for each value of \code{alpha}. Via
\code{W_function} a custom function to create \eqn{W} with the function
signature \verb{function(n, sigma, verbose)} can be specified. The parameter
\code{verbose} can be used to display progress information.
\item "QAP_LS", "QAP_2SUM", "QAP_BAR", "QAP_Inertia" Quadratic assignment problem
formulations for seriation using a simulated annealing solver.

These methods minimize the
\strong{Linear Seriation Problem} (LS) formulation (Hubert and Schultz 1976),
the \strong{2-Sum Problem} formulation (Barnard, Pothen, and Simon 1993), the
\strong{banded anti-Robinson form} (BAR) or the \strong{inertia criterion.}

\code{control} parameters are passed on to \code{\link[qap:qap]{qap::qap()}}.
An important parameter is \code{rep} to return the best result out of the
given number of repetitions with random restarts. Default is 1, but bigger
numbers result in better and more stable results.
\item "GA" Use a genetic algorithm to optimize for various criteria.

The GA code has to be first registered. A detailed description can
be found in the manual page for \code{\link[=register_GA]{register_GA()}}.
\item "DendSer" Use heuristic dendrogram seriation to optimize for various criteria.

The DendSer code has to be first registered. A
detailed description can be found in the manual page for
\code{\link[=register_DendSer]{register_DendSer()}}.
\item "Identity" Produces an identity permutation.
\item "Random"  Produces a random permutation.
}

\strong{Seriation methods for matrices (matrix or data.frame)}

Two-mode two-way data are general matrices.
Some methods also require that the matrix is positive. Data frames are just a
different representation of a matrix and all seriation methods for matrix can
be also used for data frames. The default method for data frames is heatmap
seriation which calculates distances between rows and between columns and
then applies seriation on these using hierarchical clustering and optimal
leaf ordering (OLO).

Currently the
following methods are implemented for matrix:
\itemize{
\item "BEA" Bond Energy Algorithm (BEA; McCormick 1972).

The algorithm tries to maximize the \strong{Measure of Effectiveness.} of a
non-negative matrix. Due to the definition of this measure, the tasks of
ordering rows and columns is separable and can be solved independently.

A row is arbitrarily placed; then rows are positioned one by one. When this
is completed, the columns are treated similarly. The overall procedure
amounts to two approximate traveling salesperson problems (TSP), one on the
rows and one on the columns. The so-called `best insertion strategy' is
used: rows (or columns) are inserted into the current permuted list of rows
(or columns). Several consecutive runs of the algorithm might improve the
energy.

Note that Arabie and Hubert (1990) question its use with non-binary data if
the objective is to find a seriation or one-dimensional orderings of rows
and columns.

The BEA code used in this package was implemented by Fionn Murtagh.

\code{control} parameter:
\itemize{
\item \code{"rep"}: the number of runs can be specified.
The results of the best run will be returned.
}
\item "BEA_TSP" Use a TSP to optimize the \strong{Measure of Effectiveness} (Lenstra 1974).

\code{control} parameter:
\itemize{
\item \code{"method"}: a TSP solver method (see \code{\link[TSP:solve_TSP]{TSP::solve_TSP()}}).
}
\item "CA" Correspondence analysis for a table/matrix of frequencies.

This function is designed to help simplify a mosaic plot or other displays of a
matrix of frequencies.  It calculates a correspondence analysis of the matrix and
an order for rows and columns according to the scores on a correspondence analysis dimension.

\code{control} parameters:
\itemize{
\item \code{"dim"}: CA dimension used for reordering.
\item \code{"ca_param"}: List with parameters for the call to \code{\link[ca:ca]{ca::ca()}}.
}
\item "Heatmap" Heatmap seriation

Calculates distances between
rows and between columns and then applies seriation on these using
hierarchical clustering and optimal leaf ordering (method \code{"OLO"} for distance matrices).
\item "PCA" Order by the first principal component.

Uses the projection of the data on its first principal component to
determine the order.

Note that for a distance matrix calculated from \code{x} with Euclidean
distance, this methods minimizes the least square criterion.
\item "PCA_angle" Order using the first two principal components.

Projects the data on the first two principal components
and then orders by the angle in this space. The order is split by the larges
gap between adjacent angles. A similar method was used for ordering
correlation matrices by Friendly (2002).
\item "Identity" Produces an identity permutation.
\item "Random" Produces a random permutation.
}

For \strong{general arrays} no built-in methods are currently available.
}
\examples{
# Show available seriation methods (for dist and matrix)
list_seriation_methods()

### Seriate as distance matrix (for the iris dataset)
data("iris")
x <- as.matrix(iris[-5])
x <- x[sample(1:nrow(x)), ]
d <- dist(x)

order <- seriate(d)
order

pimage(d, main = "Distances (Random Order)")
pimage(d, order, main = "Distances (Reordered)")

# Compare seriation quality
rbind(
        random = criterion(d),
        reordered = criterion(d, order)
     )

# Reorder the distance matrix
d_reordered <-  permute(d, order)
pimage(d_reordered, main = "Distances (Reordered)")


### Seriate a matrix
data("iris")
x <- as.matrix(iris[-5])

# To make the variables comparable, we scale the data
x <- scale(x, center = FALSE)

# The iris flowers are ordered by species in the data set
pimage(x, main = "original data", prop = FALSE)
criterion(x)

# Apply some methods
order <- seriate(x, method = "BEA_TSP")
pimage(x, order, main = "TSP to optimize ME", prop = FALSE)
criterion(x, order)

order <- seriate(x, method = "PCA")
pimage(x, order, main = "First principal component", prop = FALSE)
criterion(x, order)

order <- seriate(x, method = "heatmap")
pimage(x, order, main = "Heatmap seriation", prop = FALSE)
criterion(x, order)

# reorder the matrix
x_reordered <- permute(x, order)

# create a heatmap seriation manually by calculating
# distances between rows and between columns
order <- c(
    seriate(dist(x), method = "OLO"),
    seriate(dist(t(x)), method = "OLO")
)
pimage(x, order, main = "Heatmap seriation", prop = FALSE)
criterion(x, order)

### Seriate a correlation matrix
corr <- cor(x)
pimage(corr, upper_tri = FALSE, main = "Correlation matrix")

# we need to define a distance (we used d = sqrt(2(1 - r))) and
# then reorder the matrix (rows and columns).
d <- as.dist(sqrt(2 * (1 - corr)))
o <- seriate(d)
corr_reordered <- permute(corr, order = c(o, o))
pimage(corr_reordered, upper_tri = FALSE, main = "Correlation matrix (reordered)")
}
\references{
Arabie, P. and L.J. Hubert (1990): The bond energy algorithm
revisited, \emph{IEEE Transactions on Systems, Man, and Cybernetics,}
\bold{20}(1), 268--274.
\doi{10.1109/21.47829}

Bar-Joseph, Z., E. D. Demaine, D. K. Gifford, and T. Jaakkola. (2001): Fast
Optimal Leaf Ordering for Hierarchical Clustering. \emph{Bioinformatics,}
\bold{17}(1), 22--29.
\doi{10.1093/bioinformatics/17.suppl_1.S22}

Barnard, S. T., A. Pothen, and H. D. Simon (1993): A Spectral Algorithm for
Envelope Reduction of Sparse Matrices. \emph{In Proceedings of the 1993
ACM/IEEE Conference on Supercomputing,} 493--502. Supercomputing '93. New
York, NY, USA: ACM. \url{https://ieeexplore.ieee.org/document/1263497}

Bezdek, J.C. and Hathaway, R.J. (2002): VAT: a tool for visual assessment of
(cluster) tendency. \emph{Proceedings of the 2002 International Joint
Conference on Neural Networks (IJCNN '02)}, Volume: 3, 2225--2230.
\doi{10.1109/IJCNN.2002.1007487}

Brusco, M., Koehn, H.F., and Stahl, S. (2008): Heuristic Implementation of
Dynamic Programming for Matrix Permutation Problems in Combinatorial Data
Analysis. \emph{Psychometrika,} \bold{73}(3), 503--522.
\doi{10.1007/s11336-007-9049-5}

Brusco, M., and Stahl, S. (2005): \emph{Branch-and-Bound Applications in
Combinatorial Data Analysis.} New York: Springer.
\doi{10.1007/0-387-28810-4}

Chen, C. H. (2002): Generalized Association Plots: Information Visualization
via Iteratively Generated Correlation Matrices. \emph{Statistica Sinica,}
\bold{12}(1), 7--29.

Ding, C. and Xiaofeng He (2004): Linearized cluster assignment via spectral
ordering. \emph{Proceedings of the Twenty-first International Conference on
Machine learning (ICML '04)}.
\doi{10.1145/1015330.1015407}

Climer, S. and Xiongnu Zhang (2006): Rearrangement Clustering: Pitfalls,
Remedies, and Applications, \emph{Journal of Machine Learning Research,}
\bold{7}(Jun), 919--943.

Friendly, M. (2002): Corrgrams: Exploratory Displays for Correlation
Matrices. \emph{The American Statistician}, \bold{56}(4), 316--324.
\doi{10.1198/000313002533}

Gruvaeus, G. and Wainer, H. (1972): Two Additions to Hierarchical Cluster
Analysis, \emph{British Journal of Mathematical and Statistical Psychology,}
\bold{25}, 200--206.
\doi{10.1111/j.2044-8317.1972.tb00491.x}

Hahsler, M. (2017): An experimental comparison of seriation methods for
one-mode two-way data. \emph{European Journal of Operational Research,}
\bold{257}, 133--143.
\doi{10.1016/j.ejor.2016.08.066}

Hubert, Lawrence, and James Schultz (1976): Quadratic Assignment as a
General Data Analysis Strategy. \emph{British Journal of Mathematical and
Statistical Psychology} \bold{29}(2). Blackwell Publishing Ltd. 190--241.
\doi{10.1111/j.2044-8317.1976.tb00714.x}

Hurley, Catherine B. (2004): Clustering Visualizations of Multidimensional
Data. \emph{Journal of Computational and Graphical Statistics,}
\bold{13}(4), 788--806.
\doi{10.1198/106186004X12425}

Lenstra, J.K (1974): Clustering a Data Array and the Traveling-Salesman
Problem, \emph{Operations Research,} \bold{22}(2) 413--414.
\doi{10.1287/opre.22.2.413}

Mair P., De Leeuw J. (2015). Unidimensional scaling. In \emph{Wiley
StatsRef: Statistics Reference Online,} Wiley, New York.
\doi{10.1002/9781118445112.stat06462.pub2}

McCormick, W.T., P.J. Schweitzer and T.W. White (1972): Problem
decomposition and data reorganization by a clustering technique,
\emph{Operations Research,} \bold{20}(5), 993--1009.
\doi{10.1287/opre.20.5.993}

Tsafrir, D., Tsafrir, I., Ein-Dor, L., Zuk, O., Notterman, D.A. and Domany,
E. (2005): Sorting points into neighborhoods (SPIN): data analysis and
visualization by ordering distance matrices, \emph{Bioinformatics,}
\bold{21}(10) 2301--8.
\doi{10.1093/bioinformatics/bti329}
}
\seealso{
Other seriation: 
\code{\link{register_DendSer}()},
\code{\link{register_GA}()},
\code{\link{register_optics}()},
\code{\link{register_tsne}()},
\code{\link{register_umap}()},
\code{\link{registry_seriate}}
}
\author{
Michael Hahsler
}
\concept{seriation}
\keyword{cluster}
\keyword{optimize}

\name{criterion}
\alias{criterion}
\alias{criterion.dist}
\alias{criterion.matrix}
\alias{criterion.array}
\title{Criterion for a Loss/Merit Function for Data Given a Permutation}
\description{
Compute the value for different loss functions \eqn{L} and
merit function \eqn{M} for
  data given a permutation.
}
\usage{
criterion(x, order = NULL, method = NULL, force_loss = FALSE, ...)
}
\arguments{
  \item{x}{an object of class \code{dist} or a matrix (currently no
    functions are implemented for array).}
  \item{order}{an object of class \code{ser_permutation} suitable for
    \code{x}.  If \code{NULL}, the identity permutation is used.
  }
  \item{method}{a character vector with the names of the criteria to be
    employed, or \code{NULL} (default) in which case all available
    criteria are used.}
  \item{...}{additional parameters passed on to the criterion method.}
  \item{force_loss}{logical; should merit function be converted into
    loss functions by multiplying with -1?}
}
\details{
For a symmetric dissimilarity matrix
\eqn{D} with elements \eqn{d(i,j)} where \eqn{i, j = 1 \ldots n},
the aim is generally to place low distance values close to the diagonal.
The following criteria to judge the quality of a
certain permutation of the objects in a dissimilarity matrix
are currently implemented (for a more detailed description and an
experimental comparison see Hahsler (2017)):
\describe{

\item{\code{"Gradient_raw", "Gradient_weighted"}}{Gradient measures
    (Hubert et al 2001).
    A symmetric dissimilarity matrix
    where the values in all rows and columns only increase when
    moving away from the main diagonal is called a perfect
    \emph{anti-Robinson matrix} (Robinson 1951). A suitable merit
    measure which quantifies the divergence of a matrix from the
    anti-Robinson form is

    \deqn{
    M(D) = \sum_{i=1}^n \sum_{i<k<j} f(d_{ij}, d_{ik}) + \sum_{i<k<j} f(d_{ij}, d_{kj})}
    where \eqn{f(.,.)}
    is a function which defines how a violation
    or satisfaction
    of a gradient condition for an object triple (\eqn{O_i, O_k, O_j})
    is counted.

    Hubert et al (2001) suggest two functions. The first
    function is given by:
    \deqn{f(z,y) = sign(y-z) = +1 if z < y; 0 if z = y; and -1 if z > y.}

    It results in raw number of triples satisfying the gradient
    constraints minus triples which violate the constraints.

    The second function is defined as:
    \deqn{f(z,y) = |y-z| sign(y-z) = y-z}
    It weights the each satisfaction or violation by the difference by
    its magnitude given by the absolute difference between the values.
}

\item{\code{"AR_events", "AR_deviations"}}{Anti-Robinson events (Chen 2002).
    An even simpler loss function can be created in the same way as the gradient
    measures above by concentrating on violations only.
    \deqn{
        L(D) =
        \sum_{i=1}^n \sum_{i<k<j} f(d_{ik}, d_{ij}) + \sum_{i<k<j} f(d_{kj}, d_{ij})
    }

    To only count the violations we use
    \deqn{
        f(z, y) = I(z, y) = 1 if z < y and 0 otherwise.
   }


   \eqn{I(\cdot)} is an indicator function returning 1 only for violations.
   Chen (2002) presented a formulation for an equivalent
loss function and called the violations \emph{anti-Robinson events}
and also introduced a weighted versions of the loss
function resulting in
\deqn{
        f(z, y) = |y-z|I(z, y)
    }
    using the absolute deviations as weights.
}

\item{\code{"RGAR"}}{Relative generalized Anti-Robinson events (Tien et al 2008).
    Counts Anti-Robinson events in a variable band
    (window specified by \code{w} defaults to the maximum of \eqn{n-1}) around the main diagonal and normalizes by
    the maximum of possible events.

    \deqn{
        L(D) = 1/m
        \sum_{i=1}^n \sum_{(i-w)\le j<k<i} I(d_{ij} < d_{ik}) + \sum_{i<j<k\le(i+w))} I(d_{ij} > d_{ik})
    }

    where \eqn{m=(2/3-n)w + nw^2 - 2/3 w^3}, the maximal number
    of possible anti-Robinson events in the window.
  The window size \eqn{w} represents the number of neighboring objects (number of entries from the diagonal of the distance matrix) are considered. The window size is \eqn{2 \le w < n}, where smaller values result in
  focusing on the local structure while larger values look at the global structure. Alternatively, \code{pct} can be used instead of \code{w} to
  specify the window as a percentage of \eqn{n}. \code{relative=FALSE} can be
 to get the GAR, i.e., the absolute number of AR events in the window.
}

\item{\code{"BAR"}}{Banded Anti-Robinson Form (Earle and Hurley 2015).

    Simplified measure for closeness to the anti-Robinson form in a band of
    size \eqn{b} with \eqn{1 <= b < n} around the diagonal.

    \deqn{
        L(D) =
        \sum_{|i-j|<=b} (b+1-|i-j|) d_{ij}
    }

    For \eqn{b=1} the measure reduces to the Hamiltonian path length.
    For \eqn{b=n-1} the measure is equivalent to ARc defined
    (Earle and Hurley, 2015). Note that ARc is equivalent to the
    Linear Seriation criterion (scaled by 1/2).

    \eqn{b} defaults to a band of 20\% of \eqn{n}.
}



\item{\code{"Path_length"}}{Hamiltonian path length (Caraux and Pinloche 2005).

    The order of the objects in a dissimilarity matrix corresponds to a path
    through a graph where each node represents an object and is visited exactly
    once, i.e., a Hamilton path. The length of the path is defined as the sum
    of the edge weights, i.e., dissimilarities.

    \deqn{L(D) = \sum_{i=1}^{n-1} d_{i,i+1}}

    The length of the Hamiltonian path is equal to the
    value of the minimal span loss function (as used by Chen 2002).
    Both notions are related to the \emph{traveling salesperson problem (TSP).}

    If \code{order} is not unique or
    there are non-finite distance values \code{NA} is returned.}

\item{\code{"Lazy_path_length"}}{Lazy path length (Earl and Hurley 2015).

    A weighted version of the Hamiltonian path criterion. This loss function
    postpones larger distances to later in the order (i.e., a lazy traveling
    sales person).

    \deqn{L(D) = \sum_{i=1}^{n-1} (n-i) d_{i,i+1}}

    Earl and Hurley (2015) proposed this criterion for reordering in
    visualizations to concentrate on closer objects first.
    }


\item{\code{"Inertia"}}{Inertia criterion (Caraux and Pinloche 2005).

    Measures the moment of the inertia of dissimilarity values
    around the diagonal as

    \deqn{M(D) = \sum_{i=1}^n \sum_{j=1}^n d(i,j)|i-j|^2}

    \eqn{|i-j|} is used as a measure for the distance to the diagonal and
    \eqn{d(i,j)} gives the weight. This criterion gives higher weight
    to values farther away from the diagonal. It increases with quality.}


\item{\code{"Least_squares"}}{Least squares criterion (Caraux and Pinloche 2005).

    The sum of squares of deviations between the
    dissimilarities and rank differences (in the matrix) between two
    elements:
    \deqn{L(D) = \sum_{i=1}^n \sum_{j=1}^n (d(i,j) - |i-j|)^2,}
    where \eqn{d(i,j)} is an element of the dissimilarity matrix \eqn{D} and
    \eqn{|i-j|} is the rank difference between the objects.

    Note that if Euclidean distance is used to calculate \eqn{D} from
    a data matrix \eqn{X}, the order of the elements in \eqn{X} by projecting
    them on the first principal component of \eqn{X} minimizes this criterion.
    The least squares criterion is related to
    \emph{unidimensional scaling.}
    }

    \item{\code{"LS"}}{Linear Seriation Criterion (Hubert and Schultz 1976).

    Weights the distances with the absolute rank differences.

    \deqn{L(D) \sum_{i,j=1}^n d(i,j) (-|i-j|)}
    }

    \item{\code{"2SUM"}}{2-Sum Criterion (Barnard, Pothen, and Simon 1993).

    The 2-Sum
    loss criterion multiplies the similarity between objects
    with the squared rank differences.

    \deqn{L(D) \sum_{i,j=1}^n 1/(1+d(i,j)) (i-j)^2,}

    where \eqn{s(i,j) = 1/(1+d(i,j))} represents the similarity between
    objects \eqn{i} and \eqn{j}.
    }

    \item{\code{"ME"}, \code{"Moore_stress"},
          \code{"Neumann_stress"}, \code{"Cor_R"}}{These
    criteria are defined on general matrices (see below for definitions).
    The dissimilarity matrix is first converted into a similarity matrix
    using \eqn{S = 1/(1+D)}. If a different transformation is required, then
    perform the transformation first and supply a matrix instead
    of a dist object.}
    }

For a general matrix
\eqn{X = x_{ij}}, \eqn{i = 1 \ldots n} and \eqn{j = 1 \ldots m},
currently the
following loss/merit functions
are implemented:

\describe{
\item{\code{"ME"}}{Measure of
    Effectiveness (McCormick 1972).

    The measure of effectiveness (ME) for matrix
    \eqn{X}, is defined as

    \deqn{M(X) = 1/2 \sum_{i=1}^{n} \sum_{j=1}^{m} x_{i,j}(x_{i,j-1}+x_{i,j+1}+x_{i-1,j}+x_{i+1,j})}

    with, by convention

    \deqn{x_{0,j}=x_{m+1,j}=x_{i,0}=x_{i,n+1}=0.}

    ME is a merit measure,
    i.e. a higher ME indicates a better arrangement.
    Maximizing ME is the objective of the bond energy algorithm
    (BEA).
}

\item{\code{"Cor_R"}}{Weighted correlation coefficient R developed
     as the Measure of Effectiveness for the Moment Ordering
     Algorithm (Deutsch and Martin 1971).

     R is a merit measure normalized so that its value always
     lies in \eqn{[-1,1]}.
     For the special case of a square matrix \eqn{R=1} corresponds to only the
     main diagonal being filled, \eqn{R=0} to a random distribution of value
     throughout the array, and \eqn{R=-1} to the opposite diagonal only being
     filled.
}

\item{\code{"Moore_stress"}, \code{"Neumann_stress"}}{ Stress (Niermann 2005).

Stress measures the conciseness of the presentation of a matrix/table and can
be seen as a purity function which compares the values in a matrix/table with
its neighbors. The stress measure used here is computed as the sum of squared
distances of each matrix entry from its adjacent entries.

\deqn{
    L(X) =
    \sum_{i=1}^n \sum_{j=1}^m \sigma_{ij}
}

The following types
of neighborhoods are available:

  \describe{
    \item{Moore:}{comprises the eight adjacent entries.
    \deqn{   \sigma_{ij} = \sum_{k=\max(1,i-1)}^{\min(n,i+1)}
                \sum_{l=\max(1,j-1)}^{\min(m,j+1)}
                (x_{ij} - x_{kl})^2
}
    }
    \item{Neumann:}{comprises the four adjacent entries.
    \deqn{
     \sigma_{ij} =
            \sum_{k=\max(1,i-1)}^{\min(n,i+1)} (x_{ij} - x_{kj})^2 +
            \sum_{l=\max(1,j-1)}^{\min(m,j+1)} (x_{ij} - x_{il})^2
    }
    }
  }

  The major difference between the Moore and the Neumann
  neighborhood is that for
  the later the contribution of row and column permutations to stress are
  independent and thus can be optimized independently.
  }
}
}
\value{
  A named vector of real values.
}
\references{
Barnard, S.T.,  A. Pothen, and H. D. Simon (1993): A Spectral Algorithm for Envelope Reduction of Sparse Matrices. \emph{In Proceedings of the 1993
ACM/IEEE Conference on Supercomputing,} 493--502. Supercomputing '93.
New York, NY, USA: ACM.

Caraux, G. and S. Pinloche (2005): Permutmatrix: A Graphical Environment
to Arrange Gene Expression Profiles in Optimal Linear Order,
\emph{Bioinformatics,}
\bold{21}(7), 1280--1281.

Chen, C.-H. (2002): Generalized association plots:
Information visualization via iteratively generated correlation matrices,
\emph{Statistica Sinica,} \bold{12}(1), 7--29.

Deutsch, S.B. and J.J. Martin (1971):
An ordering algorithm for analysis of data arrays.
\emph{Operational Research,} \bold{19}(6), 1350--1362.
\doi{10.1287/opre.19.6.1350}


Earle, D. and C.B. Hurley (2015): Advances in Dendrogram Seriation for Application to Visualization. \emph{Journal of Computational and Graphical Statistics,}
\bold{24}(1), 1--25.
\doi{10.1080/10618600.2013.874295}

Hahsler, M. (2017): An experimental comparison of seriation methods for one-mode two-way data. \emph{European Journal of Operational Research,} \bold{257}, 133--143. \doi{10.1016/j.ejor.2016.08.066}

Hubert, L. and J. Schultz (1976): Quadratic Assignment as a General Data Analysis Strategy. \emph{British Journal of Mathematical and Statistical
Psychology,} \bold{29}(2). Blackwell Publishing Ltd. 190--241.
\doi{10.1111/j.2044-8317.1976.tb00714.x}


Hubert, L., P. Arabie, and J. Meulman (2001): \emph{Combinatorial Data Analysis:
Optimization by Dynamic Programming.} Society for Industrial Mathematics. \doi{10.1137/1.9780898718553}

Niermann, S. (2005): Optimizing the Ordering of Tables
With Evolutionary Computation, \emph{The American Statistician,}
\bold{59}(1), 41--46.
\doi{10.1198/000313005X22770}

McCormick, W.T., P.J. Schweitzer and T.W. White (1972):
Problem decomposition and data reorganization by a clustering technique,
\emph{Operations Research,}
\bold{20}(5), 993-1009.
\doi{10.1287/opre.20.5.993}

Robinson, W.S. (1951): A method for chronologically ordering archaeological deposits,
\emph{American Antiquity,} \bold{16}, 293--301.
\doi{10.2307/276978}

Tien, Y-J., Yun-Shien Lee, Han-Ming Wu and Chun-Houh Chen (2008):
Methods for simultaneously identifying coherent local clusters with
smooth global patterns in gene expression profiles,
\emph{BMC Bioinformatics,} \bold{9}(155), 1--16.
\doi{10.1186/1471-2105-9-155}
}

\seealso{
\code{\link{list_criterion_methods}} to query the criterion registry.
}


\author{Christian Buchta and Michael Hahsler}
\examples{
## create random data and calculate distances
m <- matrix(runif(20),ncol=2)
d <- dist(m)

## get an order for rows (optimal for the least squares criterion)
o <- seriate(d, method = "MDS")
o

## compare the values for all available criteria
rbind(
    unordered = criterion(d),
    ordered = criterion(d, o)
)

## compare RGAR by window size (from local to global)
w <- 2:(nrow(m)-1)
RGAR <- sapply(w, FUN = function (w)
  criterion(d, o, method="RGAR", w = w))
plot(w, RGAR, type = "b", ylim = c(0,1),
  xlab = "Windows size (w)", main = "RGAR by window size")
}
\keyword{cluster}

\name{dissimilarity}
\alias{dist}
\alias{ser_dist}
\alias{ser_cor}
\alias{ser_align}
\title{Dissimilarities and Correlations Between Seriation Orders}
\description{
Calculates dissimilarities/correlations between seriation orders in a list.
}
\usage{
ser_cor(x, y = NULL, method = "spearman", reverse = TRUE, test = FALSE)
ser_dist(x, y = NULL, method = "spearman", reverse = TRUE, ...)
ser_align(x, method = "spearman")
}
\arguments{
  \item{x}{set of seriation orders as a list with elements which can be
  coerced into \code{ser_permutation_vector} objects. }
  \item{y}{ if not \code{NULL} then a single seriation order can be specified.   In this case \code{x} has to be a single seriation order and not a list. }
  \item{method}{ a character string with the name of the used measure. Available
    measures are:
    \code{"kendall"}, \code{"spearman"}, \code{"manhattan"},
    \code{"euclidean"}, \code{"hamming"},
    \code{"ppc"} (positional proximity coefficient), and
    \code{"aprd"} (absolute pairwise rank differences).}
  \item{reverse}{a logical indicating if the orders should also be checked in
    reverse order and the best value (highest correlation, lowest distance) is
    reported. This only affect ranking-based measures and not precedence
    invariant measures (e.g., ppc, aprd). }
  \item{test}{a logical indicating if a correlation test should be performed. }
  \item{...}{Further arguments passed on to the method. }
}
\details{
\code{ser_cor} calculates the correlation between two sequences (orders).
Note that a seriation order and its reverse are identical and purely an artifact
due to the method that creates the order. This is a major difference to
rankings.
For ranking-based correlation measures (Spearman and Kendall)
the absolute value of the correlation is returned for \code{reverse = TRUE}
(in effect returning the correltation for the reversed order).
If \code{test = TRUE} then the appropriate test for association is performed
and a matrix with p-values is returned as the attribute \code{"p-value"}. Note
that no correction for multiple testing is performed.

For \code{ser_dist},
the correlation coefficients (Kendall's tau and Spearman's rho) are converted
into a dissimilarity by taking one minus the correlation value.
Note that Manhattan distance between the
ranks in a linear order is equivalent to Spearman's footrule
metric (Diaconis 1988). \code{reverse = TRUE} returns the pairwise minima
using also reversed orders.


The positional proximity coefficient (ppc) is a precedence invariant measure based on product of the squared positional distances in two permutations defined as (see Goulermas et al 2016):

\deqn{d_{ppc}(R, S) =
1/h  \sum_{j=2}^n \sum_{i=1}^{j-1} (\pi_R(i)-\pi_R(j))^2 * (\pi_S(i)-\pi_S(j))^2,}

where
\eqn{R} and \eqn{S} are two seriation orders, \eqn{pi_R} and \eqn{pi_S}
are the associated permutation vectors and
\eqn{h} is a normalization factor.
The associatied generalized correlation coefficient is defined as \eqn{1-d_{ppc}}.
For this precedence invariant measure \code{reverse} is ignored.

The absolute pairwise rank difference (aprd) is also precedence invariant
and defined as a distance measure:

\deqn{d_{aprd}(R, S) =
\sum_{j=2}^n \sum_{i=1}^{j-1} | |\pi_R(i)-\pi_R(j)| - |\pi_S(i)-\pi_S(j)| |^p,}

where \eqn{p} is the power which can be passed on as
parameter \code{p} and is by default set to 2.
For this precedence invariant measure \code{reverse} is ignored.

\code{ser_align} tries to normalize the direction in a list of seriations such
that ranking-based methods can be used.
We add for each permutation also the reversed order to the set and then
use a modified version of Prim's
algorithm for finding a minimum spanning tree (MST) to choose if the original seriation order or its reverse should be used. We use the orders first added to
the MST. Every time an order is added, its reverse is removed from the possible
remaining orders.
}
\value{
\code{ser_dist} returns an object of class \code{dist}.
\code{ser_align} returns a new list with elements of class
    \code{ser_permutation}.
}
\references{
P. Diaconis (1988): Group Representations in Probability and Statistics. Institute of Mathematical Statistics, Hayward, CA.

J.Y. Goulermas, A. Kostopoulos, and T. Mu (2016): A New Measure for Analyzing and Fusing Sequences of Objects. \emph{IEEE Transactions on Pattern Analysis and Machine Intelligence} 38(5):833-48.
\doi{10.1109/TPAMI.2015.2470671}
}
\seealso{
\code{\link{ser_permutation_vector}}
}
\author{Michael Hahsler}
\examples{
set.seed(1234)
## seriate dist of 50 flowers from the iris data set
data("iris")
x <- as.matrix(iris[-5])
x <- x[sample(1:nrow(x), 50),]
rownames(x) <- 1:50
d <- dist(x)

## Create a list of different seriations
methods <- c("HC_single", "HC_complete", "OLO", "GW", "R2E", "VAT",
  "TSP", "Spectral", "SPIN", "MDS", "Identity", "Random")

os <- sapply(methods, function(m) {
  cat("Doing", m, "... ")
  tm <- system.time(o <- seriate(d, method = m))
  cat("took", tm[3],"s.\n")
  o
})

## Compare the methods using distances. Default is based on
## Spearman's rank correlation coefficient. Reverse orders are considered
## equivalent.
ds <- ser_dist(os)
hmap(ds, margin=c(7,7))

## Compare using actual correlation between orders. Reversed orders have
## negative correlation!
cs <- ser_cor(os, reverse = FALSE)
hmap(cs, margin=c(7,7))

## Also check reversed seriation orders.
## Now all but random and identity are highly positive correlated
cs2 <- ser_cor(os, reverse = TRUE)
hmap(cs2, margin=c(7,7))

## Use Manhattan distance of the ranks (i.e., Spearman's foot rule)
ds <- ser_dist(os, method="manhattan")
plot(hclust(ds))
}
\keyword{cluster}

\name{criterion_methods}
\alias{registry_criterion}
\alias{set_criterion_method}
\alias{get_criterion_method}
\alias{list_criterion_methods}
\alias{show_criterion_methods}
\title{Registry for Criterion Methods}
\description{
A registry to manage methods to calculate a criterion value given
data and a permutation.
}
\usage{
list_criterion_methods(kind)
show_criterion_methods(kind)
get_criterion_method(kind, name)
set_criterion_method(kind, name, fun, description = NULL, merit = NA, ...)
}
\arguments{
  \item{kind}{the data type the method works on. For example, \code{"dist"},
  \code{"matrix"} or \code{"array"}.}
  \item{name}{a short name for the method used to refer to the method in
  the function \code{criterion()}.}
  \item{fun}{a function containing the method's code.} 
  \item{description}{a description of the method. For example, a long name.} 
  \item{merit}{a boolean indicating if the criterion measure is a merit
  (\code{TRUE}) or a loss (\code{FALSE}) measure.} 
  \item{...}{further information that is stored for the method in the 
  registry.} 
}
\details{
All methods below are convenience methods for the registry named
\code{registry_criterion}.

\code{list_criterion_method()} lists all available methods for a given
data type (\code{kind}). The result is a vector of character strings
with the short names of the methods. 

\code{show_criterion_method()} shows all available methods for a given
data type (\code{kind}) including a description.

\code{get_criterion_method()} returns information (including the
implementing function) about a given method in form of an object of
class \code{"criterion_method"}.

With \code{set_criterion_method()} new criterion methods can be added by the 
user. The implementing function (\code{fun}) needs to have the
formal arguments \code{x, order, ...}, where \code{x} is the data object,
order is an object of class \code{permutation_vector} and \code{...} can
contain additional information for the method passed on from \code{criterion()}.
The implementation has to return the criterion value as a scalar.
}
\author{Michael Hahsler}
\examples{
## the registry
registry_criterion

## use the convenience functions
list_criterion_methods("dist")

show_criterion_methods("dist")

get_criterion_method("dist", "AR_d")

## define a new method

## a function that return sum of the diagonal elements
criterion_method_matrix_foo <- function(x, order, ...) {
if(!is.null(order)) x <- permute(x,order)
    sum(diag(x))
}

## set new method
set_criterion_method("matrix", "foo", criterion_method_matrix_foo, 
    "foo: a useless demo criterion", FALSE)

list_criterion_methods("matrix")

##use all criterion methods (including the new one)
criterion(matrix(1:9, ncol=3))
}
\keyword{misc}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ser_permutation_vector2matrix.R
\name{permutation_vector2matrix}
\alias{permutation_vector2matrix}
\alias{permutation_matrix2vector}
\title{Conversion Between Permutation Vector and Permutation Matrix}
\usage{
permutation_vector2matrix(x)

permutation_matrix2vector(x)
}
\arguments{
\item{x}{A permutation vector (any object that can be converted into a
permutation vector, e.g., a integer vector or a \code{hclust} object) or a
matrix representing a permutation. Arguments are checked.}
}
\value{
\itemize{
\item \code{permutation_vector2matrix()}: returns a permutation matrix.
\item \code{permutation_matrix2vector()}: returns the permutation as a integer vector.
}
}
\description{
Converts between permutation vectors and matrices.
}
\examples{
## create a random permutation vector
pv <- sample(1:5)
pv

## convert into a permutation matrix
pm <- permutation_vector2matrix(pv)
pm

## convert back
permutation_matrix2vector(pm)
}
\seealso{
Other permutation: 
\code{\link{get_order}()},
\code{\link{permute}()},
\code{\link{ser_dist}()},
\code{\link{ser_permutation_vector}()},
\code{\link{ser_permutation}()}
}
\author{
Michael Hahsler
}
\concept{permutation}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/register_GA.R
\name{register_GA}
\alias{register_GA}
\alias{GA}
\alias{ga}
\alias{gaperm_mixedMutation}
\title{Register a Genetic Algorithm Seriation Method}
\usage{
register_GA()

gaperm_mixedMutation(ismProb = 0.8)
}
\arguments{
\item{ismProb}{probability to use \code{\link[GA:ga_Mutation]{GA::gaperm_ismMutation()}} (inversion) versus \code{\link[GA:ga_Mutation]{GA::gaperm_simMutation()}} (simple insertion).}
}
\value{
Nothing.
}
\description{
Register a GA-based seriation metaheuristic for use with \code{\link[=seriate]{seriate()}}.
}
\details{
Registers the method \code{"GA"} for \code{\link[=seriate]{seriate()}}. This method can be used
to optimize any criterion in package \pkg{seriation}.

\code{control} for
\code{seriate} with method \code{"GA"} accepts the following parameters:
\itemize{
\item "criterion" criterion to optimize. Default: BAR
\item "suggestions" suggestions to warm start the GA.
\code{NA} means no warm start. Default: TSP, QAP_LS and Spectral.
\item "selection" Selection operator.
\item "crossover" Crossover operator.
\item "mutation" Mutation operator. Default: a
mixture of the simple insertion (80\% chance) and simple inversion (20\%
chance) operators.
\item "pmutation" probability for permutations. Default: .5
\item "pcrossover" probability for crossover. Default: .2
\item "popsize" the population size. Default: 100
\item "maxiter" maximum number of generations. Default: 1000
\item "run" stop after \code{run} generations without improvement. Default: 50
\item "parallel" use multiple cores? Default: TRUE
\item "verbose" a logical; report progress? Default: TRUE
}

The GA uses by default the ordered cross-over (OX) operator. For mutation,
the GA uses a mixture of simple insertion and simple inversion operators.
This mixed operator is created using
\code{seriation::gaperm_mixedMutation(ismProb = .8)}, where \code{ismProb}
is the probability that the simple insertion mutation operator is used. See
package \pkg{GA} for a description of other available cross-over and
mutation operators for permutations. The appropriate operator functions in
\pkg{GA} start with \code{gaperm_}.

We warm start the GA using \code{"suggestions"} given by several heuristics.
Set \code{"suggestions"} to \code{NA} to start with a purely random initial
population.

\bold{Note:} Package \pkg{GA} needs to be installed.
}
\examples{

\dontrun{
register_GA()
get_seriation_method("dist", "GA")

d <- dist(random.robinson(50, pre=TRUE, noise=.1))

## use default settings: Banded AR form
o <- seriate(d, "GA")
pimage(d, o)

## optimize for linear sertiation criterion (LS)
o <- seriate(d, "GA", control = list(criterion = "LS"))
pimage(d, o)

## no warm start
o <- seriate(d, "GA", control = list(criterion = "LS", suggestions = NA))
pimage(d, o)
}
}
\references{
Luca Scrucca (2013): GA: A Package for Genetic Algorithms in R.
\emph{Journal of Statistical Software,} \bold{53}(4), 1--37. URL
\doi{10.18637/jss.v053.i04}.
}
\seealso{
Other seriation: 
\code{\link{register_DendSer}()},
\code{\link{register_optics}()},
\code{\link{register_tsne}()},
\code{\link{register_umap}()},
\code{\link{registry_seriate}},
\code{\link{seriate}()}
}
\author{
Michael Hahsler
}
\concept{seriation}
\keyword{cluster}
\keyword{optimize}

\name{seriation_data}
\alias{create_lines_data}
\alias{create_ordered_data}
\title{Create Simulated Data for Seriation Evaluation}
\description{
Several functions to create simulated data to evaluate different aspects
of seriation algorithms and criterion functions. 
}
\usage{
create_lines_data(n = 250)
create_ordered_data(n = 250, k = 2, size = NULL, spacing = 6,  path = "linear",
  sd1 = 1, sd2 = 0)
}
\arguments{
  \item{n}{number of data points to create.}
  \item{k}{number of Gaussian components.}
  \item{size}{relative size (number of points) of components (length of k). 
    If \code{NULL} then all components have the same size. }
  \item{spacing}{ space between the centers of components. The default of 6
    means that the components will barely touch at \code{ds1=1} 
    (3 standard deviations for each Gaussian component). }
  \item{path}{ Are the components arranged along a \code{"linear"} or 
  \code{"circular"} path? }
  \item{sd1}{ variation in the direction along the components. 
    A value greater than one means the
    components are mixing. }
  \item{sd2}{ variation perpendicular to the direction along the components. 
    A value greater than 0 will introduce anti-Robinson violation events. }
}
\details{
\code{create_lines_data} creates the lines data set used in 
for iVAT in Havens and Bezdeck (2012).


\code{create_ordered_data} is a versatile function which creates "orderable"
2D data using Gaussian components along a linear or circular path. The 
components are equally spaced (\code{spacing}) along the path. The 
default spacing of 6 ensures that 2 adjacent components with a standard 
deviation of one along the direction of the path will barely touch. The 
standard deviation along the path is set by \code{sd1}. The standard deviation
perpendicular to the path is set by \code{sd2}. A value larger than zero
will result in the data not being perfectly orderable (i.e., the
resulting distance matrix will not be a perfect pre-anti-Robinson matrix and
contain anti-Robinson violation events after seriation). Note that a circular
path always creates anti-Robinson violation since the circle has to be
broken at some point to create a linear order. 
}
\seealso{
\code{\link{seriate}},
\code{\link{criterion}},
\code{\link{VAT}}.
}
\references{
Havens, T.C. and Bezdek, J.C. (2012):
An Efficient Formulation of the Improved Visual Assessment of Cluster Tendency 
(iVAT) Algorithm, \emph{IEEE Transactions on Knowledge and Data Engineering,}
\bold{24}(5), 813--822. 
}
\author{Michael Hahsler}
\examples{
## lines data set from Havens and Bezdek (2011) 
x <- create_lines_data(250)
plot(x, xlim=c(-5,5), ylim=c(-3,3), cex=.2, col = attr(x, "id"))  
d <- dist(x)
pimage(d, seriate(d, "OLO_single"), col = bluered(100, bias=.5), key = TRUE)
  
## create_ordered_data can produce many types of "orderable" data

## perfect pre-Anti-Robinson matrix (with a single components)
x <- create_ordered_data(250, k = 1)  
plot(x, cex=.2, col = attr(x, "id"))  
d <- dist(x)
pimage(d, seriate(d, "MDS"), col = bluered(100, bias=.5), key = TRUE)  
  
## separated components
x <- create_ordered_data(250, k = 5)  
plot(x, cex=.2, col = attr(x, "id"))  
d <- dist(x)
pimage(d, seriate(d, "MDS"), col = bluered(100, bias=.5), key = TRUE)  
  
## overlapping components
x <- create_ordered_data(250, k = 5, sd1 = 2)  
plot(x, cex=.2, col = attr(x, "id"))
d <- dist(x)
pimage(d, seriate(d, "MDS"), col = bluered(100, bias=.5), key = TRUE)  
    
## introduce anti-Robinson violations (a non-zero y value)
x <- create_ordered_data(250, k = 5, sd1 = 2, sd2 = 5)  
plot(x, cex=.2, col = attr(x, "id"))  
d <- dist(x)
pimage(d, seriate(d, "MDS"), col = bluered(100, bias=.5), key = TRUE)  
  
## circular path (has always violations)  
x <- create_ordered_data(250, k = 5, path = "circular", sd1=2)  
plot(x, cex=.2, col = attr(x, "id"))
d <- dist(x)
pimage(d, seriate(d, "OLO"), col = bluered(100, bias=.5), key = TRUE)  

## circular path (with more violations violations)  
x <- create_ordered_data(250, k = 5, path = "circular", sd1=2, sd2=1)  
plot(x, cex=.2, col = attr(x, "id"))
d <- dist(x)
pimage(d, seriate(d, "OLO"), col = bluered(100, bias=.5), key = TRUE) 
}
\keyword{data}

\name{Munsingen}
\alias{Munsingen}
\docType{data}
\encoding{UTF-8}
\title{Hodson's Munsingen Data Set}
\description{
  This data set contains a grave times artifact incidence matrix for the
  Celtic Münsingen-Rain cemetery in Switzerland as provided by 
  Hodson (1968) and published by Kendall 1971.
}
\usage{data("Munsingen")}
\format{
  A 59 x 70 0-1 matrix. Rows (graves) and columns (artifacts) 
  are in the order determined by Hodson (1968).
}
\references{
Hodson, F.R. (1968): \emph{The La Tene Cemetery at Münsingen-Rain}.
Stämpfli, Bern.
    
Kendall, D.G. (1971): Seriation from abundance matrices. In: Hodson, F.R.,
Kendall, D.G. and Tautu, P., (Editors). \emph{Mathematics in the Archaeological
and Historical Sciences,} Edinburgh University Press, Edinburgh, 215--232.  
}
\examples{
data("Munsingen")

## Seriation method after Kendall (1971)
## Kendall's square symmetric matrix S and SoS
S <- function(x, w = 1) {
  sij <- function(i , j) w * sum(pmin(x[i,], x[j,]))
  h <- nrow(x)
  r <- matrix(ncol = h, nrow =h)
  for(i in 1:h) for (j in 1:h)  r[i,j] <- sij(i,j)
  r
}

SoS <- function(x) S(S(x))

## Kendall's horse shoe (Hamiltonian arc)
horse_shoe_plot <- function(mds, sigma, threshold = mean(sigma), ...) {
    plot(mds, main = paste("Kendall's horse shoe with th =", threshold), ...)  
    l <- which(sigma > threshold, arr.ind=TRUE)
    for(i in 1:nrow(l))  lines(rbind(mds[l[i,1],], mds[l[i,2],])) 
}

## shuffle data
x <- Munsingen[sample(nrow(Munsingen)),]

## calculate matrix and do isoMDS (from package MASS)
sigma <- SoS(x)
library("MASS")
mds <- isoMDS(1/(1+sigma))$points

## plot Kendall's horse shoe
horse_shoe_plot(mds, sigma)

## find order using a TSP 
library("TSP")
tour <- solve_TSP(insert_dummy(TSP(dist(mds)), label = "cut"),
    method = "2-opt", control = list(rep = 15))
tour <- cut_tour(tour, "cut")
lines(mds[tour,], col = "red", lwd = 2)

## create and plot order
order <- ser_permutation(tour, 1:ncol(x))
bertinplot(x, order, options= list(panel=panel.circles,
    rev = TRUE))

## compare criterion values    
rbind(
    random = criterion(x),
    reordered = criterion(x, order),
    Hodson = criterion(Munsingen)
   )
}
\keyword{datasets}

\name{color_palettes}
\alias{greenred}
\alias{bluered}
\alias{grays}
\alias{greys}
\title{Different Useful Color Palettes}
\description{
Defines several color palettes for \code{pimage}, \code{dissplot}
and \code{hmap}.
}
\usage{
bluered(n, bias = 1)
greenred(n, bias = 1)
grays(n, power = 1)
greys(n, power = 1)
}
\arguments{
  \item{n}{ number of different colors produces. }
  \item{bias}{a positive number. Higher values give more widely 
    spaced colors at the high end.}
  \item{power}{ control parameter determining how luminance should be 
    increased (1 = linear, 2 = quadratic, etc.).}
}
\details{
\code{bluered} creates a blue-red color palette.

\code{greenred} creates a green-red color palette.

\code{greys} and \code{grays} creates gray scales.

See \code{colorRampPalette} to create your own color palettes.
}
\value{
A vector with \code{n} colors.
}
\seealso{
  \code{\link[grDevices]{colorRampPalette}},
  \code{\link{pimage}},
  \code{\link{dissplot}},
  \code{\link{hmap}}.
}
\author{Michael Hahsler}
\examples{
pimage(rbind(1:100))
pimage(rbind(1:100), col = greys(100, power=2))
pimage(rbind(1:100), col = bluered(100))
pimage(rbind(1:100), col = bluered(100, bias = 2))
pimage(rbind(-100:100), col = greenred(10))

## create your own color palettes 
## red to green (with 10 colors)
pimage(rbind(1:100), 
  col = colorRampPalette(colors = c("red", "yellow", "green"))(10))
## white to blue (with 100 colors)
pimage(rbind(1:100), 
  col = colorRampPalette(colors = c("white", "blue"))(100))
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ser_permutation_vector.R
\name{ser_permutation_vector}
\alias{ser_permutation_vector}
\alias{c.ser_permutation_vector}
\alias{rev.ser_permutation_vector}
\alias{get_method}
\alias{length.ser_permutation_vector}
\alias{print.ser_permutation_vector}
\alias{summary.ser_permutation_vector}
\title{Class ser_permutation_vector -- A Single Permutation Vector for Seriation}
\usage{
ser_permutation_vector(x, method = NULL)

\method{c}{ser_permutation_vector}(..., recursive = FALSE)

\method{rev}{ser_permutation_vector}(x)

get_method(x, printable = FALSE)

\method{length}{ser_permutation_vector}(x)

\method{print}{ser_permutation_vector}(x, ...)

\method{summary}{ser_permutation_vector}(object, ...)
}
\arguments{
\item{x, object}{an object which contains a permutation vector (currently an
integer vector or an object of class \link{hclust}). The value \code{NA}
creates an identity permutation.}

\item{method}{a string representing the method used to obtain the
permutation vector.}

\item{...}{further arguments.}

\item{recursive}{ignored}

\item{printable}{a logical; prints "unknown" instead of \code{NULL} for non-existing methods.}
}
\value{
An object of class \code{ser_permutation_vector}.
}
\description{
The class \code{ser_permutation_vector}
represents a single permutation vector.
}
\details{
A permutation vector
maps a set of \eqn{n} objects \eqn{\{O_1, O_2, ..., O_n\}}{{O_1, O_2, ..., O_n}} onto itself.

\strong{Ordering Representation:}
In \pkg{seriation} we represent a permutation \eqn{\pi}{\pi}
as a vector which lists the objects' indices in their permuted order. This can
be seen as replacing the object in position \eqn{i} with the object
in position \eqn{\pi(i)}.
For example, the permutation vector \eqn{\langle3, 1, 2\rangle}{<3, 1, 2>} indicates that in
first position is the object with index 3 then the object with index 1 and finally
the object with index 2. This representation is often called a (re)arrangement or ordering.
The ordering can be extracted from a permutation vector object
via \code{\link[=get_order]{get_order()}}. Such an ordering can be directly used
to subset the list of original objects with \code{"["} to apply the permutation.

\strong{Rank Representation:}
An alternative way to specify a permutation is via a list of the ranks
of the objects after permutation. This representation is often called
a map or substitution. Ranks can be extracted from a permutation vector using \code{\link[=get_rank]{get_rank()}}.

\strong{Permutation Matrix:}
Another popular representation is a permutation matrix which performs
permutations using matrix multiplication. A permutation matrix can be obtained
using \code{\link[=get_permutation_matrix]{get_permutation_matrix()}}.

\code{ser_permutation_vector} objects are usually packed into
a \link{ser_permutation} object
which is a collection (a \code{list}) of \eqn{k} permutation vectors for \eqn{k}-mode data.

The constructor \code{ser_permutation_vector()}
checks if the permutation vector is valid
(i.e. if all integers occur exactly once).
}
\examples{
p <- ser_permutation_vector(sample(10), "random")
p

## some methods
length(p)
get_method(p)
get_order(p)
get_rank(p)
get_permutation_matrix(p)

r <- rev(p)
r
get_order(r)

## create a identity permutation vector (with unknown length)
ip <- ser_permutation_vector(NA)
ip

}
\seealso{
Other permutation: 
\code{\link{get_order}()},
\code{\link{permutation_vector2matrix}()},
\code{\link{permute}()},
\code{\link{ser_dist}()},
\code{\link{ser_permutation}()}
}
\author{
Michael Hahsler
}
\concept{permutation}
\keyword{classes}

\name{pimage}
\alias{pimage}
\alias{pimage.matrix}
\alias{pimage.dist}
\title{Permutation Image Plot}
\description{
  Provides methods for plotting image plots for
  matrix and \code{dist} objects given a permutation.
  By default, no permutation is performed.
  This plot can also
  be used as a more versatile replacement of \code{image} plot
  in \pkg{graphics} based on \pkg{grid}.
}
\usage{
pimage(x, order = NULL, col = NULL, main ="", xlab = "", ylab = "",
  axes = "auto", zlim=NULL, key=TRUE, key.lab="", symkey=TRUE,
  upper.tri = TRUE, lower.tri = TRUE, prop = NULL,
  ..., newpage=TRUE, pop=TRUE, gp=NULL)
}

\arguments{
  \item{x}{a matrix or an object of class \code{dist}.}
  \item{order}{an object of class \code{ser_permutation}. If \code{NULL} the order
      in \code{x} is plotted.}
  \item{col}{a list of colors used. If \code{NULL}, a gray scale is used (for
      matrix larger values are displayed darker and for \code{dist} smaller
      distances are darker). For matrices containing logical data, black and
      white is used. For matrices containing negative values a symmetric
      diverging color palette is used.}
  \item{main}{plot title.}
  \item{xlab, ylab}{labels for the x and y axes.}
  \item{axes}{ a character string indicating if axes labels (column and row
    names of \code{x}) should be potted. Possible values are
    \code{"auto"} (only plot if less then 25 labels),
    \code{"x"},
    \code{"y"},
    \code{"both"} and
    \code{"none"}.}
  \item{zlim}{ vector with two elements giving the range (min, max) for
      representing the values in the matrix. }
  \item{key}{ logical; add a color key? No key is available for logical
    matrices. }
  \item{key.lab}{ string plotted next to the color key.}
  \item{symkey}{ logical; if \code{x} contains negative values, should the
    color palate be symmetric (zero is in the middle)>}
  \item{upper.tri, lower.tri}{ a logical indicating whether to show the
      upper or lower triangle of the distance matrix. }
  \item{prop}{logical; draw the cells in the image proportional (defaults to
    \code{TRUE} for \code{dist} and \code{FALSE} for matrix).}
  \item{\dots}{further arguments passed on to \code{image} in \pkg{graphics}.}
  \item{newpage, pop}{two logical. Start plot on a new page and pop the viewports after plotting (see \pkg{grid}).}
  \item{gp}{a \code{gpar} object (see \pkg{grid}).}
}
\details{
  Plots a matrix in its original row and column orientation. This means, in a
  plot the columns become the x-coordinates and the reversed rows the
  y-coordinates.

  If \code{x} is of class \code{dist} it is converted to full-storage
  representation before plotting.

  The viewports used for plotting are called: \code{"plot"}, \code{"image"}
  and \code{"colorkey"}.

  \bold{Note:} Since \code{pimage} uses \pkg{grid}, it should not be mixed with base R primitive plotting
  functions, but the appropriate functions in \code{\link[grid]{grid-package}}.
}
\seealso{
\code{\link[grid]{grid-package}},
\code{\link{seriate}},
\code{\link{hmap}},
\code{\link{dissplot}} and
\code{\link[graphics]{image}}.}
\author{Christian Buchta and Michael Hahsler}
\examples{
x <- matrix(sample(c(FALSE, TRUE), 300, rep=TRUE), ncol=10,
  dimnames = list(1:30, LETTERS[1:10]))

## matrix (large values are dark/black)
pimage(x, main = "Random data", key = FALSE)

## plot seriated matrix (use red, proportional display and plot all axes)
pimage(x, seriate(x), col = c("white", "red"),
  prop = TRUE, axes="both", main = "Reordered data", key = FALSE)

## show correlation (for neg. values a diverging color scheme is
## used automatically)
pimage(cor(x), prop=TRUE)

## distances (note that low distances are represented dark!)
d <- dist(x, method = "binary")
pimage(d, upper.tri = FALSE, main = "Distances")

pimage(d, seriate(d), upper.tri = FALSE, main = "Distances", axes = "both")

## add to the plot using functions in package grid
library("grid")
pimage(x, pop = FALSE)           ### pop = FALSE allows us to manipulate viewports
downViewport(name = "image")

## highlight cell 7/5 with a red arrow
grid.lines(x = c(5, 7), y = c(3, 5), arrow = arrow(),
  default.units = "native", gp = gpar(col="red", lwd = 3))

## add a red box around rows 15 and 16
grid.rect(x = 0.5, y = 15.5, width = ncol(x), height = 2,
  just = "left",
  default.units = "native", gp = gpar(col="red", lwd = 3, fill = NA))

## remove the viewports
popViewport(0)

## put several pimages on a page (uses viewports and newpage = FALSE)
library(grid)
grid.newpage()
pushViewport(viewport(layout=grid.layout(nrow = 1, ncol = 2)))
pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 1))

## seriate matrix
o <- seriate(x)
pimage(x, o, main = "Random data", prop = TRUE, axes = "both", key = FALSE,
  newpage = FALSE)

upViewport(1)
pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 2))

## add the reordered disimilarity matrix for rowa
pimage(d, o[[1]], upper.tri = FALSE, main = "Distances", axes = "both",
  key = FALSE, newpage = FALSE)

upViewport(1)
popViewport(0)
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permute.R
\name{permute}
\alias{permute}
\alias{permute.array}
\alias{permute.matrix}
\alias{permute.data.frame}
\alias{permute.table}
\alias{permute.numeric}
\alias{permute.character}
\alias{permute.list}
\alias{permute.dist}
\alias{permute.dendrogram}
\alias{permute.hclust}
\title{Permute the Order in Various Objects}
\usage{
permute(x, order, ...)

\method{permute}{array}(x, order, margin = NULL, ...)

\method{permute}{matrix}(x, order, margin = NULL, ...)

\method{permute}{data.frame}(x, order, margin = NULL, ...)

\method{permute}{table}(x, order, margin = NULL, ...)

\method{permute}{numeric}(x, order, ...)

\method{permute}{character}(x, order, ...)

\method{permute}{list}(x, order, ...)

\method{permute}{dist}(x, order, ...)

\method{permute}{dendrogram}(x, order, ...)

\method{permute}{hclust}(x, order, ...)
}
\arguments{
\item{x}{an object (a list, a vector, a \code{dist} object, a matrix, an
array or any other object which provides \code{dim} and standard subsetting
with \code{"["}).}

\item{order}{an object of class \link{ser_permutation} which contains
suitable permutation vectors for \code{x}. Alternatively, a character string with the
name of a seriation method appropriate for \code{x} can be specified (see \code{\link[=seriate]{seriate()}}).
This will perform seriation and permute \code{x}.}

\item{...}{if \code{order} is the name of a seriation method, then additional arguments are
passed on to \code{\link[=seriate]{seriate()}}.}

\item{margin}{specifies the dimensions to be permuted as a vector with dimension indices.
If \code{NULL}, \code{order} needs to contain a permutation for all dimensions.
If a single margin is specified, then \code{order} can also contain
a single permutation vector.
\code{margin} are ignored.}
}
\value{
A permuted object of the same class as \code{x}.
}
\description{
Provides the generic function and methods for permuting the order of various
objects including vectors, lists, dendrograms (also \code{hclust} objects),
the order of observations in a \code{dist} object, the rows and columns of a
matrix or data.frame, and all dimensions of an array given a suitable
\link{ser_permutation} object.
}
\details{
The permutation vectors in \link{ser_permutation} are suitable if the number
of permutation vectors matches the number of dimensions of \code{x} and if
the length of each permutation vector has the same length as the
corresponding dimension of \code{x}.

For 1-dimensional/1-mode data (list, vector, \code{dist}), \code{order} can
also be a single permutation vector of class \link{ser_permutation_vector}
or data which can be automatically coerced to this class (e.g. a numeric
vector).

For \code{dendrogram} and \code{hclust}, subtrees are rotated to represent
the order best possible. If the order is not achieved perfectly then the
user is warned. This behavior can be changed with the extra parameter
\code{incompatible} which can take the values \code{"warn"} (default),
\code{"stop"} or \code{"ignore"}.
}
\examples{
# List data types for permute
methods("permute")

# Permute matrix
m <- matrix(rnorm(10), 5, 2, dimnames = list(1:5, LETTERS[1:2]))
m

# Permute rows and columns
o <- ser_permutation(5:1, 2:1)
o

permute(m, o)

## permute only columns
permute(m, o, margin = 2)

## permute using PCA seriation
permute(m, "PCA")

## permute only rows using PCA
permute(m, o, margin = 1)

# Permute data.frames
df <- as.data.frame(m)
permute(df, o)

# Permute objects in a dist object
d <- dist(m)
d

permute(d, c(3, 2, 1, 4, 5))

permute(d, "Spectral")

# Permute a list
l <- list(a = 1:5, b = letters[1:3], c = 0)
l

permute(l, c(2, 3, 1))

# Permute a dendrogram
hc <- hclust(d)
plot(hc)
plot(permute(hc, 5:1))
}
\seealso{
Other permutation: 
\code{\link{get_order}()},
\code{\link{permutation_vector2matrix}()},
\code{\link{ser_dist}()},
\code{\link{ser_permutation_vector}()},
\code{\link{ser_permutation}()}
}
\author{
Michael Hahsler
}
\concept{permutation}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_order.R
\name{get_order}
\alias{get_order}
\alias{get_order.ser_permutation_vector}
\alias{get_order.ser_permutation}
\alias{get_order.hclust}
\alias{get_order.dendrogram}
\alias{get_order.integer}
\alias{get_rank}
\alias{get_permutation_matrix}
\title{Extracting Order Information from a Permutation Object}
\usage{
get_order(x, ...)

\method{get_order}{ser_permutation_vector}(x, ...)

\method{get_order}{ser_permutation}(x, dim = 1, ...)

\method{get_order}{hclust}(x, ...)

\method{get_order}{dendrogram}(x, ...)

\method{get_order}{integer}(x, ...)

get_rank(x, ...)

get_permutation_matrix(x, ...)
}
\arguments{
\item{x}{an object of class \link{ser_permutation} or
\link{ser_permutation_vector}.}

\item{...}{further arguments are ignored for \code{get_order()}.  For
\code{get_rank()} and for \code{get_permutation_matrix()} the additional
arguments are passed on to \code{get_order()} (e.g., as \code{dim}).}

\item{dim}{order information for which dimension should be returned?}
}
\value{
Returns an integer permutation vector/a permutation matrix.
}
\description{
Method to get the order information from an object of class
\link{ser_permutation} or \link{ser_permutation_vector}. Order information
can be extracted as an integer permutation vector, a vector containing the
object ranks or a permutation matrix.
}
\details{
\code{get_order()} returns the seriation as an integer vector containing the
order of the objects after permutation. That is, the index of the first,
second, \eqn{..., n}-th object. These permutation vectors can directly be
used to reorder objects using subsetting with \code{"["}.  \emph{Note:} In
\pkg{seriation} we usually use these order-based permutation vectors.

\code{get_rank()} returns the seriation as an integer vector containing the
rank/position for each objects in the permutation. That is, position of the
first, second, \eqn{..., n}-th object after permutation.  Note: Use
\code{order()} to convert ranks back to an order.

\code{get_permutation_matrix()} returns a \eqn{n \times n}{n x n} permutation
matrix.
}
\examples{
## ser_permutation_vector is a single permutation vector
o <- ser_permutation_vector(sample(10))
o

get_order(o)
get_rank(o)
get_permutation_matrix(o)

## ser_permutation contains one permutation vector for each dimension
o2 <- ser_permutation(o, sample(5))
o2

get_order(o2, dim = 2)
get_rank(o2, dim = 2)
get_permutation_matrix(o2, dim = 2)
}
\seealso{
Other permutation: 
\code{\link{permutation_vector2matrix}()},
\code{\link{permute}()},
\code{\link{ser_dist}()},
\code{\link{ser_permutation_vector}()},
\code{\link{ser_permutation}()}
}
\author{
Michael Hahsler
}
\concept{permutation}
\keyword{manip}

\name{Robinson}
\alias{Robinson}
\alias{is.robinson}
\alias{random.robinson}
\title{Create and Recognize Robinson and Pre-Robinson Matrices}
\description{
Provides functions to create and
recognize (anti) Robinson and pre-Robinson matrices.
A (anti) Robinson matrix has strictly decreasing (increasing) values when
moving away from the main diagonal. A pre-Robinson matrix is a matrix which
can be transformed into a perfect Robinson matrix using simultaneous
permutations of rows and columns.
}
\usage{
is.robinson(x, anti = TRUE, pre = FALSE)
random.robinson(n, anti = TRUE, pre = FALSE, noise = 0)
}
\arguments{
  \item{x}{a symmetric, positive matrix or a dissimilarity matrix
  (a \code{dist} object). }
  \item{anti}{logical; check for anti Robinson structure? Note
  that for distances, anti Robinson structure is appropriate. }
  \item{pre}{logical; recognize/create pre-Robinson matrices. }
  \item{n}{ number of objects. }
  \item{noise}{ noise intensity between 0 and 1. Zero means no noise.
    Noise more than zero results in
    non-Robinson matrices. }
}
\details{
Note that the default matrices are anti Robinson matrices. This is done because
distance matrices (the default in R) are typically anti Robinson matrices
with values increasing when moving away from the diagonal.

Robinson matrices are recognized using the fact that they have zero anti Robinson events. For pre-Robinson matrices we use spectral seriation first since spectral seriation is guaranteed to perfectly reorder pre-Robinson matrices
(see Laurent and Seminaroti, 2015).

Random pre-Robinson matrices are generated by reversing the process of
unidemensional scaling. We randomly
(uniform distribution with range \eqn{[0,1]}) choose \eqn{x} coordinates
for \code{n} points on a straight line
and calculate the pairwise distances. For Robinson matrices,
the points are sorted first according to \eqn{x}.
For noise, \eqn{y} coordinates is added. The coordinates
are chosen
uniformly between 0 and \code{noise}, with \code{noise} between 0 and 1.
}
\value{
A single logical value.
}
%\seealso{
%}
\references{
M. Laurent, M. Seminaroti (2015): The quadratic assignment problem is easy for
Robinsonian matrices with Toeplitz structure,
\emph{Operations Research Letters} 43(1), 103--109.
}
\examples{
## create a perfect anti Robinson structure
m <- random.robinson(10)
pimage(m)

is.robinson(m)

## permute the structure to make it not Robinsonian. However,
## it is still pre-Robinson.
o <- sample(10)
m2 <- permute(m, ser_permutation(o,o))
pimage(m2)

is.robinson(m2)
is.robinson(m2, pre = TRUE)

## create a binary random Robinson matrix (not anti Robinson)
m3 <- random.robinson(10, anti = FALSE) > .7
pimage(m3)
is.robinson(m3, anti = FALSE)

## create matrices with noise (as distance matrices)
m4 <- as.dist(random.robinson(50, pre = FALSE, noise = .1))
pimage(m4)
criterion(m4, method = "AR")

m5 <- as.dist(random.robinson(50, pre = FALSE, noise = .5))
pimage(m5)
criterion(m5, method = "AR")
}
%\keyword{manip}

\name{seriation_methods}
\alias{registry_seriate}
\alias{set_seriation_method}
\alias{get_seriation_method}
\alias{list_seriation_methods}
\alias{show_seriation_methods}
\title{Registry for Seriation Methods}
\description{
A registry to manage methods for seriation.
}
\usage{
list_seriation_methods(kind)
show_seriation_methods(kind)
get_seriation_method(kind, name)
set_seriation_method(kind, name, definition, description = NULL, control = list(), ...)
}
\arguments{
  \item{kind}{the data type the method works on. For example, \code{"dist"},
  \code{"matrix"} or \code{"array"}. If missing, then methods for any type are shown. }
  \item{name}{a short name for the method used to refer to the method in
  \code{seriate()}.}
  \item{definition}{a function containing the method's code.}
  \item{description}{a description of the method. For example, a long name.}
  \item{control}{a list with control arguments and default values.}
  \item{...}{further information that is stored for the method in the
  registry.}
}
\details{
The functions below are convenience function for the registry
\code{registry_seriate}.

\code{list_seriation_method()} lists all available methods for a given
data type (\code{kind}). The result is a vector of character strings
with the short names of the methods.

\code{show_seriation_method()} shows all available methods including
a description.

\code{get_seriation_method()} returns information (including the
implementing function) about a given method in form of an object of
class \code{"seriation_method"}.

With \code{set_seriation_method()} new seriation methods can be added by the
user. The implementing function (\code{definition}) needs to have the formal
arguments \code{x, control}, where \code{x} is the data object and
\code{control} contains a list with additional information for the method
passed on from \code{seriate()}.  The implementation has to return
a list of objects which can be coerced into \code{ser_permutation_vector}
objects (e.g., integer vectors). The elements in the list
have to be in corresponding order to the dimensions of \code{x}.
}
\author{Michael Hahsler}
\examples{
## registry
registry_seriate

## convenience functions
show_seriation_methods()
show_seriation_methods("matrix")

list_seriation_methods("matrix")

get_seriation_method(name = "BEA")

## define a new method

## create a identity function which returns the identity order
seriation_method_identity <- function(x, control) {
   lapply(dim(x), seq)
}

## set new method
set_seriation_method("matrix", "identity", seriation_method_identity,
    "Identity order")

set_seriation_method("array", "identity", seriation_method_identity,
    "Identity order")

show_seriation_methods("matrix")

##use all criterion methods (including the new one)
seriate(matrix(1:12, ncol=3), "identity")
}
\keyword{misc}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ser_permutation.R
\name{ser_permutation}
\alias{ser_permutation}
\alias{print.ser_permutation}
\alias{summary.ser_permutation}
\alias{c.ser_permutation}
\alias{[.ser_permutation}
\title{Class ser_permutation -- A Collection of Permutation Vectors for Seriation}
\usage{
ser_permutation(x, ...)

\method{print}{ser_permutation}(x, ...)

\method{summary}{ser_permutation}(object, ...)

\method{c}{ser_permutation}(..., recursive = FALSE)

\method{[}{ser_permutation}(object, i, ...)
}
\arguments{
\item{x, object}{an object of class \code{ser_permutation_vector} or
any object which can be converted into
a object of class \code{ser_permutation} (e.g. an integer
vector).}

\item{...}{vectors for further dimensions.}

\item{recursive}{ignored.}

\item{i}{index of the dimension(s) to extract.}
}
\value{
An object of class \code{ser_permutation}.
}
\description{
The class \code{ser_permutation} is a collection of permutation vectors
(see class \link{ser_permutation_vector}), one for each dimension (mode)
of the data to be permuted.
}
\examples{
o <- ser_permutation(1:5, 10:1)
o

## length (number of dimensions)
length(o)

## get permutation vector for 2nd dimension
get_order(o, 2)

## reverse dimensions
o[2:1]

## combine
o <- c(o, ser_permutation(1:15))
o

## get an individual permutation
o[[2]]

## reverse the order of a permutation
o[[2]] <- rev(o[[2]])
get_order(o,2)
}
\seealso{
Other permutation: 
\code{\link{get_order}()},
\code{\link{permutation_vector2matrix}()},
\code{\link{permute}()},
\code{\link{ser_dist}()},
\code{\link{ser_permutation_vector}()}
}
\author{
Michael Hahsler
}
\concept{permutation}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/register_tsne.R
\name{register_tsne}
\alias{register_tsne}
\alias{tsne}
\alias{tSNE}
\title{Register Seriation Based on 1D t-SNE}
\usage{
register_tsne()
}
\value{
Nothing.
}
\description{
Use t-distributed stochastic neighbor embedding (t-SNE) for \code{\link[=seriate]{seriate()}}.
}
\details{
Registers the method \code{"tsne"} for \code{\link[=seriate]{seriate()}}. This method applies
1D t-SNE to data represented by a distance matrix and extracts the order
from the 1D embedding. To speed up the process, an initial embedding is
created using multi-dimensional scaling (MDS) which is improved by t-SNE.

The \code{control} parameter \code{mds} controls if MDS is used to create an
initial embedding. See \code{\link[Rtsne:Rtsne]{Rtsne::Rtsne()}} to learn about the other
available \code{control} parameters.

\bold{Note:} Package \pkg{Rtsne} needs to be installed.
}
\examples{

\dontrun{
register_tsne()
get_seriation_method("dist", "tsne")

d <- dist(random.robinson(50, pre=TRUE, noise=.1))

o <- seriate(d, method = "tsne")
pimage(d, o)
}

}
\references{
van der Maaten, L.J.P. & Hinton, G.E., 2008. Visualizing
High-Dimensional Data Using t-SNE. Journal of Machine Learning Research, 9,
pp.2579-2605.
}
\seealso{
\code{\link[Rtsne:Rtsne]{Rtsne::Rtsne()}} in \pkg{Rtsne}.

Other seriation: 
\code{\link{register_DendSer}()},
\code{\link{register_GA}()},
\code{\link{register_optics}()},
\code{\link{register_umap}()},
\code{\link{registry_seriate}},
\code{\link{seriate}()}
}
\concept{seriation}
\keyword{cluster}
\keyword{optimize}

\name{Zoo}
\alias{Zoo}
\docType{data}
\title{Zoo Data Set}
\description{
A database containing characteristics of different animals.
The database was created and donated by Richard S. Forsyth
and is available from the UCI Machine Learning Repository 
(Newman et al, 1998).
}
\usage{data("Zoo")}
\format{
  A data frame with 101 observations on the following 17 variables.
  \describe{
    \item{\code{hair}}{\{0, 1\}}
    \item{\code{feathers}}{\{0, 1\}}
    \item{\code{eggs}}{\{0, 1\}}
    \item{\code{milk}}{\{0, 1\}}
    \item{\code{airborne}}{\{0, 1\}}
    \item{\code{aquatic}}{\{0, 1\}}
    \item{\code{predator}}{\{0, 1\}}
    \item{\code{toothed}}{\{0, 1\}}
    \item{\code{backbone}}{\{0, 1\}}
    \item{\code{breathes}}{\{0, 1\}}
    \item{\code{venomous}}{\{0, 1\}}
    \item{\code{fins}}{\{0, 1\}}
    \item{\code{legs}}{Numeric (set of values: \{0, 2, 4, 5, 6, 8\})}
    \item{\code{tail}}{\{0, 1\}}
    \item{\code{domestic}}{\{0, 1\}}
    \item{\code{catsize}}{\{0, 1\}}
    \item{\code{class}}{a factor with levels \code{amphibian} \code{bird} \code{fish} \code{insect} \code{invertebrate} \code{mammal} \code{reptile}}
  }
}
%\details{
%}
\source{
D.J. Newman, S. Hettich, C.L. Blake and C.J. Merz (1998):
UCI Repository of machine learning databases,
\url{https://www.ics.uci.edu/~mlearn/MLRepository.html},
University of California, Irvine, Dept. of Information and Computer Sciences.
}
\examples{
data("Zoo")
x <- scale(Zoo[, -17])


d <- dist(x)
pimage(d)

order <- seriate(d, method = "tsp")
pimage(d, order)
}
\keyword{datasets}

\name{permutation_matrix}
\alias{permutation_matrix2vector}
\alias{permutation_vector2matrix}
\title{Conversion Between Permutation Vector and Permutation Matrix}
\description{
Converts between permutation vectors and matrices.
}
\usage{
permutation_matrix2vector(x)
permutation_vector2matrix(x)
}
\arguments{
  \item{x}{A permutation vector (any object that can be converted into a 
    permutation vector, e.g., a integer vector or a \code{hclust} object) or a 
    matrix representing a permutation. Arguments are checked.} 
}
\seealso{
\code{\link{ser_permutation}},
\code{\link{permute}}
}
\author{Michael Hahsler}
\examples{
## create a random permutation vector
pv <- sample(1:5)
pv
  
## convert into a permutation matrix
pm <- permutation_vector2matrix(pv)
pm
  
## convert back  
permutation_matrix2vector(pm)
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Chameleon.R
\docType{data}
\name{Chameleon}
\alias{Chameleon}
\alias{chameleon}
\alias{chameleon_ds4}
\alias{chameleon_ds5}
\alias{chameleon_ds7}
\alias{chameleon_ds8}
\title{2D Data Sets used for the CHAMELEON Clustering Algorithm}
\format{
\code{chameleon_ds4}: The format is a 8,000 x 2 data.frame.

\code{chameleon_ds5}: The format is a 8,000 x 2 data.frame.

\code{chameleon_ds7}: The format is a 10,000 x 2 data.frame.

\code{chameleon_ds8}: The format is a 8,000 x 2 data.frame.
}
\description{
Several 2D data sets created to evaluate the CHAMELEON clustering algorithm in
the paper by Karypis et al (1999).
}
\examples{
data(Chameleon)

plot(chameleon_ds4, cex = .1)
plot(chameleon_ds5, cex = .1)
plot(chameleon_ds7, cex = .1)
plot(chameleon_ds8, cex = .1)
}
\references{
Karypis, G., EH. Han, V. Kumar (1999): CHAMELEON: A Hierarchical
Clustering Algorithm Using Dynamic Modeling, \emph{IEEE Computer,}
\strong{32}(8): 68--75.
\doi{10.1109/2.781637}
}
\keyword{datasets}

\name{Wood}
\alias{Wood}
\docType{data}
\title{Gene Expression Data for Wood Formation in Poplar Trees}
\description{
A data matrix containing a sample of the normalized gene expression data for
6 locations in the stem of Popla trees 
published in the study by Herzberg et al (2001).
The sample of 136 genes selected by 
Caraux and Pinloche (2005).
}
\usage{data(Wood)}
\format{
  The format is a 136 x 6 matrix.
}
\source{
The data was obtained from 
\url{http://www.atgc-montpellier.fr/permutmatrix/manual/Exemples/Wood/Wood.htm}.
}
\references{
Hertzberg M., H. Aspeborg, J. Schrader, A. Andersson, R.Erlandsson,
K. Blomqvist, R. Bhalerao, M. Uhlen, T. T. Teeri,
J. Lundeberg, Bjoern Sundberg, P. Nilsson and Goeran Sandberg (2001): 
A transcriptional roadmap to wood formation, 
\emph{PNAS,} \bold{98}(25), 14732--14737.

Caraux G. and Pinloche S. (2005):
PermutMatrix: a graphical environment to arrange gene expression profiles in optimal linear order, 
\emph{Bioinformatics,} \bold{21}(7) 1280--1281. 
}
\examples{
data(Wood)
head(Wood)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dissplot.R, R/ggdissplot.R
\name{dissplot}
\alias{dissplot}
\alias{plot.reordered_cluster_dissimilarity_matrix}
\alias{print.reordered_cluster_dissimilarity_matrix}
\alias{ggdissplot}
\title{Dissimilarity Plot}
\usage{
dissplot(
  x,
  labels = NULL,
  method = "Spectral",
  control = NULL,
  lower_tri = TRUE,
  upper_tri = "average",
  diag = TRUE,
  cluster_labels = TRUE,
  cluster_lines = TRUE,
  reverse_columns = FALSE,
  options = NULL,
  ...
)

\method{plot}{reordered_cluster_dissimilarity_matrix}(
  x,
  lower_tri = TRUE,
  upper_tri = "average",
  diag = TRUE,
  options = NULL,
  ...
)

\method{print}{reordered_cluster_dissimilarity_matrix}(x, ...)

ggdissplot(
  x,
  labels = NULL,
  method = "Spectral",
  control = NULL,
  lower_tri = TRUE,
  upper_tri = "average",
  diag = TRUE,
  cluster_labels = TRUE,
  cluster_lines = TRUE,
  reverse_columns = FALSE,
  ...
)
}
\arguments{
\item{x}{an object of class \link{dist}.}

\item{labels}{\code{NULL} or an integer vector of the same length as
rows/columns in \code{x} indicating the cluster membership for each object
in \code{x} as consecutive integers starting with one. The labels are used
to reorder the matrix.}

\item{method}{A single character string indicating the seriation method used
to reorder the clusters (inter cluster seriation) as well as the objects
within each cluster (intra cluster seriation).  If different algorithms for
inter and intra cluster seriation are required, \code{method} can be a
\code{list} of two named elements (\code{inter_cluster} and
\code{intra_cluster} each containing the name of the respective seriation
method. Use \code{\link[=list_seriation_methods]{list_seriation_methods()}} with \code{kind = "dist"} to find available algorithms.

Set method to \code{NA} to plot the matrix as is (no or, if cluster labels
are supplied, only coarse seriation). For intra cluster reordering with the
special method \code{"silhouette width"} is available (for \code{dissplot()}
only). Objects in clusters are then ordered by silhouette width (from
silhouette plots).  If no \code{method} is given, the default method of
\code{\link[=seriate.dist]{seriate.dist()}} is used.

A third list element (named \code{aggregation}) can be added to control how
inter cluster dissimilarities are computed from from the given dissimilarity
matrix. The choices are \code{"avg"} (average pairwise dissimilarities;
average-link), \code{"min"} (minimal pairwise dissimilarities; single-link),
\code{"max"} (maximal pairwise dissimilarities; complete-link), and
\code{"Hausdorff"} (pairs up each point from one cluster with the most
similar point from the other cluster and then uses the largest dissimilarity
of paired up points).}

\item{control}{a list of control options passed on to the seriation
algorithm.  In case of two different seriation algorithms, \code{control}
can contain a list of two named elements (\code{inter_cluster} and
\code{intra_cluster}) containing each a list with the control options for
the respective algorithm.}

\item{upper_tri, lower_tri, diag}{a logical indicating whether to show the upper triangle, the
lower triangle or the diagonal of the distance matrix. The string "average" can also be used
to display within and between cluster averages in the two triangles.}

\item{cluster_labels}{a logical indicating whether to display cluster labels
in the plot.}

\item{cluster_lines}{a logical indicating whether to draw lines to separate
clusters.}

\item{reverse_columns}{a logical indicating if the clusters are displayed on
the diagonal from north-west to south-east (\code{FALSE}; default) or from
north-east to south-west (\code{TRUE}).}

\item{options}{a list with options for plotting the matrix (\code{dissplot}
only).
\itemize{
\item \code{plot} a logical indicating if a plot should
be produced.  if \code{FALSE}, the returned object can be plotted later
using the function \code{plot} which takes as the second argument a list of
plotting options (see \code{options} below).
\item \code{silhouettes} a logical indicating whether to include a silhouette plot
(see Rousseeuw, 1987).
\item \code{threshold} a numeric. If used, only plot distances
below the threshold are displayed. Consider also using \code{zlim} for this
purpose.
\item \code{col} colors used for the image plot.
\item \code{key} a logical indicating whether to place a color key below the plot.
\item \code{zlim} range of values to display (defaults to range \code{x}).
\item \code{axes} \code{"auto"} (default; enabled for less than 25 objects), \code{"y"} or \code{"none"}.
\item \code{main} title for the plot.
\item \code{newpage} a logical indicating whether to start plot on a new page
(see \code{\link[=grid.newpage]{grid.newpage()}}.
\item \code{pop} a logical indicating whether to pop the created viewports?
(see package \pkg{grid})
\item \code{gp}, \code{gp_lines}, \code{gp_labels} objects of class \code{gpar} containing graphical parameters for the plot
lines and labels (see \code{\link[=gpar]{gpar()}}.
}}

\item{...}{\code{dissplot()}: further arguments are added to \code{options}.
\code{ggdissplot()} further arguments are passed on to \code{\link[=ggpimage]{ggpimage()}}.}
}
\value{
\code{dissplot()} returns an invisible object of class
\code{cluster_proximity_matrix} with the following elements:
\item{order}{\code{NULL} or integer vector giving the order used to plot \code{x}.}
\item{cluster_order}{ \code{NULL} or integer vector giving the order of the
clusters as plotted.}
\item{method}{ vector of character strings indicating
the seriation methods used for plotting \code{x}.}
\item{k}{ \code{NULL} or integer scalar giving the number of clusters generated.}
\item{description}{ a \code{data.frame} containing information (label, size, average
intra-cluster dissimilarity and the average silhouette) for the clusters as
displayed in the plot (from top/left to bottom/right).}

This object can be used for plotting via \code{plot(x, options = NULL, ...)},
where \code{x} is the object and \code{options} contains a list with
plotting options (see above).

\code{ggdissplot()} returns a ggplot2 object representing the plot.

The plot description as an object of class \code{reordered_cluster_dissimilarity_matrix}.
}
\description{
Visualizes a dissimilarity matrix using seriation and matrix shading using
the method developed by Hahsler and Hornik (2011). Entries with lower
dissimilarities (higher similarity) are plotted darker. Dissimilarity plots
can be used to uncover hidden structure in the data and judge cluster
quality.
}
\details{
The plot can also be used to visualize cluster quality (see Ling 1973).
Objects belonging to the same cluster are displayed in consecutive order.
The placement of clusters and the within cluster order is obtained by a
seriation algorithm which tries to place large similarities/small
dissimilarities close to the diagonal. Compact clusters are visible as dark
squares (low dissimilarity) on the diagonal of the plot. Additionally, a
Silhouette plot (Rousseeuw 1987) is added. This visualization is similar to
CLUSION (see Strehl and Ghosh 2002), however, allows for using arbitrary
seriating algorithms.

\strong{Note:} Since \code{\link[=pimage]{pimage()}} uses \pkg{grid}, it should not be mixed
with base R primitive plotting functions.
}
\examples{
data("iris")

# shuffle rows
x_iris <- iris[sample(seq(nrow(iris))), -5]
d <- dist(x_iris)

# Plot original matrix
dissplot(d, method = NA)

# Plot reordered matrix using the nearest insertion algorithm (from tsp)
dissplot(d, method = "TSP", main = "Seriation (TSP)")

# Cluster iris with k-means and 3 clusters and reorder the dissimality matrix
l <- kmeans(x_iris, centers = 3)$cluster
dissplot(d, labels = l, main = "k-means")

# show only distances as lower triangle
dissplot(d, labels = l, main = "k-means", lower_tri = TRUE, upper_tri = FALSE)

# Use a grid layout to place several plots on a page
library("grid")
grid.newpage()
pushViewport(viewport(layout=grid.layout(nrow = 2, ncol = 2),
    gp = gpar(fontsize = 8)))
pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 1))

# Visualize the clustering (using Spectral between clusters and MDS within)
res <- dissplot(d, l, method = list(inter = "Spectral", intra = "MDS"),
  main = "K-Means + Seriation", newpage = FALSE)

popViewport()
pushViewport(viewport(layout.pos.row = 1, layout.pos.col = 2))

# More visualization options. Note that we reuse the reordered object res!
# color: use 10 shades red-blue, biased towards small distances
plot(res, main = "K-Means + Seriation (red-blue + biased)",
    col= bluered(10, bias = .5), newpage = FALSE)

popViewport()
pushViewport(viewport(layout.pos.row = 2, layout.pos.col = 1))

# Threshold (using zlim) and cubic scale to highlight differences
plot(res, main = "K-Means + Seriation (cubic + threshold)",
    zlim = c(0, 2), col = grays(100, power = 3), newpage = FALSE)

popViewport()
pushViewport(viewport(layout.pos.row = 2, layout.pos.col = 2))

# Use gray scale with logistic transformation
plot(res, main = "K-Means + Seriation (logistic scale)",
  col = gray(
    plogis(seq(max(res$x_reordered), min(res$x_reordered), length.out = 100),
      location = 2, scale = 1/2, log = FALSE)
    ),
  newpage = FALSE)

popViewport(2)

# The reordered_cluster_dissimilarity_matrix object
res
names(res)

## --------------------------------------------------------------------
## ggplot-based dissplot
if (require("ggplot2")) {

library("ggplot2")

# Plot original matrix
ggdissplot(d, method = NA)

# Plot seriated matrix
ggdissplot(d, method = "TSP") +
  labs(title = "Seriation (TSP)")

# Cluster iris with k-means and 3 clusters
l <- kmeans(x_iris, centers = 3)$cluster

ggdissplot(d, labels = l) +
  labs(title = "K-means + Seriation")

# show only lower triangle
ggdissplot(d, labels = l, lower_tri = TRUE, upper_tri = FALSE) +
  labs(title = "K-means + Seriation")

# No lines or cluster labels and add a label for the color key (fill)
ggdissplot(d, labels = l, cluster_lines = FALSE, cluster_labels = FALSE) +
  labs(title = "K-means + Seriation", fill = "Distances\n(Euclidean)")

# Diverging color palette with manual set midpoint and different seriation methods
ggdissplot(d, l, method = list(inter = "Spectral", intra = "MDS")) +
  labs(title = "K-Means + Seriation", subtitle = "biased color scale") +
  scale_fill_gradient2(midpoint = median(d))

# Use manipulate scale using package scales
library("scales")

# Threshold (using limit and na.value) and cubic scale to highlight differences
cubic_dist_trans <- trans_new(
  name = "cubic",
  # note that we have to do the inverse transformation for distances
  trans = function(x) x^(1/3),
  inverse = function(x) x^3
)

ggdissplot(d, l, method = list(inter = "Spectral", intra = "MDS")) +
  labs(title = "K-Means + Seriation", subtitle = "cubic + biased color scale") +
  scale_fill_gradient(low = "black", high = "white",
    limit = c(0,2), na.value = "white",
    trans = cubic_dist_trans)

# Use gray scale with logistic transformation
logis_2_.5_dist_trans <- trans_new(
  name = "Logistic transform (location, scale)",
  # note that we have to do the inverse transformation for distances
  trans = function(x) plogis(x, location = 2, scale = .5, log = FALSE),
  inverse = function(x) qlogis(x, location = 2, scale = .5, log = FALSE),
)

ggdissplot(d, l, method = list(inter = "Spectral", intra = "MDS")) +
  labs(title = "K-Means + Seriation", subtitle = "logistic color scale") +
  scale_fill_gradient(low = "black", high = "white",
    trans = logis_2_.5_dist_trans,
    breaks = c(0, 1, 2, 3, 4))
}
}
\references{
Hahsler, M. and Hornik, K. (2011): Dissimilarity plots: A visual
exploration tool for partitional clustering. \emph{Journal of Computational
and Graphical Statistics,} \bold{10}(2):335--354.
\doi{10.1198/jcgs.2010.09139}

Ling, R.F. (1973): A computer generated aid for cluster analysis.
\emph{Communications of the ACM,} \bold{16}(6), 355--361.
\doi{10.1145/362248.362263}

Rousseeuw, P.J. (1987): Silhouettes: A graphical aid to the interpretation
and validation of cluster analysis. \emph{Journal of Computational and
Applied Mathematics,} \bold{20}(1), 53--65.
\doi{10.1016/0377-0427(87)90125-7}

Strehl, A. and Ghosh, J. (2003): Relationship-based clustering and
visualization for high-dimensional data mining. \emph{INFORMS Journal on
Computing,} \bold{15}(2), 208--230.
\doi{10.1287/ijoc.15.2.208.14448}
}
\seealso{
Other plots: 
\code{\link{VAT}()},
\code{\link{bertinplot}()},
\code{\link{hmap}()},
\code{\link{palette}()},
\code{\link{pimage}()}
}
\author{
Michael Hahsler
}
\concept{plots}
\keyword{cluster}
\keyword{hplot}

\name{register_DendSer}
\alias{register_DendSer}
\alias{DendSer}
\alias{dendser}
\title{Register Seriation Methods from Package DendSer}
\description{
Register the DendSer dendrogram seriation method and
the ARc criterion (Earle and Hurley, 2015).
}
\usage{
register_DendSer()
}
\details{
Registers the method \code{"DendSer"} for \code{seriate}.
DendSer is a fast heuristic for reordering dendrograms developed by
Earle and Hurley (2015) able to use different criteria.
\code{control} for \code{seriate} with
method \code{"DendSer"} accepts the following parameters:
\describe{
\item{\code{"h"} or \code{"method"}}{A dendrogram or a method for
hierarchical clustering (see \code{hclust}). Default: complete-link.}
\item{\code{"criterion"}}{A seriation
criterion to optimize (see \code{list_criterion_methods("dist")}).
Default: \code{"BAR"} (Banded anti-Robinson from with 20\% band width).}
\item{\code{"verbose"}}{print progress information.}
\item{\code{"DendSer_args"}}{additional arguments for \code{DendSer}.}
}

For convenience the following methods (for differnt cost functions)
are also provided:
\code{"DendSer_ARc"} (anti-robinson form),
\code{"DendSer_BAR"} (banded anti-Robinson form),
\code{"DendSer_LS"} (leaf seriation),
\code{"DendSer_PL"} (path length).

Note: Package \pkg{DendSer} needs to be installed.
}
%\value{
%}
\references{
D. Earle, C. B. Hurley (2015): Advances in dendrogram seriation for application
to visualization. \emph{Journal of Computational and Graphical Statistics,}
\bold{24}(1), 1--25.
}
\author{
Michael Hahsler based on code by Catherine B. Hurley and Denise Earle
}
\seealso{
\code{\link{seriate}},
\code{\link[DendSer]{DendSer}} in \pkg{DendSer}.
}
\examples{
\dontrun{
register_DendSer()
list_seriation_methods("dist")

d <- dist(random.robinson(20, pre=TRUE))

## use Banded AR form with default clustering (complete-link)
o <- seriate(d, "DendSer_BAR")
pimage(d, o)

## use different hclust method (Ward) and AR as the cost function for
## dendrogram reordering
o <- seriate(d, "DendSer", control = list(method = "ward.D2", criterion = "AR"))
pimage(d, o)
}
}
\keyword{optimize}
\keyword{cluster}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AAA_seriation-package.R
\docType{package}
\name{seriation-package}
\alias{seriation-package}
\title{seriation: Infrastructure for Ordering Objects Using Seriation}
\description{
Infrastructure for ordering objects with an implementation of several seriation/sequencing/ordination techniques to reorder matrices, dissimilarity matrices, and dendrograms. Also provides (optimally) reordered heatmaps, color images and clustering visualizations like dissimilarity plots, and visual assessment of cluster tendency plots (VAT and iVAT).
}
\section{Key functions}{

\itemize{
\item Seriation: \code{\link[=seriate]{seriate()}}, \code{\link[=criterion]{criterion()}}, \code{\link[=get_order]{get_order()}}, \code{\link[=permute]{permute()}}
\item Visualization: \code{\link[=pimage]{pimage()}}, \code{\link[=bertinplot]{bertinplot()}}, \code{\link[=hmap]{hmap()}}, \code{\link[=dissplot]{dissplot()}}, \code{\link[=VAT]{VAT()}}
}
}

\references{
Michael Hahsler, Kurt Hornik, and Christian Buchta. Getting things in order: An introduction to the R package seriation. Journal of Statistical Software, 25(3):1--34, March 2008. \doi{10.18637/jss.v025.i03}
}
\author{
Michael Hahsler
}

\name{reorder.hclust}
\alias{reorder}
\alias{reorder.hclust}
\title{Reorder Dendrograms using Optimal Leaf Ordering}
\description{
Reorder method for dendrograms for optimal leaf ordering.
}
\usage{
\method{reorder}{hclust}(x, dist, method = "OLO", ...)
}
\arguments{
  \item{x}{an object of class \code{hclust}.}
  \item{dist}{an object of class \code{dist} with dissimilarities between the
    objects in \code{x}.}
  \item{method}{ a character string with the name of the used measure
    Available are: 
      \code{"OLO"} (optimal leaf ordering; Bar-Joseph et al., 2001) and
      \code{"GW"} (Gruvaeus and Wainer, 1972).}
  \item{...}{ further arguments are currently ignored.}
}
\details{
Minimizes the distance between neighboring objects
(leaf nodes) in the dendrogram by flipping the order of subtrees. 
The algorithm by Gruvaeus and Wainer is implemented in
package \pkg{gclus} (Hurley 2004).
}
\value{
A reordered \code{hclust} object.
}
\references{ 
Bar-Joseph, Z., E. D. Demaine, D. K. Gifford, and T. Jaakkola. (2001): Fast
Optimal Leaf Ordering for Hierarchical Clustering.
\emph{Bioinformatics,} \bold{17}(1), 22--29.
  
Gruvaeus, G. and Wainer, H. (1972): Two Additions to Hierarchical Cluster
Analysis,
\emph{British Journal of Mathematical and Statistical Psychology,}
\bold{25}, 200--206.
  
Hurley, Catherine B. (2004): Clustering Visualizations of Multidimensional
Data.
\emph{Journal of Computational and Graphical Statistics,}
\bold{13}(4), 788--806.
}
\seealso{
\code{\link[gclus]{reorder.hclust}}
}
\author{Michael Hahsler}
\examples{
## cluster European cities by distance
data("eurodist")
d <- as.dist(eurodist)
hc <- hclust(eurodist)
  
## plot original dendrogram and the reordered dendrograms  
plot(hc)  
plot(reorder(hc, d, method = "GW"))  
plot(reorder(hc, d, method = "OLO"))  
}
\keyword{optimize}
\keyword{cluster}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Townships.R
\docType{data}
\name{Townships}
\alias{Townships}
\title{Bertin's Characteristics of Townships}
\format{
A matrix with 16 0-1 variables (columns) indicating the presence
(\code{1}) or absence (\code{0}) of characteristics of townships
(rows).
}
\description{
This data contains nine characteristics for 16 townships. The data
set was used by Bertin (1981) to illustrate that the conciseness
of presentation can be improved by seriating the rows and columns.
}
\examples{
data("Townships")

## original data
pimage(Townships)
criterion(Townships)

## seriated data
order <- seriate(Townships, method = "BEA", control = list(rep = 5))
pimage(Townships, order)
criterion(Townships, order)
}
\references{
Bertin, J. (1981): \emph{Graphics and Graphic Information Processing}. Berlin, Walter de Gruyter.
}
\author{
Michael Hahsler
}
\keyword{datasets}

\name{permutation}
\alias{ser_permutation}
\alias{permutation}
\alias{print.ser_permutation}
\alias{c.ser_permutation}
\alias{"[.ser_permutation"}
\title{Class ser_permutation -- A Collection of 
Permutation Vectors for Seriation}
\description{
The class \code{ser_permutation} is a collection of permutation vectors
(see class \code{ser_permutation_vector}), one for each dimension (mode) 
of the data to be permuted.
}
\usage{
## constructor
ser_permutation(x, \ldots)
}
\arguments{
  \item{x}{an object of class \code{ser_permutation_vector} or
      any object which can be converted into
      a object of class \code{ser_permutation} (e.g. an integer
      vector).}
  \item{\ldots}{permutation vectors for further dimensions}
    }
\details{
The basic functions \code{print}, \code{"["}, \code{"[["} and \code{c}
are provided.
}
\value{
An object of class \code{ser_permutation}.
}
\seealso{
\code{\link{ser_permutation_vector}},
\code{\link{get_order}},
\code{\link{get_permutation_matrix}}
}
\author{Michael Hahsler}
\examples{
o <- ser_permutation(1:5, 10:1)
o

## length (number of dimensions)
length(o)

## get permutation vector for 2nd dimension
get_order(o, 2)

## reverse dimensions
o[2:1]

## combine
o <- c(o, ser_permutation(1:15))
o

## get an individual permutation
o[[2]]

## reverse the order of a permutation
o[[2]] <- rev(o[[2]])
get_order(o,2)
}
\keyword{classes}% at least one, from doc/KEYWORDS

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/register_umap.R
\name{register_umap}
\alias{register_umap}
\alias{umap}
\title{Register Seriation Based on 1D UMAP}
\usage{
register_umap()
}
\value{
Nothing.
}
\description{
Use uniform manifold approximation and projection (UMAP) to embed the data
on the number line and create a order for \code{\link[=seriate]{seriate()}}.
}
\details{
Registers the method \code{"umap"} for \code{\link[=seriate]{seriate()}}. This method applies
1D UMAP to data represented by a distance matrix and extracts the order from
the 1D embedding.

\bold{Note:} Package \pkg{umap} needs to be installed.
}
\examples{

\dontrun{
register_umap()
get_seriation_method("dist", "umap")

d <- dist(random.robinson(50, pre=TRUE, noise=.1))

o <- seriate(d, method = "umap")
pimage(d, o)
}

}
\references{
McInnes, L, Healy, J, UMAP: Uniform Manifold Approximation and
Projection for Dimension Reduction, ArXiv e-prints 1802.03426, 2018
}
\seealso{
\code{\link[umap:umap]{umap::umap()}} in \pkg{umap}.

Other seriation: 
\code{\link{register_DendSer}()},
\code{\link{register_GA}()},
\code{\link{register_optics}()},
\code{\link{register_tsne}()},
\code{\link{registry_seriate}},
\code{\link{seriate}()}
}
\concept{seriation}
\keyword{cluster}
\keyword{optimize}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Irish.R
\docType{data}
\name{Irish}
\alias{Irish}
\title{Irish Referendum Data Set}
\format{
The format is a 41 x 9 matrix. Two values are missing.
}
\source{
The data was kindly provided by Guenter Sawitzki.
}
\description{
A data matrix containing the results of 8 referenda for 41 Irish communities
used in Falguerolles et al (1997).
}
\details{
Column 6 contains the size of the Electorate in 1992.
}
\examples{

data(Irish)

}
\references{
de Falguerolles, A., Friedrich, F., Sawitzki, G. (1997): A
Tribute to J. Bertin's Graphical Data Analysis. In: Proceedings of the
SoftStat '97 (Advances in Statistical Software 6), 11--20.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AAA_color_palette.R
\name{palette}
\alias{palette}
\alias{bluered}
\alias{palette,}
\alias{colors}
\alias{greenred}
\alias{reds}
\alias{blues}
\alias{greens}
\alias{greys}
\alias{grays}
\title{Different Useful Color Palettes}
\usage{
bluered(n = 100, bias = 1, power = 1, ...)

greenred(n = 100, bias = 1, power = 1, ...)

reds(n = 100, bias = 1, power = 1, ...)

blues(n = 100, bias = 1, power = 1, ...)

greens(n = 100, bias = 1, power = 1, ...)

greys(n = 100, bias = 1, power = 1, ...)

grays(n = 100, bias = 1, power = 1, ...)
}
\arguments{
\item{n}{number of different colors produces.}

\item{bias}{a positive number. Higher values give more widely spaced colors
at the high end.}

\item{power}{used to control how chroma and luminance is increased (1 =
linear, 2 = quadratic, etc.)}

\item{...}{further parameters are passed on to \code{\link[=sequential_hcl]{sequential_hcl()}}
or \code{\link[=diverging_hcl]{diverging_hcl()}}.}
}
\value{
A vector with \code{n} colors.
}
\description{
Defines several color palettes for \code{\link[=pimage]{pimage()}}, \code{\link[=dissplot]{dissplot()}} and
\code{\link[=hmap]{hmap()}}.
}
\details{
The color palettes are created with \code{\link[=sequential_hcl]{sequential_hcl()}} and
\code{\link[=diverging_hcl]{diverging_hcl()}} from package \pkg{colorspace}.

The two sequential palettes are: \code{reds()} and \code{grays()} (or
\code{greys()}).

The two diverging palettes are: \code{bluered()} and \code{greenred()}.
}
\examples{
m <- outer(1:10,1:10)
m

pimage(m)
pimage(m, col = greys(100, power = 2))
pimage(m, col = greys(100, bias = 2))
pimage(m, col = bluered(100))
pimage(m, col = bluered(100, power = .5))
pimage(m, col = bluered(100, bias = 2))
pimage(m - 25, col = greenred(20, bias = 2))

## choose your own color palettes
library(colorspace)
hcl_palettes(plot = TRUE)

## blues (with 20 shades)
pimage(m,
  col = colorspace::sequential_hcl(20, "Blues", rev = TRUE))
## blue to green (aka "Cork")
pimage(m,
  col = colorspace::diverging_hcl(100, "Cork"))
}
\seealso{
Other plots: 
\code{\link{VAT}()},
\code{\link{bertinplot}()},
\code{\link{dissplot}()},
\code{\link{hmap}()},
\code{\link{pimage}()}
}
\author{
Michael Hahsler
}
\concept{plots}
\keyword{hplot}
